/*****************************************************************************/

/*
 *      main.c -- REALmagic simple windowing application in GTK
 *
 *      Copyright (C) 2000 Sigma Designs
 *                    written by Nicolas Vignal <nicolas_vignal@sdesigns.com>
 *                    modified by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/

#include "define.h"

#include "interface.h"

Display *dpy;
int scr;

void loadconfig()
{
	DWORD FMPreturn;
	DWORD xresolution;
	DWORD yresolution;
	DWORD depth;
	DWORD freq;
	
	G_MESSAGE("Ok clicked\nLoad parameters");

	getres(&xresolution,&yresolution,&depth,&freq);
	FMPreturn=LoadParamForThisResolution(xresolution,yresolution,depth,freq);
	
	G_MESSAGE("Load parameters: %ld",FMPreturn);	
	
	if (FMPreturn==2) {
		printf("Please, make the autocalibration for this resolution before launching miniplayer\n");
		exit(0);
	}
}

int main (int argc, char *argv[])
{
	GtkWidget *window;
	
	GtkWidget *drawingarea;
	GdkColor keycolor;
	GdkColormap *colormap;
	DWORD FMPreturn;

	dpy = XOpenDisplay (NULL);
	if (!dpy) exit(1);
  
	scr=DefaultScreen(dpy);
	
	gtk_init (&argc, &argv);
	
	window = create_window ();
	
	gtk_widget_show (window);

	drawingarea=GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (window), "drawingarea"));

	// Get drawingarea colormap
	colormap = gdk_window_get_colormap (drawingarea->window);
	keycolor.red = 0;
	keycolor.green = 10280;
	keycolor.blue = 0;

	// Allocate color
	gdk_color_alloc (colormap, &keycolor);
	
	// Set drawingarea background color
	gdk_window_set_background (drawingarea->window, &keycolor);

	// Clear drawingarea
	gdk_window_clear (drawingarea->window);
	
	// Connect to the server
	FMPConnectToMessageQueue();
	
	// Test the VGA cable
	if (!FMPAnalogOverlay(FMP_ANALOG_OVERLAY_GET,FMP_ANALOG_OVERLAY_ATTRIBUTE_OVERLAY_CABLE_PRESENT,
			      UNUSED_ARG,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG)) {
		fprintf(stderr,"Please attach VGA cable.\n");
		exit(1);
	}
	
	// Switch output to VGA
	FMPreturn = FMPSet (FMPI_VIDEOOUT, FMPV_VIDEOOUT_VGA);

	// Set overlay mode
	FMPreturn=FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_MODE,2,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);
	
	// Set key color
	FMPreturn=FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_VGAKEY,0x004000,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);

	loadconfig();
	
	gtk_main ();

	// Switch back overlay mode
	FMPreturn=FMPAnalogOverlay(FMP_ANALOG_OVERLAY_SET,FMP_ANALOG_OVERLAY_ATTRIBUTE_MODE,0,UNUSED_ARG,UNUSED_ARG,UNUSED_ARG);

	// Switch back output to TV
	FMPreturn = FMPSet (FMPI_VIDEOOUT, FMPV_VIDEOOUT_TV);
	
	return 0;
}
