/*
 * $Workfile: geodefb.c $
 * $Revision: 67 $
 *
 * File Contents: This file contains the main functions of the Geode 
 *                frame buffer device driver, which includes functions 
 *                init, setup and ioctl etc.
 * [Notes:]	      All the geode frame buffer driver functions can be 
 *                invoked through fbmem.c file functions
 *
 * Project:       Geode Frame buffer device driver
 */

/* 
 * NSC_LIC_ALTERNATIVE_PREAMBLE
 *
 * Revision 1.0
 *
 * National Semiconductor Alternative GPL-BSD License
 *
 * National Semiconductor Corporation licenses this software 
 * ("Software"):
 *
 * Geode frame buffer driver
 *
 * under one of the two following licenses, depending on how the 
 * Software is received by the Licensee.
 * 
 * If this Software is received as part of the Linux Framebuffer or
 * other GPL licensed software, then the GPL license designated 
 * NSC_LIC_GPL applies to this Software; in all other circumstances 
 * then the BSD-style license designated NSC_LIC_BSD shall apply.
 *
 * END_NSC_LIC_ALTERNATIVE_PREAMBLE */

/* NSC_LIC_BSD
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * Geode frame buffer driver
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * END_NSC_LIC_BSD */

/* NSC_LIC_GPL
 *
 * National Semiconductor Corporation Gnu General Public License for 
 *
 * Geode frame buffer driver
 *
 * (GPL License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted under the terms of the GNU General 
 * Public License as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version  
 *
 * In addition to the terms of the GNU General Public License, neither 
 * the name of the National Semiconductor Corporation nor the names of 
 * its contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE, 
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. See the GNU General Public License for more details. 
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this file; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * END_NSC_LIC_GPL */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/malloc.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/console.h>
#include <linux/selection.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/devfs_fs_kernel.h>

#include <asm/io.h>
#include <asm/mtrr.h>

#include <video/fbcon.h>
#include <video/fbcon-cfb8.h>
#include <video/fbcon-cfb16.h>
#include <video/fbcon-cfb32.h>

#include <asm/uaccess.h>

#include "geodeioctl.h"

#include "gfx_rtns.h"
#include "gfx_defs.h"
#include "gfx_regs.h"

#include "panel.h"
#include "ddc.h"

#include "galproto.h"

#define PM 1
#ifdef PM
#include <linux/pm.h>
static int geode_pm_callback(struct pm_dev *dev, pm_request_t rqst, void *data);
#endif

/* Geode Frame buffer driver version */
#define NSCFB_MAJOR 1
#define NSCFB_MINOR 1
#define NSCFB_SUB 5
#define PUT_TVO      0x4701 /* enable debug output? */

#define GEODEFBDEBUG 0

#if GEODEFBDEBUG
#define DPRINTK(fmt, args...) printk(KERN_DEBUG "%s: " fmt, __FUNCTION__ , ## args)
#else
#define DPRINTK(fmt, args...)
#endif

/* disable runtime assertions? */

#define GEODE_NDEBUG 0

/* debugging assertions */
#if GEODE_NDEBUG
#define assert(expr)\
	if(!(expr)) {\
		printk( "Assertion failed! %s,%s,%s,line=%d\n",\
		#expr,__FILE__,__FUNCTION__,__LINE__);\
	}
#else
#define assert(expr)
#endif

static struct geodefb_par current_par;
static char default_fontname[40] = { 0 };
static int current_par_valid = 0;
static int currcon = 0;
static struct display disp;
static struct fb_info_gen gen;
static int is_tv_set=-1;
static int panel_selected=0;
static Pnl_PanelParams PanelParam;
static int ddc_selected=0;
static unsigned int ddc_bpp=8;	/* default ddc depth */
static unsigned int compression_enabled=1; /* default compression enabled */

static int Panel_Support_From_Bios=1; /*defaults to bios support */
static int FPBX, FPBY, FPBD, FPBF;

static unsigned int GeodeMem_phys;
static unsigned int GeodeRegs_phys;
static unsigned int GeodeRegs_phys_size;
static unsigned int GeodeMem_virt;
static unsigned int GeodeSize;
static int blit_maybe_busy = 0;
static int Sxres, Syres, Sbpp, Shz, SDisplayType;

/*  dynamic offsets parameters */

struct TVO_scan 
{
  int TV_Overscan_On;
  int TVOx;
  int TVOy;
  int TVOw;
  int TVOh;
};


#define arraysize(x) (sizeof(x)/sizeof(*(x)))

struct geodefb_par 
{
	int xres;
	int yres;
	int xres_virtual;
	int yres_virtual;
	int bpp;
	int refresh;
	int accel;
	unsigned long left_margin; /* x offset */
	unsigned long right_margin;/* width */
	unsigned long upper_margin; /* y offset */
	unsigned long lower_margin; /* height */
	unsigned long hsync_len; /* flag */
};

#ifdef FBCON_HAS_CFB16
static union 
{
	u16 cfb16[16];
} fbcon_cmap;
#endif

#ifdef FBCON_HAS_CFB32
static union 
{
	u32 cfb32[16];
} fbcon_cmap;
#endif

/*  Frame Buffer Name   */
static char geodefb_name[16] = "Geode FB";

static struct 
{
	unsigned short red;
	unsigned short green;
	unsigned short blue;
} Geode_palette [256];
	
#define FB_ACCEL_NSC_GEODE   40

typedef struct _MemOffset 
{
	unsigned short xres;
	unsigned short yres;
	unsigned short bpp;
	unsigned long CBOffset;
	unsigned short CBPitch;
	unsigned short CBSize;
	unsigned long CurOffset;
	unsigned long OffScreenOffset;

} MemOffset_t;

/*
 * predefined memory address for compression and cursor offsets
 * if COMPRESSION enabled.
 */
static struct TVO_scan defaultTVO;

static MemOffset_t GeodeMemOffset[] = 
{
	/* CRT and TFT modes */

	{640,  480,  8,  640,      1024, 272, 0x78000,  0x78100},
	{640,  480,  16, 1280,     2048, 272, 0x610,    0xF0000},
	{640,  480,  32, 2560,     4096, 272, 0xB10,    0x1E0000},
	{800,  600,  8,  800,      1024, 224, 0x96000,  0x96100},
	{800,  600,  16, 1600,     2048, 272, 0x12C000, 0x12C100},
	{800,  600,  32, 3200,     4096, 272, 0xD90,    0x258000},
	{1024, 768,  8,  0xC0000,  272,  272, 0xF3000,  0xF3100},
	{1024, 768,  16, 0x180000, 272,  272, 0x1B3000, 0x1B3100},
	{1024, 768,  32, 0x300000, 272,  272, 0x303000, 0x303100},
	{1152, 864,  8,  1152,     2048, 272, 0x590,    0x1B0000},
	{1152, 864,  16, 2304,     4096, 272, 0xA10,    0x360000},
	{1280, 1024, 8,  1280,     2048, 272, 0x610,    0x200000},
	{1280, 1024, 16, 2560,     4096, 272, 0xB10,    0x400000},
	{1600, 1200, 8,  1600,     2048, 272, 0x750,    0x258000},
	{1600, 1200, 16, 3200,     4096, 272, 0xD90,    0x4B0000},

	/* PAL TV modes */

	{704,  576,  16, 1408,     2048, 272, 0x690,    0x120000},
	{720,  576,  16, 1440,     2048, 272, 0x6B0,    0x120000},
	{768,  576,  16, 1536,     2048, 256, 0x700,    0x120000},

	/* NTSC TV modes */

	{704,  480,  16, 1408,     2048, 272, 0x690,    0xF0000},
	{720,  480,  16, 1440,     2048, 272, 0x6B0,    0xF0000}
};

#define NUM_TOTAL_OFFSET (arraysize(GeodeMemOffset))

/* display types */

#define	Display_CRT              0x01
#define	Display_FP               0x02
#define	Display_TV_NTSC          0x04
#define	Display_TV_PAL           0x08
#define	Display_TV               0x10
#define	Display_TV_Square_Pixels 0x20
#define	Display_TV_No_Scaling    0x40
#define	Display_TVO              0x80

static int Display_Type = Display_CRT;

/* Predefined Video Modes */

static struct {

	const char *name;
	unsigned int vesa;
	unsigned int Display_Type;
	struct fb_var_screeninfo var;

} geodefb_predefined[] = {

	{ "640x480-8", 0x101, Display_CRT | Display_FP,
     
		{ 	640, 480, 640, 480, 0, 0, 8, 0,{0, 8, 0}, 
			{0, 8, 0}, {0, 8, 0}, {0, 0, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

	{ "800x600-8", 0x103, Display_CRT | Display_FP,

		{ 	800, 600, 800, 600, 0, 0, 8, 0, {0, 8, 0}, 
			{0, 8, 0}, {0, 8, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}

	}, 

	{ "1024x768-8", 0x105, Display_CRT | Display_FP, 

     
		{ 	1024, 768, 1024, 768, 0, 0, 8, 0, {0, 8, 0},
			{0, 8, 0}, {0, 8, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	},

	{ "1152x864-8", 0x190, Display_CRT | Display_FP, 
     
		{	1152, 864, 1152, 864, 0, 0, 8, 0, {0, 8, 0}, 
			{0, 8, 0}, {0, 8, 0}, {0, 0, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

	{ "1280x1024-8", 0x107, Display_CRT | Display_FP, 
     
		{	1280, 1024, 1280, 1024, 0, 0, 8, 0, {0, 8, 0}, 
			{0, 8, 0}, {0, 8, 0}, {0, 0, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

	{ "1600x1200-8", 0x11C, Display_CRT | Display_FP, 
     
		{	1600, 1200, 1600, 1200, 0, 0, 8, 0, {0, 8, 0}, 
			{0, 8, 0}, {0, 8, 0}, {0, 0, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

	{ "640x480-16", 0x111, Display_CRT | Display_FP,
      
		{	640, 480, 640, 480, 0, 0, 16, 0, {11, 5, 0},
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	 },

	{ "800x600-16", 0x114, Display_CRT | Display_FP, 

		{	800, 600, 800, 600, 0, 0, 16, 0,{11, 5, 0}, 
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	},

	{ "1024x768-16", 0x117, Display_CRT | Display_FP, 
   
		{	1024, 768, 1024, 768, 0, 0, 16, 0, {11, 5, 0}, 
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

    { "1152x864-16", 0x192, Display_CRT | Display_FP, 

		{	1152, 864, 1152, 864, 0, 0, 16, 0,{11, 5, 0},
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

    { "1280x1024-16", 0x11A, Display_CRT | Display_FP, 

		{	1280, 1024, 1280, 1024, 0, 0, 16, 0,{11, 5, 0},
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

	{ "1600x1200-16", 0x11E, Display_CRT | Display_FP, 
     
		{	1600, 1200, 1600, 1200, 0, 0, 16, 0,{11, 5, 0}, 
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 

	{ "640x480-32", 0x112, Display_CRT | Display_FP,
      
		{	640, 480, 640, 480, 0, 0, 32, 0, {16, 8, 0},
			{8, 8, 0}, {0, 8, 0}, {24, 8, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	 },

	{ "800x600-32", 0x115, Display_CRT | Display_FP,
      
		{	800, 600, 800, 600, 0, 0, 32, 0, {16, 8, 0},
			{8, 8, 0}, {0, 8, 0}, {24, 8, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	 },

	{ "1024x768-32", 0x118, Display_CRT | Display_FP,
      
		{	1024, 768, 1024, 768, 0, 0, 32, 0, {16, 8, 0},
			{8, 8, 0}, {0, 8, 0}, {24, 8, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	 },

		/* TV Modes to the end of the list */

	{	/* 720 x 576 (PAL) */

	  "PAL-720x576", 0, Display_TV | Display_TV_PAL |
           Display_TV_No_Scaling,

		{	720, 576, 720, 576, 0, 0, 16, 0,{11, 5, 0},
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 50, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}, 


	{	/* 768 x 576 (PAL) */
      
	  "PAL-768x576", 0, Display_TV | Display_TV_PAL |
          Display_TV_Square_Pixels,

		{	768, 576, 768, 576, 0, 0, 16, 0, {11, 5, 0},
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 50, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	},
	{	/* 640 x 480 (NTSC) */

	  "NTSC-640x480", 0,Display_TV | Display_TV_NTSC | 
          Display_TV_Square_Pixels,

		{	640, 480, 640, 480, 0, 0, 16, 0,{11, 5, 0}, 
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0, 
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	},
	{	/* 720 x 480 (NTSC) */
 
	  "NTSC-720x480", 0, Display_TV | Display_TV_NTSC | 
	  Display_TV_No_Scaling, 

		{	720, 480, 720, 480, 0, 0, 16, 0,{11, 5, 0}, 
			{5, 6, 0}, {0, 5, 0}, {0, 0, 0}, 0,
			FB_ACTIVATE_NOW, -1, -1, FB_ACCELF_TEXT, 60, 
			0, 0, 0, 0, 0, 0, 
			FB_SYNC_COMP_HIGH_ACT| FB_SYNC_VERT_HIGH_ACT, 
			FB_VMODE_NONINTERLACED
		}
	}
};

#define NUM_TOTAL_MODES    (arraysize(geodefb_predefined))

/* total number of TV modes */
#define NUM_TV_MODES       4

static unsigned int vesa = ~0;		/* "geode:vesa:xxxxx" */
static unsigned int vfreq = 60;		/* "geode:vfreq:xxxxx" */
static int Geodefb_inverse = 0;

/*
 * Some default modes 
 * 640x480 16Bpp 
 */
#define GEODE_DEFMODE     6  

static struct fb_var_screeninfo geodefb_default;
static int geodefb_default_valid = 0;

static int geodefb_set_cmap(struct fb_cmap *cmap, int kspc,int con,
				struct fb_info *info);

static int geodefb_pan_display(struct fb_var_screeninfo *var,int con,
				struct fb_info *info);

static int geodefb_ioctl(struct inode *inode,struct file *file, u_int cmd,
				u_long arg,int con,struct fb_info *info);
#ifdef MMAP
static int geodefb_mmap(struct fb_info *info, struct file *file,
                         struct vm_area_struct *vma);
#endif

/* Interface used by the world */

/* The geodefb_setup function set the display mode */
int geodefb_setup(char*);

/* Interface to the low level console driver */

/*
 * The geodefb_init function intialiazes the Geode frame buffer
 * driver
 */        
int geodefb_init(void);

/* Text console acceleration */

#ifdef FBCON_HAS_CFB8
static struct display_switch fbcon_geode8;
#endif

#ifdef FBCON_HAS_CFB16
static struct display_switch fbcon_geode16;
#endif

#ifdef FBCON_HAS_CFB32
static struct display_switch fbcon_geode32;
#endif

/* Hardware Specific Routines */
static void Geode_detect(void);

static int Geode_encode_fix(struct fb_fix_screeninfo *fix, const void *par,
				struct fb_info_gen *info);

static int Geode_get_fix(struct fb_fix_screeninfo *fix, int con, 
				struct fb_info *info);

static int do_fb_set_var(struct fb_var_screeninfo *var, int isactive,
		     struct fb_info_gen *info);

static int Geode_set_var(struct fb_var_screeninfo *var, int con, 
				struct fb_info *info);

static int Geode_decode_var(const struct fb_var_screeninfo *var, void *par,
				struct fb_info_gen *info);

static int Geode_get_var(struct fb_var_screeninfo *var, int con, 
				struct fb_info *info);

static int Geode_encode_var(struct fb_var_screeninfo *var, const void *par,
				struct fb_info_gen *info);

static void Geode_get_par(void *par, struct fb_info_gen *info);

static void Geode_set_par(const void *par, struct fb_info_gen *info);
 
static int Geode_getcolreg(unsigned regno, unsigned *red, unsigned *green,
				unsigned *blue, unsigned *transp,
				struct fb_info *info);

static int Geode_setcolreg(unsigned regno, unsigned red, unsigned green,
				unsigned blue, unsigned transp,			
				struct fb_info *info);

static int Geode_blank(int blank_mode, struct fb_info_gen *info);

static void Geode_set_disp(const void *par, struct display *disp,
				struct fb_info_gen *info);
static void Geode_WaitBusy(void);

static void get_ddc_mode(void);	

/* Internal routines */
static int get_video_mode(const char *name);
static void set_mode(int xres, int yres, int bpp, int hz, int DType);
static void rectfill(int sx, int sy, int width, int height, unsigned long bg);

/* powermanagement related functions */
#include "savreg.c"
/*------------- Hardware specific routines --------------------*/
/*-----------------------------------------------------------------
 * Geode_WaitBusy
 *
 * Description:	This function sychronize the Geode graphics engine.
 *		While accessing the geode frame buffer it makes 
 *		the graphics engine idle.
 *  parameters:	none
 *      return:	none
 *---------------------------------------------------------------*/
static void Geode_WaitBusy(void)
{
	gfx_wait_until_idle();
	blit_maybe_busy = 0;
}

/*-----------------------------------------------------------------
 * get_stride 
 *
 * Description:	This function gets the stride (line lenth) of the screen.
 *	 
 *  parameters:
 *	  xres: X-resolution of the display screen.
 *	  yres: Y-resolution of the display screen.
 *	   bpp: This gives the bits-per-pixel of the diplay screen. 
 *      return:	It returns the screen line lenth.
 *---------------------------------------------------------------*/
static int get_stride(int xres, int yres, int bpp)
{
	int line_delta = xres * (bpp >> 3);

	if(line_delta > 2048) line_delta = 4096;
	else if(line_delta > 1024) line_delta = 2048;
	else line_delta = 1024;
	return(line_delta);
}

/*-----------------------------------------------------------------
 * is_tv_enabled 
 *
 * Description:	This function checks whether tv is enabled or not.
 *  parameters:	none
 *      return:	If tv is enabled it returns 0 else it returns 1.
 *---------------------------------------------------------------*/
int is_tv_enabled(void)
{
#if 0 
	is_tv_set = 1; /* for testing */
#else
	if(is_tv_set == -1) 
	{
		gfx_get_tv_enable(&is_tv_set);
	}
#endif
	return(is_tv_set);
}
  
/*-----------------------------------------------------------------
 * Geode_detect
 *
 * Description:	This routine will search the display mode , which 
 *				can be supported or not. If the video mode
 *				was specified on the command line, it will override
 *				the default mode. This is the first function called
 *				in a module to setup the default mode.
 *	parameters:	none
 *		return:	none
 *-----------------------------------------------------------------*/
static void Geode_detect(void)
{
	int i;
	int xres, yres, bpp, hz, ret;

	DPRINTK("%d\n", geodefb_default_valid);

	if (!geodefb_default_valid) 
	{
		int i, found = -1;

		gfx_get_display_mode(&xres, &yres, &bpp, &hz);

		DPRINTK("gfx_get_display_mode %d %d %d %d, %x\n",xres,
			yres, bpp, hz, is_tv_enabled());

		/*
		 * search if we can support this mode, else set default 
		 */

		for (i=0; i < NUM_TOTAL_MODES; i++) 
		{
			if ((xres == geodefb_predefined[i].var.xres) && 
			   (yres == geodefb_predefined[i].var.yres)  &&
           		   (bpp == geodefb_predefined[i].var.bits_per_pixel))
			{
				if((gfx_detect_video() == GFX_VID_SC1200) && (is_tv_enabled() == 1))
				{
					if (geodefb_predefined[i].Display_Type & Display_TV) 
					{
						found = i;
						break;
             		}
				} 
				else 
				{
					found = i;
					break;
				}
			}
		}
		if (found >= 0) 
		{
			geodefb_default = geodefb_predefined[found].var;
			Display_Type = geodefb_predefined[found].Display_Type;
		}
		else 
		{
			/* Default mode */
			DPRINTK("%s", "Default mode\n");
			geodefb_default = geodefb_predefined[GEODE_DEFMODE].var;
			Display_Type = geodefb_predefined[GEODE_DEFMODE].Display_Type;
		}
	}
	else /* do a sanity check on the mode accepted */
	{ 
		xres = geodefb_default.xres; 
		yres = geodefb_default.yres;
		bpp = geodefb_default.bits_per_pixel;
		hz = vfreq;

		/* check if the requested mode can be supported */
		ret = gfx_is_display_mode_supported(xres, yres, bpp, hz);

		DPRINTK("gfx_is_display_mode_supported %d\n", ret);

		if(ret >= 0) /* CRT mode */
		{
			/* Check if a TV mode, if platform supports it */

			if((gfx_detect_video() == GFX_VID_SC1200) && (is_tv_enabled() == 1))
			{
				for(i=(NUM_TOTAL_MODES - NUM_TV_MODES); i<NUM_TOTAL_MODES; i++) 
				{
					if ((xres == geodefb_predefined[i].var.xres)	&&
						(yres == geodefb_predefined[i].var.yres) &&
						(bpp == 
							geodefb_predefined[i].var.bits_per_pixel)) 
					{
						Display_Type = geodefb_predefined[i].Display_Type;
						hz = geodefb_predefined[i].var.pixclock;
						break;
					}   
				} /* for end */
			} 

			/* Enable FP if CRT selected */
			if((Display_Type == Display_CRT) && 
			((PanelParam.PanelPresent == PNL_PANELPRESENT) ||
				Panel_Support_From_Bios))
				Display_Type = Display_FP;
		}
		else if(gfx_detect_video() == GFX_VID_SC1200)
		{
			/* if Not a CRT mode try if its TV mode, 
			 * check if running on SC1200 with TV enabled/support
			 * based system 
			 */
			ret = -1;

			if (is_tv_enabled() == 1) 
			{
				for (i=(NUM_TOTAL_MODES - NUM_TV_MODES); i<NUM_TOTAL_MODES; i++)
				{
					DPRINTK("Test %dx%d %d\n",
						geodefb_predefined[i].var.xres,
						geodefb_predefined[i].var.yres,
						geodefb_predefined[i].var.bits_per_pixel);
					if ((xres == geodefb_predefined[i].var.xres) 
						&& (yres == geodefb_predefined[i].var.yres) 
						&& (bpp == geodefb_predefined[i].var.bits_per_pixel)) 
					{
							ret = i;
							break;
					}
				}
			}
			printk(" return  value after tv_mode test = %d",ret);

			if (ret < 0) 
			{
	   
				printk("Cannot support requested TV mode %dx%d %d @%dHz, %X\n", 
					xres, yres, bpp, hz, Display_Type);
				/* default to 640x480 8 60Hz */
				geodefb_default = geodefb_predefined[0].var;
				vfreq=60;
				Display_Type = geodefb_predefined[0].Display_Type;
				DPRINTK("%d %d %d, %X\n",
					geodefb_default.xres, geodefb_default.yres,
					geodefb_default.bits_per_pixel,	Display_Type);
			}
		} else 
		{ /* wrong mode, fall back */
			/* default to 640x480 8 60Hz */
			geodefb_default = geodefb_predefined[0].var;
			vfreq=60;
			Display_Type = geodefb_predefined[0].Display_Type;
			DPRINTK("%d %d %d, %X\n",
				geodefb_default.xres, geodefb_default.yres,
				geodefb_default.bits_per_pixel,	Display_Type);
		}
	}

	for (i = 0; i < 256; i++) 
	{
		Geode_palette[i].red = i;
		Geode_palette[i].green = i;
		Geode_palette[i].blue = i;
	}

	memset ((char*)GeodeMem_virt, 0, GeodeSize);
	DPRINTK("%d %d %d, %X\n",
		geodefb_default.xres, geodefb_default.yres,
		geodefb_default.bits_per_pixel,	Display_Type);

	geodefb_default.hsync_len &= ~Display_TVO;
	/* If TV Overscan setup by user */
	if(defaultTVO.TV_Overscan_On)
	{
		if( ( defaultTVO.TVOx >= 0 && defaultTVO.TVOy >= 0 ) && 
			( defaultTVO.TVOw > 0 && defaultTVO.TVOh > 0 ) ) 
		{
			if(((defaultTVO.TVOx + defaultTVO.TVOw) <= geodefb_default.xres) && 
				((defaultTVO.TVOy + defaultTVO.TVOh) <= geodefb_default.yres )) 
			{
				geodefb_default.left_margin = defaultTVO.TVOx;
				geodefb_default.right_margin = defaultTVO.TVOw;
				geodefb_default.upper_margin = defaultTVO.TVOy;
				geodefb_default.lower_margin = defaultTVO.TVOh;
				geodefb_default.hsync_len |= Display_TVO;
			}
		}
	}


}

/*-----------------------------------------------------------------
 * Geode_encode_fix
 *
 * Description:	This function should fill in the fix structure
 *				based on the values in the `par' structure.
 *  parameters:
 *		   fix:	It is a pointer to the frame buffer fixed screen 
 *				information.    
 *   	fb_par:	It is a void pointer to the frame buffer parameters. 
 *     	  info:	This info pointer holds the generic frame buffer 
 *				device information.
 *      return: It returns 0 on success. 
 *		On failure it returns erro value.  
 *------------------------------------------------------------------*/
static int Geode_encode_fix(struct fb_fix_screeninfo *fix, const void *fb_par,
			     struct fb_info_gen *info)
 {
	/* parameters are used to get the display mode */ 
	struct geodefb_par *par = (struct geodefb_par *)fb_par;
	int stride = get_stride(par->xres_virtual, 
		par->yres_virtual, par->bpp);

	DPRINTK("%dx%d %d\n", par->xres_virtual, 
		par->yres_virtual, par->bpp);
	assert (par != NULL);
	assert (fix != NULL);
	assert (info != NULL);

	if ((info == NULL) || (fix == NULL) || (par == NULL))
		return(-EINVAL);

	memset(fix, 0, sizeof(struct fb_fix_screeninfo));
	strcpy(fix->id, geodefb_name);


	if((par->hsync_len & Display_TVO) && (is_tv_enabled()==1)) 
	{
		fix->smem_start = (char*) (GeodeMem_phys + 
					(stride * par->upper_margin) + 
					(par->left_margin << ((par->bpp >> 3) -1)));
		fix->smem_len = GeodeSize - 
				((stride * par->upper_margin) + 
				(par->left_margin << ((par->bpp >> 3) -1)));
	} else 
	{
		fix->smem_start = GeodeMem_phys;
		fix->smem_len = GeodeSize;
	}
	fix->mmio_start = GeodeRegs_phys;
	fix->mmio_len = GeodeRegs_phys_size;
    
	fix->type = FB_TYPE_PACKED_PIXELS;
	fix->type_aux = 0;
	if (par->bpp == 8)
		fix->visual = FB_VISUAL_PSEUDOCOLOR;
	else
		fix->visual = FB_VISUAL_TRUECOLOR;
    
	fix->xpanstep = 0;
	fix->ypanstep = 0;
	fix->ywrapstep = 0;
	fix->line_length = stride;
	fix->accel = FB_ACCEL_NSC_GEODE;
#if 0
	{
	int xres, yres, bpp, hz;
		gfx_get_display_mode(&xres, &yres, &bpp, &hz);
		DPRINTK("%d %d %d %d %d\n", xres, yres, 
			bpp, hz, fix->line_length);
	}
#endif
	return(0);
}

/*-----------------------------------------------------------------
 * Geode_get_var
 *
 * Description:	This function get the user defined part of the display.
 *              This function will call the geode  specific functions 
 *              Geode_get_par, Geode_encode_var to get the user defined
 *              part of the display.  
 *
 *  parameters:
 *         var: It is a pointer to the frame buffer variable screen information.     
 *	   con: It specifies the virtual console number.
 *	  info: It is a pointer to the frame buffer information structure.
 *
 *      return: On success it returns the 0 value. 
 *
 *------------------------------------------------------------------*/
	
static int
Geode_get_var(struct fb_var_screeninfo *var, int con, struct fb_info *info)
{
    struct fb_info_gen *info2 = (struct fb_info_gen *)info;
	struct geodefb_par par;
	if (con == -1) {
		Geode_get_par(&par, info2);
		Geode_encode_var(var, &par, info2);
	}
	else
		*var=fb_display[con].var;
	return 0;
}

/*-----------------------------------------------------------------
 * Geode_get_fix
 *
 * Description:	  This function gets the fixed part of the display.
 *                This function calls the Geode_get_par, Geode_decode_var and 
 *                Geode_encode_fix to get the fixed screen information.
 *
 *  parameters:
 *         fix: Pointer to fb_fix_screeninfo which has to be updated
 *              with the data for console indicated by con.
 *         con: The console whose data has to be returned in fix structure.
 *        Info: Pointer to fb_info.
 *
 *      return: It returns 0 on success. 
 *		On failure it returns erro value.  
 *------------------------------------------------------------------*/
static int
Geode_get_fix(struct fb_fix_screeninfo *fix, int con, struct fb_info *info)
{
    struct fb_info_gen *info2 = (struct fb_info_gen *)info;
	struct geodefb_par par;
	if (con == -1)
		Geode_get_par(&par, info2);
	else {
	  int err;
		if ((err=Geode_decode_var(&fb_display[con].var, &par, info2)))
		  return err;
	}
	memset(fix, 0, sizeof(struct fb_fix_screeninfo));
	return Geode_encode_fix(fix, &par, info2);
}

/*-----------------------------------------------------------------
 * do_fb_set_var
 *
 * Description:	This function change the video mode of the frame buffer device. 
 *              This function make use of the  function Geode_decode_var, 
 *              Geode_set_par and Geode_encode_var.
 *
 *  parameters:
 *         var: Pointer to fb_var_screeninfo which has the parameters
 *              to set the hardware. 
 *    isactive: It specifies the current console number.
 *        info: This pointer points to the generic frame buffer
 * 	        device information structure.  
 *
 *      return: On success it returns the 0 value. 
 *              On failure it returns the error value.
 *
 *------------------------------------------------------------------*/	

static int
do_fb_set_var(struct fb_var_screeninfo *var, int isactive,
		     struct fb_info_gen *info)
{
	int err,activate;
	struct geodefb_par par;

	DPRINTK("%d\n", isactive);

	if ((err=Geode_decode_var(var, &par, info)))
		return err;
	activate=var->activate;
	if (((var->activate & FB_ACTIVATE_MASK) == FB_ACTIVATE_NOW) && isactive)
		Geode_set_par(&par, info);
	Geode_encode_var(var, &par, info);
	var->activate=activate;
	return 0;
}


/*-----------------------------------------------------------------
 * Geode_set_var
 *
 * Description:	This function set the user defined part of the display. 
 *              This function change the video mode using fbgen_do_set_var
 *              function.It calls the function fbgen_set_disp to set 
 *              the frame buffer display switch. It allocates and installs
 *              the frame buffer color map using  the functions 
 *              fb_alloc_cmap, and fbgen_install_cmap functions.
 *
 *  parameters:
 *         var: It is a pointer to the frame buffer variable screen information. *         con: It specifies the virtual console number.
 *	  info: It is a pointer to the frame buffer information structure.
 *
 *      return: On success it returns the 0 value.
 *              On failure it returns the error value.
 *------------------------------------------------------------------*/
static int
Geode_set_var(struct fb_var_screeninfo *var, int con, struct fb_info *info)
{
    struct fb_info_gen *info2 = (struct fb_info_gen *)info;
	int err,oldxres,oldyres,oldbpp,oldxres_virtual,
	    oldyres_virtual,oldyoffset;

	DPRINTK("%d\n", con);

	if((err=do_fb_set_var(var, con==currcon, info2)))
		return err;
	if((var->activate & FB_ACTIVATE_MASK) == FB_ACTIVATE_NOW) {
		DPRINTK("%s","- activate\n");
		oldxres=fb_display[con].var.xres;
		oldyres=fb_display[con].var.yres;
		oldxres_virtual=fb_display[con].var.xres_virtual;
		oldyres_virtual=fb_display[con].var.yres_virtual;
		oldbpp=fb_display[con].var.bits_per_pixel;
		oldyoffset=fb_display[con].var.yoffset;
		fb_display[con].var=*var;
		if((oldxres != var->xres || oldyres != var->yres 
		    || oldxres_virtual != var->xres_virtual
		    || oldyres_virtual != var->yres_virtual
		    || oldbpp != var->bits_per_pixel
		    || oldyoffset != var->yoffset) ||
			(var->hsync_len & Display_TVO)) /* if TVO, force mode change */
		{
			fbgen_set_disp(con, info2);
			if (info->changevar)
				(*info->changevar)(con);
			fb_alloc_cmap(&fb_display[con].cmap, 0, 0);
			fbgen_install_cmap(con, info2);
		}
	}
	var->activate=0;
	return 0;
}


/*-----------------------------------------------------------------
 * Geode_decode_var
 *
 * Description:	This routine get the video parameters out
 *				of `var'. If a value doesn't fit, round it up, 
 *				if it's too big, return -EINVAL.
 *  parameters:
 *     	   var:	Pointer to fb_var_screeninfo which has the 
 *				parameters to set the hardware. 
 *      fb_par:	This is void pointer which points to the frame 
 *				buffer parameters.  
 *        info:	This pointer points to the generic frame buffer
 *				device structure.  
 *      return:	On decoding the variable screen information 
 *				it returns 0.  
 *-----------------------------------------------------------------*/
static int Geode_decode_var(const struct fb_var_screeninfo *var, 
							void *fb_par, struct fb_info_gen *info)
{
 	int ret, i;
	struct geodefb_par *par = (struct geodefb_par *)fb_par;

	assert (par != NULL);
	assert (var != NULL);
	assert (info != NULL);

	if ((info == NULL) || (var == NULL) || (par == NULL))
		return(-EINVAL);

	DPRINTK("%dx%d  %dx%d %d (%d, %d), %X\n", 
		var->xres,
		var->yres, 
		var->xres_virtual,
		var->yres_virtual, 
		var->bits_per_pixel, 
		var->pixclock, vfreq,
		Display_Type);
	DPRINTK("%dx%d %d %d %dx%d %d\n", 
		var->xres_virtual,
		var->yres_virtual,
		var->left_margin,
		var->upper_margin,
		var->right_margin,
		var->lower_margin,
		var->hsync_len); 

	/* check if the requested mode can be supported */

	if((var->hsync_len & Display_TVO) &&  (is_tv_enabled()==1)) 
	{
		ret = gfx_is_display_mode_supported(var->xres_virtual,
				 var->yres_virtual,var->bits_per_pixel, vfreq);
	}
	else 
	{
		ret = gfx_is_display_mode_supported(var->xres, var->yres,
				var->bits_per_pixel, vfreq);
	}

	DPRINTK("gfx_is_display_mode_supported %d\n", ret);
	if(ret >= 0) /* CRT mode */
	{ 
		par->refresh = var->pixclock;

		/* Should be a CRT mode*/
		Display_Type = Display_CRT;

		/* check if this is a tv mode, if platform supports it */

		if ((var->hsync_len & Display_TVO) && (gfx_detect_video() == GFX_VID_SC1200) && (is_tv_enabled() == 1)) 
		{
			for(i=(NUM_TOTAL_MODES - NUM_TV_MODES); i<NUM_TOTAL_MODES; i++) 
			{
				if ((var->xres_virtual ==
					geodefb_predefined[i].var.xres_virtual) &&
					(var->yres_virtual == 
					geodefb_predefined[i].var.yres_virtual) &&
					(var->bits_per_pixel == 
					geodefb_predefined[i].var.bits_per_pixel)) 
				{
					Display_Type = 
						geodefb_predefined[i].Display_Type;
					par->refresh = 
						geodefb_predefined[i].var.pixclock;
					break;
				}   
			} /* for end */
		} 
		else 
		if ((gfx_detect_video() == GFX_VID_SC1200) &&
				 (is_tv_enabled() == 1)) 
		{
				for(i=(NUM_TOTAL_MODES - NUM_TV_MODES); 
							i<NUM_TOTAL_MODES; i++) 
				{
					if ((var->xres == 
					geodefb_predefined[i].var.xres)	&& 
					(var->yres == geodefb_predefined[i].var.yres) &&
					(var->bits_per_pixel == 
					geodefb_predefined[i].var.bits_per_pixel)) 
					{
						Display_Type = geodefb_predefined[i].Display_Type;
						par->refresh = geodefb_predefined[i].var.pixclock;
						break;
					}   
				} /* for end */
		}
		          
		 
		DPRINTK("PanelParam.PanelPresent : %d\n", PanelParam.PanelPresent);
	
		/* Enable FP if CRT selected */
		if((Display_Type == Display_CRT) && 
			((PanelParam.PanelPresent == PNL_PANELPRESENT) ||
			Panel_Support_From_Bios))
		{
			Display_Type = Display_FP;
		}

		DPRINTK("Display_Type: %d\n", Display_Type);
	}
	else if(gfx_detect_video() == GFX_VID_SC1200) 
	{

 		/* if Not a CRT mode try if its TV mode, 
		 * check if running on SC1200 with TV enabled/support
		 * based system 
		 */

		if (is_tv_enabled() == -1) 
		{
			printk("Cannot support TV on this platform\n");
			return(-EINVAL);
		}
	
		if(var->hsync_len & Display_TVO) 
		{
			for (i=(NUM_TOTAL_MODES - NUM_TV_MODES);
				 i<NUM_TOTAL_MODES; i++) 
			{

				DPRINTK("Test %dx%d %d\n",
					geodefb_predefined[i].var.xres_virtual,
					geodefb_predefined[i].var.yres_virtual,
					geodefb_predefined[i].var.bits_per_pixel);

				if ((var->xres_virtual == 
					geodefb_predefined[i].var.xres_virtual) 
					&& (var->yres_virtual == 
					geodefb_predefined[i].var.yres_virtual) 
					&& (var->bits_per_pixel == 
					geodefb_predefined[i].var.bits_per_pixel)) 
				{
					ret = i;
					break;
				}
			}
		}
		else {

			for (i=(NUM_TOTAL_MODES - NUM_TV_MODES);
				 i<NUM_TOTAL_MODES; i++) 
			{

				DPRINTK("Test %dx%d %d\n",
					geodefb_predefined[i].var.xres,
					geodefb_predefined[i].var.yres,
					geodefb_predefined[i].var.bits_per_pixel);
				if ((var->xres == geodefb_predefined[i].var.xres) 
					&& (var->yres == geodefb_predefined[i].var.yres) 
					&& (var->bits_per_pixel == 
					geodefb_predefined[i].var.bits_per_pixel)) 
				{
					ret = i;
					break;
				}
			}
		}

		if (ret < 0) 
		{
			printk("Cannot support requested TV mode %dx%d %d (%d, %d), %X\n", 
				var->xres, var->yres,
				var->bits_per_pixel, var->pixclock,vfreq,
				Display_Type);
			return(-EINVAL);
		}

		Display_Type = geodefb_predefined[ret].Display_Type;
		par->refresh = geodefb_predefined[ret].var.pixclock;
	} else {
		printk("Cannot support this CRT mode on this platform\n");
		return(-EINVAL);
	}

	if((var->hsync_len & Display_TVO) &&  (is_tv_enabled()==1)) 
	{

		par->xres = var->right_margin;
		par->yres = var->lower_margin;
		par->xres_virtual = var->xres_virtual; 
		par->yres_virtual = var->yres_virtual;
		
		par->left_margin = var->left_margin;
		par->upper_margin = var->upper_margin;
		par->right_margin = var->right_margin;
		par->lower_margin = var->lower_margin;
		par->hsync_len = var->hsync_len; 
		DPRINTK("TVOh :%d", par->lower_margin);
		DPRINTK("TVOw :%d", par->right_margin);
	}
	else {
		par->xres = var->xres;
		par->yres = var->yres;
		par->xres_virtual = var->xres_virtual; 
		par->yres_virtual = var->yres_virtual;

		par->left_margin=0;
		par->right_margin=0;
		par->upper_margin=0;
		par->lower_margin=0;
		par->hsync_len=0;
	}
	
	par->bpp = var->bits_per_pixel;

	if (var->accel_flags & FB_ACCELF_TEXT)
		par->accel = FB_ACCELF_TEXT;
	else
		par->accel = 0;

	if((var->hsync_len & Display_TVO) &&  (is_tv_enabled()==1)) 
	{
		DPRINTK("%dx%d(%dx%d) -  %d (%d, %d), %X\n", 
		var->xres,
		var->yres, 
		var->xres_virtual,
		var->yres_virtual, 
		var->bits_per_pixel, 
		var->pixclock, vfreq,
		Display_Type);
	}
	else 
	{
		DPRINTK("%dx%d %d (%d, %d), %X\n",	
		par->xres, par->yres, par->bpp,	par->refresh, vfreq,
		Display_Type);
	}

	DPRINTK("%dx%d %d %d %dx%d %d\n", 
		var->xres_virtual,
		var->yres_virtual,
		var->left_margin,
		var->upper_margin,
		var->right_margin,
		var->lower_margin,
		var->hsync_len); 
	return(0);
}

/*-----------------------------------------------------------------
 * Geode_encode_var
 * 
 * Description:	Fill the `var' structure based on the values in `par' 
 *				and other values read out of the hardware.
 *  parameters:
 *         var:	Pointer to fb_var_screeninfo which has the parameters
 *				to set the hardware. 
 *       b_par:	This is void pointer which points to the frame buffer
 *				parameters.  
 *        info:	This pointer points to the generic frame buffer
 *				device structure.  
 *      return:	On encoding the variable screen information 
 *				it returns 0.  
 *-------------------------------------------------------------------*/
static int Geode_encode_var(struct fb_var_screeninfo *var,
				const void *fb_par,
				struct fb_info_gen *info)
{
	struct geodefb_par *par = (struct geodefb_par *)fb_par;
	assert (par != NULL);
	assert (var != NULL);
	assert (info != NULL);
	
	if((info == NULL) || (var == NULL) || (par == NULL))
		return(-EINVAL);

	DPRINTK("%dx%d %d (%d,%d), %X\n", par->xres,
		par->yres, par->bpp, par->refresh, vfreq, Display_Type);
	DPRINTK("%dx%d %d %d %dx%d %d\n", 
		par->xres_virtual,
		par->yres_virtual,
		par->left_margin,
		par->upper_margin,
		par->right_margin,
		par->lower_margin,
		par->hsync_len); 

	memset(var, 0, sizeof(struct fb_var_screeninfo));
	var->xres = par->xres;
	var->yres = par->yres;

	if((par->hsync_len & Display_TVO) && (is_tv_enabled()== 1) ) 
	{ 
		var->xres_virtual = par->xres_virtual;
		var->yres_virtual = par->yres_virtual;

		var->left_margin = par->left_margin;
		var->upper_margin = par->upper_margin;
		var->right_margin = par->right_margin;
		var->lower_margin = par->lower_margin;
		var->hsync_len = par->hsync_len; 
		var->vsync_len=0;

		DPRINTK("virtual %dx%d\n", var->xres_virtual,var->yres_virtual);
	} 
 	else 
	{
		var->xres_virtual = par->xres;
		var->yres_virtual = par->yres;

		var->left_margin=0;
		var->right_margin=0;
		var->upper_margin=0;
		var->lower_margin=0;
		var->hsync_len=0;
	}
	var->xoffset = 0;
	var->yoffset = 0;
	var->bits_per_pixel = par->bpp;
	var->grayscale = 0;

	switch (var->bits_per_pixel) 
	{
  #ifdef FBCON_HAS_CFB8

		case 8:	
			/* CLUT */
			var->red.offset = 0;
			var->red.length = 6;
			var->red.msb_right = 0;
			var->blue = var->green = var->red;
			break;
#endif

#ifdef FBCON_HAS_CFB16
       
		case 16:	
			/* RGB 565 */
			var->red.offset = 11;
			var->red.length = 5;
			var->green.offset = 5;
			var->green.length = 6;
			var->blue.offset = 0;
			var->blue.length = 5;
			var->transp.offset = 0;
			var->transp.length = 0;
			break;
#endif

#ifdef FBCON_HAS_CFB32
       
		case 32:	
			/* RGBA 8888 */
			var->red.offset = 16;
			var->red.length = 8;
			var->green.offset = 8;
			var->green.length = 8;
			var->blue.offset = 0;
			var->blue.length = 8;
			var->transp.offset = 24;
			var->transp.length = 8;
			break;
#endif
	}

	var->red.msb_right = 0;
	var->green.msb_right = 0;
	var->blue.msb_right = 0;
	var->transp.msb_right = 0;
	var->nonstd = 0;
	var->activate = 0;
	var->height = -1;
	var->width = -1;
	var->accel_flags = (par->accel && ((par->bpp == 8) || 
		               (par->bpp == 16))) ? FB_ACCELF_TEXT : 0;

	var->vmode = FB_VMODE_NONINTERLACED;

	/* Clock is a 16:16 fixed point number */
	DPRINTK("%X\n", par->refresh);
	var->pixclock = par->refresh;
	var->sync = 0;
#if 0 
	var->hsync_len = gfx_get_hsync_end() - gfx_get_hsync_start();
	var->vsync_len = gfx_get_vsync_end() - gfx_get_vsync_start();
	var->right_margin = var->left_margin = 
		(gfx_get_htotal() - gfx_get_hactive() - var->hsync_len) / 2;
	var->upper_margin = var->lower_margin = 
		(gfx_get_vtotal() - gfx_get_vactive() - var->vsync_len) / 2;
#endif
	DPRINTK("res%dx%d, virt%dx%d, offset%dx%d,wh%dx%d\n", 
		var->xres, 
		var->yres, 
		var->xres_virtual,
		var->yres_virtual, 
		var->xoffset, 
		var->yoffset,var->width,
		var->height);

	DPRINTK("sync=%dx%d, margin=%dx%d, bpp=%d, accel=%x, sync=%x,gray=%x\n",
		var->hsync_len, 
		var->vsync_len,	
		var->right_margin,
		var->upper_margin, 
		var->bits_per_pixel,	
		var->accel_flags,
		var->sync, 
		var->grayscale);

	DPRINTK("RGB %d %d,%d %d, %d %d, %d %d, msb%d %d %d %d, nostd%d,activ%d\n",
		var->red.offset, 
		var->red.length, 
		var->green.offset, 
		var->green.length, 
		var->blue.offset, 
		var->blue.length,
		var->transp.offset, 
		var->transp.length,	
		var->red.msb_right,
		var->green.msb_right, 
		var->blue.msb_right,
		var->transp.msb_right,
		var->nonstd,var->activate);

	return(0);
}

/*-----------------------------------------------------------------
 * Geode_get_par
 * 
 * Description:	This function fill the Geode hardware's 'par' 
 *				structure.
 * parameters:
 *     fb_par:	This is void pointer which points to the frame buffer
 *				parameters.  
 *       info:	This pointer points to the generic frame buffer
 *				device structure.  
 *     return:	none.
 *------------------------------------------------------------------*/
static void Geode_get_par(void *fb_par, struct fb_info_gen *info)
{
	struct geodefb_par *par = (struct geodefb_par *)fb_par;
	assert (par != NULL);
 	assert (info != NULL);

	if ((info == NULL) || (par == NULL))
		return;

	DPRINTK("%s", "\n");

	if (current_par_valid)
		*par = current_par;
	else
		Geode_decode_var(&geodefb_default, par, info);
}

/*-----------------------------------------------------------------
 * Geode_set_par
 *
 * Description:	This function set the Geode frame buffer device
 *              according to par structure and  
 *		display mode based on display type ( TV or CRT or PANEL).
 * parameters:
 *     fb_par:	This is void pointer which points to the frame
 *				buffer parameters.  
 *       info:	This pointer points to the generic frame buffer
 *				device structure.  
 *     return:	none.
 *----------------------------------------------------------------*/
static void Geode_set_par(const void *fb_par, struct fb_info_gen *info)
{
	struct geodefb_par *par = (struct geodefb_par *)fb_par;
	assert (par != NULL);
	assert (info != NULL);
   
 	if ((info == NULL) || (par == NULL))
		return;

	DPRINTK("%d %d, %d %d, %d %d %d, %X\n", par->xres, par->yres,
		par->xres_virtual, par->yres_virtual,
		par->bpp, par->refresh,	par->accel, Display_Type);

	current_par = *par;
	current_par_valid = 1;

	/* Save the current mode, for pwrmgmt */
        
	if((par->hsync_len & Display_TVO) && (is_tv_enabled()==1)) 
	{
		Sxres = par->xres_virtual;
		Syres = par->yres_virtual;
	}
	else 
	{
		Sxres = par->xres;
		Syres = par->yres;
	}

	Sbpp = par->bpp;
	Shz = vfreq;
	SDisplayType = Display_Type; 

	set_mode(Sxres, Syres, Sbpp, Shz, SDisplayType);
}
 	
	
/*-----------------------------------------------------------------
 * set_mode 
 *
 * Description:	This function sets the display mode.
 *	 
 *  parameters:
 *	  xres: X-resolution of the display screen.
 *	  yres: Y-resolution of the display screen.
 *	   bpp: Bits-per-pixel of the diplay screen. 
 *          hz: Vertical frequency of the display screen.
 *      return:	none.
 *---------------------------------------------------------------*/
static void set_mode(int xres, int yres, int bpp, int hz, int DType)
{
	int i;
	int pxres, pyres, pbpp, phz, pstride, pheight;

	DPRINTK("%d %d %d %d, %X\n", xres, yres, bpp, hz, DType);

	/* Get the current operating display mode */
	gfx_get_display_mode(&pxres, &pyres, &pbpp, &phz);
	DPRINTK("prev mode %d %d %d %d\n", pxres, pyres, pbpp, phz);

	/* get the stride in bytes for this resolution */
	pstride = get_stride(pxres, pyres, pbpp);
	pheight = 0x400000/pstride;
	/* get the stride in pixels for this resolution */
	pstride >>= (pbpp >> 3) - 1;

	/* disable compression, since we clear the compression area also */
	if (compression_enabled) 
		gfx_set_compression_enable(0);

	/* clean the frame buffer */
	rectfill(0, 0, pstride, pheight, 0x0);

	if (DType & Display_TV) 
	{
		gfx_set_tv_display(xres, yres);

 		/* TV stuff goes here */
		if (DType & Display_TV_NTSC) 
		{
			if (DType & Display_TV_Square_Pixels) 
				gfx_set_tv_format(TV_STANDARD_NTSC,
						GFX_ON_TV_SQUARE_PIXELS);
			else 
				gfx_set_tv_format(TV_STANDARD_NTSC,
						GFX_ON_TV_NO_SCALING);
	 
		} 
		else if (DType & Display_TV_PAL) 
		{
			if (DType & Display_TV_Square_Pixels) 
				gfx_set_tv_format(TV_STANDARD_PAL,
						GFX_ON_TV_SQUARE_PIXELS);
			else 
			   	gfx_set_tv_format(TV_STANDARD_PAL,
						GFX_ON_TV_NO_SCALING);

		}
		gfx_set_tv_output(TV_OUTPUT_S_VIDEO);
		gfx_set_tv_enable(1);
	}
	else 
	{
		/* disable TV if TV is being supported. CRT modes
		 * don't go well with TV.
		*/
		if(is_tv_enabled()==1) 
			gfx_set_tv_enable(0);

		if (DType & Display_CRT) 
		{
			gfx_set_display_mode(xres, yres, bpp, hz);
		} else if (DType & Display_FP) 
		{
			if(Panel_Support_From_Bios)
			{
				gfx_set_fixed_timings(FPBX, FPBY,
					xres,
					yres, 
					bpp);
			} else 
			{
				gfx_set_fixed_timings(PanelParam.PanelStat.XRes, 
					PanelParam.PanelStat.YRes, 
					xres,
					yres, 
					bpp);
				Pnl_InitPanel(&PanelParam);
			}
		}
		gfx_set_crt_enable(CRT_ENABLE);
	}


	/* Set the display depth */
	gfx_set_display_bpp(bpp);

	if (compression_enabled) 
	{
		int match = -1;
		/* find the index to our operating mode the offsets are located */

		for (i=0; i<NUM_TOTAL_OFFSET; i++) 
		{
			if ((xres == GeodeMemOffset[i].xres) &&
				(yres == GeodeMemOffset[i].yres) &&
				(bpp == GeodeMemOffset[i].bpp)) 
			{
				match = i;
				break;
			}
		}
		DPRINTK("Found the Match @ %d\n", match);
		if(match >= 0)
		{
				/* set the compression parameters */
				gfx_set_compression_offset(GeodeMemOffset[i].CBOffset);
				gfx_set_compression_pitch(GeodeMemOffset[i].CBPitch);
				gfx_set_compression_size((GeodeMemOffset[i].CBSize -16));
		
				/* enable compression buffer, all parameters already set */
				gfx_set_compression_enable(1);
		}
	}

}

/*-----------------------------------------------------------------
 * Geode_setcolreg
 *
 * Description:	This function set a single color register. 
 *				The values supplied have a 16 bit magnitude.
 *				This function make the first 16 colors of the 
 *				palette available to frame buffer console.
 * parameters:
 *      regno:	It is an integer which represents color register number.
 *        red:    
 *      green:
 *       blue:
 *     transp:	These red, green, blue, and transp are the inter values
 *				which represents the color attributes for geode palette.
 *       info:	This is a pointer to the frame buffer information 
 *				structure.  
 *     return:	Return != 0 for invalid regno.
 *----------------------------------------------------------------*/
static int Geode_setcolreg(u_int regno,	u_int red, u_int green,	u_int blue,
				u_int transp, struct fb_info *info)
{
	assert (info != NULL);
	if (((current_par.bpp==8) && (regno>255)) ||
		((current_par.bpp!=8) && (regno>15)))
		return (1);

	if (((current_par.bpp==8) && (regno<256)) || 
		((current_par.bpp==16) && (regno<16))) 
	{
		Geode_palette[regno].red = red >> 8;
		Geode_palette[regno].green = green >> 8;
		Geode_palette[regno].blue = blue >> 8;
	}

	switch (current_par.bpp) 
	{
#ifdef FBCON_HAS_CFB8
	case 8:
		gfx_set_display_palette_entry(regno,
					(Geode_palette[regno].red << 16) |
					(Geode_palette[regno].green << 8) |
					Geode_palette[regno].blue);
		break;
#endif

#ifdef FBCON_HAS_CFB16
	case 16:
		fbcon_cmap.cfb16[regno] = ((red  & 0xf800) |
				((green & 0xfc00) >> 5) | 
				((blue  & 0xf800) >> 11));
		break;
#endif

#ifdef FBCON_HAS_CFB32
	case 32:
		fbcon_cmap.cfb32[regno] = ((red  << 16) |
				(green << 8) | blue);
		break;
#endif
	}
	return (0);

}

/*-----------------------------------------------------------------
 * Geode_getcolreg
 *
 * Description:	This function read  a single color register. 
 *				and split it into colors/transparent.The return 
 *				values must have 16 bit magnitude.
 *  parameters:
 *   	 regno:	It is an integer which represents color register number.
 *         red:    
 *       green:
 *        blue:
 *      transp:	These red, green, blue, and transp are the inter values
 *				which represents the color attributes for geode palette.
 *        info:	This is a pointer to the frame buffer information 
 *				structure.  
 *      return:	Return != 0 for invalid regno.
 *---------------------------------------------------------------*/
static int Geode_getcolreg(u_int regno,	u_int *red, u_int *green,
				u_int *blue, u_int *transp,struct fb_info *info)
{
	int t;
	assert (red != NULL);
	assert (green != NULL);
	assert (blue != NULL);
	assert (transp != NULL);
	assert (info != NULL);
	if (regno > 255)
		return (1);
	if (((current_par.bpp==8) && (regno<256)) ||
		((current_par.bpp==16) && (regno<16))) 
	{
		t = Geode_palette[regno].red;
		*red = (t<<8) | t;
		t = Geode_palette[regno].green;
		*green = (t<<8) | t;
		t = Geode_palette[regno].blue;
	       *blue = (t<<8) | t;
	}
  
	*transp = 0;
	return (0);
}


/*----------- Interfaces to hardware functions -------------- */

static struct fbgen_hwswitch Geode_hwswitch = 
{
	detect:	    Geode_detect, 
	encode_fix: Geode_encode_fix, 
	decode_var: Geode_decode_var, 
	encode_var: Geode_encode_var,
	get_par:    Geode_get_par, 
	set_par:    Geode_set_par,
	getcolreg:  Geode_getcolreg, 
	setcolreg:  Geode_setcolreg, 
	blank:      Geode_blank,
	set_disp:   Geode_set_disp
};


/*--------------- Generic routines ----------------------------*/

/*-----------------------------------------------------------------
 * Geode_set_disp
 *
 * Description:	This function fill in a pointer to appropriate
 *				low level text console operations for the video 
 *				mode 'par' of the video hardware. If it don't 
 *				have any appropriate operations, it will fill in a 
 *				pointer to dummy operations, and there will be no 
 *				text output.
 *  parameters:
 *      fb_par:	This is a void pointer, it points to frame
 *				buffer parameters.  
 *        disp:	This is a pointer to the Geode frame buffer display 
 *				structure.
 *        info:	This is a pointer to the generic frame buffer 
 *				information structure.  
 *      return:	none.
 *----------------------------------------------------------------*/
static void Geode_set_disp(const void *fb_par, struct display *disp,
				struct fb_info_gen *info)
{
	
	int stride = get_stride(disp->var.xres_virtual, 
		disp->var.yres_virtual, disp->var.bits_per_pixel);
	assert (fb_par != NULL);
	assert (disp != NULL);
	assert (info != NULL);
 	
	if ((info == NULL) || (disp == NULL) || (fb_par == NULL))
		return;

	if((disp->var.hsync_len & Display_TVO) && (is_tv_enabled()==1)) 
	{
		disp->screen_base = (char*) (GeodeMem_virt + 
				(stride * disp->var.upper_margin) + 
				(disp->var.left_margin << ((disp->var.bits_per_pixel >> 3) -1)));
	} else 
	{
		disp->screen_base = (char*)GeodeMem_virt;
	}

	DPRINTK("= %d, 0x%8X\n", stride, disp->screen_base);


	switch (disp->var.bits_per_pixel) 
	{
   #ifdef FBCON_HAS_CFB8
		case 8:
			if (disp->var.accel_flags & FB_ACCELF_TEXT) 
			{    
				disp->dispsw = &fbcon_geode8;
			}
			else
				disp->dispsw = &fbcon_cfb8;
			break;
#endif

#ifdef FBCON_HAS_CFB16
   
		case 16:
			if (disp->var.accel_flags & FB_ACCELF_TEXT) 
			{
				disp->dispsw = &fbcon_geode16;
			}
			else
				disp->dispsw = &fbcon_cfb16;
			disp->dispsw_data = &fbcon_cmap.cfb16;
			break;
#endif

#ifdef FBCON_HAS_CFB32
   
		case 32:
			if (disp->var.accel_flags & FB_ACCELF_TEXT) 
			{
				disp->dispsw = &fbcon_geode32;
			}
			else
				disp->dispsw = &fbcon_cfb32;
			disp->dispsw_data = &fbcon_cmap.cfb32;
			break;
#endif
		default:
			disp->dispsw = &fbcon_dummy;
			break;
	}
	disp->scrollmode = SCROLL_YREDRAW;
}

/*-------------------------------------------------------------
 * geodefb_set_cmap
 *
 * Description:	This function will set the color map to the
 *				console frame buffer device.
 *  parameters:
 *        cmap:	This is a fb_cmap pointer, which has the colormap
 *				to be set for the console indicated by 'con'.
 *        kspc:	It is an integer for kernel/user mode switch. 
 *         con:	The console for which the colormap has to be set.
 *        info:	It is a pointer to the frame buffer information
 *				structure.
 *      return:	It returns 0 on success otherwise it returns error value. 
 *------------------------------------------------------------*/   
static int geodefb_set_cmap(struct fb_cmap *cmap, int kspc, int con,
				struct fb_info *info)
{
	int err;
	int cmap_len = 0;
	assert (cmap != NULL);
	assert (info != NULL);
	if (info->var.bits_per_pixel == 8)
		cmap_len = 256;
	else
		cmap_len = 16;

	if (!fb_display[con].cmap.len) 
	{      
		/* no colormap allocated? */
		if ((err = fb_alloc_cmap(&fb_display[con].cmap,cmap_len, 0)))
		return(err);
   	}

	/* current console? */

	if (con == currcon) 
		return(fb_set_cmap(cmap, kspc, Geode_setcolreg, info));
	else
		fb_copy_cmap(cmap, &fb_display[con].cmap, kspc ? 0 : 1);
	return(0);
}


/*-----------------------------------------------------------------
 * geodefb_pan_display
 *
 * Description:	This function Pan or Wrap the Display
 *				and it looks only at xoffset, yoffset and
 *				the FB_VMODE_YWRAP flag.
 *	parameters:
 *		   var:	Pointer to fb_var_screeninfo which has the parameters
 *				to set the hardware. 
 *        con:	The console which has to be updated with parameters 
 *				in 'var' structure.
 *       info:	This is a pointer to the frame buffer information 
 *				structure.  
 *     return:	error value 
 *----------------------------------------------------------------*/
static int geodefb_pan_display(struct fb_var_screeninfo *var,
				int con,
				struct fb_info *info)
{
	assert (var != NULL);
	return(-EINVAL);
}

/*-----------------------------------------------------------------
 * geodefb_ioctl
 *
 * Description:	This function provide an interface to the Geode frame
 *				buffer specific input output control commands.
 *  parameters:
 *		 inode:	Pointer to "inode" structure which contains the device
 *				reference.
 *        file:	Pointer to a "file" structure. 
 *		   cmd:	The IOCTL code which was issued by the application.
 *         arg: Pointer to the data which was passed to the IOCTL by
 *				the application.
 *         con:	The console from which the ioctl was issued.
 *        info:	This is a pointer to the frame buffer information 
 *				structure.  
 *      return:	Returns 0 if the ioctl was handled successfully.
 *----------------------------------------------------------------*/
static int geodefb_ioctl(struct inode *inode, struct file *file,
				u_int cmd, u_long arg, int con,
				struct fb_info *info)
{
	int fbidx = GET_FB_IDX(inode->i_rdev);
	struct fb_ops *fb = info->fbops;
	struct fb_var_screeninfo var;
	int i;

	switch(cmd) 
	{
	case FBIOGAL_API: 
		return geodefb_ioctl_ext(inode, file, cmd, arg, con, info);
	case PUT_TVO:
		{
			if(is_tv_enabled()!=1) 
			{
				printk("This is not a TV supported Platform\n");
				return(-EINVAL);
			}

			if (copy_from_user(&var, (void *) arg, sizeof(var)))
				return -EFAULT;
			if(var.activate & FB_ACTIVATE_ALL)
			{
				DPRINTK("PUT_TVO %d\n", fbidx);
				i = set_all_vcs(fbidx, fb, &var, info);
			} else
			{

				DPRINTK("PUT_TVO %dx%d  %dx%d %d (%d, %d)\n", 
					var.xres,
					var.yres, 
					var.xres_virtual,
					var.yres_virtual, 
					var.bits_per_pixel, 
					var.pixclock, vfreq);
				DPRINTK("PUT_TVO %d %d %dx%d %d\n", 
					var.left_margin,
					var.upper_margin,
					var.right_margin,
					var.lower_margin,
					var.hsync_len); 

				/* if requesting for TVO */
				if(var.hsync_len==1)
				{
					/* check if parameters valid */
					if((var.left_margin >=0) && 
						(var.upper_margin >= 0) && 
						(var.right_margin > 0) && 
						(var.lower_margin > 0) && 
						((var.left_margin + var.right_margin) <=
							var.xres_virtual) &&
						((var.upper_margin + var.lower_margin) <=
							var.yres_virtual))
					{	
						struct fb_var_screeninfo cur_var;
						i=fb->fb_get_var(&cur_var, con, info);

						DPRINTK("get:PUT_TVO %dx%d  %dx%d %d (%d, %d)\n", 
							cur_var.xres,
							cur_var.yres, 
							cur_var.xres_virtual,
							cur_var.yres_virtual, 
							cur_var.bits_per_pixel, 
							cur_var.pixclock, i);
						DPRINTK("get:PUT_TVO %d %d %dx%d %d\n", 
							cur_var.left_margin,
							cur_var.upper_margin,
							cur_var.right_margin,
							cur_var.lower_margin,
							cur_var.hsync_len); 

						/* If currently TVO already active */
						if(cur_var.hsync_len & Display_TVO)
						{
							var.xres = cur_var.xres_virtual;
							var.yres = cur_var.yres_virtual;
							var.hsync_len = Display_TVO;
						}
						else /* TVO applied for first time on this console */
						{
							var.xres_virtual = var.xres;
							var.yres_virtual = var.yres;
							var.hsync_len = Display_TVO;
						}
					}
					else /* return error */
					{
						return -EFAULT;
					}
				}
				else /* disabling TVO */
				{

					if(var.hsync_len==0)
					{	
						var.xres = var.xres_virtual;
						var.yres = var.yres_virtual;
						var.hsync_len = 0;
					}
					else
						return(-EINVAL);
				}
			}
			i = Geode_set_var(&var, PROC_CONSOLE(info), info);
			if (i)
				return i;
			return 0;	
		}
	}
	return(-EINVAL);
}

#ifdef MMAP

/* Private mmap since we want to have a different caching on the framebuffer
 * for geodefb.
 * Note there's no locking in here; it's done in fb_mmap() in fbmem.c.
 */

/*-----------------------------------------------------------------
 * geodefb_mmap
 *
 * Description: This function map the frame buffer device memory.
 *			
 *  parameters:
 *	  info: It is a pointer to the frame buffer information structure.
 *        file:	Pointer to a "file" structure.
 *	   vma: It is a pointer to virtual memory area structure.   
 *      return:	Returns 0 if the ioctl was handled successfully.
 *----------------------------------------------------------------*/
static int geodefb_mmap(struct fb_info *info, struct file *file,
		struct vm_area_struct *vma)
{
	struct fb_ops *fb = info->fbops;
	unsigned long off=0;
	struct fb_fix_screeninfo fix;
	struct fb_var_screeninfo var;
	unsigned long start;
	u32 len;

	if (vma->vm_pgoff > (~0UL >> PAGE_SHIFT))
		return -EINVAL;
	off = vma->vm_pgoff << PAGE_SHIFT;

	Geode_get_fix(&fix, PROC_CONSOLE(info), info);

	/* frame buffer memory */
	start = fix.smem_start;
	DPRINTK("%x %x\n", start, fix.smem_len);
	len = PAGE_ALIGN((start & ~PAGE_MASK)+fix.smem_len);
	DPRINTK("off %x %x\n", off, len);
	if (off >= len) {
		/* memory mapped io */
		off -= len;
		fb->fb_get_var(&var, PROC_CONSOLE(info), info);
		if (var.accel_flags)
			return -EINVAL;
		start = fix.mmio_start;
		len = PAGE_ALIGN((start & ~PAGE_MASK)+fix.mmio_len);
	}
	DPRINTK("start2 %x %x\n", start, off);
#if 0
	start &= PAGE_MASK;
#endif
	if ((vma->vm_end - vma->vm_start + off) > len)
		return -EINVAL;
	off += start;
	DPRINTK("start3 %x %x\n", start, off);

	DPRINTK("remap %x %x %x %x\n", 
		vma->vm_start, 
		off, 
		vma->vm_end - vma->vm_start, 
		vma->vm_page_prot);

	if (io_remap_page_range(vma->vm_start, off,
			     vma->vm_end - vma->vm_start, vma->vm_page_prot))
		return -EAGAIN;

	return 0;
}
#endif

/* Geode frame buffer driver file operations */

static struct fb_ops geodefb_ops = 
{
	owner      : THIS_MODULE,
	fb_get_fix : Geode_get_fix,
	fb_get_var : Geode_get_var,
	fb_set_var : Geode_set_var,
	fb_get_cmap: fbgen_get_cmap,
	fb_set_cmap: geodefb_set_cmap,
	fb_ioctl   : geodefb_ioctl,
#ifdef MMAP
	fb_mmap    : geodefb_mmap,
#endif
};

/*----------------------------------------------------------------
 * get_ddc_mode	
 *
 * Description:	This function gets the display mode supported by 
 * 		ddc. If no mode is supported by ddc the default mode
 *		set to  640x480-16 with ddc parameters.Default vfreq
 *		set to 60 hz.
 *  parameters:	none. 
 *      return:	none  
 *--------------------------------------------------------------*/   

static void get_ddc_mode(void)	
{
	int i, mode_res, found=-1;

	unsigned long ddc_xres, ddc_yres, ddc_hz;

	printk("Quering monitor for Display modes supported :\n");

	/*intialization*/

	ddc_init();

	/* Find the best ddc available and use the Geode's mode
	* values to program the timings for the chip
	*/		
	while(found == -1)
	{
		/* Get the next resolution supported by Monitor */

		nextFromList(&ddc_xres, &ddc_yres, &ddc_hz);

		printk("Do we Support %ld %ld %ld %d",
			ddc_xres, ddc_yres, ddc_hz, ddc_bpp);
		
		if( ddc_xres == 0  ) /* if end of the list */
			break;
		/* check if this mode can be supported by our hardware */
		mode_res = gfx_is_display_mode_supported(ddc_xres, ddc_yres, 
			ddc_bpp, ddc_hz);

		/* if not supported test next supported resolution */

		if( mode_res < 0 ) 
		{
			printk(" - NO \n");
			continue;
		}
		printk(" - YES \n");

		/*
		 * search if we can support this mode from the predefined table, 
		 * else set default 
		 */

		for (i=0; i < NUM_TOTAL_MODES; i++) 
		{
			if ((ddc_xres == geodefb_predefined[i].var.xres) && 
				(ddc_yres == geodefb_predefined[i].var.yres)  &&
				(ddc_bpp == geodefb_predefined[i].var.bits_per_pixel) &&
				(geodefb_predefined[i].Display_Type & Display_CRT))
			{
				/* set geodefb default mode */
				geodefb_default = geodefb_predefined[i].var;
				Display_Type = geodefb_predefined[i].Display_Type;
				geodefb_default_valid = 1;
				vfreq = ddc_hz;
				found = i;
				break;
			}
		}
	}

	/*cleanup*/      

	ddc_cleanup();
}

/*----------------------------------------------------------------
 * get_video_mode
 *
 * Description:	This function get the video mode specified by
 *				the pointer name if it supports otherwise it set
 *				the default video mode.
 *  parameters:
 *        name:	It is a character pointer holds requested 
 *				video mode by the user.       
 *      return:	Return >= 0 if it succeeds the requested mode else 
 *				it return 0 with default mode.
 *--------------------------------------------------------------*/   
static int __init get_video_mode(const char *name)
{
	int i;
	DPRINTK("%s\n", name);
	for (i = 0; i < NUM_TOTAL_MODES; i++) 
	{
		if (!strcmp(name, geodefb_predefined[i].name)) 
		{
			DPRINTK("Found mode %s @ %d\n", name, i);
			geodefb_default = geodefb_predefined[i].var;
			Display_Type = 
				geodefb_predefined[i].Display_Type;
			return(i);
		}
	}
   
	/* set geodefb default mode */
	geodefb_default = geodefb_predefined[GEODE_DEFMODE].var;
	Display_Type = geodefb_predefined[GEODE_DEFMODE].Display_Type;
	return(0);
}

/*----------------------------------------------------------------
 * get_tv_overscan_geom
 *
 * Description:	This function parse the tv-overscan parameters 
 *		X-offset, Y-offset, width, and height of the screen.
 *
 *  parameters:
 *     options: Pointer to the tv-over-scan parameters.
 *	     X:	Pointer to the X-offset of the display screen.
 *	     Y: Pointer to the Y-offset of the display screen.
 *	     W: Pointer to the width of the display screen.
 *	     H: Pointer to the height of the display screen.
 *
 *      return:	none. 
 *--------------------------------------------------------------*/   

void get_tv_overscan_geom(char *options, int *X, int *Y, int *W, int *H)
{
	char *tv_opt;
	DPRINTK("geom %s\n", options);
	tv_opt = strtok(options, ":");
	DPRINTK("this %s, tv %s\n", options, tv_opt);
	*X= simple_strtoul(tv_opt,NULL,0); 
	tv_opt = strtok(NULL, ":");
	DPRINTK("this %s, tv %s\n", options, tv_opt);
	*Y= simple_strtoul(tv_opt,NULL,0); 
	tv_opt = strtok(NULL, ":");
	DPRINTK("this %s, tv %s\n", options, tv_opt);
	*W= simple_strtoul(tv_opt,NULL,0); 
	tv_opt = strtok(NULL, ":");
	DPRINTK("this %s, tv %s\n", options, tv_opt);
	*H= simple_strtoul(tv_opt,NULL,0); 
}

/*---------------------------------------------------------------------
 * geodefb_setup
 *
 * Description:	This function setup the display mode based 
 *		on the option specified by the user if it 
 *		supports, other wise it will set the default 
 *		display mode.
 *  parameters:
 *	   options:	This is character pointer holds the display mode 
 *				specified from user.
 *      return:	Returns 0 if the display mode was set successfully. 
 *---------------------------------------------------------------------*/   
int __init geodefb_setup(char *options)
{

	char *this_opt;
	assert (options != NULL);
	DPRINTK("%s", "\n");

	if (!options || !*options)
		return(0);

	for (this_opt = strtok(options, ",");this_opt; this_opt = 
		strtok(NULL, ","))

	if (!strcmp(this_opt, "inverse")) 
	{
		Geodefb_inverse = 1;
		fb_invert_cmaps();
	} 
	else if (!strncmp(this_opt, "font:", 5)) 
	{
		strncpy(default_fontname, this_opt+5, sizeof(
						default_fontname));
	} 
	else if (!strncmp(this_opt, "vesa:", 5)) 
	{
		vesa = simple_strtoul(this_opt+5, NULL, 0);
		DPRINTK("vesa: %x\n", vesa);

		/* fix VESA number */
		if (vesa != ~0) 
		{
			int i;
			vesa &= 0x1DFF;
			/* mask out clearscreen, acceleration and so on */
			/* static settings */
			for (i = 0; i < NUM_TOTAL_MODES; i++) 
			{
				if (vesa == geodefb_predefined[i].vesa) 
				{
					geodefb_default  = geodefb_predefined[i].var;
					Display_Type = geodefb_predefined[i].Display_Type;
					geodefb_default_valid = 1;
					break;
				}
			}
		}
	} 
	else if (!strncmp(this_opt, "vfreq:", 6)) 
	{
		vfreq = simple_strtoul(this_opt+6, NULL, 0);
		DPRINTK("vfreq: %x\n", vfreq);
	} 
	else if (!strncmp(this_opt, "vmode:", 6)) 
	{
		get_video_mode(this_opt+6);
		geodefb_default_valid = 1;
	}
	else if (!strncmp(this_opt, "flatpanel:", 10)) 
	{
		panel_selected = simple_strtoul(this_opt+10, NULL, 0);
	}
	else if (!strncmp(this_opt, "disable_fp_in_bios:", 19)) 
	{
		int ret = simple_strtoul(this_opt+19, NULL, 0);
		if(ret > 0)
			Panel_Support_From_Bios = 0;
	}
	else if (!strncmp(this_opt, "ddc:",4)) {
		ddc_selected = simple_strtoul(this_opt+4, NULL, 0);
	}
	else if (!strncmp(this_opt, "ddc_depth:", 10)) 
	{
		ddc_bpp = simple_strtoul(this_opt+10, NULL, 0);
		DPRINTK("ddc_depth: %d\n", ddc_bpp);
	}
	else if (!strncmp(this_opt, "compression_enabled:",20)) 
	{
		compression_enabled = simple_strtoul(this_opt+20, NULL, 0);
	}
    else if (!strncmp(this_opt, "TVO:",4 )) 
	{
		get_tv_overscan_geom((this_opt+4), &defaultTVO.TVOx, 
			&defaultTVO.TVOy, &defaultTVO.TVOw, &defaultTVO.TVOh);
		defaultTVO.TV_Overscan_On = 1;

		DPRINTK("XYWH: %d %d %d %d\n", 
			defaultTVO.TVOx,
			defaultTVO.TVOy,
			defaultTVO.TVOw,
			defaultTVO.TVOh);
	}

#if 0

	/* if out put request is TV findout if we can support it */
	if (Display_Type & Display_TV) 
	{
		if (is_tv_enabled()) 
		{
			geodefb_default = 
				geodefb_predefined[GEODE_DEFMODE].var;
			Display_Type = Display_CRT;
		}
	}
#endif

	DPRINTK("Geode Options: %x %d", vesa, vfreq);
	DPRINTK("default mode: xres=%d, yres=%d, bpp=%d, DType=%X\n",
		geodefb_default.xres, geodefb_default.yres,
		geodefb_default.bits_per_pixel, Display_Type);
	return(0);
}

/*----------------------------------------------------------------
 * nscgal_open
 *
 * Description:	This function will open the galdevice.
 *  parameters:
 *   	 inode:	Pointer to "inode" structure which contains the 
 *		device reference.
 *        file:	Pointer to a "file" structure. 
 *		return:	Returns 0 if the galdevice was opened  
 *				successfully. 
 *--------------------------------------------------------------*/   
static int nscgal_open(struct inode *inode, struct file *file)
{
	return(0);
}

/*----------------------------------------------------------------
 * nscgal_release
 *
 * Description:	This function will release the  galdevice.
 *				parameters:
 *		 inode:	Pointer to "inode" structure which contains the 
 *				device reference.
 *       file:	Pointer to a "file" structure. 
 *     return:	Returns 0 if the galdevice was released  
 *				successfully. 
 *--------------------------------------------------------------*/   
static int nscgal_release(struct inode *inode, struct file *file)
{
	return(0);
}

/*-----------------------------------------------------------------
 * nscgal_ioctl
 *
 * Description:	This function provide an interface to the galdevice
 *				specific input output control commands.
 *  parameters:
 *       inode:	Pointer to "inode" structure which contains the 
 *				device reference.
 *        file:	Pointer to a "file" structure. 
 *	       cmd:	The IOCTL code which was issued by the application.
 *         arg:	Pointer to the data which was passed to the IOCTL 
 *				by the application.
 *      return:	Returns 0 if the ioctl was handled successfully.
 *----------------------------------------------------------------*/
static int nscgal_ioctl(struct inode *inode, struct file *file, 
			unsigned int cmd, unsigned long arg)
{
	return(geodefb_ioctl_ext(inode, file, cmd, arg, 0, NULL));
}

/* Galdevice file operations */
static struct file_operations nscgal_fops = 
{
	ioctl: nscgal_ioctl,
	open: nscgal_open,
	release: nscgal_release,
};

static devfs_handle_t devfs_handle = NULL;

/*----------------------------------------------------------------
 * geode_create_galdevice
 *
 * Description:This function creates the galdevice.
 *  parameters:none. 
 *      return:none  
 *--------------------------------------------------------------*/   
void geode_create_galdevice(void)
{
	/* We get a major device handle out. */
	if ((devfs_handle = devfs_register_chrdev(UNNAMED_MAJOR,
				"nscgal", &nscgal_fops)) <= 0)
	DPRINTK("unable to get major %d for fb devs\n", FB_MAJOR);
}

/*----------------------------------------------------------------
 * geodefb_init
 *
 * Description:	This is the entry point to the driver. This 
 *		function detects a National display controller 
 *		(CS5530, CS1200) and initializes the durango 
 *		interface by memory mapping the physical address 
 *		space required for the controller.
 *		A software device /dev/nscgal is created to receive 
 *		IOCTLs from GAL.Then it registers the  geode frame 
 *		buffer device.
 *  Parameters:	none
 *		return:	0 returned if the geode frame buffer device 
 *				is created < 0 returned on error.
 *---------------------------------------------------------------*/
int __init geodefb_init(void)
{
	unsigned int cpu_version, address, address_size;
	DPRINTK("%s", "\n");
	cpu_version = gfx_detect_cpu();

	printk("National Geode FrameBuffer Driver Version %d.%d.%d\n", 
		NSCFB_MAJOR, NSCFB_MINOR, NSCFB_SUB);

	if (!cpu_version) 
	{
		/* NO GEODE PROCESSOR DETECTED */
		return( -ENXIO);
	}
#ifdef PM
{
        struct pm_dev *pmdev;

        pmdev = pm_register(PM_SYS_DEV, PM_SYS_VGA, geode_pm_callback);
}
#endif
	/* DISPLAY DETECTION MESSAGE */
	printk("Geode Processor Detected (Type = %d,Version = %d.%d)\n", 
		cpu_version & 0xFF, (cpu_version >> 8) & 0xFF,
		(cpu_version >> 16) & 0xFF);

	/* Panel is selected from bios, check if user turned on panel */
	if(!panel_selected)
		Panel_Support_From_Bios = 0;

	/* Check if bios supports panel. 
	 * To be checked before SoftVGA put to sleep 
	 */
	if(Panel_Support_From_Bios && panel_selected) 
	{
		Panel_Support_From_Bios = Pnl_IsPanelEnabledInBIOS();

		/* if OK, get the panel details from the Softvga */
		if(Panel_Support_From_Bios) {
			FPBX = FPBY = FPBD = FPBF = 0;
			Pnl_GetPanelInfoFromBIOS(&FPBX, &FPBY, &FPBD, &FPBF);
		}
	}
	DPRINTK("PanelSupportInBIOS %d\n", Panel_Support_From_Bios);

	/* Initialize durango API here  SET DURANGO REGISTER 
	 * POINTERS. The method of mapping from a physical address
	 * to a linear address is operating system independent.
	 * Set variables to linear address 
	 */
    
	GeodeRegs_phys = address = gfx_get_cpu_register_base();
	GeodeRegs_phys_size = address_size = 0x10000;
	gfx_virt_regptr = (unsigned char *)ioremap(address,
				address_size);

	gfx_virt_spptr = gfx_virt_regptr; /* scratch buffer */

	DPRINTK("Geodefb: Gfx Registers at 0x%x,mapped to 0x%p, size %dk\n", 
		address, gfx_virt_regptr, 
		address_size/1024);

	address = gfx_get_vid_register_base();
	address_size = 0x1000;

	gfx_virt_vidptr = (unsigned char *)ioremap(address, 
				address_size);

	DPRINTK("Geodefb: 5530 registers at 0x%x, mapped to 0x%p, size %dk\n", 
		address, (unsigned int *)gfx_virt_vidptr,
		address_size/1024);

	GeodeMem_phys = address = gfx_get_frame_buffer_base();
	GeodeSize = address_size = gfx_get_frame_buffer_size();

	gfx_virt_fbptr = (unsigned char *)ioremap(address, 
				address_size);

	GeodeMem_virt = (unsigned int) gfx_virt_fbptr;

	DPRINTK("framebuffer at 0x%x, mapped to 0x%p, size %dk\n", 
		address, (unsigned char *)gfx_virt_fbptr,
		address_size/1024);

	/* CHECK IF REGISTERS WERE MAPPED SUCCESSFULLY */
	if (!((unsigned)gfx_virt_regptr & (unsigned)gfx_virt_vidptr &
		(unsigned)gfx_virt_fbptr)) 
	{	

		DPRINTK("%s", "Could not map hardware registers.\n");

		return(-EBUSY);
	}

	/* try ddc only if only selected */
	if(ddc_selected == 1)
		get_ddc_mode();
	
	geode_create_galdevice();

	/* disable softvga and never enable it again, to take complete
	 * control 
	 */
	gfx_disable_softvga();

	/* Tell softvga we are changing values and never call this with 0 */
	gfx_vga_mode_switch(1);

	/* time to init the flatpanel */

	DPRINTK("Panel_Support_From_Bios: %d, panel_selected : %d ",Panel_Support_From_Bios,panel_selected);
 
	if((!Panel_Support_From_Bios) && panel_selected)
	{

		PanelParam.PanelPresent = PNL_PANELPRESENT;
        PanelParam.Flags = PNL_PLATFORM |
						PNL_PANELCHIP |
						PNL_PANELSTAT;

		Pnl_GetPanelParam(&PanelParam);
		DPRINTK("%s Platform with %s Chip, %s Panel %dx%d %dBpp%sColor \n",
			((PanelParam.Platform == CENTAURUS_PLATFORM) ?  "Centaurus" :\
			((PanelParam.Platform == DORADO_PLATFORM) ? "Dorado" :\
			"Unknown")),
			((PanelParam.PanelChip & PNL_9210) ? "9210" :\
			((PanelParam.PanelChip & PNL_9211_A) ? "9211_A" :\
			((PanelParam.PanelChip & PNL_9211_C) ? "9211_C" :\
			"Unknown"))),
			((PanelParam.PanelStat.Type & PNL_TFT) ? "TFT" :\
			((PanelParam.PanelStat.Type & PNL_SSTN) ? "SSTN" :\
			((PanelParam.PanelStat.Type & PNL_DSTN) ? "DSTN" :\
			"Unknown"))),
			PanelParam.PanelStat.XRes,
			PanelParam.PanelStat.YRes,
			PanelParam.PanelStat.Depth,
			((PanelParam.PanelStat.MonoColor & PNL_MONO_PANEL) ?  " Mono" :\
			((PanelParam.PanelStat.MonoColor & PNL_COLOR_PANEL) ? " ":\
			" Unknown"))
			);

		/* check if we found a vaild configuration */
		if((PanelParam.Platform == OTHER_PLATFORM) ||
			(PanelParam.PanelChip & PNL_UNKNOWN_CHIP) ||
			(PanelParam.PanelStat.Type & PNL_UNKNOWN_PANEL) ||
			(PanelParam.PanelStat.MonoColor & PNL_UNKNOWN_COLOR))
		{
			PanelParam.PanelPresent = 0;
			panel_selected = 0;
		}

		DPRINTK("PanelParam.PanelPresent : %d\n", PanelParam.PanelPresent);
	}
    
	strcpy(gen.info.modename, geodefb_name);
	gen.info.node = -1;
	gen.info.flags = FBINFO_FLAG_DEFAULT;
	gen.info.fbops = &geodefb_ops;
	gen.info.disp = &disp;
	gen.info.changevar = NULL;
	gen.info.switch_con = &fbgen_switch;
	gen.info.updatevar = &fbgen_update_var;
	gen.info.blank = &fbgen_blank;
	strcpy(gen.info.fontname, default_fontname);
	gen.parsize = sizeof (struct fb_info_gen);
	gen.fbhw = &Geode_hwswitch;
	
	gen.fbhw->detect();

	/* This should give a reasonable default video mode */

	Geode_get_var(&disp.var, -1, &gen.info);
	disp.var.activate = FB_ACTIVATE_NOW;
	do_fb_set_var(&disp.var, 1, &gen);

	fbgen_set_disp(-1, &gen);
	fbgen_install_cmap(0, &gen);
	if (register_framebuffer(&gen.info) < 0) 
	{
		DPRINTK("%s", "register_framebuffer failed\n");
		return(-EINVAL);
	}

	DPRINTK("fb%d: %s frame buffer device, using %ldK of video memory\n", 
		GET_FB_IDX(gen.info.node),gen.info.modename,
		GeodeSize>>10);
	return(0);	
}

static SAV_REG_GP SaveRegGp;
static SAV_REG_DP SaveRegDp;
static SAV_REG_5530 SaveReg5530;
/*----------------------------------------------------------------
 * Geode_blank
 *
 * Description:	This function blank the screen if
 *				blank_mode != 0, else unblank the screen.
 *  parameters:
 *       blank:	This is an integer which specify the blank_mode.
 *      return:	Return 0 if blanking succeeded, != 0 if
 *				unblanking failed.
 *--------------------------------------------------------------*/   
static int Geode_blank(int blank, struct fb_info_gen *info)
{
	DPRINTK("%d\n", blank);

	/*
	 * (Un)Blank the screen
	 * 0 unblank, 1 blank, 2 no vsync, 3 no hsync, 4 off 
	 */
	switch(blank)
	{
		case VESA_POWERDOWN:
		case VESA_VSYNC_SUSPEND:
			gfx_set_crt_enable(CRT_DISABLE);
			if (PanelParam.PanelPresent == PNL_PANELPRESENT)
				Pnl_PowerDown();
			if (is_tv_enabled())
				gfx_set_tv_enable(0);
			break;

		case VESA_HSYNC_SUSPEND:
			gfx_set_crt_enable(CRT_STANDBY);
			if (PanelParam.PanelPresent == PNL_PANELPRESENT)
				Pnl_PowerDown();
			if (is_tv_enabled())
				gfx_set_tv_enable(0);
			break;

		case VESA_NO_BLANKING:
			gfx_set_crt_enable(CRT_ENABLE);
			if (PanelParam.PanelPresent == PNL_PANELPRESENT)
				Pnl_PowerUp();
			if (is_tv_enabled())
				gfx_set_tv_enable(1);
			break;
	}
	return(0);
}

 
/*---------------------------------------------------------------
 * geode_pm_callback
 *
 * Description: Geode Power management request callback.
 *		The device request callback will be called before the
 * 		device/system enters a suspend state (ACPI D1-D3) or
 *		or after the device/system resumes from suspend (ACPI D0).
 *		For PM_SUSPEND, the ACPI D-state being entered is passed
 *		as the "data" argument to the callback.  The device
 *		driver should save (PM_SUSPEND) or restore (PM_RESUME)
 *		device context when the request callback is called.
 *		Once a driver returns 0 (success) from a suspend
 *		request, it should not process any further requests or
 *		access the device hardware until a call to "pm_access" is 
 *		made.
 *
 *  Parameters:
 *         dev: M device previously returned from pm_register.
 *   	  rqst: request type.
 *        data: data, if any, associated with the request.
 *
 *     Returns: 0 if the request is successful
 *     		EINVAL if the request is not supported
 *          	EBUSY if the device is now busy and can not handle the
 *		request.
 *          	ENOMEM if the device was unable to handle the request due
 *		to memory.
 *          
 *------------------------------------------------------------------------- */  
#ifdef PM
static int geode_pm_callback(struct pm_dev *dev, pm_request_t rqst, void *data)
{
	switch (rqst) {
		case PM_SUSPEND:
			DPRINTK("%s", "suspend\n");
			gfx_set_crt_enable(CRT_DISABLE);
			GXSaveRestoreGPReg(REG_SAVE, &SaveRegGp);
			GXSaveRestoreDPReg(REG_SAVE, &SaveRegDp);
			GXSaveRestore5530Reg(REG_SAVE, &SaveReg5530);
			if (PanelParam.PanelPresent == PNL_PANELPRESENT)
				Pnl_PowerDown();
			if (is_tv_enabled())
				gfx_set_tv_enable(0);
		break;
		case PM_RESUME:
			DPRINTK("%s", "resume\n");
			GXSaveRestoreDPReg(REG_RESTORE, &SaveRegDp);
			GXSaveRestore5530Reg(REG_RESTORE, &SaveReg5530);
			GXSaveRestoreGPReg(REG_RESTORE, &SaveRegGp);

			set_mode(Sxres, Syres, Sbpp, Shz, SDisplayType);

			if (PanelParam.PanelPresent == PNL_PANELPRESENT)
				Pnl_PowerUp();
			if (is_tv_enabled())
				gfx_set_tv_enable(1);
		break;
	}
	return 0;
}
#endif

/*----------------------------------------------------------------
 * geodefb_bmove 
 *
 * Description:	This function provide text console acceleration.
 *				It accelerates screen to screen blts.
 * parameters:
 *          p:	It is a pointer to the display structure.
 *         sy:	It holds the integer value of font height.
 *         sx:	It holds the integer value of font width.
 *         dy:	It holds the integer value of font height.
 *         dx:	It holds the integer value of font width.
 *     height:	height of blt, in pixels.
 *      width:	width of blt, in pixels.
 *     return:	none
 *--------------------------------------------------------------*/   
static void geodefb_bmove(struct display *p, int sy, int sx, int dy,
				int dx, int height, int width)
{
	assert (p != NULL);

    if (fontwidthlog(p)) 
	{
		sx <<= fontwidthlog(p); 
		dx <<= fontwidthlog(p); 
		width <<= fontwidthlog(p);
	} else
	{
		sx *= fontwidth(p); 
		dx *= fontwidth(p); 
		width *= fontwidth(p);
	}

	sy *= fontheight(p);
	dy *= fontheight(p);
	height *= fontheight(p);
	gfx_set_solid_pattern(0);
	gfx_set_raster_operation(0xCC);

    if((p->var.hsync_len & Display_TVO) && (is_tv_enabled()==1)) {
		gfx_screen_to_screen_blt(sx+p->var.left_margin, sy+p->var.upper_margin, 
			dx+p->var.left_margin, dy+p->var.upper_margin, width, height);
	} else {
		gfx_screen_to_screen_blt(sx, sy, dx, dy, width, height);
	}	
		
	blit_maybe_busy = 1;

}

/*----------------------------------------------------------------
 * rectfill 
 *
 * Description:	This function accelerates fill rectangles of the
 *				display screen.
 * parameters:
 *         sx: screen X location
 *         sy: screen Y location
 *     height: height of rectangle, in pixels.
 *      width: width of rectangle, in pixels.
 *	   bg: It gives the back ground color of the pattern.
 *     return: none.
 *-----------------------------------------------------------------*/

static void rectfill(int sx, int sy, int width, int height, 
	unsigned long bg)
{
	gfx_set_solid_pattern(bg);
	gfx_set_raster_operation(0xF0);
	gfx_pattern_fill(sx, sy, width, height);
}	
/*----------------------------------------------------------------
 * geodefb_clear 
 *
 * Description:	This function accelerates fill rectangles of the
 *				display screen.
 * parameters:
 *       conp:	It is a pointer to  virtual console data strucure.
 *          p:	It is a pointer to the display structure.
 *         sy:	It holds the integer value of font height.
 *         sx:	It holds the integer value of font width.
 *         dy:	It holds the integer value of font height.
 *         dx:	It holds the integer value of font width.
 *     height:	height of rectangle, in pixels.
 *      width:	width of rectangle, in pixels.
 *     return:	none
 *-----------------------------------------------------------------*/

static void geodefb_clear(struct vc_data *conp, struct display *p,
				int sy, int sx, int height, int width)
{
	unsigned long bg = 0;
	assert (p != NULL);
	assert (conp != NULL);
    if (fontwidthlog(p)) 
	{
		sx <<= fontwidthlog(p); 
		width <<= fontwidthlog(p);
	} else
	{
		sx *= fontwidth(p); 
		width *= fontwidth(p);
	}

	sy *= fontheight(p);
	height *= fontheight(p);
	if (p->var.bits_per_pixel == 8) 
	{
		bg = attr_bgcol_ec(p,conp);
	}
	else if (p->var.bits_per_pixel == 16) 
	{
		bg = ((u16 *)p->dispsw_data)[attr_bgcol_ec(p, conp)];
	}
	else
	{
		bg = ((u32 *)p->dispsw_data)[attr_bgcol_ec(p, conp)];
	}

	if((p->var.hsync_len & Display_TVO) && (is_tv_enabled()==1) ) 
	{
		rectfill( p->var.left_margin + sx, 
			p->var.upper_margin+sy, width, height, bg);
	} else 
	{
		rectfill(sx, sy, width, height, bg);
	}

	blit_maybe_busy = 1;
}

/*----------------------------------------------------------------
 * geodefb_putc
 *
 * Description:	This functions calls the console frame buffer driver
 *				putc function.          
 *
 * parameters:
 *       conp:	It is a pointer to the virtual console data structure. 
 *          p:	It is a pointer to the display data structure. 
 *          c:	It is an integer used as mask to get character source
 *				pixel data.	    
 *    	   yy:	It gives destination Y position 
 *				(offset into frame buffer)
 *         xx:	It gives destination X position 
 *     return:	none
 *-----------------------------------------------------------------*/
static void geodefb_putc(struct vc_data *conp, struct display *p, 
				int c, int yy, int xx)
{
	u32 fg, bg;
	int fw=fontwidth(p);
	int fh=fontheight(p);
	int fbw=(fw+7)>>3;
	u8 *cdata=p->fontdata+(c & p->charmask)*fh;

	assert (p != NULL);
	assert (conp != NULL);

	if (blit_maybe_busy)
		Geode_WaitBusy();

	if (p->var.bits_per_pixel == 8) 
	{
		fg=attr_fgcol(p, c);
		bg=attr_bgcol(p, c);
	}
	else if (p->var.bits_per_pixel == 16) 
	{
		fg = ((u16 *)p->dispsw_data)[attr_fgcol(p, c)];
		bg = ((u16 *)p->dispsw_data)[attr_bgcol(p, c)];
	}
	else
	{
		fg = ((u32 *)p->dispsw_data)[attr_fgcol(p, c)];
		bg = ((u32 *)p->dispsw_data)[attr_bgcol(p, c)];
	}

/*	DPRINTK("%d %d %d\n", fbw, fw, fh); */
	xx *= fw;
	yy *= fh;

	if((p->var.hsync_len & Display_TVO) && (is_tv_enabled()==1) ) 
	{
		xx += p->var.left_margin;
		yy += p->var.upper_margin;
	}

	gfx_set_solid_pattern(0);
	gfx_set_mono_source(bg, fg, 0);
	gfx_set_raster_operation(0xcc);
	gfx_mono_bitmap_to_screen_blt(0, 0, xx, yy, fw, fh, cdata, fbw);
	blit_maybe_busy = 1;
}

/*----------------------------------------------------------------
 * geodefb_puts
 *
 * Description:	This functions calls the console frame buffer driver
 *				putcs function.          
 * parameters:
 *       conp:	It is a pointer to the virtual console data structure. 
 *          p:	It is a pointer to the display data structure. 
 *          s:	It is  a pointer used as mask to get string of source
 *				pixel data.	    
 *      count:
 *    	   yy:	It gives destination Y position 
 *				(offset into frame buffer)
 *         xx:	It gives destination X position 
 *     return:	none
 *-----------------------------------------------------------------*/
static void geodefb_putcs(struct vc_data *conp, struct display *p,
				const unsigned short *s, int count,
				int yy,int xx)
{
	u32 fg,bg;
	int fw=fontwidth(p);
	int fh=fontheight(p);
	int fbw=(fw+7)>>3;

	assert (p != NULL);
	assert (conp != NULL);

	if (p->var.bits_per_pixel == 8) 
	{
		fg=attr_fgcol(p, *s);
		bg=attr_bgcol(p, *s);
	}
	else if (p->var.bits_per_pixel == 16) 
	{
		fg = ((u16 *)p->dispsw_data)[attr_fgcol(p, scr_readw(s))];
		bg = ((u16 *)p->dispsw_data)[attr_bgcol(p, scr_readw(s))];
	}
	else
	{
		fg = ((u32 *)p->dispsw_data)[attr_fgcol(p, scr_readw(s))];
		bg = ((u32 *)p->dispsw_data)[attr_bgcol(p, scr_readw(s))];
	}

	if (blit_maybe_busy)
		Geode_WaitBusy();
/*	DPRINTK("%d %d %d\n", fbw, fw, fh); */

	xx *= fw;
	yy *= fh;

	if((p->var.hsync_len & Display_TVO) && (is_tv_enabled()==1) ) 
	{
		xx += p->var.left_margin;
		yy += p->var.upper_margin;
	}

	gfx_set_solid_pattern(0);
	gfx_set_mono_source(bg, fg, 0);
	gfx_set_raster_operation(0xcc);
	while (count--) 
	{
		u8 *cdata=p->fontdata+(scr_readw(s++) & 
				p->charmask)*fh;
		gfx_mono_bitmap_to_screen_blt(0, 0, xx, yy, fw, fh, cdata, fbw);
		xx+=fw;
	}
	blit_maybe_busy = 1;
}

/*----------------------------------------------------------------
 * geodefb_clear_margins
 *
 *  Description: This function clears the margins of the screen.
 *			          
 *   parameters:
 *         conp: It is a pointer to the virtual console data structure. 
 *            p: It is a pointer to the display data structure.
 *  bottom_only: It gives the value of the bottom only flag.
 *       return: none.
 *-----------------------------------------------------------------*/
static void geodefb_clear_margins(struct vc_data *conp,
				struct display *p, int bottom_only)
{
	unsigned long bg;
	unsigned int sx = conp->vc_cols*fontwidth(p);
	unsigned int sy = conp->vc_rows*fontheight(p);
	unsigned int width, height;

	assert (p != NULL);
	assert (conp != NULL);

	if (blit_maybe_busy)
		Geode_WaitBusy();

	DPRINTK("%s", "\n");

	if (p->var.bits_per_pixel == 8) 
	{
		bg = attr_bgcol_ec(p,conp);
	}
	else if (p->var.bits_per_pixel == 16) 
	{
		bg = ((u16 *)p->dispsw_data)[attr_bgcol_ec(p, conp)];
	}
	else
	{
		bg = ((u32 *)p->dispsw_data)[attr_bgcol_ec(p, conp)];
	}

	if (!bottom_only && (width = p->var.xres-sx))
	{
		if((p->var.hsync_len & Display_TVO) && (is_tv_enabled()==1) ) 
		{
			rectfill(sx + p->var.left_margin, p->var.upper_margin, 
				width, p->var.yres_virtual, bg);
		} else 
		{
			rectfill(sx, 0, width, p->var.yres_virtual, bg);
		}
	}


	if ((height = p->var.yres-sy))
	{
		if((p->var.hsync_len & Display_TVO) && (is_tv_enabled()==1) ) 
		{
			rectfill( p->var.left_margin, 
				(p->var.yoffset+sy) + p->var.upper_margin, 
				sx, height, bg);
		} else 
		{
			rectfill( 0, (p->var.yoffset+sy), sx, height, bg);
		}
	}
	blit_maybe_busy = 1;
}

/*----------------------------------------------------------------
 * geodefb_revc
 *
 *  Description: This function reverse the character color.
 *			          
 *   parameters:
 *            p: It is a pointer to the display data structure.
 *  	     xx: It is the X-offset of the character.
 *  	     yy: It is the Y-offset of the character.
 *       return: none.
 *-----------------------------------------------------------------*/
static void geodefb_revc(struct display *p, int xx, int yy)
{
	int fw=fontwidth(p);
	int fh=fontheight(p);

	assert (p != NULL);

	if (blit_maybe_busy)
		Geode_WaitBusy();
	xx *= fw;
	yy *= fh;

	gfx_set_solid_pattern(0);
	gfx_set_raster_operation(0x55); /* XOR */

	if((p->var.hsync_len & Display_TVO) && (is_tv_enabled()==1) ) 
	{	
/*		DPRINTK("TVO revc %d %d %dx%d, %dx%d\n", 
			xx, yy, p->var.left_margin, p->var.upper_margin, fw, fh);
*/
		gfx_screen_to_screen_blt(p->var.left_margin+xx, p->var.upper_margin+yy, 
			p->var.left_margin+xx, p->var.upper_margin+yy, fw, fh);
	}
	else 
	{
/*		DPRINTK("revc %d %d, %dx%d\n", xx, yy, fw, fh); */
		gfx_screen_to_screen_blt(xx, yy, xx, yy, fw, fh);	
	}
	blit_maybe_busy = 1;
}

/*
 * Switch for low level operations 
 */
#ifdef FBCON_HAS_CFB8
static struct display_switch fbcon_geode8 = 
{
	setup: fbcon_cfb8_setup, 
	bmove: geodefb_bmove, 
	clear: geodefb_clear, 
	putc: geodefb_putc,
	putcs: geodefb_putcs, 
	revc: fbcon_cfb8_revc, /* our func not working for 8bpp - Why not known */
	clear_margins: geodefb_clear_margins,
	fontwidthmask: FONTWIDTH(4) | FONTWIDTH(8) |
			FONTWIDTH(12) | FONTWIDTH(16)
};
#endif  /* endif for FBCON_HAS_CFB8 */
#ifdef FBCON_HAS_CFB16
static struct display_switch fbcon_geode16 = 
{
	setup: fbcon_cfb16_setup, 
	bmove: geodefb_bmove,
	clear: geodefb_clear, 
	putc: geodefb_putc,
	putcs: geodefb_putcs, 
	revc: geodefb_revc, 
	clear_margins: geodefb_clear_margins,
	fontwidthmask: FONTWIDTH(4) | FONTWIDTH(8) |
			FONTWIDTH(12) | FONTWIDTH(16)
};
#endif /* endif  for FBCON_HAS_CFB16 */

#ifdef FBCON_HAS_CFB32
static struct display_switch fbcon_geode32 = 
{
	setup: fbcon_cfb32_setup, 
	bmove: geodefb_bmove, 
	clear: geodefb_clear, 
	putc: geodefb_putc,
	putcs: geodefb_putcs, 
	revc: geodefb_revc, 
	clear_margins: geodefb_clear_margins,
	fontwidthmask: FONTWIDTH(4) | FONTWIDTH(8) |
			FONTWIDTH(12) | FONTWIDTH(16)
};

#endif /* endif  for FBCON_HAS_CFB32 */

#ifdef MODULE

MODULE_AUTHOR("(c) 1999,2001 National Semiconductor Corporation");
MODULE_DESCRIPTION("Accelerated FBDev driver for GX1/SCx200/GX2");
MODULE_PARM(vesa, "i");
MODULE_PARM_DESC(vesa, "Startup videomode (0x000-0x1FF) (default=0x101)");
MODULE_PARM(vfreq, "i");
MODULE_PARM_DESC(vfreq, "Startup vertical frequency (60-85 MHz) (default=60)");
MODULE_PARM(vmode, "s");
MODULE_PARM_DESC(vmode, "Name of the selected resolution in text (default=640x480-16)");
MODULE_PARM(font, "s");
MODULE_PARM_DESC(font, "Name of the font");
MODULE_PARM(flatpanel, "i");
MODULE_PARM_DESC(flatpanel, "Flatpanel support (0=disabled or 1=enabled) (default=0)");
MODULE_PARM(disable_fp_in_bios, "i");
MODULE_PARM_DESC(disable_fp_in_bios, "Flatpanel support from BIOS (0=disabled or 1=enabled) (default=1)");
MODULE_PARM(compression_enabled, "i");
MODULE_PARM_DESC(compression_enabled, "Compression support (0=disabled or 1=enabled) (default=1)");
MODULE_PARM(ddc, "i");
MODULE_PARM_DESC(ddc, "DDC support (0=disabled or 1=enabled) (default=0)");
MODULE_PARM(ddc_depth, "i");
MODULE_PARM_DESC(ddc_depth, "DDC Depth when supported  8 or 16 Bpp (default=8)");

/*------------------------------------------------------------------------
 * init_module 
 *
 * Description:	This function will call geodefb_init to creat 
 *				geode frame buffer driver as a module.
 *  parameters:	none. 
 *      return:	0 returned if the geode frame buffer device is created.
 *				< 0 returned on error.
 *----------------------------------------------------------------------*/   
int init_module(void)
{
	return(geodefb_init());
}

/*----------------------------------------------------------------
 * cleanup_module 
 *
 * Description:	This function unregister the geode frame buffer
 *				driver.
 *  parameters:	none. 
 *      return:	none  
 *--------------------------------------------------------------*/   
void cleanup_module(void)
{
        /* unregister all usages of geode_pm_callback */
        pm_unregister_all(geode_pm_callback);
	unregister_framebuffer((struct fb_info *)&gen.info);
}
#endif /* MODULE */
