/* NSC_LIC_BSD
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    Video library for Geode SCx2x0
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 2000-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */
 
/*--------------------------------------------------------------------------
 * Copyright (c) 2000-2001 by National Semiconductor Corporation
 * All rights reserved.
 *--------------------------------------------------------------------------
 * File Contents:
 *           Constants, macros, variable and function declarations, and
 *           data type definitions for internal use of the SCx2x0 Video
 *           Library.
 * Project : Video library for Geode SCx2x0
 * Created : November, 2000
 * Author  : Galit Heller & Eyal Cohen
 *--------------------------------------------------------------------------
 * $Date: 8/10/01 6:58a $
 * $RCSfile: vl_defs.h $
 * $ProjectRevision: 3.1 $
 * $Revision: 6 $
 *------------------------------------------------------------------------*/

#ifndef _vl_defs_h
#define _vl_defs_h

/*------------------------------------------------------------------------*/
/*-------------------------   Include files   ----------------------------*/
/*------------------------------------------------------------------------*/

/*------------------------------------------------------------------------*/
/*----------------   Constants and macros definitions  -------------------*/
/*------------------------------------------------------------------------*/
// Field toggle definitions
#define VPHASE_OFFSET (VidTimings.VerticalTotal/2 + 1)
#define ASPECT_RATIO_VIDEO_PRESCALE(x) (x * VidDisplay.horzFactor.num / VidDisplay.horzFactor.denom)

// DVIP video window setting
#define EXTRA_Y_FOR_DVIP_NTSC (-1)
#define EXTRA_Y_FOR_DVIP_PAL    3

/*------------------------------------------------------------------------*/
/*--------------------   Data Types definitions   ------------------------*/
/*------------------------------------------------------------------------*/
 
/* Structure for keeping information of the video display preferred
 * and actual settings
 */
typedef struct VidDisplayState {

	/* VIDEO PREFERENCES */

	/* The default video source to be used for full screen */
	/* (For scaled video memory is always used) */
	
	VL_VideoSourceType full_screen_source;
									
	/* The deinterlacing method to be used for full screen video (for both DVIP and VIP) */
	VL_VideoDeinterlacingMethod full_screen_deinterlacing;
	
	/* The deinterlacing method to be used for scaled video windows */
	VL_VideoDeinterlacingMethod scaled_video_deinterlacing;
	
	/* The graphics aspect ratio on TV - TRUE if correct graphics aspect ratio is kept on TV */
	boolean graphics_scale;
	
	/* CURRENT VIDEO STATE */
	
	/* The video source from which video is currently shown */
	VL_VideoSourceType current_video_source;
	
	/* The deinterlacing method which is currently active */
	VL_VideoDeinterlacingMethod current_deinterlacing_method;
	
	/* Tearing correction currently active */
	boolean tearing_correction_active;
	
	/* The size type of the current video window */
	VL_VideoWindowSize current_window_size;
	
	/* Whether video is currently paused */
	boolean paused;
	
	/* MEMORY VALUES */
	
	/* The memory address of the displayed odd field */
	unsigned long memory_address_odd;
	
	/* The memory address of the displayed even field */
	unsigned long memory_address_even;
	
	/* CURRENT WINDOW SIZE */
	
	/* The X start of the video window on the display */
	short current_window_x;
	
	/* The Y start of the video window on the display */
	short current_window_y;
	
	/* The width of the video window on the display */
	unsigned short current_window_width;
	
	/* The height start of the video window on the display */
	unsigned short current_window_height;
	
	/* CURRENT WINDOW VALUES */
	
	/* The horizontal factor which operates on the video before it reaches the overlay window */
	factor horzFactor;
	
	/* The value to add to Y offset (needed when showing DVIP) */
	short extra_y_offset;
	
} VidDisplayState;

/* Structure for keeping information on the characteristics of the digitized
 * video source. These characteristics are determined according to the TV standard used.
 */
typedef struct VidTimingsChar {

	unsigned short Width;            /* Horizontal resolution */
	unsigned short Height;           /* Vertical resolution */
	unsigned short VerticalTotal;
	unsigned short RefreshRate;      /* Fields per second */
	unsigned short LineTime;         /* Total TV line time in 27 MHz units */
									 /* Note: TV line is 2 progressive lines i.e. 2 x HTOTAL */ 
} VidTimingsChar;

/*------------------------------------------------------------------------*/
/*-----------------------   Function headers   ---------------------------*/
/*------------------------------------------------------------------------*/
void VidSetDirectVIP(boolean onOff, int debug);
void VidSetDeinterlacingMethod(void);
void VidSendCCDataToTVenc(void);
VL_STATUS VidSetTVTimings(void);

/*------------------------------------------------------------------------*/
/*----------------------   Variable definitions   ------------------------*/
/*------------------------------------------------------------------------*/

/*-------------*/
/*-- Externs --*/
/*-------------*/
extern VL_VideoStruct VidAttr;
extern VidDisplayState VidDisplay;

#endif /* _vl_defs_h */
