/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Geode Video 4 Linux 2 Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>

#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>

int
main(int argc, char *argv[])
{
	char			*device_video = "/dev/video";
	char			*device_vbi = "/dev/vbi";
	int			vid;
	int			vbi;
	int			err;
	int			n;
	struct v4l2_capability	cap;
        struct v4l2_format	fmt;
	char			*data;

	vid = open(device_video, O_RDONLY);
	if (vid < 0)
	{
		fprintf(stderr, "Can't open %s\n", device_video);
		return 1;
	}

	vbi = open(device_vbi, O_RDONLY);
	if (vbi < 0)
	{
		fprintf(stderr, "Can't open %s\n", device_vbi);
		return 1;
	}

	err = ioctl(vid, VIDIOC_QUERYCAP, &cap);
	if (err)
	{
		fprintf(stderr, "QUERYCAP returned error %d\n", errno);
		return 1;
	}
	if (cap.type != V4L2_TYPE_CAPTURE)
	{
		fprintf(stderr, "Device %s is not a video capture device.\n",
			device_video);
		return 1;
	}
	if (!(cap.flags & V4L2_FLAG_READ))
	{
		fprintf(stderr, "Device %s doesn't support read().\n", device_video);
		return 1;
	}

	fmt.type = V4L2_BUF_TYPE_CAPTURE;
	err = ioctl(vid, VIDIOC_G_FMT, &fmt);
	if (err)
	{
		fprintf(stderr, "G_FMT returned error %d\n", errno);
		return 1;
	}

	close(vbi);
	close(vid);

	return 0;
}
