/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 *    Geode Video4linux2 driver
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */
 /* File Name		: ioctl_geode_v.h
 * File Contents        :
 * 	ioctl constants for V4L2 Geode driver
 *
 * [Notes]
 *   This file contains the definitions for all the additional
 *   ioctls and the structure definitions for the V4L2 Geode
 *   driver. The ioctls provided by V4L2 are found in
 *   linux/videodev.h
 *   This file should also be included by the application, after
 *   including videodev.h. Please add all new ioctl definitions
 *   in this file. Structures that the application need to know
 *   of, should be defined here.	
 *
 * Project		: GeodeV4l2 linux driver
 *  Author		: Sojan James
 *    Date		: Dec 18, 2000
 *-----------------------------------------------------------------------*/

#ifndef _MACHINE_IOCTL_GEODE_V_H_
#define _MACHINE_IOCTL_GEODE_V_H_

#include <linux/ioctl.h>

typedef struct overlay_info_t
{
        int            TypeOfData;  /* Deprecated */
        short          StartX;      /* Deprecated */
        short          StartY;      /* Deprecated */
        unsigned short Width;       /* Deprecated */
        unsigned short Height;      /* Deprecated */
        unsigned long  BufferOffset;/* Deprecated */
        unsigned short BufferWidth; /* Deprecated */
        unsigned short BufferHeight;/* Deprecated */
	unsigned long  ColorKey;    /* Colorkey to be used for Overlay. */
	unsigned long  ColorMask;   /* Mask bits of the graphics or video stream being */
        unsigned long  VideoFormat; /* Deprecated */

	int Graphics;      // Not used, retained to maintain compatibility with
	int Enable;        // old applications.
} overlay_info_t;


/* moved in from geodedrv.h */
typedef struct alpha_blend_info_t
{
	int            TypeOfData;   /* Is this data for Overlay or AlphaBlending or Palette */
				     /* '1' for Overlay,'2' for Alpha and '3' for Palette data */
	int            WindowNumber; /* The Alpha Window to be used. Can be '0' '1' or '2'   */
	int            Priority;     /* Window priority */
	unsigned short StartX;       /* Start x coordinate of Alpha window */
	unsigned short StartY;       /* Start y coordinate of Alpha window */
	unsigned short Width;        /* Width of Alpha window */
	unsigned short Height;       /* Height of Alpha window */
	unsigned char  Alpha;        /* Alpha Value */
	char           Delta;        /* Value by which Alpha is incremented at every frame. */
	unsigned long  Color;        /* If bit-24 is set, then display this color in the */
				     /* Alpha window when there is a color key match. */
	int            Enable;       /* '0' for disable Alpha Window & '1' for enable */
} alpha_blend_info_t;


typedef struct signal_info_t
{
	int signal_number;	 /* the signal to send to the application */
	unsigned long signal_on_length;	/* the length of the buffer when */
}signal_info_t;


typedef struct tag_geode_ioctl_debug_phys_t
{
	unsigned long	control;
	unsigned long	addr;
	unsigned long	data;
} geode_ioctl_debug_phys_t;

typedef struct tag_geode_ioctl_debug_i2c_t
{
	unsigned long	control;
	unsigned char	addr;
	unsigned char	subaddr;
	unsigned char	data;
} geode_ioctl_debug_i2c_t;

typedef struct tag_geode_ioctl_debug_portio_t
{
	unsigned long	control;
	unsigned short	addr;
	unsigned long	data;
} geode_ioctl_debug_portio_t;


typedef struct geode_general_param_t {
   unsigned long p_1;
   unsigned long p_2;
   unsigned long p_3;
   unsigned long p_4;
   unsigned long p_5;
   unsigned long p_6;
   unsigned long p_7;
} geode_general_param_t ;


/* ioctl definitions  */
/* Add all new ioctl definitions here */

#define GEODE_IOCTL_S_SIGNAL	_IOW('v',BASE_VIDIOCPRIVATE+0,signal_info_t)
#define GEODE_IOCTL_DEBUG_PHYS  _IOWR('v',BASE_VIDIOCPRIVATE+1,geode_ioctl_debug_phys_t)

#define GEODE_IOCTL_DEBUG_PORTIO 	_IOWR('v',BASE_VIDIOCPRIVATE+2,geode_ioctl_debug_portio_t)
#define GEODE_IOCTL_SET_OVERLAY_INFO 	_IOW('v',BASE_VIDIOCPRIVATE+3,overlay_info_t)
#define GEODE_IOCTL_SET_ALPHA_BLEND_INFO _IOW('v',BASE_VIDIOCPRIVATE+4,alpha_blend_info_t)
#define GEODE_IOCTL_DVIP 		_IO('v',BASE_VIDIOCPRIVATE+5)
#define GEODE_IOCTL_PAL  		_IO('v',BASE_VIDIOCPRIVATE+6)
#define GEODE_IOCTL_VBI_DUMP 		_IOW('v',BASE_VIDIOCPRIVATE+7,unsigned long)
#define GEODE_IOCTL_DEBUG_I2C		_IOWR('v',BASE_VIDIOCPRIVATE+8,geode_ioctl_debug_i2c_t)

#define GEODE_IOCTL_VBI_PASSTHRU	_IOWR('v',BASE_VIDIOCPRIVATE+11,geode_ioctl_vbi_passthru_t)

#define GEODE_IOCTL_S_COLOR_KEY	_IOW('v',BASE_VIDIOCPRIVATE+12,geode_general_param_t)
#define GEODE_IOCTL_S_ALPHA_WINDOW	_IOW('v', BASE_VIDIOCPRIVATE+14, geode_general_param_t)
#define GEODE_IOCTL_S_ALPHA_OPER	_IOW('v', BASE_VIDIOCPRIVATE+15, geode_general_param_t)
#define GEODE_IOCTL_S_DVIP	_IOW('v', BASE_VIDIOCPRIVATE+17, int)
#define GEODE_IOCTL_G_DVIP	_IOWR('v', BASE_VIDIOCPRIVATE+18, geode_general_param_t)
#define GEODE_IOCTL_DEINTERLACE_FULL	_IOWR('v',BASE_VIDIOCPRIVATE+19, int)
#define GEODE_IOCTL_DEINTERLACE_SCALED	_IOWR('v',BASE_VIDIOCPRIVATE+20, int)
#define GEODE_IOCTL_GET_DECODER_REG     _IOWR('v' ,BASE_VIDIOCPRIVATE+21,geode_ioctl_debug_i2c_t)
#define GEODE_IOCTL_SET_DECODER_REG     _IOWR('v' ,BASE_VIDIOCPRIVATE+22,geode_ioctl_debug_i2c_t)
#define GEODE_IOCTL_SET_GENLOCK     _IOWR('v' ,BASE_VIDIOCPRIVATE+23,int)
#define GEODE_VIDEO_OVERLAY_MODE	_IOW('v', BASE_VIDIOCPRIVATE+24,int)

/* Deprecated ioctls, kept here just for backward compatibility */
#define GEODE_IOCTL_S_GRAPHICS_ON_VIDEO   _IOW('v', BASE_VIDIOCPRIVATE+13, geode_general_param_t)
#define GEODE_IOCTL_VIDEO_OUTSIDE_ALPHA _IOW('v', BASE_VIDIOCPRIVATE+16, geode_general_param_t)

#define GEODE_IOCTL_S_GRAPHICS_ON_VID   GEODE_IOCTL_S_GRAPHICS_ON_VIDEO
/* Parameters for the GEODE_IOCTL_DEINTERLACE ioctl */
#define GEODE_DEINTERLACE_NONE 		1
#define GEODE_DEINTERLACE_INTERLEAVE 	2
#define GEODE_DEINTERLACE_BOB 		3


/*parameters for the PORTIO control parameters*/
#define GEODE_IOCTL_DEBUG_PHYS_READ		0x00000000
#define GEODE_IOCTL_DEBUG_PHYS_WRITE		0x00000001		

#define GEODE_IOCTL_DEBUG_PORTIO_READ_8		0x00000000
#define GEODE_IOCTL_DEBUG_PORTIO_READ_16	0x00000001
#define GEODE_IOCTL_DEBUG_PORTIO_READ_32	0x00000002
#define GEODE_IOCTL_DEBUG_PORTIO_WRITE_8	0x00000003
#define GEODE_IOCTL_DEBUG_PORTIO_WRITE_16	0x00000004
#define GEODE_IOCTL_DEBUG_PORTIO_WRITE_32	0x00000005

#define GEODE_IOCTL_DEBUG_I2C_READ              0x00000000
#define GEODE_IOCTL_DEBUG_I2C_WRITE             0x00000001

typedef enum VideoSource_t {
	VSOURCE_TUNER,
	VSOURCE_COMP,
	VSOURCE_SVIDEO,	
	VSOURCE_MPEG_DEC,
	VSOURCE_COUNT } VideoSource_t ;	

typedef enum VideoStandard_t {
	VSOURCE_STD_NTSC = 1,
	VSOURCE_STD_PAL,	
	VSOURCE_STD_SECAM,
	VSOURCE_STD_SIGMA_PAL,
	VSOURCE_STD_SIGMA_NTSC,
	VSOURCE_STD_NONE,
	VSOURCE_STD_MAX } VideoStandard_t;

#define GEODE_V4L2_VBI_SF_MULTILINE             0x80000000
#define GEODE_V4L2_VBI_SF_CC_US                 0x80010000
#define GEODE_V4L2_VBI_SF_CC_EURO               0x80020000
#define GEODE_V4L2_VBI_SF_VPS                   0x80040000
#define GEODE_V4L2_VBI_SF_WSS                   0x80080000
#define GEODE_V4L2_VBI_SF_WST                   0x80100000
#define GEODE_V4L2_VBI_SF_RAW                   0x80200000
#define GEODE_V4L2_VBI_SF_TELETEXT              0x80400000
#define GEODE_V4L2_VBI_SF_NABTS                 0x80800000
#define GEODE_V4L2_VBI_SF_MOJI                  0x81000000
#define GEODE_V4L2_VBI_SF_JFS                   0x82000000
#define GEODE_V4L2_VBI_SF_VIDEO                 0x84000000
#define GEODE_V4L2_VBI_SF_TS_EURO		0x88000000
#define GEODE_V4L2_VBI_SF_TS_US			0x80001000
#define GEODE_V4L2_VBI_SF_INTERCAST             GEODE_V4L2_VBI_SF_RAW
#define GEODE_V4L2_VBI_SF_APPEND		0xFFFFFFFF

/* New VBI flag to tell the driver to add the new set of lines to
   the previous set of lines */
#define V4L2_VBI_APPEND_FMT			(1 << 2)


/* New CIDs */
#define GEODE_CID_OVERLAY_MODE			V4L2_CID_PRIVATE_BASE+1
#define GEODE_CID_LAST				V4L2_CID_PRIVATE_BASE+2
#define MAX_VOUT_CONTROLS			1
#define MAX_VOUT_EFFECTS			1

/* Integer values to passed into VIDIOC_S_CTRL of vout device to set the
   different overlay options. Note: These values are defined by VideoLib and will
   have to be changed if they are changed in VideoLib
 */
   
#define OVERLAY_COLOR_KEY			1
#define OVERLAY_CHROMA_KEY			2
#define OVERLAY_COLRK_IN_ALPHA_VIDEO_OUTSIDE	3
#define OVERLAY_CHROMAK_IN_ALPHA_GFX_OUTSIDE	4
#define OVERLAY_ALWAYS_ON			5




#endif /* _MACHINE_IOCTL_GEODE_V_H_ */
