/* Test Suite for GeodeV4l2 driver
   Uses   : Check Library
   Author : Sojan James, Lakshmi Gopinath, Chitra Shankaran
   Date: Aug 27, 2001
 */
   
#include "include.h"

#define OVERLAY_KEY 0x00f000f0
#define OVERLAY_MASK 0x00f0f0f0

#define TRUE 1
#define FALSE 0
int interactive; //flag to say if appln is interactive or not
int channel;

int dvip(int fd_vout,int enable)
{
	if(ioctl(fd_vout,GEODE_IOCTL_S_DVIP,&enable)) 
		return FALSE;
	else
		return TRUE;	
}

int open_video_device()
{
	int fd_video = open(CAPTURE_DEVICE, O_RDWR);
	if(fd_video < 0)
		return FALSE;
	else
		return fd_video;	
}
int open_vout_device()
{
	int fd_vout = open(VOUT_DEVICE,O_RDWR);
	if(fd_vout < 0)
		return FALSE;
	else
		return fd_vout;	
}

void close_devices(int fd_video,int fd_vout)
{
	close(fd_video);
	close(fd_vout);
}

int get_vout_format(int fd_vout,int * width, int* height)
{
	struct v4l2_format fmt;
	if(ioctl(fd_vout,VIDIOC_G_FMT,&fmt)) 
		return FALSE;
	*width = fmt.fmt.pix.width;
	*height = fmt.fmt.pix.height;
	return TRUE;
}

int get_video_format(int fd_video,int * width, int* height, int * size)
{
	struct v4l2_format fmt;
	if(ioctl(fd_video,VIDIOC_G_FMT,&fmt))
		return FALSE;
	*width  = fmt.fmt.pix.width;
	*height = fmt.fmt.pix.height;
	*size   = fmt.fmt.pix.sizeimage;
	return TRUE;
}

int set_overlay_info(int fd_vout, unsigned long key, unsigned long mask)
{
	geode_general_param_t ck_info;
	ck_info.p_1 = key;
	ck_info.p_2 = mask;
	if(ioctl(fd_vout, GEODE_IOCTL_S_COLOR_KEY, &ck_info))
		return FALSE;
	else
		return TRUE;	
}

int set_alpha_blend_info(int fd_vout,int window, int x, int y, int w, int h, 
			  int priority, int alpha, int delta, int enable,
			  unsigned long color)
{
	alpha_blend_info_t alp_info;
	alp_info.WindowNumber = window;
	alp_info.StartX = x;
	alp_info.StartY = y;
	alp_info.Width = w;
	alp_info.Height = h;
	alp_info.Alpha = alpha;
	alp_info.Delta = delta;
	alp_info.Enable = enable;
	alp_info.Color = color;
	if(ioctl(fd_vout, GEODE_IOCTL_SET_ALPHA_BLEND_INFO, &alp_info))
		return FALSE;
	else
		return TRUE;	
}

int set_video_fmt(int fd_video,int width, int height)
{
	struct v4l2_format fmt;
	fmt.fmt.pix.width = width;
	fmt.fmt.pix.height = height;
	if(ioctl(fd_video, VIDIOC_S_FMT, &fmt))
		return FALSE;
	else
		return TRUE;	
}

int set_vout_window(int fd_vout,int x, int y, int width, int height)
{
	struct v4l2_window win;
	win.x = x;
	win.y = y;
	win.width = width;
	win.height = height;
	win.clipcount = 0; /* unused by the driver */
	if(ioctl(fd_vout, VIDIOC_S_WIN, &win))
		return FALSE;
	else
		return TRUE;	
}

int set_input(int fd_video,int inp)
{
	if(ioctl(fd_video, VIDIOC_S_INPUT,&inp))
		return FALSE;
	else
		return TRUE;	
}

int capture_enable(int fd_video, int enable)
{
	int dummy;
	if(enable) {
		if(ioctl(fd_video, VIDIOC_STREAMON, &dummy))
			return FALSE;
	}
	else {
		if(ioctl(fd_video, VIDIOC_STREAMOFF, &dummy))
			return FALSE;
	}	
	return TRUE;
}

int preview(int fd_vout,int enable)
{
		if(ioctl(fd_vout, VIDIOC_PREVIEW,&enable))
			return FALSE;
		else
			return TRUE;	
}

int set_chan(int fd_video,int ch)
{
	unsigned long c = ch;
	if(ioctl(fd_video, VIDIOC_S_FREQ,&c))
		return FALSE;
	else
		return TRUE;	
}

int enum_fx(int fd_vout,int index)
{
        struct v4l2_fxdesc fxdesc;
        fxdesc.index = index;
        if(ioctl(fd_vout,VIDIOC_ENUMFX,&fxdesc))
		return FALSE;
	else
		return TRUE;
}

int set_vout_control(int fd_vout,int val)
{
        int ret;
        int i = 0;
        struct v4l2_control ctrl;
        if(ioctl(fd_vout,VIDIOC_S_EFFECT,&i))
		return FALSE;
        ctrl.id = GEODE_CID_OVERLAY_MODE;
        ctrl.value = val;
        if(ioctl(fd_vout, VIDIOC_S_CTRL, &ctrl))
		return FALSE;
	else 
		return TRUE;
}


int video_control(int fd_video, unsigned long control, int value)
{
	struct v4l2_control  vcontrol;
	vcontrol.value = value;

	switch(control)
	{
		case V4L2_CID_BRIGHTNESS:
                        vcontrol.id = V4L2_CID_BRIGHTNESS;
	                break;

                case V4L2_CID_HUE:
                        vcontrol.id = V4L2_CID_HUE;
        	        break;

                case V4L2_CID_SATURATION:
                        vcontrol.id = V4L2_CID_SATURATION;
                	break;
                /* add more as we go on */
                default:
                        break;
	}

	if(ioctl( fd_video, VIDIOC_S_CTRL, &vcontrol))
		return FALSE;
	else
		return TRUE;
	

}
/* Test to check for hue video control */ 
START_TEST(hue_control_test)
{
	int flag = 0;
	int fd_video, fd_vout;
	int width1, height1,xpos1,ypos1;
	char ch;
	int value;
	int original;

	printf(" \n Hue Video Control Test\n");
	xpos1 = ypos1 = 0;

	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	xpos1 = ypos1 = 0;
	width1 = 400; height1 = 200;
	
	flag = set_video_fmt(fd_video,width1,height1);
	fail_unless(flag == 1,"set video format failed");

	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	
	flag = set_input(fd_video,1);
	fail_unless(flag == 1,"Set input failed ");

	flag = set_vout_control(fd_vout,5);
	fail_unless(flag == 1,"set vout control failed");

	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	if(interactive)
	{
		printf("\nValue:");
		scanf("%d",&value);
	}	
	else
	{
		value = 100;
	}

	flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
	fail_unless(flag == 1,"set vout window failed");

	flag = video_control(fd_video,V4L2_CID_HUE,value);
	fail_unless(flag == 1,"hue video control failed");

	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail("Hue video control failed");
	}
	else
		sleep(1);


	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");
	value = 1;
	flag = video_control(fd_video,V4L2_CID_HUE,value);
	fail_unless(flag == 1,"hue video control failed");

	close(fd_vout);
	close(fd_video);
}
END_TEST

/* Test to check for saturation video control */ 
START_TEST(saturation_control_test)
{
	int flag = 0;
	int fd_video, fd_vout;
	int width1, height1,xpos1,ypos1;
	char ch;
	int value;

	printf(" \n Saturation Video Control Test\n");
	xpos1 = ypos1 = 0;

	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	flag = set_vout_control(fd_vout,5);
        fail_unless(flag == 1,"set vout control failed");

	xpos1 = ypos1 = 0;	
	width1 = 400; height1 = 200;

	flag = set_video_fmt(fd_video,width1,height1);
	fail_unless(flag == 1,"set video format failed");

	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	

	flag = set_input(fd_video,1);
	fail_unless(flag == 1,"Set input failed ");

	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	if(interactive)
	{
		printf("\nValue:");
		scanf("%d",&value);
	}	
	else
	{
		value = 100;
	}

	flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
	fail_unless(flag == 1,"set vout window failed");

	flag = video_control(fd_video,V4L2_CID_SATURATION,value);
	fail_unless(flag == 1,"saturation video control failed");

	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail("Saturation video control failed");
	}
	else
		sleep(1);


	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");
	value = 0x40 << 1;
	flag = video_control(fd_video,V4L2_CID_SATURATION,value);
	fail_unless(flag == 1,"saturation video control failed");

	close(fd_vout);
	close(fd_video);
}
END_TEST

/* Test to check for brightness video control */ 
START_TEST(brightness_control_test)
{
	int flag = 0;
	int fd_video, fd_vout;
	int width1, height1,xpos1,ypos1;
	char ch;
	int value;

	printf(" \n Brightness Video Control Test\n");
	xpos1 = ypos1 = 0;

	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	flag = set_vout_control(fd_vout,5);
        fail_unless(flag == 1,"set vout control failed");

	xpos1=ypos1 = 0;
	width1 = 400; height1 = 200;
	
	flag = set_video_fmt(fd_video,width1,height1);
	fail_unless(flag == 1,"set video format failed");

	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	

	flag = set_input(fd_video,1);
	fail_unless(flag == 1,"Set input failed ");

	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	if(interactive)
	{
		printf("\nValue:");
		scanf("%d",&value);
	}	
	else
	{
		value = 100;
	}

	flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
	fail_unless(flag == 1,"set vout window failed");

	flag = video_control(fd_video,V4L2_CID_BRIGHTNESS,value);
	fail_unless(flag == 1,"brightness video control failed");

	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail("Brightness video control failed");
	}
	else
		sleep(1);


	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");

	close(fd_vout);
	close(fd_video);
}
END_TEST



/* Basic Tests  */
START_TEST(capture_open_close_multiple)
{
	int fd[MAX_OPENS],i;
	int fd_video = -1, fd_vout=-1;

	printf(" \nMultiple Open Test for Capture\n");
	fd[0] = open(CAPTURE_DEVICE,O_WRONLY); /* capture open */
	fail_unless(fd[0] > 0, "Capture open failed on Capture device" );
	for(i = 1; i < MAX_OPENS; i++) {
		fd[i] = open(CAPTURE_DEVICE,O_WRONLY|O_NOIO);
		fail_unless(fd[i] > 0,"NO_IO open failed on Capture device" );
	}
	for(i = 0; i < MAX_OPENS; i++) {
		close(fd[i]);
	}

}
END_TEST

START_TEST(vout_open_close_multiple)
{
	int fd[MAX_OPENS],i;

	printf("Vout Open Test\n");
	fd[0] = open(VOUT_DEVICE,O_WRONLY); /* capture open */
	fail_unless(fd[0] > 0, "Capture open failed on vout device" );
	for(i = 1; i < MAX_OPENS; i++) {
		fd[i] = open(VOUT_DEVICE,O_WRONLY|O_NOIO);
		fail_unless(fd[i] > 0,"NO_IO open failed on vout device");
	}
	for(i = 0; i < MAX_OPENS; i++) {
		close(fd[i]);
	}
}
END_TEST

/* Interface tests: To ensure all the ioctls and other system calls work ok */
START_TEST(capture_interface)
{
	char buffer[200] ; /* should be enough? */
	void *joker = (void*)buffer;
	
	int fd = open(CAPTURE_DEVICE,O_WRONLY);
	printf("Capture Interface Test\n");

	if(fd < 0) fail(" Open failed on Capture device");


	if(ioctl(fd,VIDIOC_QUERYCAP,(struct v4l2_capability*)joker) )
		fail("Failed ioctl : VIDIOC_QUERYCAP");

	/* This should return an error */
	if(!ioctl(fd,VIDIOC_ENUM_PIXFMT,(struct v4l2_fmtdesc*)joker) )
		fail("Failed ioctl : VIDIOC_ENUM_PIXFMT");

	if(ioctl(fd,VIDIOC_G_FMT,(struct v4l2_format*)joker) )
		fail("Failed ioctl : VIDIOC_G_FMT");

	if(ioctl(fd,VIDIOC_G_INPUT,(int*)joker) )
		fail("Failed ioctl : VIDIOC_G_INPUT");

	if(ioctl(fd,VIDIOC_G_STD,(struct v4l2_standard*)joker) )
		fail("Failed ioctl : VIDIOC_G_STD");

	((struct v4l2_enumstd*)buffer)->index = 0;

	if(ioctl(fd,VIDIOC_ENUMSTD,(struct v4l2_enumstd*)joker) )
		fail("Failed ioctl : VIDIOC_ENUMSTD"); 

	((struct v4l2_input*)buffer)->index = 0;
	if(ioctl(fd,VIDIOC_ENUMINPUT,(struct v4l2_input*)joker) )
		fail("Failed ioctl : VIDIOC_ENUMINPUT"); 
	
	if(ioctl(fd,VIDIOC_G_TUNER,(struct v4l2_tuner*)joker) )
		fail("Failed ioctl : VIDIOC_G_TUNER");

	((struct v4l2_control*)buffer)->id = V4L2_CID_BRIGHTNESS;
	if(ioctl(fd,VIDIOC_G_CTRL,(struct v4l2_control*)joker) )
		fail("Failed ioctl : VIDIOC_G_CTRL");

	if(ioctl(fd,VIDIOC_G_FREQ,(int*)joker) )
		fail("Failed ioctl : VIDIOC_G_FREQ");

	/* Setting functions */
	*(int*)buffer = 0;	
	if(ioctl(fd,VIDIOC_S_FREQ,(int*)joker) )
		fail("Failed ioctl : VIDIOC_S_FREQ");
	
	((struct v4l2_control*)buffer)->id=V4L2_CID_BRIGHTNESS;
	((struct v4l2_control*)buffer)->value = 100;
	if(ioctl(fd,VIDIOC_S_CTRL,(struct v4l2_control*)joker) )
		fail("Failed ioctl : VIDIOC_S_CTRL");

	*(int*)buffer = 0;	
	if(ioctl(fd,VIDIOC_S_INPUT,(int*)joker) )
		fail("Failed ioctl : VIDIOC_S_INPUT");
	((struct v4l2_format*)buffer)->fmt.pix.width = 400;
	((struct v4l2_format*)buffer)->fmt.pix.height = 200;
	if(ioctl(fd,VIDIOC_S_FMT,(struct v4l2_format*)joker) )
		fail("Failed ioctl : VIDIOC_S_FMT");

	((struct v4l2_window*)buffer)->x = 10;
	((struct v4l2_window*)buffer)->y = 10;
	((struct v4l2_window*)buffer)->width = 400;
	((struct v4l2_window*)buffer)->height = 200;
	if(ioctl(fd,VIDIOC_S_WIN,(struct v4l2_window*)joker) )
		fail("Failed ioctl : VIDIOC_S_WIN");


	/* check VIDIOC_S_STD */
	close(fd);
}
END_TEST

START_TEST(vout_interface)
{
	char buffer[200] ; /* should be enough? */
	void *joker = (void*)buffer;
	int ret;
	
	int fd = open(VOUT_DEVICE,O_WRONLY);
	printf("Vout Interface Test\n");
	if(fd < 0) fail(" Open failed on Vout device");

	/* This should return an error */
	if(!ioctl(fd,VIDIOC_ENUM_PIXFMT,(struct v4l2_fmtdesc*)joker) )
		fail("Failed ioctl : VIDIOC_ENUM_PIXFMT");

	if(ioctl(fd,VIDIOC_G_FMT,(struct v4l2_format*)joker) )
		fail("Failed ioctl : VIDIOC_G_FMT");


	if(ioctl(fd,VIDIOC_G_STD,(struct v4l2_standard*)joker) )
		fail("Failed ioctl : VIDIOC_G_STD");


	/* Setting functions */

	((struct v4l2_format*)buffer)->fmt.pix.width = 400;
	((struct v4l2_format*)buffer)->fmt.pix.height = 200;
	if(ioctl(fd,VIDIOC_S_FMT,(struct v4l2_format*)joker) )
		fail("Failed ioctl : VIDIOC_S_FMT");

	((struct v4l2_window*)buffer)->x = 10;
	((struct v4l2_window*)buffer)->y = 10;
	((struct v4l2_window*)buffer)->width = 400;
	((struct v4l2_window*)buffer)->height = 200;
	if(ioctl(fd,VIDIOC_S_WIN,(struct v4l2_window*)joker) )
		fail("Failed ioctl : VIDIOC_S_WIN");

	((struct v4l2_fxdesc*)buffer)->index = 0;
	if(ioctl(fd,VIDIOC_ENUMFX,(struct v4l2_fxdesc*)joker))
		fail("Failed ioctl : VIDIOC_ENUMFX");

	//*((int*)buffer) = 0;
	ret = 0;
	if(ioctl(fd,VIDIOC_S_EFFECT,&ret))
		fail("Failed ioctl : VIDIOC_S_EFFECT");
	((struct v4l2_control*)buffer)->id = GEODE_CID_OVERLAY_MODE;
	((struct v4l2_control*)buffer)->value = 2;
	if(ioctl(fd, VIDIOC_S_CTRL,(struct v4l2_control*)joker))
		fail("Failed ioctl : VIDIOC_S_CTRL");

	((geode_general_param_t*)buffer)->p_1 = OVERLAY_KEY;
	((geode_general_param_t*)buffer)->p_2 = OVERLAY_MASK;
        if(ioctl(fd, GEODE_IOCTL_S_COLOR_KEY,(geode_general_param_t*)joker ))
		fail("Failed ioctl : GEODE_IOCTL_S_COLOR_KEY");
	
	((struct v4l2_fmtdesc*)buffer)->index = 0;
        if(ioctl(fd, VIDIOC_ENUM_FBUFFMT,(struct v4l2_fmtdesc*)joker ))
		fail("Failed ioctl : VIDIOC_ENUM_FBUFFMT");
	
        if(ioctl(fd, VIDIOC_G_FBUF,(struct v4l2_framebuffer*)joker ))
		fail("Failed ioctl : VIDIOC_G_FBUF");
	
	((struct v4l2_framebuffer*)buffer)->base[0] = (void*)0x200000 ;
	((struct v4l2_framebuffer*)buffer)->fmt.width = 400 ;
	((struct v4l2_framebuffer*)buffer)->fmt.height = 200 ;
        if(ioctl(fd, VIDIOC_S_FBUF,(struct v4l2_framebuffer*)joker ))
		fail("Failed ioctl : VIDIOC_S_FBUF");

		

	/* check VIDIOC_S_STD */
	close(fd);
}
END_TEST

/* Test to check for switching between different sources */
START_TEST(source_switching_test)
{
	int flag = 0,src=0;
	int fd_video, fd_vout;
	int width1, height1,xpos1,ypos1;
	char ch;

	printf(" \nSource Switching Test\n");
	xpos1 = ypos1 = 0;

	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	flag = set_vout_control(fd_vout,5);
        fail_unless(flag == 1,"set vout control failed");

	xpos1 = ypos1 = 0;
	width1 = 400; height1 = 200;
	
	flag = set_video_fmt(fd_video,width1,height1);
	fail_unless(flag == 1,"set video format failed");

	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	

	
	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	if(interactive)
	{
		printf("\nSource:");
		scanf("%d",&src);
		flag = set_input(fd_video,src);
		fail_unless(flag == 1,"set input for composite failed");
		flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
		fail_unless(flag == 1,"set vout window failed");
	}	

	else
	{
		for(src=0;src<4;src++)
		{
			printf("\nSOURCE %d\n",src);

			flag = set_input(fd_video,src);
			fail_unless(flag == 1,"set input for composite failed");
			
			flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
			fail_unless(flag == 1,"set vout window failed");
			sleep(1);
		}
	}


	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail("Source Switching Failed");
	}
	else
		sleep(1);


	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");

	close(fd_vout);
	close(fd_video);
}
END_TEST

START_TEST(tv_channel_test)
{
	int flag = 0,src=0;
	int fd_video, fd_vout;
	int width1, height1,xpos1,ypos1;
	char ch;

	printf(" \nTV Channel Switching Test\n");
	xpos1 = ypos1 = 0;

	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	flag = set_input(fd_video,0);
	fail_unless(flag == 1,"set input for TV tuner failed");

	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	flag = set_vout_control(fd_vout,5);
        fail_unless(flag == 1,"set vout control failed");


	xpos1 = ypos1 = 0;
	width1 = 400; height1 = 200;
	
	flag = set_video_fmt(fd_video,width1,height1);
	fail_unless(flag == 1,"set video format failed");

	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	

	
	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	if(interactive)
	{
		printf("\nChannel:");
		scanf("%d",&src);
	}	
	else
	{
		src = channel; 
	}

	flag = set_chan(fd_video,src);		
	fail_unless(flag == 1,"set channel failed");

	flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
	fail_unless(flag == 1,"set vout window failed");

	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail("TV Set Channel Failed");
	}
	else
		sleep(1);


	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");

	close(fd_vout);
	close(fd_video);
}
END_TEST
/* Test to check for Video Window */
START_TEST(video_window_test)
{
	int flag = 0;
	int fd_video, fd_vout;
	int width1, height1,xpos1,ypos1;
	char ch;

	printf("Video Window Test\n");
	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	/* get the origin and width and height from the user if interactive */

	if(interactive)
	{
		printf("\n\nEnter xpos ypos width height:");
		scanf("%d %d %d %d",&xpos1,&ypos1,&width1,&height1);
	}
	else	
	{
		width1 = 400; height1 = 200;
		xpos1=ypos1=0;
	}
	
	flag = set_input(fd_video,1);
	fail_unless(flag == 1,"set input for composite failed");

	flag = set_video_fmt(fd_video,width1,height1);
	fail_unless(flag == 1,"set video format failed");

	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	flag = set_vout_control(fd_vout,5);
        fail_unless(flag == 1,"set vout control failed");


	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	

	flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
	fail_unless(flag == 1,"set vout window failed");
	
	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	ch =0;
	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail(" Video window Failed ");
	}
	else
		sleep(1);

	if(!interactive)
	{
		xpos1 = ypos1 = 0;
		get_vout_format(fd_vout,&width1, &height1);
//		width1 = 720; height1 = 480;
		
		flag = set_video_fmt(fd_video,width1,height1);
		fail_unless(flag == 1,"set video format failed");

		flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
		fail_unless(flag == 1,"set vout window failed");
		
		sleep(1);
	}

	xpos1 = ypos1 = 100;
	width1 = 400;
	height1 = 200;

	flag = set_video_fmt(fd_video,width1,height1);
	fail_unless(flag == 1,"set video format failed");


	flag = set_vout_window(fd_vout,xpos1,ypos1,width1,height1);
	fail_unless(flag == 1,"set vout window failed");

	ch = 0;
	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail(" Video window Failed ");
	}
	else
		sleep(1);


	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");

	close(fd_vout);
	close(fd_video);
}
END_TEST

/* Test to check the three alpha blended windows */
START_TEST(alpha_blend_window_test)
{
	int flag = 0;
	int fd_video, fd_vout, i;
	int width1, height1,xpos1,ypos1;
	int width2, height2,xpos2,ypos2;
	int width3, height3,xpos3,ypos3;
	int maxx, maxy;
	char ch;

	printf("Alpha Blend Window Test\n");

	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	/* get the origin and width and height from the user if interactive */

	if(interactive)
	{
		printf("***FOR ALPHA BLEND WINDOW 1***\n");
		printf("\nEnter xpos ypos width height:");
		scanf("%d %d %d %d",&xpos1,&ypos1,&width1,&height1);

		printf("***FOR ALPHA BLEND WINDOW 2***\n");
		printf("\nEnter xpos ypos width height:");
		scanf("%d %d %d %d",&xpos2,&ypos2,&width2,&height2);

		printf("***FOR ALPHA BLEND WINDOW 3***\n");
		printf("\nEnter xpos ypos width height:");
		scanf("%d %d %d %d",&xpos3,&ypos3,&width3,&height3);
	}
	else	
	{
		width1=height1=100;
		xpos1=ypos1=0;
		width2=height2=200;
		xpos2=ypos2=100;
//		width3=720; height3=480;
		get_vout_format(fd_vout,&width3,&height3);
		xpos3=ypos3=0;

	}
	
	flag = set_input(fd_video,1);
	fail_unless(flag == 1,"set input for composite failed");

	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	get_vout_format(fd_vout,&maxx,&maxy);

	flag = set_video_fmt(fd_video,maxx,maxy);
	fail_unless(flag == 1,"set video format failed");

	flag = enum_fx(fd_vout,0);
	fail_unless(flag == 1,"enum_fx failed");

//	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
//	fail_unless(flag == 1,"set overlay info failed");
	
	
	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	/* Alpha blend window #1 */
	flag = set_alpha_blend_info(fd_vout,0,xpos1,ypos1, width1,height1, 1,10,0,1,0xFFFF0000);
	fail_unless(flag == 1,"alpha blend info failed");

	/* Alpha blend window #2 */

	//flag = set_video_fmt(fd_video,width2,height2);
	//fail_unless(flag == 1,"set video format failed");


	flag = set_alpha_blend_info(fd_vout,1,xpos2,ypos2, width2,height2, 1,60,0,1,0xFFFF0000);
	fail_unless(flag == 1,"alpha blend info failed");


	/* Alpha blend window #3 */

	flag = set_alpha_blend_info(fd_vout,2,xpos3,ypos3, width3,height3, 1,120,0,1,0xFFFF0000);
	fail_unless(flag == 1,"alpha blend info failed");
	
	i = 0;	
	flag = set_vout_control(fd_vout,i);
	fail_unless(flag == 0,"set vout control should fail here, did not");
	i = 6;	
	flag = set_vout_control(fd_vout,i);
	fail_unless(flag == 0,"set vout control should fail here, did not");
	i = 2;	
	flag = set_vout_control(fd_vout,i);
	fail_unless(flag == 1,"set vout control failed");

	flag = set_vout_window(fd_vout,0,0,maxx,maxy);
	fail_unless(flag == 1,"set vout window failed");
		

	if(interactive){
		printf("Video Window visible?(y/n): ");
		scanf("%s",&ch);
		if(ch != 'y' && ch != 'Y')
			fail(" Video window Failed ");
	}
	else
		sleep(1);

	flag = set_alpha_blend_info(fd_vout,0,xpos1,ypos1, width1,height1, 
						1,50,0,0,0xFFFF0000);
	fail_unless(flag == 1,"alpha blend disable failed");

	flag = set_alpha_blend_info(fd_vout,1,xpos1,ypos1, width1,height1, 
						1,50,0,0,0xFFFF0000);
	fail_unless(flag == 1,"alpha blend disable failed");

	flag = set_alpha_blend_info(fd_vout,2,xpos1,ypos1, width1,height1, 
						1,50,0,0,0xFFFF0000);
	fail_unless(flag == 1,"alpha blend disable failed");

	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");

	close(fd_video);
	close(fd_vout);
}
END_TEST

/* Test to check for preview on off */
START_TEST(preview_on_off_test)
{
	int flag = 0;
	int size,enable;
	int fd_video, fd_vout, i;
	int width, height,xpos,ypos;
	char ch;
	//width = height = 100;

	printf("Preview Test\n");
	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	/* get the origin and width and height from the user if interactive */

	if(interactive)
	{
		printf("\n\nEnter xpos ypos width height:");
		scanf("%d %d %d %d",&xpos,&ypos,&width,&height);
	}
	else	
	{
		xpos = 0; ypos = 0;
		width = 400; height = 200;
	}
	
	flag = set_input(fd_video,1);
	fail_unless(flag == 1,"set input for composite failed");

	flag = set_video_fmt(fd_video,width,height);
	fail_unless(flag == 1,"set video format failed");

	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	flag = set_vout_control(fd_vout,5);
        fail_unless(flag == 1,"set vout control failed");


	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	

	flag = set_vout_window(fd_vout,xpos,ypos,width,height);
	fail_unless(flag == 1,"set vout window failed");
	

	for(i=0;i<3;i++){

		flag = preview(fd_vout,1);
		fail_unless(flag == 1,"preview failed");
		
		if(interactive){
			printf("You must now see video.Okay?(y/n): ");
			scanf("%s",&ch);
			if(ch != 'y' && ch != 'Y')
				fail("Preview Failed");
		}
		else
			sleep(1);
		flag = preview(fd_vout,0);
		fail_unless(flag == 1,"preview failed");
		if(interactive){
			printf("Video window off.Okay?(y/n): ");
			scanf("%s",&ch);
			if(ch != 'y' && ch != 'Y')
				fail("Preview Failed");
		}
		else
			sleep(1);
	}	
			

	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");

	close(fd_video);
	close(fd_vout);

	
}
END_TEST

/* Test to check for Video Pause */
START_TEST(capture_on_off_test)
{
	int flag = 0;
	int size,enable;
	int fd_video, fd_vout, i;
	int width, height,xpos,ypos;
	char ch;

	printf("Video Pause Test\n");
	fd_vout = open_vout_device();
	if(fd_vout < 0) fail(" Open failed on Vout device");
	
	fd_video = open_video_device();
	if(fd_video < 0) fail(" Open failed on Capture device");
	
	/* get the origin and width and height from the user if interactive */

	if(interactive)
	{
		printf("\n\nEnter xpos ypos width height:");
		scanf("%d %d %d %d",&xpos,&ypos,&width,&height);
	}
	else	
	{
		xpos = 0; ypos = 0;
		width = 400; height = 200;
	}
	
	flag = set_input(fd_video,1);
	fail_unless(flag == 1,"set input for composite failed");

	flag = set_video_fmt(fd_video,width,height);
	fail_unless(flag == 1,"set video format failed");

	flag = set_overlay_info(fd_vout,OVERLAY_KEY,OVERLAY_MASK);
	fail_unless(flag == 1,"set overlay info failed");
	
	flag = capture_enable(fd_video,1);
	fail_unless(flag == 1,"capture enable failed");

	flag = set_vout_control(fd_vout,5);
        fail_unless(flag == 1,"set vout control failed");


	flag = set_vout_window(fd_vout,xpos,ypos,width,height);
	fail_unless(flag == 1,"set vout window failed");
	

	flag = preview(fd_vout,1);
	fail_unless(flag == 1,"preview failed");

	for(i=0;i<3;i++){
		flag = capture_enable(fd_video,1);
		fail_unless(flag == 1,"capture enable failed");

		flag = set_vout_window(fd_vout,xpos,ypos,width,height);
		fail_unless(flag == 1,"set vout window failed");

		if(interactive){
			printf("Video active. Okay?(y/n): ");
			scanf("%s",&ch);
			if(ch != 'y' && ch != 'Y')
				fail("Video Pause Failed");
		}
		else
			sleep(1);

		flag = capture_enable(fd_video,0);
		fail_unless(flag == 1,"capture enable failed");

		if(interactive){
			printf("Video paused. Okay?(y/n): ");
			scanf("%s",&ch);
			if(ch != 'y' && ch == 'Y')
				fail("Video Pause Failed");
		}
		else
			sleep(1);
	}	
			

	flag = capture_enable(fd_video,0);
	fail_unless(flag == 1,"capture disable failed");

	flag = preview(fd_vout,0);
	fail_unless(flag == 1,"preview off failed");

	close(fd_video);
	close(fd_vout);

	
}
END_TEST

/* Test Suite that calls the different test cases written above */
Suite * test_suite_capture (void)
{
	/* Test suite to which cases are added */
	Suite * s = suite_create("V4l2 Capture and Vout device");

	/* Test cases */
	TCase * tc_basic = tcase_create("Basic tests");
	TCase * tc_interface ;
	TCase * tc_video_window;
	TCase * tc_capture;
	TCase * tc_alpha_blend_window;
	TCase * tc_preview_on_off;
	TCase * tc_source_switching;
	TCase * tc_tv_channel;
	TCase * tc_bright_control;
	TCase * tc_hue_control;
	TCase * tc_saturation_control;

	/* Add the test case to the suite */
	suite_add_tcase(s,tc_basic);
	
	/* Add individual tests to the test cases */
	tcase_add_test(tc_basic,capture_open_close_multiple);
	tcase_add_test(tc_basic,vout_open_close_multiple);

	/* Interface tests */	
	tc_interface = tcase_create("Interface tests (testing all ioctls)");
	suite_add_tcase(s,tc_interface);
	tcase_add_test(tc_interface,capture_interface);
	tcase_add_test(tc_interface,vout_interface);

	/* Video Window Test */
	tc_video_window = tcase_create("video_window tests ");
	suite_add_tcase(s,tc_video_window);
	tcase_add_test(tc_video_window,video_window_test);	
	
	/* Alpha Blend Window Test */
	tc_alpha_blend_window = tcase_create("Alpha Blend window tests ");
	suite_add_tcase(s,tc_alpha_blend_window);
	tcase_add_test(tc_alpha_blend_window,alpha_blend_window_test);	

	/* Preview On Off Test */
	tc_preview_on_off = tcase_create("Preview On Off tests ");
	suite_add_tcase(s,tc_preview_on_off);
	tcase_add_test(tc_preview_on_off,preview_on_off_test);	

	/* Capture On Off Test */
	tc_capture = tcase_create("Capture tests ");
	suite_add_tcase(s,tc_capture);
	tcase_add_test(tc_capture,capture_on_off_test);	

	/* Source Switching Test */
	tc_source_switching = tcase_create("Source switching tests ");
	suite_add_tcase(s,tc_source_switching);
	tcase_add_test(tc_source_switching,source_switching_test);	

	/* TV Channel Setting Test */
	tc_tv_channel = tcase_create("TV Set Channel tests ");
	suite_add_tcase(s,tc_tv_channel);
	tcase_add_test(tc_tv_channel,tv_channel_test);	

	/* Brightness Control Test */
	tc_bright_control = tcase_create("Brightness Control tests ");
	suite_add_tcase(s,tc_bright_control);
	tcase_add_test(tc_bright_control,brightness_control_test);	

	/* hue Control Test */
	tc_hue_control = tcase_create("Hue Control tests ");
	suite_add_tcase(s,tc_hue_control);
	tcase_add_test(tc_hue_control,hue_control_test);	

	/* Saturation Control Test */
	tc_saturation_control = tcase_create("Saturation Control tests ");
	suite_add_tcase(s,tc_saturation_control);
	tcase_add_test(tc_saturation_control,saturation_control_test);	
	return s;
}


int main(int argc,char **argv)
{
	int nf;
	int c;
	int repeat_count = 1;
	char *optstring="if:r:c:";
	char filename[100];
	Suite * s = test_suite_capture();

	SRunner *sr = srunner_create(s);

	/* To check for the command line arguements			 */
	/* -i option is for interactive check-----No args reqd     	 */
	/* -f option is the file option----log file name given as arg	 */
	/* -r option for no of times to repeat				 */
	/* -c option for TV channel to use				 */

	interactive = FALSE;
	strncpy(filename,"v4l2_test.log",99);
	while((nf = getopt(argc,argv,optstring)) != -1)
	{
		switch(nf){
			case 'i':
				interactive = TRUE;
				break;

			case 'f':
				printf("Log file: %s\n",optarg);
				bzero(filename,sizeof(filename));
				strncpy(filename,optarg,99);	
				break;
			case 'r':
				repeat_count = atoi(optarg);
				printf("Repeating test %d times\n",repeat_count);	
				break;
			case 'c':
				channel = atoi(optarg);
				break;
		}
	}	

	
	srunner_set_log(sr,filename);
	printf("Running tests on /dev/video\n");
	while(repeat_count)
	{	
		srunner_run_all(sr, CRVERBOSE);
		repeat_count --;
	}
	nf = srunner_ntests_failed(sr);

	srunner_free(sr);
	suite_free(s);
	return (nf == 0)? EXIT_SUCCESS : EXIT_FAILURE;
}
