/* NSC_LIC_BSD
 *
 * Revision 1.2
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    TVLIB
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) July 19, 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 19, 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */

/* This file contains routines to assist debugging.
 *
 * Physical memory access:
 * 
 *    tv_dbg_phys_read
 *    tv_dbg_phys_write
 *
 * I2C access:
 * 
 *    tv_dbg_i2c_read
 *    tv_dbg_i2c_write
 *
 * Port access:
 * 
 *    tv_dbg_portio_read_8
 *    tv_dbg_portio_read_16
 *    tv_dbg_portio_read_32
 *    tv_dbg_portio_write_8
 *    tv_dbg_portio_write_16
 *    tv_dbg_portio_write_32
 *
 * Project: TVLIB
 * Author:  Peter Lohmann
 * Date:    April 10, 2001
 *---------------------------------------------------------------------*/
#include "tv_debug.h"


/*---------------------------------------------------------------------------
 * tv_dbg_phys_read
 * 
 * This routine reads physical memory at the requested physical address
 * and puts the data in the supplied buffer.
 *---------------------------------------------------------------------------
 */
int tv_dbg_phys_read( ULONG addr, ULONG* data ) 
{ 
	BYTE*	pB; 
	volatile void*  pregs = (void*)0; 
	
	if( ( addr >= 0x40008300 ) && ( addr <= 0x40008400 ) )
	{
		//	We must map GXm from 0x40008000
		pregs = tv_os_kmalloc_phys( 0x40008000, 0x1000 ); 
		if( pregs == 0 ) 
		{ 
			tv_os_err_msg( "tv_dbg_phys_write(): failed to unlock gxm\n" ); 
			return 0; 
		}
		pB = (BYTE*)pregs; 

		*data = *( (ULONG*)( pB + ( addr - 0x40008000 ) ) ); 
#ifdef WIN98
		tv_os_kfree_phys( (void*)pregs, 0x1000 ); 
#else
		tv_os_kfree_phys( (void*)pregs ); 
#endif
		return 1;
	}

	if( ( addr >= 0x40010400 ) && ( addr <= 0x40010800 ) )
	{
		pregs = tv_os_kmalloc_phys( 0x40010000, 0x1000 ); 
		if( pregs == 0 ) 
		{ 
			tv_os_err_msg( "tv_dbg_phys_write(): failed to unlock 0x40010000\n" ); 
			return 0; 
		}
		pB = (BYTE*)pregs; 

		*data = *( (ULONG*)( pB + ( addr - 0x40010400 ) ) ); 

#ifdef WIN98
		tv_os_kfree_phys( (void*)pregs, 0x1000 ); 
#else
		tv_os_kfree_phys( (void*)pregs ); 
#endif
		return 1;
	}

	pregs = tv_os_kmalloc_phys( ( addr & 0xFFFFF000 ), 0x1000 ); 
	//pregs = tv_os_kmalloc_phys( addr, 0x10 ); 
	
	if( pregs == (void*)0 ) 
	{ 
		tv_os_err_msg( "tv_dbg_phys_read( 0x%08X ): failed to map regs\n", addr ); 
		return 0; 
	} 
	
	pB = (BYTE*)pregs; 
	
	*data = *( (ULONG*)( pB + ( addr & 0x00000FFF ) ) ); 
	//*data = *( (ULONG*)pB ); 
	
#ifdef WIN98
		tv_os_kfree_phys( (void*)pregs, 0x1000 ); 
#else
		tv_os_kfree_phys( (void*)pregs ); 
#endif
	
	return 1; 
} 


/*---------------------------------------------------------------------------
 * tv_dbg_phys_write
 * 
 * This routine writes physical memory at the requested physical address
 * with the data in the supplied buffer.  It unlocks the video memory,
 * if needed.
 *---------------------------------------------------------------------------
 */
int tv_dbg_phys_write( ULONG addr, ULONG data ) 
{ 
	BYTE*	pB; 
	volatile void*  pregs = 0; 

	if( ( addr >= 0x40008300 ) && ( addr <= 0x40008400 ) )
	{
		//	We must unlock GXm registers
		//	We must map GXm from 0x40008000
		pregs = tv_os_kmalloc_phys( 0x40008000, 0x1000 ); 
		if( pregs == 0 ) 
		{ 
			tv_os_err_msg( "tv_dbg_phys_write(): failed to unlock gxm\n" ); 
			return 0; 
		}
		pB = (BYTE*)pregs; 

		*( (ULONG*)( pB + 0x0300 ) ) = 0x30004758; 
		*( (ULONG*)( pB + ( addr - 0x40008000 ) ) ) = data; 

#ifdef WIN98
		tv_os_kfree_phys( (void*)pregs, 0x1000 ); 
#else
		tv_os_kfree_phys( (void*)pregs ); 
#endif
		return 1;
	}	
	if( ( addr >= 0x40010400 ) && ( addr <= 0x40010800 ) )
	{
		pregs = tv_os_kmalloc_phys( 0x40010000, 0x1000 ); 
		if( pregs == 0 ) 
		{ 
			tv_os_err_msg( "tv_dbg_phys_write(): failed to map other regs\n" ); 
			return 0; 
		}
		pB = (BYTE*)pregs; 

		*( (ULONG*)( pB + ( addr - 0x40010000 ) ) ) = data; 

#ifdef WIN98
		tv_os_kfree_phys( (void*)pregs, 0x1000 ); 
#else
		tv_os_kfree_phys( (void*)pregs ); 
#endif
		return 1;
	}	

	pregs = tv_os_kmalloc_phys( ( addr & 0xFFFFF000 ), 0x1000 ); 
	//pregs = tv_os_kmalloc_phys( addr, 0x1000 ); 
	
	if( pregs == 0 ) 
	{ 
		tv_os_err_msg( "tv_dbg_phys_write(): failed to map regs\n" ); 
		return 0; 
	} 
	
	pB = (BYTE*)pregs; 
	
	*( (ULONG*)( pB + ( addr & 0x00000FFF ) ) ) = data; 
	//*( (ULONG*)pB ) = data; 
	
#ifdef WIN98
		tv_os_kfree_phys( (void*)pregs, 0x1000 ); 
#else
		tv_os_kfree_phys( (void*)pregs ); 
#endif
	
	return 1; 
} 


int tv_dbg_i2c_read( BYTE addr, BYTE subaddr, BYTE* data )
{
/*
	acb_reset_i2c( 0x840 );
	if( subaddr != 0x00 )
	{
		tv_os_dbg_msg("dbg_i2c_read(): 0x%02X 0x%02X\n", addr, subaddr);
		*data = acb_read_i2c( GEODE_ACB2_BASE_ADDR, addr, subaddr );
	}
	else
	{
		tv_os_dbg_msg("dbg_i2c_read(): 0x%02X\n", addr );
		*data = acb_read_bus_i2c( GEODE_ACB2_BASE_ADDR, addr );
	}
*/
	return 1;
}


int tv_dbg_i2c_write( BYTE addr, BYTE subaddr, BYTE data )
{
/*
	if( subaddr != 0x00 )
	{
		acb_write_i2c( GEODE_ACB2_BASE_ADDR, addr, subaddr, data );
	}
	else
	{
		acb_write_bus_i2c( GEODE_ACB2_BASE_ADDR, addr, data );
	}
*/
	return 1;
}


int tv_dbg_portio_read_8( ULONG addr, ULONG* data )
{
	*data = (ULONG)tv_os_io_read_byte( (WORD)addr );
	return 1;
}


int tv_dbg_portio_read_16( ULONG addr, ULONG* data )
{
	*data = (ULONG)tv_os_io_read_word( (WORD)addr );
	return 1;
}


int tv_dbg_portio_read_32( ULONG addr, ULONG* data )
{
	*data = (ULONG)tv_os_io_read_ulong( (WORD)addr );
	return 1;
}


int tv_dbg_portio_write_8( ULONG addr, ULONG data )
{
	tv_os_io_write_byte( (WORD)data, (BYTE)addr );
	return 1;
}


int tv_dbg_portio_write_16( ULONG addr, ULONG data )
{
	tv_os_io_write_word( (WORD)data, (BYTE)addr );
	return 1;
}


int tv_dbg_portio_write_32( ULONG addr, ULONG data )
{
	tv_os_io_write_ulong( (WORD)data, (BYTE)addr );
	return 1;
}


