/* NSC_LIC_BSD
 *
 * Revision 1.2
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    TVLIB
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) July 19, 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 19, 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */

/*
 * This file contains routines to abstract the operating system from the 
 * drivers.  This includes handling sleep/delay, memory access, printing
 * debug output, system timers, etc.
 *
 * OS-abstraction functions:
 * 
 *    tv_os_dbg_msg
 *    tv_os_err_msg              
 *    tv_os_get_tick_count
 *    tv_os_sleep                
 *    tv_os_kmalloc              
 *    tv_os_kfree                
 *    tv_os_kmalloc_phys         
 *    tv_os_kfree_phys           
 *    tv_os_pci_dump             
 *    tv_os_pci_find_device      
 *    tv_os_pci_read_ulong       
 *    tv_os_pci_write_ulong      
 *    tv_os_pci_read_byte        
 *    tv_os_pci_write_byte       
 *    tv_os_pciconfig_read_byte  
 *    tv_os_pciconfig_write_byte 
 *    tv_os_pciconfig_read_word  
 *    tv_os_pciconfig_write_word 
 *    tv_os_pciconfig_read_ulong 
 *    tv_os_pciconfig_write_ulong
 *    tv_os_io_read_byte         
 *    tv_os_io_write_byte        
 *    tv_os_io_read_word         
 *    tv_os_io_write_word        
 *    tv_os_io_read_ulong        
 *    tv_os_io_write_ulong       
 *    
 * Project: TVLIB
 * Author:  Peter Lohmann
 * Date:    April 10, 2001
 *---------------------------------------------------------------------*/

#include "tv_os_geode.h"


#ifdef WINCE
// For now, these fns are the same.
void	tv_os_dbg_msg( LPCSTR szFormat, ... )
{
	wchar_t	szWBuf[ 128 ];
	char		szBuf[ 128 ];
	va_list	arglist;
	memset( szWBuf, 0, sizeof(szWBuf) );
	memset( szBuf, 0, sizeof(szBuf) );
	va_start( arglist, szFormat );
	_vsnprintf( 
			szBuf, 
			127, 
			szFormat, 
			arglist 
			);
	va_end( arglist );
	mbstowcs( szWBuf, szBuf, strlen(szBuf) );
	//RETAILMSG(1,(szWBuf));
	NKDbgPrintfW( szWBuf );
}
void	tv_os_err_msg( LPCSTR szFormat, ... )
{
	wchar_t	szWBuf[ 128 ];
	char		szBuf[ 128 ];
	va_list	arglist;
	memset( szWBuf, 0, sizeof(szWBuf) );
	memset( szBuf, 0, sizeof(szBuf) );
	va_start( arglist, szFormat );
	_vsnprintf( 
			szBuf, 
			127, 
			szFormat, 
			arglist 
			);
	va_end( arglist );
	mbstowcs( szWBuf, szBuf, strlen(szBuf) );
	//RETAILMSG(1,(szWBuf));
	NKDbgPrintfW( szWBuf );
}
#elif defined(WIN98)
// For now, these fns are the same.
void	tv_os_dbg_msg( LPCSTR szFormat, ... )
{
	char		szBuf[ 128 ];
	va_list	arglist;
	memset( szBuf, 0, sizeof(szBuf) );
	va_start( arglist, szFormat );
	sprintf( 
			szBuf, 
			szFormat, 
			arglist 
			);
	va_end( arglist );
	DbgPrint( szBuf );
}
void	tv_os_err_msg( LPCSTR szFormat, ... )
{
	char		szBuf[ 128 ];
	va_list	arglist;
	memset( szBuf, 0, sizeof(szBuf) );
	va_start( arglist, szFormat );
	sprintf( 
			szBuf, 
			szFormat, 
			arglist 
			);
	va_end( arglist );
	DbgPrint( szBuf );
}
#endif

void tv_os_sleep( ULONG milliseconds )
{
#ifdef LINUX
#ifndef TV_OS_LINUX_2_4
	current->state = TASK_INTERRUPTIBLE;
	schedule_timeout( ( milliseconds * HZ ) / 1000 );
#else
	current->state = TASK_INTERRUPTIBLE;
	schedule_timeout( milliseconds );
#endif
#elif defined(WINCE)
	Sleep( milliseconds );
#elif defined(WIN98)
	LARGE_INTEGER	SysTimeout;
	if( KeGetCurrentIrql() >= DISPATCH_LEVEL )
	{
		ULONG	Cnt = 20 * milliseconds;
		while( Cnt-- )
		{
			//	KeStallExecutionProcessor: counted in us
			KeStallExecutionProcessor( 50 );
		}
	}
	else
	{
		SysTimeout.QuadPart = (LONGLONG)-10 * 1000 * milliseconds;
		//	KeDelayExecutionThread: counted in 100 ns
		KeDelayExecutionThread( KernelMode, FALSE, &SysTimeout );
	}
#elif defined(VXWORKS)

#endif
}

#ifdef LINUX
ULONG tv_os_get_tick_count()
{
	struct timeval tv1;
	ULONG	ultime;
	
	do_gettimeofday( &tv1 );
	ultime = tv1.tv_sec; 
	
	return ( ( ultime * 1000 ) + tv1.tv_usec );
}
#elif defined(WINCE)
// GetTickCount() is defined by OS.
ULONG tv_os_get_tick_count()
{
	// GetSystemTime() goes by 100 ns ticks.
	//return( GetSystemTime() / 10000 );
	return GetTickCount();
}
#elif defined(WIN98)
ULONG tv_os_get_tick_count()
{
    ULONGLONG ticks;
    ULONGLONG rate;

    ticks = (ULONGLONG)KeQueryPerformanceCounter((PLARGE_INTEGER)&rate).QuadPart;

    //
    // convert from ticks to 100ns clock
    //
    ticks = (ticks & 0xFFFFFFFF00000000) / rate * 10000000 +
    		(ticks & 0xFFFFFFFF) * 10000000 / rate;

	// GetSystemTime() goes by 100 ns ticks.
    return( (ULONG) ( ticks / 10000 ) );
}
#elif defined(VXWORKS)
ULONG tv_os_get_tick_count()
{
	return tickGet();
}
#endif

void* tv_os_kmalloc( ULONG size )
{
#ifdef LINUX
	return kmalloc( size, GFP_KERNEL ); 
#elif defined(WINCE)
	return VirtualAlloc( 0, size, MEM_COMMIT, PAGE_READWRITE | PAGE_NOCACHE );
#elif defined(WIN98)
	return ExAllocatePool( NonPagedPool, size );
#elif defined(VXWORKS)
#endif
}

void tv_os_kfree( void* data )
{
#ifdef LINUX
	kfree( data );
#elif defined(WINCE)
	VirtualFree( data, 0, MEM_RELEASE );
#elif defined(WIN98)
	ExFreePool( data );
#elif defined(VXWORKS)
#endif
}

void* tv_os_kmalloc_phys(  ULONG address, ULONG size )
{
#ifdef LINUX
	return ioremap( address, size );
#elif defined(WINCE)
	void*	pv = VirtualAlloc( 0, size, MEM_RESERVE, PAGE_NOACCESS );
	if( pv == NULL )
	{
		return NULL;
	}
	if( !VirtualCopy( pv, (LPVOID)(address >> 8), size, PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL ) )
	{
		VirtualFree( pv, 0, MEM_RELEASE );
		return NULL;
	}
	return pv;
#elif defined(WIN98)
	PHYSICAL_ADDRESS	phys;
	phys.QuadPart = address;
	return MmMapIoSpace( phys, size, MmNonCached );
#elif defined(VXWORKS)

#endif
}

#ifdef WIN98
void tv_os_kfree_phys( void* data, ULONG size )
#else
void tv_os_kfree_phys( void* data )
#endif
{
#ifdef LINUX
	iounmap( data );
#elif defined(WINCE)
	VirtualFree( data, 0, MEM_RELEASE );
#elif defined(WIN98)
	MmUnmapIoSpace( data, size );
#elif defined(VXWORKS)
#endif
}

#if defined(WINCE) || defined(WIN98)
BOOL 
tv_os_pci_dump( WORD bus, WORD vendor, WORD device, WORD fn )
{
	int		i;
	int		j;
	ULONG	pci_config_addr = 0; 
	ULONG	config_data = 0;
	ULONG	vendor_device = ( device << 16 ) | vendor;
	ULONG	vendor_pci_fn = 0;

	for( i = 0; i < 32; i++ ) 
	{
		pci_config_addr = 0x80000000 | (i<<11) ;  
		tv_os_io_write_ulong( 0x0CF8, (unsigned long)pci_config_addr );  
		config_data = tv_os_io_read_ulong( 0x0CFC ); 

		if( config_data == vendor_device )
		{
			tv_os_dbg_msg( "found: 0x%08X 0x%08X\n", pci_config_addr, config_data );
			tv_os_io_write_ulong( 0x0CF8, (ULONG)( pci_config_addr + ( fn * 256 ) + 0x2C ) );  
			vendor_pci_fn = tv_os_io_read_ulong( 0x0CFC ); 
			if( vendor_pci_fn == (ULONG)( ( ( device | fn ) << 16 ) | vendor ) )
			{
				for( j = 0; j < 256; j += 4 )
				{
					tv_os_io_write_ulong( 0x0CF8, (ULONG)( pci_config_addr + ( fn * 256 ) + j ) );  
					config_data = tv_os_io_read_ulong( 0x0CFC ); 
					tv_os_dbg_msg( "0x%08X 0x%08X\n", (ULONG)( pci_config_addr + ( fn * 256 ) + j ), config_data );
				}
				return TRUE;
			}
			else
			{
				tv_os_err_msg( "no such fn\n" );
				return FALSE;
			}
		}
	}
	return FALSE;
}

BOOL 
tv_os_pci_find_device( WORD bus, WORD vendor, WORD device, WORD fn, ULONG* config_addr )
{
	int		i;
	ULONG	pci_config_addr = 0; 
	ULONG	config_data = 0;
	ULONG	vendor_device = ( device << 16 ) | vendor;
	ULONG	vendor_pci_fn = 0;

	for( i = 0; i < 32; i++ ) 
	{
		pci_config_addr = 0x80000000 | (i<<11) ;  
		tv_os_io_write_ulong( 0x0CF8, (unsigned long)pci_config_addr );  
		config_data = tv_os_io_read_ulong( 0x0CFC ); 
		if( config_data == vendor_device )
		{
			//tv_os_dbg_msg( "found: 0x%08X 0x%08X\n", pci_config_addr, config_data );
			tv_os_io_write_ulong( 0x0CF8, (ULONG)( pci_config_addr + ( fn * 256 ) + 0x2C ) );  
			vendor_pci_fn = tv_os_io_read_ulong( 0x0CFC ); 
			if( vendor_pci_fn == (ULONG)( ( ( device | fn ) << 16 ) | vendor ) )
			{
				*config_addr = (ULONG)( pci_config_addr + ( fn * 256 ) );
				return TRUE;
			}
			else
			{
				//tv_os_err_msg( "no such fn\n" );
				return FALSE;
			}
		}
	}
	return FALSE;
}

BOOL 
tv_os_pci_read_ulong( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, ULONG* data )
{
	ULONG	pci_config_addr = 0; 
	if( !tv_os_pci_find_device( bus, vendor, device, fn, &pci_config_addr ) )
	{
		return FALSE;
	}
	WRITE_PORT_ULONG( (ULONG*)0x0CF8, (ULONG)( pci_config_addr + reg ) );
	*data = READ_PORT_ULONG( (ULONG*)0x0CFC );
	//tv_os_io_write_ulong( 0x0CF8, (ULONG)( pci_config_addr + reg ) );  
	//*data = tv_os_io_read_ulong( 0x0CFC ); 
	return TRUE;
}

BOOL 
tv_os_pci_write_ulong( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, ULONG data )
{
	ULONG	pci_config_addr = 0; 
	if( !tv_os_pci_find_device( bus, vendor, device, fn, &pci_config_addr ) )
	{
		return FALSE;
	}
	WRITE_PORT_ULONG( (ULONG*)0x0CF8, (ULONG)( pci_config_addr + reg ) );
	WRITE_PORT_ULONG( (ULONG*)0x0CFC, data );
	//tv_os_io_write_ulong( 0x0CF8, pci_target );  
	//tv_os_io_write_ulong( 0x0CFC, data ); 
	return TRUE;
}

BOOL 
tv_os_pci_read_byte( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, BYTE* data )
{
	ULONG	pci_config_addr = 0; 
	if( !tv_os_pci_find_device( bus, vendor, device, fn, &pci_config_addr ) )
	{
		return FALSE;
	}
	WRITE_PORT_ULONG( (ULONG*)0x0CF8, (ULONG)( pci_config_addr + reg ) );
	*data = READ_PORT_UCHAR( (UCHAR*)0x0CFC );
	//tv_os_io_write_ulong( 0x0CF8, (ULONG)( pci_config_addr + reg ) );  
	//*data = tv_os_io_read_byte( 0x0CFF ); 
	return TRUE;
}

BOOL 
tv_os_pci_write_byte( WORD bus, WORD vendor, WORD device, WORD fn, BYTE reg, BYTE data )
{
	ULONG	pci_config_addr = 0; 
	if( !tv_os_pci_find_device( bus, vendor, device, fn, &pci_config_addr ) )
	{
		return FALSE;
	}
	WRITE_PORT_ULONG( (ULONG*)0x0CF8, (ULONG)( pci_config_addr + reg ) );
	WRITE_PORT_UCHAR( (UCHAR*)0x0CFC, data );
	//tv_os_io_write_ulong( 0x0CF8, (ULONG)( pci_config_addr + reg ) );  
	//tv_os_io_write_byte( 0x0CFC, data ); 
	return TRUE;
}
#endif

BOOL tv_os_pciconfig_find_dev( WORD bus, WORD dev, WORD func, ULONG* address )
{
	int		i;
	ULONG	pci_config_addr = 0; 
	ULONG	config_data = 0;
	ULONG	id = ( func << 16 ) | dev;

	for( i = 0; i < 32; i++ ) 
	{
		pci_config_addr = 0x80000000 | (i<<11) ;  
		tv_os_io_write_ulong( 0x0CF8, (unsigned long)pci_config_addr );  
		config_data = tv_os_io_read_ulong( 0x0CFC ); 

		if( config_data == id )
		{
			*address = pci_config_addr;
			return TRUE;
		}
	}

	return FALSE;
}

BOOL tv_os_pciconfig_read_byte( int bus, int dev, int func, int address, BYTE* data )
{
#ifdef LINUX
	if( pcibios_read_config_byte( (BYTE)bus, (BYTE)PCI_DEVFN(dev,func), (BYTE)address, data ) )
	{
		return FALSE;
	}
	else
	{
		return TRUE;    
	}
#elif defined(WINCE) || defined(WIN98)
//#elif defined(WINCE)
	ULONG	pci_address = 0;
	if( !tv_os_pciconfig_find_dev( (WORD)bus, (WORD)dev, (WORD)func, &pci_address )	)
	{
		return FALSE;
	}
	tv_os_io_write_ulong( 0x0CF8, pci_address + address );
	*data = (BYTE)tv_os_io_read_ulong( 0x0CFC );
#elif defined(WIN98)
/*
	//BUGBUG: Check this.
	ULONG ulConfigSpace;
	ulConfigSpace = PCIReadConfig( bus, ( func << 3 ) | dev, address & ~3 );
	*data = (BYTE)( ulConfigSpace >> ( 8 * ( address & 3 ) ) );
	return TRUE;
*/
#elif defined(VXWORKS)
#endif
	return TRUE;
}

BOOL tv_os_pciconfig_write_byte( int bus, int dev, int func, int address, BYTE data )
{
#ifdef LINUX
	if( pcibios_write_config_byte( (BYTE) bus, (BYTE) PCI_DEVFN(dev,func), (BYTE)address, data ) )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
#elif defined(WINCE) || defined(WIN98)
	ULONG	pci_address = 0;
	if( !tv_os_pciconfig_find_dev( (WORD)bus, (WORD)dev, (WORD)func, &pci_address ) )
	{
		return FALSE;
	}
	tv_os_io_write_ulong( 0x0CF8, pci_address + address );
	tv_os_io_write_byte( 0x0CFC, data );
#elif defined(VXWORKS)
#endif
	return TRUE;
}

BOOL tv_os_pciconfig_read_word( int bus, int dev, int func, int address, WORD* data )
{
#ifdef LINUX
	if( pcibios_read_config_word( (BYTE)bus, (BYTE)PCI_DEVFN(dev,func), (BYTE)address, data ) )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
#elif defined(WINCE) || defined(WIN98)
	ULONG	pci_address = 0;
	if( !tv_os_pciconfig_find_dev( (WORD)bus, (WORD)dev, (WORD)func, &pci_address )	)
	{
		return FALSE;
	}
	tv_os_io_write_ulong( 0x0CF8, pci_address + address );
	*data = (WORD)tv_os_io_read_ulong( 0x0CFC );
#elif defined(VXWORKS)
#endif
	return TRUE;
}

BOOL tv_os_pciconfig_write_word( int bus, int dev, int func, int address, WORD data)
{
#ifdef LINUX
	if( pcibios_write_config_word( (BYTE) bus, (BYTE)PCI_DEVFN(dev,func), (BYTE)address, data ) )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
#elif defined(WINCE) || defined(WIN98)
	ULONG	pci_address = 0;
	if( !tv_os_pciconfig_find_dev( (WORD)bus, (WORD)dev, (WORD)func, &pci_address ) )
	{
		return FALSE;
	}
	tv_os_io_write_ulong( 0x0CF8, pci_address + address );
	tv_os_io_write_word( 0x0CFC, data );
#elif defined(VXWORKS)
#endif
	return TRUE;
}

BOOL tv_os_pciconfig_read_ulong( int bus, int dev, int func, int address, ULONG* data )
{
#ifdef LINUX
	if( pcibios_read_config_dword( (BYTE)bus, (BYTE)PCI_DEVFN(dev,func), (BYTE)address, (unsigned int*)data ) )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
#elif defined(WINCE) || defined(WIN98)
	ULONG	pci_address = 0;
	if( !tv_os_pciconfig_find_dev( (WORD)bus, (WORD)dev, (WORD)func, &pci_address )	)
	{
		return FALSE;
	}
	tv_os_io_write_ulong( 0x0CF8, pci_address + address );
	*data = (ULONG)tv_os_io_read_ulong( 0x0CFC );
#elif defined(VXWORKS)
#endif
	return TRUE;
}

BOOL tv_os_pciconfig_write_ulong( int bus, int dev, int func, int address, ULONG data )
{
#ifdef LINUX
	if( pcibios_write_config_dword( (BYTE)bus, (BYTE)PCI_DEVFN(dev,func), (BYTE)address, (ULONG)data ) )
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
#elif defined(WINCE) || defined(WIN98)
	ULONG	pci_address = 0;
	if( !tv_os_pciconfig_find_dev( (WORD)bus, (WORD)dev, (WORD)func, &pci_address ) )
	{
		return FALSE;
	}
	tv_os_io_write_ulong( 0x0CF8, pci_address + address );
	tv_os_io_write_ulong( 0x0CFC, data );
#elif defined(VXWORKS)
#endif
	return TRUE;
}


BYTE tv_os_io_read_byte( WORD port )
{
#ifdef LINUX
	return inb( port );
#elif defined(WINCE)
	return _inp( port );
#elif defined(WIN98)
	return READ_PORT_UCHAR( (UCHAR*)port );
#elif defined(VXWORKS)
#endif
}

void tv_os_io_write_byte( WORD port, BYTE data )
{
#ifdef LINUX
    outb( data, port );
#elif defined(WINCE)
    _outp( port, data );
#elif defined(WIN98)
    WRITE_PORT_UCHAR( (BYTE*)port, data );
#elif defined(VXWORKS)
#endif
}

unsigned short tv_os_io_read_word( WORD port )
{
#ifdef LINUX
	return inw( port );
#elif defined(WINCE)
	return _inpw( port );
#elif defined(WIN98)
	return READ_PORT_USHORT( (WORD*)port );
#elif defined(VXWORKS)
#endif
}

void tv_os_io_write_word( WORD port, WORD data )
{
#ifdef LINUX
    outw( data, port );
#elif defined(WINCE)
    _outpw( port, data );
#elif defined(WIN98)
    WRITE_PORT_USHORT( (WORD*)port, data );
#elif defined(VXWORKS)
#endif
}

ULONG tv_os_io_read_ulong( WORD port )
{
#ifdef LINUX
	return inl( port );
#elif defined(WINCE)
	return _inpd( port );
#elif defined(WIN98)
	return READ_PORT_ULONG( (ULONG*)port );
#elif defined(VXWORKS)
#endif
}

void tv_os_io_write_ulong( WORD port, ULONG data )
{
#ifdef LINUX
    outl( data, port );
#elif defined(WINCE)
    _outpd( port, data );
#elif defined(WIN98)
    WRITE_PORT_ULONG( (ULONG*)port, data );
#elif defined(VXWORKS)
#endif
}
