/* NSC_LIC_BSD
 *
 * Revision 1.2
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    TVLIB
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) July 19, 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 19, 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */

/*
 * This file contains routines to program the SAA7114 video decoder.
 *
 * Decoder functions:
 * 
 *    tv_saa7114_detect_hw
 *    tv_saa7114_software_reset
 *    tv_saa7114_set_input_win
 *    tv_saa7114_set_output_win
 *    tv_saa7114_set_decoder_vbi_upscale
 *    tv_saa7114_decoder_detect_macrovision
 *    tv_saa7114_decoder_detect_video
 *    tv_saa7114_set_decoder_analog_input
 *    tv_saa7114_set_decoder_brightness
 *    tv_saa7114_set_decoder_contrast
 *    tv_saa7114_set_decoder_hue
 *    tv_saa7114_set_decoder_saturation
 *    tv_saa7114_set_decoder_input_offset
 *    tv_saa7114_set_decoder_input_size
 *    tv_saa7114_set_decoder_output_size
 *    tv_saa7114_set_decoder_scale
 *    tv_saa7114_set_decoder_vbi_format
 *    tv_saa7114_set_decoder_vbi_enable
 *    tv_saa7114_get_decoder_brightness
 *    tv_saa7114_get_decoder_contrast
 *    tv_saa7114_get_decoder_hue
 *    tv_saa7114_get_decoder_saturation
 *    tv_saa7114_get_decoder_input_offset
 *    tv_saa7114_get_decoder_input_size
 *    tv_saa7114_get_decoder_output_size
 *    tv_saa7114_get_decoder_vbi_format
 *    tv_saa7114_set_tv_standard
 *    tv_saa7114_set_luminance
 *    tv_saa7114_initialize_ntsc
 *    tv_saa7114_xport_enable
 *    tv_saa7114_set_input
 *    tv_saa7114_set_scaling_parameters
 *    tv_saa7114_scale_video
 *    
 * Project: TVLIB
 * Author:  Peter Lohmann
 * Date:    April 10, 2001
 *---------------------------------------------------------------------*/

#ifndef TV_SAA7114_H
#define TV_SAA7114_H

#include "tv_i2c.h"
#include "tv_os_geode.h"


#ifdef __cplusplus
extern "C" {
#endif


#define TV_SAA7114_STANDARD_NTSC	1
#define TV_SAA7114_STANDARD_PAL		2

#define VBI_FORMAT_VIDEO            0x1
#define VBI_FORMAT_RAW              0x2
#define VBI_FORMAT_CC               0x4
#define VBI_FORMAT_NABTS            0x8


int tv_saa7114_detect_hw();
int tv_saa7114_software_reset();
//void tv_saa7114_init();
void tv_saa7114_set_input_win( int std, WORD width, WORD height );
void tv_saa7114_set_output_win( WORD srcWidth, WORD srcHeight, WORD dstWidth, WORD dstHeight );
int tv_saa7114_set_decoder_vbi_upscale();
int tv_saa7114_decoder_detect_macrovision();
int tv_saa7114_decoder_detect_video();
int tv_saa7114_set_decoder_analog_input( BYTE input );
int tv_saa7114_set_decoder_brightness( BYTE brightness );
int tv_saa7114_set_decoder_contrast( BYTE contrast );
int tv_saa7114_set_decoder_hue( char hue );
int tv_saa7114_set_decoder_saturation( BYTE saturation );
int tv_saa7114_set_decoder_input_offset( WORD x, WORD y );
int tv_saa7114_set_decoder_input_size( WORD width, WORD height );
int tv_saa7114_set_decoder_output_size( WORD width, WORD height );
//int tv_saa7114_set_decoder_scale( WORD srcw, WORD srch, WORD dstw, WORD dsth );
int tv_saa7114_set_decoder_vbi_format( int start, int end, int format );
int tv_saa7114_set_decoder_vbi_enable( int enable );
BYTE tv_saa7114_get_decoder_brightness();
BYTE tv_saa7114_get_decoder_contrast();
char tv_saa7114_get_decoder_hue();
BYTE tv_saa7114_get_decoder_saturation();
ULONG tv_saa7114_get_decoder_input_offset();
ULONG tv_saa7114_get_decoder_input_size();
ULONG tv_saa7114_get_decoder_output_size();
int tv_saa7114_get_decoder_vbi_format( int line );
int tv_saa7114_set_tv_standard( int standard );
void tv_saa7114_set_luminance( BYTE val );
//void tv_saa7114_set_brightness( BYTE val );
//void tv_saa7114_set_contrast( BYTE val );
//void tv_saa7114_set_saturation( BYTE val );
//void tv_saa7114_set_hue( BYTE val );
void tv_saa7114_initialize_ntsc();
void tv_saa7114_xport_enable( int enable );
void tv_saa7114_set_vbi_input_offset(WORD x, WORD y);
void tv_saa7114_set_vbi_input_size(WORD width, WORD height);
void tv_saa7114_set_vbi_output_size(WORD width, WORD height);
#if 1
int tv_saa7114_set_input( int current_input, int new_input,
	int xoffset, int yoffset, int sourceW, int sourceH,
	 int width, int height, int standard );
#else
int tv_saa7114_set_input( int current_input, int new_input, int width, int height );
#endif

#define TV_SAA7114_REG_LUMINANCE		0x09
//#define TV_SAA7114_REG_BRIGHTNESS		0x0A
//#define TV_SAA7114_REG_CONTRAST		0x0B
//#define TV_SAA7114_REG_SATURATION		0x0C
//#define TV_SAA7114_REG_HUE			0x0D

#define TV_SAA7114_ANALOG_INPUT_CTRL1			0x02
#define TV_SAA7114_BRIGHTNESS					0x0A
#define TV_SAA7114_CONTRAST					0x0B
#define TV_SAA7114_SATURATION					0x0C
#define TV_SAA7114_HUE							0x0D
#define TV_SAA7114_STATUS						0x1F
#define TV_SAA7114_IPORT_CONTROL				0x86

/* TASK A REGISTER DEFINITIONS */

#define TV_SAA7114_TASK_A_HORZ_OUTPUT_LO		0x9C
#define TV_SAA7114_TASK_A_HORZ_OUTPUT_HI		0x9D
#define TV_SAA7114_TASK_A_HSCALE_LUMA_LO		0xA8
#define TV_SAA7114_TASK_A_HSCALE_LUMA_HI		0xA9
#define TV_SAA7114_TASK_A_HSCALE_CHROMA_LO		0xAC
#define TV_SAA7114_TASK_A_HSCALE_CHROMA_HI		0xAD

/* TASK B REGISTER DEFINITIONS */

#define TV_SAA7114_HORZ_OFFSET_LO				0xC4
#define TV_SAA7114_HORZ_OFFSET_HI				0xC5
#define TV_SAA7114_HORZ_INPUT_LO				0xC6
#define TV_SAA7114_HORZ_INPUT_HI				0xC7
#define TV_SAA7114_VERT_OFFSET_LO				0xC8
#define TV_SAA7114_VERT_OFFSET_HI				0xC9
#define TV_SAA7114_VERT_INPUT_LO				0xCA
#define TV_SAA7114_VERT_INPUT_HI				0xCB
#define TV_SAA7114_HORZ_OUTPUT_LO				0xCC
#define TV_SAA7114_HORZ_OUTPUT_HI				0xCD
#define TV_SAA7114_VERT_OUTPUT_LO				0xCE
#define TV_SAA7114_VERT_OUTPUT_HI				0xCF
#define TV_SAA7114_HORZ_PRESCALER				0xD0
#define TV_SAA7114_HORZ_ACL					0xD1
#define TV_SAA7114_HORZ_FIR_PREFILTER			0xD2
#define TV_SAA7114_FILTER_CONTRAST				0xD5
#define TV_SAA7114_FILTER_SATURATION			0xD6
#define TV_SAA7114_HSCALE_LUMA_LO				0xD8
#define TV_SAA7114_HSCALE_LUMA_HI				0xD9
#define TV_SAA7114_HSCALE_CHROMA_LO			0xDC
#define TV_SAA7114_HSCALE_CHROMA_HI			0xDD
#define TV_SAA7114_VSCALE_LUMA_LO				0xE0
#define TV_SAA7114_VSCALE_LUMA_HI				0xE1
#define TV_SAA7114_VSCALE_CHROMA_LO			0xE2
#define TV_SAA7114_VSCALE_CHROMA_HI			0xE3
#define TV_SAA7114_VSCALE_CONTROL				0xE4
#define TV_SAA7114_VSCALE_CHROMA_OFFS0			0xE8
#define TV_SAA7114_VSCALE_CHROMA_OFFS1			0xE9
#define TV_SAA7114_VSCALE_CHROMA_OFFS2			0xEA
#define TV_SAA7114_VSCALE_CHROMA_OFFS3			0xEB
#define TV_SAA7114_VSCALE_LUMINA_OFFS0			0xEC
#define TV_SAA7114_VSCALE_LUMINA_OFFS1			0xED
#define TV_SAA7114_VSCALE_LUMINA_OFFS2			0xEE
#define TV_SAA7114_VSCALE_LUMINA_OFFS3			0xEF

/* Still need to determine PHO value (common phase offset) */
#define TV_SAA7114_VSCALE_PHO					0xFF

typedef struct tagTV_SAA7114_FIR_PREFILTER
{
	unsigned char prescaler;
	unsigned char acl_low;
	unsigned char prefilter_low;
	unsigned char acl_high;	
	unsigned char prefilter_high;
} TV_SAA7114_FIR_PREFILTER;





enum tagDECODERREGS
{
        ChipVersion=            0x00,
        // front-end part
        IncrementDelay=         0x01,
        AnalogInputControl1=    0x02,
        AnalogInputControl2=    0x03,
        AnalogInputControl3=    0x04,
        AnalogInputControl4=    0x05,
        // decoder part
        HorzSyncStart=          0x06,
        HorzSyncStop=           0x07,
        SyncControl=            0x08,
        LuminanceControl=       0x09,
        LuminanceBrightness=    0x0A,
        LuminanceContrast=      0x0B,
        ChromaSaturation=       0x0C,
        ChromaHueControl=       0x0D,
        ChromaControl1=         0x0E,
        ChromaGainControl=      0x0F,
        ChromaControl2=         0x10,

        ModeDelayControl=       0x11,
        RTSignalControl=        0x12,
        RTOutputControl=        0x13,
        CompatibilityControl=   0x14,

        VGateStart=             0x15,
        VGateStop=              0x16,
        VGateMSB=               0x17,

        RawDataGain=            0x18,
        RawDataOffset=          0x19,

        StatusByte=             0x1F,
        // Textslicer
        AC1=                    0x40,
        LCR2=                   0x41,
        LCR3=                   0x42,
        LCR4=                   0x43,
        LCR5=                   0x44,
        LCR6=                   0x45,
        LCR7=                   0x46,
        LCR8=                   0x47,
        LCR9=                   0x48,
        LCR10=                  0x49,
        LCR11=                  0x4A,
        LCR12=                  0x4B,
        LCR13=                  0x4C,
        LCR14=                  0x4D,
        LCR15=                  0x4E,
        LCR16=                  0x4F,
        LCR17=                  0x50,
        LCR18=                  0x51,
        LCR19=                  0x52,
        LCR20=                  0x53,
        LCR21=                  0x54,
        LCR22=                  0x55,
        LCR23=                  0x56,
        LCR24=                  0x57,
        FC=                     0x58,
        HOFF=                   0x59,
        VOFF=                   0x5A,
        HVOFF=                  0x5B,
        DID=                    0x5D,
        SDID=                   0x5E,
        DR=                     0x60,
        LN1=                    0x61,
        LN2=                    0x62,
        LineFlagsEven1=         0x63,
        LineFlagsEven2=         0x64,
        LineFlagsEven3=         0x65,
        LineFlagsOdd1=          0x66,
        LineFlagsOdd2=          0x67,
        LineFlagsOdd3=          0x68,

        // X-Port, I-Port and scaler
        GlobalControl1=         0x80,
        XPortIO=                0x83,
        IPortSignalDef=         0x84,
        IPortSignalPol=         0x85,
        IPortFIFOFlag=          0x86,
        IPortIODelay=           0x87,
        PowerSaveControl=       0x88,

        ScalerStatus=           0x8F,

        // task A
        ATaskHandlingControl=   0x90,
        AXPortFormat=           0x91,
        AXPortInputRef=         0x92,
        AIPortFormat=           0x93,
        AHorzInputStart=        0x94,
        AHorzInputStartMSB=     0x95,
        AHorzInputLength=       0x96,
        AHorzInputLengthMSB=    0x97,
        AVertInputStart=        0x98,
        AVertInputStartMSB=     0x99,
        AVertInputLength=       0x9A,
        AVertInputLengthMSB=    0x9B,
        AHorzOutputLength=      0x9C,
        AHorzOutputLengthMSB=   0x9D,
        AVertOutputLength=      0x9E,
        AVertOutputLengthMSB=   0x9F,

        AHorzPrescale=          0xA0,
        AAcl=                   0xA1,
        ADCGain=                0xA2,

        ALuminanceBrightness=   0xA4,
        ALuminanceContrast=     0xA5,
        AChromaSaturation=      0xA6,

        AHorzScalingLuma=       0xA8,
        AHorzScalingLumaMSB=    0xA9,
        AHorzPhaseLuma=         0xAA,

        AHorzScalingChroma=     0xAC,
        AHorzScalingChromaMSB=  0xAD,
        AHorzPhaseChroma=       0xAE,

        AVertScalingLuma=       0xB0,
        AVertScalingLumaMSB=    0xB1,
        AVertScalingChroma=     0xB2,
        AVertScalingChromaMSB=  0xB3,
        AVertScalingMode=       0xB4,

        AVertPhaseChroma00=     0xB8,
        AVertPhaseChroma01=     0xB9,
        AVertPhaseChroma10=     0xBA,
        AVertPhaseChroma11=     0xBB,
        AVertPhaseLuma00=       0xBC,
        AVertPhaseLuma01=       0xBD,
        AVertPhaseLuma10=       0xBE,
        AVertPhaseLuma11=       0xBF,
        // task B
        BTaskHandlingControl=   0xC0,
        BXPortFormat=           0xC1,
        BXPortInputRef=         0xC2,
        BIPortFormat=           0xC3,
        BHorzInputStart=        0xC4,
        BHorzInputStartMSB=     0xC5,
        BHorzInputLength=       0xC6,
        BHorzInputLengthMSB=    0xC7,
        BVertInputStart=        0xC8,
        BVertInputStartMSB=     0xC9,
        BVertInputLength=       0xCA,
        BVertInputLengthMSB=    0xCB,
        BHorzOutputLength=      0xCC,
        BHorzOutputLengthMSB=   0xCD,
        BVertOutputLength=      0xCE,
        BVertOutputLengthMSB=   0xCF,

        BHorzPrescale=          0xD0,
        BAcl=                   0xD1,
        BDCGain=                0xD2,

        BLuminanceBrightness=   0xD4,
        BLuminanceContrast=     0xD5,
        BChromaSaturation=      0xD6,

        BHorzScalingLuma=       0xD8,
        BHorzScalingLumaMSB=    0xD9,
        BHorzPhaseLuma=         0xDA,

        BHorzScalingChroma=     0xDC,
        BHorzScalingChromaMSB=  0xDD,
        BHorzPhaseChroma=       0xDE,

        BVertScalingLuma=       0xE0,
        BVertScalingLumaMSB=    0xE1,
        BVertScalingChroma=     0xE2,
        BVertScalingChromaMSB=  0xE3,
        BVertScalingMode=       0xE4,

        BVertPhaseChroma00=     0xE8,
        BVertPhaseChroma01=     0xE9,
        BVertPhaseChroma10=     0xEA,
        BVertPhaseChroma11=     0xEB,
        BVertPhaseLuma00=       0xEC,
        BVertPhaseLuma01=       0xED,
        BVertPhaseLuma10=       0xEE,
        BVertPhaseLuma11=       0xEF,
};

typedef struct tagScaleBuf
{
		BYTE Buf_ATaskHandlingControl;	//0x90
		BYTE Buf_AXPortFormat;				//0x91
		BYTE Buf_AXPortInputRef;			//0x92
		BYTE Buf_AIPortFormat;				//0x93
		BYTE Buf_AHorzInputStart;			//0x94
		BYTE Buf_AHorzInputStartMSB;		//0x95
		BYTE Buf_AHorzInputLength;			//0x96
		BYTE Buf_AHorzInputLengthMSB;		//0x97
		BYTE Buf_AVertInputStart;			//0x98
		BYTE Buf_AVertInputStartMSB;		//0x99
		BYTE Buf_AVertInputLength;			//0x9A
		BYTE Buf_AVertInputLengthMSB;		//0x9B
		BYTE Buf_AHorzOutputLength;		//0x9C
		BYTE Buf_AHorzOutputLengthMSB;	//0x9D
		BYTE Buf_AVertOutputLength;		//0x9E
		BYTE Buf_AVertOutputLengthMSB;	//0x9F
		BYTE Buf_AHorzPrescale;				//0xA0
		BYTE Buf_AAcl;							//0xA1
		BYTE Buf_ADCGain;						//0xA2
		BYTE Buf_Reserved1;					//0xA3
		BYTE Buf_ALuminanceBrightness;	//0xA4
		BYTE Buf_ALuminanceContrast;		//0xA5
		BYTE Buf_AChromaSaturation;		//0xA6
		BYTE Buf_Reserved2;					//0xA7
		BYTE Buf_AHorzScalingLuma;			//0xA8
		BYTE Buf_AHorzScalingLumaMSB;		//0xA9
		BYTE Buf_AHorzPhaseLuma;			//0xAA
		BYTE Buf_Reserved3;					//0xAB
		BYTE Buf_AHorzScalingChroma;		//0xAC
		BYTE Buf_AHorzScalingChromaMSB;	//0xAD
		BYTE Buf_AHorzPhaseChroma;			//0xAE
		BYTE Buf_Reserved4;					//0xAF
		BYTE Buf_AVertScalingLuma;			//0xB0
		BYTE Buf_AVertScalingLumaMSB;		//0xB1
		BYTE Buf_AVertScalingChroma;		//0xB2
		BYTE Buf_AVertScalingChromaMSB;	//0xB3
		BYTE Buf_AVertScalingMode;			//0xB4
		BYTE Buf_Reserved5;					//0xB5
		BYTE Buf_Reserved6;					//0xB6
		BYTE Buf_Reserved7;					//0xB7
		BYTE Buf_AVertPhaseChroma00;		//0xB8
		BYTE Buf_AVertPhaseChroma01; 		//0xB9
		BYTE Buf_AVertPhaseChroma10;		//0xBA
		BYTE Buf_AVertPhaseChroma11;		//0xBB
		BYTE Buf_AVertPhaseLuma00;			//0xBC
		BYTE Buf_AVertPhaseLuma01;			//0xBD
		BYTE Buf_AVertPhaseLuma10;			//0xBE
		BYTE Buf_AVertPhaseLuma11;			//0xBF
		BYTE Buf_BTaskHandlingControl;	//0xC0
		BYTE Buf_BXPortFormat;       		//0xC1
		BYTE Buf_BXPortInputRef;		     //0xC2
		BYTE Buf_BIPortFormat;       		//0xC3
		BYTE Buf_BHorzInputStart;			//0xC4
		BYTE Buf_BHorzInputStartMSB;		//0xC5
		BYTE Buf_BHorzInputLength;			//0xC6
		BYTE Buf_BHorzInputLengthMSB;		//0xC7
		BYTE Buf_BVertInputStart;			//0xC8
		BYTE Buf_BVertInputStartMSB;		//0xC9
		BYTE Buf_BVertInputLength;			//0xCA
		BYTE Buf_BVertInputLengthMSB;		//0xCB
		BYTE Buf_BHorzOutputLength;		//0xCC
		BYTE Buf_BHorzOutputLengthMSB;	//0xCD
		BYTE Buf_BVertOutputLength;		//0xCE
		BYTE Buf_BVertOutputLengthMSB;	//0xCF
		BYTE Buf_BHorzPrescale;				//0xD0
		BYTE Buf_BAcl;							//0xD1
		BYTE Buf_BDCGain;						//0xD2
		BYTE Buf_Reserved8;					//0xD3
		BYTE Buf_BLuminanceBrightness;	//0xD4
		BYTE Buf_BLuminanceContrast;		//0xD5
		BYTE Buf_BChromaSaturation;		//0xD6
		BYTE Buf_Reserved9;					//0xD7
		BYTE Buf_BHorzScalingLuma;			//0xD8
		BYTE Buf_BHorzScalingLumaMSB;		//0xD9
		BYTE Buf_BHorzPhaseLuma;			//0xDA
		BYTE Buf_Reserved10;					//0xDB
		BYTE Buf_BHorzScalingChroma;		//0xDC
		BYTE Buf_BHorzScalingChromaMSB;	//0xDD
		BYTE Buf_BHorzPhaseChroma;			//0xDE
		BYTE Buf_Reserved11;					//0xDF
		BYTE Buf_BVertScalingLuma;			//0xE0
		BYTE Buf_BVertScalingLumaMSB;		//0xE1
		BYTE Buf_BVertScalingChroma;		//0xE2
		BYTE Buf_BVertScalingChromaMSB;	//0xE3
		BYTE Buf_BVertScalingMode;			//0xE4
		BYTE Buf_Reserved12;					//0xE5
		BYTE Buf_Reserved13;					//0xE6
		BYTE Buf_Reserved14;					//0xE7
		BYTE Buf_BVertPhaseChroma00;		//0xE8
		BYTE Buf_BVertPhaseChroma01; 		//0xE9
		BYTE Buf_BVertPhaseChroma10;		//0xEA
		BYTE Buf_BVertPhaseChroma11;		//0xEB
		BYTE Buf_BVertPhaseLuma00;			//0xEC
		BYTE Buf_BVertPhaseLuma01;			//0xED
		BYTE Buf_BVertPhaseLuma10;			//0xEE
		BYTE Buf_BVertPhaseLuma11;			//0xEF
} ScaleBuf;


typedef struct tagScaleParas
{
	BYTE	bTaskSelect;
	BYTE	bSkipFields;
	BYTE	bRepeatTask;
	WORD 	wVertInputLength;
	WORD	wVertInputStart;
	WORD 	wHorzInputLength;
	WORD	wHorzInputStart;
	BYTE	bHorzPrescale;
	BYTE	bDCGain;
	BYTE	bXC2_1;
 	BYTE	bFIRLuma;
	BYTE	bFIRChroma;
	BYTE	AVertScalingMode;
	BYTE	bVertScalingMode;
	BYTE	bBrightness;
	BYTE	bContrast;
	BYTE	bSaturation;
	WORD	wHorzScalingIncrementLuma;
	WORD	wHorzScalingIncrementChroma;
	WORD	wVertScalingIncrementLuma;
	WORD	wVertScalingIncrementChroma;
	BYTE	wHorzPhaseOffsetLuma;
	BYTE	wHorzPhaseOffsetChroma;
	WORD	wVertPhaseOffsetLuma00;
	WORD	wVertPhaseOffsetLuma01;
	WORD	wVertPhaseOffsetLuma10;
	WORD	wVertPhaseOffsetLuma11;
	WORD	wVertPhaseOffsetChroma00;
	WORD	wVertPhaseOffsetChroma01;
	WORD	wVertPhaseOffsetChroma10;
	WORD	wVertPhaseOffsetChroma11;
} ScaleParas;

typedef struct tagInWindow
{
	WORD nLeft;
	WORD nWidth;
	WORD nTop;
	WORD nHeight;
} InWindow;

typedef struct tagOutWindow
{
	int nWidth;
	int nHeight;
} OutWindow;

typedef struct tagXSCALE
{
	BYTE		xACL;   // @field Accumulation length.
	BYTE		xC2_1;  // @field Filter coefficient select.
	BYTE		xDCGain;// @field DCGain setting.
} XSCALE;

#define MAX_XPSC	64

#define VBI_TASK    0x10		// Task A
#define VIDEO_TASK  0x20		// Task B
#define BOTH_TASKS  0x30

#define TaskSelect  VIDEO_TASK

#define	VD_NTSC     0
#define	VD_PAL	    1

#define DO_VIDEO    1
#define DO_VBI      1

void tv_saa7114_set_scaling_parameters( int, int, int, int, int Xpixel, int Ypixel , int);
void tv_saa7114_set_decoder_scale(int,int,int,int, int X, int Y, int );

#ifdef __cplusplus
}
#endif

#endif	// TV_SAA7114_H



