/* NSC_LIC_BSD
 *
 * Revision 1.2
 *
 * National Semiconductor Corporation Open Source License for
 *
 *    TVLIB
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) July 19, 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use by Licensee of this Software in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must either:
 *     (a) reproduce the above copyright notice, this list of conditions
 *         and the following disclaimer in the documentation and/or
 *         other materials provided with the distribution, or
 *     (b) if Licensee elects to redistribute any Software not in
 *         accordance with (a), then Licensee shall indemnify National
 *         Semiconductor Corporation and all copyright holders and
 *         contributors for any claims related to such Software (and pay
 *         all costs and damages, including all attorney fees)
 *
 *   * Neither the name of the National Semiconductor Corporation nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF
 * YOUR JURISDICTION. It is Licensee's responsibility to comply with
 * any export regulations applicable in Licensee's jurisdiction. Under
 * CURRENT (July 19, 2001) U.S. export regulations this software
 * is eligible for export from the U.S. and can be downloaded by or
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran,
 * Syria, Sudan, Afghanistan and any other country to which the U.S.
 * has embargoed goods and services.
 *
 * END_NSC_LIC_BSD */

/*
 * This file contains routines to program TV tuners (various).
 *
 * Decoder functions:
 * 
 *    tv_tuner_channel_freq
 *    tv_tuner_get_signal_strength
 *    tv_tuner_get_freq
 *    tv_tuner_change_channel
 *    
 * Project: TVLIB
 * Author:  Peter Lohmann
 * Date:    April 10, 2001
 *---------------------------------------------------------------------*/

#ifndef TV_TUNER_H
#define TV_TUNER_H

#include "tv_i2c.h"
#include "tv_os_geode.h"

#ifdef __cplusplus
extern "C" {
#endif

#define TUNER_TYPE_TEMIC_PAL 				0
#define TUNER_TYPE_PHILIPS_PAL_I			1 
#define TUNER_TYPE_PHILIPS_NTSC			2 
#define TUNER_TYPE_PHILIPS_SECAM			3 
#define TUNER_TYPE_PHILIPS_PAL			4 
#define TUNER_TYPE_TEMIC_NTSC				5 
#define TUNER_TYPE_TEMIC_PAL_I			6 
#define TUNER_TYPE_TEMIC_4036_FY5			7 
#define TUNER_TYPE_ALPS_TSBH1				8 
#define TUNER_TYPE_ALPS_TSBE1				9 
#define TUNER_TYPE_ALPS_TMDZ2				10
									

//	IF Freq
#define IFFREQ_TEMIC_PAL					623
#define IFFREQ_PHILIPS_PAL_I				623
#define IFFREQ_PHILIPS_NTSC				732
#define IFFREQ_PHILIPS_SECAM				623
#define IFFREQ_PHILIPS_PAL				623
#define IFFREQ_TEMIC_NTSC				732
#define IFFREQ_TEMIC_PAL_I				623
#define IFFREQ_TEMIC_4036_FY5_NTSC			732
#define IFFREQ_ALPS_TSBH1				732
#define IFFREQ_ALPS_TSBE1				732
#define IFFREQ_ALPS_TMDZ2				623

//	VHF low
#define VHF_LOW_TEMIC_PAL				0x8e02
#define VHF_LOW_PHILIPS_PAL_I				0x8ea0
#define VHF_LOW_PHILIPS_NTSC				0x8eA0
#define VHF_LOW_PHILIPS_SECAM				0x8eA7
#define VHF_LOW_PHILIPS_PAL				0x8eA0
#define VHF_LOW_TEMIC_NTSC				0x8e02
#define VHF_LOW_TEMIC_PAL_I				0x8e02
#define VHF_LOW_TEMIC_4036_FY5_NTSC		0x8ea0
#define VHF_LOW_ALPS_TSBH1				0x8e01
#define VHF_LOW_ALPS_TSBE1				0x8e01
#define VHF_LOW_ALPS_TMDZ2				0x8E01

//	VHF high
#define VHF_HIGH_TEMIC_PAL				0x8e04
#define VHF_HIGH_PHILIPS_PAL_I			0x8e90
#define VHF_HIGH_PHILIPS_NTSC				0x8e90
#define VHF_HIGH_PHILIPS_SECAM			0x8e97
#define VHF_HIGH_PHILIPS_PAL				0x8e90
#define VHF_HIGH_TEMIC_NTSC				0x8e04
#define VHF_HIGH_TEMIC_PAL_I				0x8e04
#define VHF_HIGH_TEMIC_4036_FY5_NTSC		0x8e90
#define VHF_HIGH_ALPS_TSBH1				0x8e02
#define VHF_HIGH_ALPS_TSBE1				0x8e02
#define VHF_HIGH_ALPS_TMDZ2				0x8e02

//	UHF
#define UHF_TEMIC_PAL					0x8e01
#define UHF_PHILIPS_PAL_I				0x8e30
#define UHF_PHILIPS_NTSC					0x8e30
#define UHF_PHILIPS_SECAM				0x8e37
#define UHF_PHILIPS_PAL					0x8e30
#define UHF_TEMIC_NTSC					0x8e01
#define UHF_TEMIC_PAL_I					0x8e01
#define UHF_TEMIC_4036_FY5_NTSC			0x8e30
#define UHF_ALPS_TSBH1					0x8e08
#define UHF_ALPS_TSBE1					0x8e08
#define UHF_ALPS_TMDZ2					0x8e08


#define TEMIC_PAL_LOWMID_THRESH			141 
#define PHILIPS_PAL_I_LOWMID_THRESH		141 
#define PHILIPS_NTSC_LOWMID_THRESH			158 
#define PHILIPS_SECAM_LOWMID_THRESH		169 
#define PHILIPS_PAL_LOWMID_THRESH			169 
#define TEMIC_NTSC_LOWMID_THRESH			158 
#define TEMIC_PAL_I_LOWMID_THRESH			170 
#define TEMIC_4036_FY5_NTSC_LOWMID_THRESH	158 
#define ALPS_TSBH1_LOWMID_THRESH			138 
#define ALPS_TSBE1_LOWMID_THRESH			138 
//BUGBUG: this is a guess
#define ALPS_TMDZ2_LOWMID_THRESH			138 
									    
#define TEMIC_PAL_MIDHIGH_THRESH			464 
#define PHILIPS_PAL_I_MIDHIGH_THRESH		464 
#define PHILIPS_NTSC_MIDHIGH_THRESH		452 
#define PHILIPS_SECAM_MIDHIGH_THRESH		448 
#define PHILIPS_PAL_MIDHIGH_THRESH			448 
#define TEMIC_NTSC_MIDHIGH_THRESH			464 
#define TEMIC_PAL_I_MIDHIGH_THRESH			450 
#define TEMIC_4036_FY5_NTSC_MIDHIGH_THRESH	464 
#define ALPS_TSBH1_MIDHIGH_THRESH			386 
#define ALPS_TSBE1_MIDHIGH_THRESH			386 
//BUGBUG: this is a guess
#define ALPS_TMDZ2_MIDHIGH_THRESH			386 


typedef struct tuner_table_entry
{
	unsigned short	iffreq;
	unsigned short	vhf_low_control;
	unsigned short	vhf_high_control;
	unsigned short	uhf_control;
	unsigned short	low_mid_thresh;
	unsigned short	mid_high_thresh;
} tuner_table_entry_t;


ULONG tv_tuner_channel_freq( int cable, int channel );
BYTE tv_tuner_get_signal_strength( BYTE tuner_i2c_read_addr );
ULONG tv_tuner_get_freq();
int tv_tuner_change_channel( BYTE tuner_i2c_write_addr, int tuner_type, int cable, int channel );
int tv_tuner_change_frequency( BYTE tuner_i2c_write_addr, int tuner_type,int freq );

#ifdef __cplusplus
}
#endif


#endif	// TV_TUNER_H


