/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Geode Video 4 Linux 2 Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */
 /*
 * File Contents:
 * 	The Implementation of the VBI module
 * 
 * Project: GeodeV4l2 linux driver
 *  Author: Sojan James
 *    Date:
 * ---------------------------------------------------------------------*/
#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#define __NO_VERSION__ 
/* System includes */
#include <linux/module.h>
#include <linux/version.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/interrupt.h>


#include "geodedrv.h"
#include "vbi.h"
#include "tv_os_geode.h"

#define NUM_VBI_SAMPLES_PER_LINE_NTSC 	720
//#define NUM_VBI_SAMPLES_PER_LINE_NTSC 	( gfx_get_display_pitch() )
#define NUM_VBI_SAMPLES_PER_LINE_PAL	720
//#define NUM_VBI_SAMPLES_PER_LINE_PAL	( gfx_get_display_pitch() )

#define VBI_START_LINE_ODD_NTSC		10
#define VBI_START_LINE_EVEN_NTSC	273

#define VBI_START_LINE_ODD_PAL		6
#define VBI_START_LINE_EVEN_PAL		319

#define VBI_NUM_LINES_NTSC              12
#define VBI_NUM_LINES_PAL               18

#define VBI_START_ADDRESS_OFFSET  0x300000   /* 3 Meg offset from fb-base */
//#define VBI_START_ADDRESS_OFFSET  0x000000   /* 0 offset for debug */
#define VBI_END_ADDRESS_OFFSET    0x400000   /* 4 Meg */

/* Macros to extract line number, field and data type from slicer output */
/* Refer Saa7114 datasheet, Table 20, Pg. 57 */
#define LINE_NUMBER(ID1,ID2) (((ID1 & 0x3f) << 3) | (( ID2 >> 4) & 0x07))
	
#define FIELD(ID1) ((ID1 >> 6) & 0x01)		
        
#define DATA_TYPE(ID2) (ID2 & 0x0F )	
	
/* 47 is the offset of the BC byte */	
#define NUM_VALID_BYTES(buffer) (buffer[47] & 0x3f)
	
	
#define SET_BIT(bit,bitfield) (bitfield |= (0x01 << bit))
        
#define UNSET_BIT(bit,bitfield) (bitfield &=  (~(0x01 << bit)) )
        
#define	GET_BIT(bit,bitfield) (bitfield & (0x01 << bit) )
	

extern int 	VIN_STD; 	/* The input standard */
extern int 	VBI_MINOR;   /* VBI minor number, defined in geodedrv.c */
extern int 	MAX_VBI_STREAM_BUFFERS;
extern int	MAX_VBI_OPEN_BUFFERS;
extern struct 	capture_device capture_dev;
struct vbi_device vbi_dev;
VL_VbiStruct	g_vl_vbi;

/* Forward function declarations */

static inline void
decode_vbi(unsigned char sdid, 
           struct vbi_device_open * o_dev, 
           struct vbi_stream_buffer * buffer);

/* DEBUG function */
#ifdef DEBUG
/* ------------------------------------------------------------------------
 * Description : 
 *      print all set vbi formats 
 * Return      : void
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
void print_all_formats()
{
	int i;
	for(i = 0; i < MAX_VBI_OPENS ; i++)
	{
		printk("vbi_dev.open_data[%d]\n",i);
		printk("samples_per_line = %d\n",
			vbi_dev.open_data[i].vbifmt.samples_per_line);
		printk("sample_format = %d\n",
			vbi_dev.open_data[i].vbifmt.sample_format);
		printk("start[0] = %d\n",
			vbi_dev.open_data[i].vbifmt.start[0]);
		printk("count[0] = %d\n",
			vbi_dev.open_data[i].vbifmt.count[0]);
		printk("start[1] = %d\n",
			vbi_dev.open_data[i].vbifmt.start[1]);
		printk("count[1] = %d\n",
			vbi_dev.open_data[i].vbifmt.count[1]);
		printk("stream_on = %d\n", vbi_dev.open_data[i].stream_on);
		printk("isopen = %d\n", vbi_dev.open_data[i].isopen);
	}
}

/* ------------------------------------------------------------------------
 * Description : 
 *      print the value of decoder slicer register 
 * Return      : void
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
void print_decoder_slicer_regs()
{
#ifndef NO_DECODER
	int i;
	unsigned long flags;
	unsigned char reg;
	
	for(i = 2; i <= 24; i ++)
	{
		save_flags(flags);
		cli();
		reg = tv_i2c_saa7114_read_byte_data(0x3f + i);
		restore_flags(flags);
		printk("%d:%x\n",i,reg);
		
	}
#endif
}
#endif /* DEBUG */
/* END of DEBUG functions */

/* ------------------------------------------------------------------------
 * Description :
 * 	disable all the LCRs of the 7114. This is required to disable
 *  	the slicer totally when we move to the MPEG input 
 * Return      : void
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
static void slicer_off()
{
#ifndef NO_DECODER		
#if 0       
	int reg;
	unsigned long flags;
	save_flags(flags);
	cli();
	for(reg = 0x41 ; reg < 0x58 ; reg++)
	{
		tv_i2c_saa7114_write_byte_data(reg,0xFF);
	}
	restore_flags(flags);
#endif        
#endif		
}

/* ------------------------------------------------------------------------
 * Description : de-allocate all the buffers
 * Return      : void
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
static void cleanup_vbi_buffers(struct vbi_device * dev 
				/* pointer to vbi device structure*/)
{
	vbi_stream_buffer_t * buf;
	while( (buf = v4l2_q_del_tail(&dev->q_free)) )
	{
#ifndef VBI_IN_FB
		free_pages((unsigned long)buf->vaddress,dev->stream_buffers_allocated);
#endif
		kfree(buf);
	}
	while( (buf = v4l2_q_del_tail(&dev->q_done)) )
	{
#ifndef VBI_IN_FB
		free_pages((unsigned long)buf->vaddress,dev->stream_buffers_allocated);
#endif
		kfree(buf);
	}
	if(dev->odd_capturing) 
	{
#ifndef VBI_IN_FB
		free_pages((unsigned long)dev->odd_capturing->vaddress,dev->stream_buffers_allocated);
#endif
		kfree(dev->odd_capturing);
	}
	if(dev->even_capturing) 
	{
#ifndef VBI_IN_FB
		free_pages((unsigned long)dev->even_capturing->vaddress,dev->stream_buffers_allocated);
#endif
		kfree(dev->even_capturing);
	}
	dev->odd_capturing = 0;
	dev->even_capturing = 0;
	dev->stream_buffers_allocated = 0;	
}

/* ------------------------------------------------------------------------
 * Description : allocate free buffers
 * Return      : int (TRUE on success and FALSE on faliure)
 * Side effects: None 
 * Notes: We assume that the durango gfx_virt_fbptr has already been allocated 
 * ------------------------------------------------------------------------*/
static int allocate_vbi_buffers(int pitch, /* length of each line */
			 int lines, /* number of lines */ 
			 int num_buffers, /* number of buffers to allocate */
			 struct vbi_device * dev)
{
	/* first, cleanup all the buffers */
	int index;
	vbi_stream_buffer_t * buf;
#ifdef VBI_IN_FB
	unsigned long offset = VBI_START_ADDRESS_OFFSET;
	unsigned long buffer_size = lines * pitch ;
#else
	unsigned long order;
	if(lines == VBI_NUM_LINES_NTSC)
		order = 5;
	else if(lines == VBI_NUM_LINES_PAL)
		order = 7;
	else
	{
		err_msg("VBI: unknown number of lines\n");
		return FALSE;
	}
#endif
	if(vbi_dev.stream_buffers_allocated)
	cleanup_vbi_buffers(dev);
	for( index = 0; index < num_buffers; index++)
	{
#ifdef VBI_IN_FB
		/* check for more than 4MB */
		if(offset+buffer_size > VBI_END_ADDRESS_OFFSET) /* abort */
		{
			cleanup_vbi_buffers(dev);
			return FALSE;
		}
#endif
		buf = kmalloc(sizeof(vbi_stream_buffer_t), GFP_KERNEL);
		if(!buf) /* abort */
		{
			err_msg("VBI: couldnt allocate VBI stream buffers\n");
			cleanup_vbi_buffers(dev);
			return FALSE;
		}
		memset(buf,0, sizeof(vbi_stream_buffer_t));
#ifdef VBI_IN_FB
		buf->paddress = (unsigned long)gfx_phys_fbptr + offset;
		buf->vaddress = (__u8*)gfx_virt_fbptr + offset;
		offset += buffer_size;
#else
		buf->vaddress=(__u8*)__get_free_pages(GFP_DMA,order);
		if(!buf->vaddress)
		{
			err_msg("VBI:unable to allocate VBI buffers\n");
			cleanup_vbi_buffers(dev);
			return FALSE;
		}
		memset(buf->vaddress,0,sizeof(order*PAGE_SIZE));
		buf->paddress=virt_to_phys(buf->vaddress);
#endif
		/* now put it into the free buffer queue */
		v4l2_q_add_head(&dev->q_free, (struct v4l2_q_node*)buf);
	}
#ifdef VBI_IN_FB
	dev->stream_buffers_allocated = TRUE;
#else
	dev->stream_buffers_allocated = order;
	debug_msg("gfx_get_vbi_base = %lX\n",gfx_get_vbi_base(1)); 
#endif
	return TRUE;	
}

/* ------------------------------------------------------------------------
 * Description : De-allocate the buffers for a particular open. We should 
 *	first de-allocate the data within the buffer before freeing the buffer 
 *	itself.
 * Return      : void
 * Side effects: None 
 * Notes:
 * ------------------------------------------------------------------------*/
static void cleanup_open_buffers(struct vbi_device_open * dev)
{
	struct vbi_buffer * buf;
	while( (buf = v4l2_q_del_tail(&dev->q_free)) )
	{
		kfree(buf->data);	
		kfree(buf);
	}	
	while( (buf = v4l2_q_del_tail(&dev->q_done)) )
	{
		kfree(buf->data);	
		kfree(buf);
	}	
	dev->num_captured_buffers = 0;
	debug_msg("Cleanup open buffers\n");
}

/* ------------------------------------------------------------------------
 * Description : 
 *      Allocate buffers for a particular open. The buffers are 
 *	    allocated according to the number of samples_per_line field and the 
 *	    number of lines in each field of the vbifmt structure for each open 
 *          New: (13th Jun 2001). We will allocate a maximum possible lines in
 *               each buffer. For NTSC, this is 12 and for PAL it is 18. Each 
 *               line will also have a two byte header. Each read now will return
 *		 only one field of data, instead of a frame. 
 * Return      : int (TRUE on success, FALSE on failure) 
 * Side effects: None 
 * Notes:
 * ------------------------------------------------------------------------*/
static int allocate_open_buffers(struct vbi_device_open * dev, int num_buffers)
{
	struct vbi_buffer * buf;
	int i,num_lines;
	if(!dev->stream_on) /* Dont do anything if !stream_on. */
		return FALSE;
	switch(VIN_STD) {
		case VSOURCE_STD_NTSC:
		num_lines = VBI_NUM_LINES_NTSC; break;
		case VSOURCE_STD_PAL:
		num_lines = VBI_NUM_LINES_PAL; break;
		default:
			err_msg("Invalid standard\n"); return FALSE;
	}		
	/* first, cleanup any existing buffers that might have been left
	   behind. This may not be necessary */
	 cleanup_open_buffers(dev);  
	 for(i = 0; i < num_buffers; i++) {
		 buf = kmalloc( sizeof(struct vbi_buffer), GFP_KERNEL);
		 if(!buf) {
			 cleanup_open_buffers(dev);
			 err_msg("Critical, unable to allocate kernel memory\n");
			 return FALSE;
		 }
		/* Compute the length of the buffer required, depending on the
		   samples_per_line field and the number of lines plus two bytes
		   for the header
                   CHANGED: The buffer size is now the minimun length required to
		   read the VBI lines from one field of data. Since both the fields
		   may have different number of VBI lines, we take the larger of the
		   two.
		 */
		 dev->buff_length = max(dev->line_count_odd, dev->line_count_even);
		 debug_msg("Min lines : %d\n", dev->buff_length);
		 //dev->buff_length = dev->buff_length * 
		 //			(dev->vbifmt.samples_per_line + 2);
		 dev->buff_length = dev->buff_length * 
		 			(dev->vbifmt.samples_per_line);
		 debug_msg("Minimum buffer length = %d\n", dev->buff_length);		

		 debug_msg("Length:%d\n", dev->buff_length);		
		 buf->data = kmalloc(dev->buff_length, GFP_KERNEL);
		 if(!buf->data) {
			cleanup_open_buffers(dev);
			err_msg("Critical: unable to allocate kernel mem\n");
			return FALSE;	 
		 }
		 v4l2_q_add_head(&dev->q_free,(struct v4l2_q_node*)buf);
	 }
	 debug_msg("Allocated open buffers\n");
	 return TRUE;
}

/* ------------------------------------------------------------------------
 * Description : 
 *      The bottom-half handler for the VBI interrupt. This routine 
 *	    runs at an elevated IRQ level, but is interruptible. This function
 *      wakes up any processes sleeping for data and also signals the process
 *	    if the process has requested it.
 * Return      : void 
 * Side effects: None 
 * Notes:
 * ------------------------------------------------------------------------*/
static void vbi_bottom_half(void * v)
{
	struct vbi_device * dev = v;
	int i;
	int start_line_odd=VBI_START_LINE_ODD_NTSC, 
		start_line_even=VBI_START_LINE_EVEN_NTSC; /* default for NTSC*/
	struct vbi_stream_buffer * buffer = NULL;

	
	buffer = v4l2_q_del_tail(&dev->q_done);
	if(!buffer)
		return;	
	
	switch(VIN_STD)
	{
		case VSOURCE_STD_NTSC:
			break; /* already set for NTSC */
		case VSOURCE_STD_PAL:
			start_line_odd = VBI_START_LINE_ODD_PAL;
			start_line_even = VBI_START_LINE_EVEN_PAL;
			break;			
        default:
            err_msg("unsupported vin standard\n");    
            return;
	}
	for(i = 0; i < MAX_VBI_OPENS; i++)
	{
		if(dev->open_data[i].stream_on )
		{
			decode_vbi(0,&dev->open_data[i],buffer);
#ifdef SIGNAL_USER		
		 /* signal the process if needed */
		 if(dev->open_data[i].proc && (dev->open_data[i].signal_on_count)
			 && (dev->open_data[i].num_captured_buffers >= 
			 dev->open_data[i].signal_on_count ))
		 {
//	 debug_msg("Sending signal %d\n",dev->open_data[i].num_captured_buffers);
			 	
			send_sig(dev->open_data[i].signal_number,
				 dev->open_data[i].proc, 1 );	 
		 }
#endif		 
		 /* wake up any process waiting on this open */
		 wake_up_interruptible(&dev->open_data[i].read_wait);	
		}
	}	
	v4l2_q_add_head(&dev->q_free,
			(struct v4l2_q_node*)buffer);	
        buffer->vidbuf.flags = 0;
}

/* ------------------------------------------------------------------------
 * Description : pulls a buffer out of the free buffer queue, if no free
 *	buffers are avalable, pull one from the head of done queue and use it
 * Return      : void 
 * Side effects: Might lose information in the buffers which are pulled out
 *               from the done list. 
 * Notes:
 * ------------------------------------------------------------------------*/
static void VbiGetEmptyBuf(unsigned long *phy, /* physical address of buffer */
                           unsigned long *virt, /* virtual address */
                           char field_type /* odd or even */
                           )
{
	/* pull one out of the free list */
	vbi_stream_buffer_t * buf = v4l2_q_del_tail(&vbi_dev.q_free);
	if(!buf)
	{
		printk("Pulling out of q_done\n");
		buf = v4l2_q_del_head(&vbi_dev.q_done);
		if(!buf) {
			err_msg("Aieeee!!, this should not happen\n");
			return;
		}
/*
		v4l2_q_add_head(&vbi_dev.q_free,(struct v4l2_q_node*)buf);
		buf = v4l2_q_del_head(&vbi_dev.q_done);
*/
		*phy = buf->paddress;
		*virt = (unsigned long)buf->vaddress;
	}
	else {
		*phy = buf->paddress;
		*virt = (unsigned long)buf->vaddress;
	}
	buf->vidbuf.flags =  0;
	if(field_type)
		vbi_dev.odd_capturing = buf;
	else
		vbi_dev.even_capturing = buf;	
}

/* ------------------------------------------------------------------------
 * Description : interrupt handler to be called whenever a captured buffer
 *	is ready for processing.  
 * Return      : void 
 * Side effects: none 
 * Notes: This is called by the VideoLib interrupt handler.
 * ------------------------------------------------------------------------*/
static void VbiNotifyFullBuf(unsigned long buf, 
                             char field_type /* odd or even */
                             )
{
//	printk("Notify %d\n",field_type); 
	if(field_type) { /* odd */
		vbi_dev.odd_capturing->vidbuf.flags = (V4L2_BUF_FLAG_DONE | 
			   	    V4L2_BUF_FLAG_ODDFIELD);
		v4l2_q_add_head(&vbi_dev.q_done,
			(struct v4l2_q_node*)vbi_dev.odd_capturing);
#if 1 /* paranoid sanity check */
		if(buf != (unsigned long)vbi_dev.odd_capturing->vaddress)
			err_msg("Oh no!! (odd)\n");
#endif				
		vbi_dev.odd_capturing = 0;	
	}
	else { /* even */
		vbi_dev.even_capturing->vidbuf.flags = (V4L2_BUF_FLAG_DONE | 
			   	    V4L2_BUF_FLAG_EVENFIELD);
		v4l2_q_add_head(&vbi_dev.q_done,(struct v4l2_q_node*)vbi_dev.even_capturing);
#if 1 /* paranoid sanity check */
		if(buf != (unsigned long)vbi_dev.even_capturing->vaddress)
			err_msg("Oh no!! (even)\n");
#endif			
		vbi_dev.even_capturing = 0;	
	}

	#if 1
	vbi_bottom_half(&vbi_dev);
	
	#else
	/* Mark the bottom-half handler */
	vbi_dev.vbi_bottom_half.routine = vbi_bottom_half;
	vbi_dev.vbi_bottom_half.data     = &vbi_dev;
	queue_task(&vbi_dev.vbi_bottom_half,&tq_immediate);
	mark_bh(IMMEDIATE_BH);	
#endif
}

/* ------------------------------------------------------------------------
 * Description : initialize the vbi module
 * Return      : int (1 for success & 0 for faliure) 
 * Side effects: none 
 * Notes:
 * ------------------------------------------------------------------------*/
int vbi_decode_init( struct vbi_device* dev )
{

	memset( &g_vl_vbi, 0, sizeof(VL_VbiStruct) );
	g_vl_vbi.VipIRQ = dev->irq;
	g_vl_vbi.VipINT = 3;/*'c'*/
	g_vl_vbi.VL_VbiGetEmptyBuf = VbiGetEmptyBuf;
	g_vl_vbi.VL_VbiNotifyFullBuf = VbiNotifyFullBuf;
	switch(VIN_STD)
	{
		case VSOURCE_STD_NTSC:		
	    	g_vl_vbi.VbiLineSize = NUM_VBI_SAMPLES_PER_LINE_NTSC;
		break;
		case VSOURCE_STD_PAL:		
    		g_vl_vbi.VbiLineSize = NUM_VBI_SAMPLES_PER_LINE_PAL;
		break;
		default:
			err_msg("Invalid input standard for VBI\n");
    		g_vl_vbi.VbiLineSize = 0;
	}	
	if( VL_VbiInit( &g_vl_vbi ) != VL_STATUS_OK )
	{
		err_msg("VL_VbiInit() failed\n");
		while(1){ printk("Damn\n");}
		return 0;
	}
	return 1;
}

/* ------------------------------------------------------------------------
 * Description : cleanup vbi buffers & stop the decoder 
 * Return      : int (1 for success & 0 for faliure) 
 * Side effects: none 
 * Notes:
 * ------------------------------------------------------------------------*/
int vbi_decode_cleanup()
{
	VL_VbiEnd();
	if(vbi_dev.stream_buffers_allocated)
		cleanup_vbi_buffers(&vbi_dev);
	return 1;	
}

/* ------------------------------------------------------------------------
 * Description : configure the vbi device  
 * Return      : int (1 on success & 0 on faliure) 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
int __init config_vbi_device(struct vbi_device *dev)
{
        sprintf(dev->shortname, "vbi0");

        sprintf(dev->v.name, "NSC Geode V4L2 VBI driver" );
        dev->v.type = V4L2_TYPE_VBI;
        dev->v.minor = VBI_MINOR;
        dev->v.open = v4l2_vbi_open;
        dev->v.close = v4l2_vbi_close;
        dev->v.read = v4l2_vbi_read;
        dev->v.write = v4l2_vbi_write;
        dev->v.ioctl = v4l2_vbi_ioctl;
        dev->v.mmap = v4l2_vbi_mmap;
        dev->v.poll = v4l2_vbi_poll;
        dev->v.initialize = v4l2_vbi_init_done;
        dev->v.priv = NULL;
        return TRUE;
}

/* ------------------------------------------------------------------------
 * Description : set the capture format for vbi device  
 * Return      : int (1 on success & 0 on faliure) 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
static int vbi_set_format(struct v4l2_vbi_format * fmt, /* requested format */
	            		  struct vbi_device_open * dev)
{
	int u_limit_odd , l_limit_odd= 0;
	int u_limit_even, l_limit_even= 0;
	int i;
	/* Identify the sampling format */
	switch(fmt->sample_format)
	{
		/* This ioctl have a cumulative effect so that
		   non consecutive multiple lines can be specified
		   in a single open. The GEODE_V4L2_VBI_SF_CLEAR_ALL will
		   clear all the sample formats 
		 */  
		case GEODE_V4L2_VBI_SF_CC_US:
		case GEODE_V4L2_VBI_SF_CC_EURO:
		case GEODE_V4L2_VBI_SF_VPS:
		case GEODE_V4L2_VBI_SF_WSS:
		case GEODE_V4L2_VBI_SF_WST:
		case GEODE_V4L2_VBI_SF_INTERCAST:
		case GEODE_V4L2_VBI_SF_TELETEXT:
		case GEODE_V4L2_VBI_SF_NABTS:
		case GEODE_V4L2_VBI_SF_MOJI:
		case GEODE_V4L2_VBI_SF_JFS:
		case GEODE_V4L2_VBI_SF_TS_EURO:
		case GEODE_V4L2_VBI_SF_TS_US:
			break;
		case V4L2_VBI_SF_UBYTE:
			return FALSE;
		default: 
			return FALSE;
	}
	/* check the flag field to find out if we need to keep the earlier
	   lines */
	if(! (fmt->flags & V4L2_VBI_APPEND_FMT))
	{
		dev->line_bitmap_odd = 0x00;
		dev->line_bitmap_even = 0x00;
		dev->line_count_odd = 0;
		dev->line_count_even = 0;
	}
	
	/* We support lines 10 to 21 for NTSC and lines 6 to 22 for PAL */	
	switch(VIN_STD)
	{
		case VSOURCE_STD_NTSC:
		    l_limit_odd = VBI_START_LINE_ODD_NTSC; u_limit_odd = 21; 
		    l_limit_even = VBI_START_LINE_EVEN_NTSC;u_limit_even = 284;
		    break;
		case VSOURCE_STD_PAL:
		    l_limit_odd = VBI_START_LINE_ODD_PAL; u_limit_odd = 22; 
		    l_limit_even= VBI_START_LINE_EVEN_PAL; u_limit_even = 335; 
		    break;
		default:
            err_msg("VBI slicer format not supported\n");
            return FALSE;
	}
	/* check limits */
	if(fmt->count[0])
		if(!( (fmt->start[0] >= l_limit_odd)
			 && (fmt->start[0] + fmt->count[0] -1 <= u_limit_odd)))
		{	debug_msg("Limit failed odd\n");
			return FALSE; 
		}		
	if(fmt->count[1])	
		if(!( (fmt->start[1] >= l_limit_even)
			  && (fmt->start[1] + fmt->count[1] -1 <= u_limit_even)))
		{	debug_msg("Limit failed even\n");
			return FALSE;
		}			
	/* new implementation, we set the bit in the bitfields indicating the 
	 *  requested lines for each field.
         *  TODO: we dont check for any overlap now. do it.
         */
         for(i = fmt->start[0] - l_limit_odd; 
	     i < (fmt->start[0] + fmt->count[0] - l_limit_odd); i++) {
                 if(!GET_BIT(i,dev->line_bitmap_odd)) {
			 SET_BIT(i,dev->line_bitmap_odd);
			 dev->line_count_odd++;
		 }	 
		 //printk("Line count odd %d\n",i);
	 }	 
         for(i = fmt->start[1] - l_limit_even ; 
	     i < (fmt->start[1] + fmt->count[1] - l_limit_even);i++) {
                 if(!GET_BIT(i,dev->line_bitmap_even)) {
                 	SET_BIT(i,dev->line_bitmap_even);
			 dev->line_count_even++;
		 }
		 //printk("Line count even %d\n",i);
	 }	 

        //err_msg("odd_bitfield: %x, even_bitfield: %x\n",
          //                      dev->line_bitmap_odd,
            //                    dev->line_bitmap_even);         
                 
	/* safe to continue */
	/* we wont set anything on the hardware yet. That will be done 
           only when we start VBI capture */
	return TRUE;
}

/* ------------------------------------------------------------------------
 * Description : open the vbi device  
 * Return      : int (0 on success & error on faliure) 
 * Side effects: for the first open device, sets the vbi format to CC 
 * Notes       : For the first open, the default format is set to capture
 *	       : closed caption data from line 21 and line 284.	
 * ------------------------------------------------------------------------*/
int v4l2_vbi_open(struct v4l2_device *v, /* pointer to the vbi device structure */
	      int l_flags, /* flag to indicate if the device is non capturing */
	      void **idptr /*out parameter, pointer to the open device structure*/
        )
{
	struct vbi_device *dev = (struct vbi_device *)v;
	int     i, n;
        int     cap;

        for (i = 0, n = -1, cap = 0; i < MAX_VBI_OPENS; ++i)
        {
                if (!dev->open_data[i].isopen)
                        n = i;/* available open_data structure */
                else if (!dev->open_data[i].noncapturing)
                        cap = 1;/* another open is already capturing */
        }
        if (n == -1)/* No available open_data structures */
        {
                err_msg("No more opens on this device\n");
                return -EBUSY;
        }
        if (l_flags & O_NONCAP)/*  Non-capturing open */
                dev->open_data[n].noncapturing = 1;
        else if (cap)
        {
                debug_msg("No more capturing opens on this device\n");
                return -EBUSY;
        }
        else
	{
                dev->open_data[n].noncapturing = 0;
                /*  Keep track of whether there is a capturing open  */
                ++dev->capturing_opens;
        }
        ++dev->open_count;
        dev->open_data[n].isopen = 1;
        dev->open_data[n].dev = dev;
	/* Initialize the queues and the wait queue */
	v4l2_q_init(&dev->open_data[n].q_done);
	v4l2_q_init(&dev->open_data[n].q_free);
/* jyothin */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	init_waitqueue(&dev->open_data[n].read_wait);
#else	
	init_waitqueue_head(&dev->open_data[n].read_wait);
#endif
        *idptr = &dev->open_data[n];
        if (dev->open_count == 1)
        {
    		struct v4l2_vbi_format * fmt = &dev->open_data[n].vbifmt;
            debug_msg("First open on capture device\n");
		/* Lets set up some default values for the format */
    		fmt->sampling_rate = 60;
	    	fmt->offset = 0;
		fmt->samples_per_line = 2;
    		fmt->sample_format = GEODE_V4L2_VBI_SF_CC_US;
	    	fmt->start[0] = 21;
		fmt->start[1] = 284;
    		fmt->count[0] = 1;
	    	fmt->count[1] = 1;
		fmt->flags = 0;
		vbi_set_format(fmt, &dev->open_data[n]);
		switch(VIN_STD) {
			case VSOURCE_STD_NTSC:
	        	        dev->open_data[n].format_set = TRUE;
        			dev->num_vbi_lines = VBI_NUM_LINES_NTSC;
	        		break;
			case VSOURCE_STD_PAL:	
	                	dev->open_data[n].format_set = FALSE;
        			dev->num_vbi_lines = VBI_NUM_LINES_PAL;
                                break;
                        default:
                                err_msg("Unsupported standard\n");         
		}	
				
        }
        debug_msg("Open succeeded\n");
        MOD_INC_USE_COUNT;
        return 0;
}

/* ------------------------------------------------------------------------
 * Description : close the vbi device  
 * Return      : void 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
void v4l2_vbi_close(void *id) /*pointer to the device to be closed*/
{
	struct vbi_device_open *o = (struct vbi_device_open *)id;
    struct vbi_device *dev = o->dev;
    if (!o->noncapturing)
    {
        --dev->capturing_opens;
        debug_msg("Close\n");
    }
    --dev->open_count;
    if (dev->open_count == 0)
    {
	    debug_msg("v4l2_vbi_close()\n");
	    VL_VbiCapture(0);
	    if(vbi_dev.stream_buffers_allocated)
	        cleanup_vbi_buffers(&vbi_dev);
	    slicer_off();
        debug_msg("Last close on capture device\n");
    }

	vbi_dev.streaming = FALSE;
	cleanup_open_buffers(o);
	MOD_DEC_USE_COUNT;
	o->isopen = FALSE;
	o->stream_on = FALSE;
#ifdef SIGNAL_USER	
	o->proc = NULL;
	o->signal_on_count = 0;
	o->signal_number = 0;
#endif	
}

/* ------------------------------------------------------------------------
 * Description : write function for vbi device  
 * Return      : error 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
long v4l2_vbi_write(void		  *id, 
	                const char	  *buf, 
	                unsigned long count, 
	                int		      noblock)
{
	return -EINVAL;
}


/* ------------------------------------------------------------------------
 * Description : return the value of slicer byte corresponding to the given
 *		 capture format.  
 * Return      : unsigned char  
 * Side effects: None 
 * Notes       : These numbers are from the saa7114 data sheet. Maybe this 
 *	       : function should be part of tvlib	
 * ------------------------------------------------------------------------*/
static unsigned char get_slicer_byte(unsigned long format)
{
    unsigned char val; 
	switch(format)
	{
		case GEODE_V4L2_VBI_SF_CC_US:
			val = 0x05; 
                        break ;
		case GEODE_V4L2_VBI_SF_CC_EURO:               
		        val = 0x01;	
                        break ;
		case GEODE_V4L2_VBI_SF_VPS:                   
			val = 0x02;
                        break ;
		case GEODE_V4L2_VBI_SF_WSS:                   
			val = 0x03;
                        break ;
		case GEODE_V4L2_VBI_SF_WST:
			val = 0x04;                   
                        break ;
		case GEODE_V4L2_VBI_SF_RAW:
			val = 0x07;                   
                        break ;
		case GEODE_V4L2_VBI_SF_TELETEXT:              
			val = 0x08;
                        break ;
		case GEODE_V4L2_VBI_SF_NABTS:                 
			val = 0x0C;
                        break ;
		case GEODE_V4L2_VBI_SF_MOJI:                  
			val = 0x0D;
                        break ;
		case GEODE_V4L2_VBI_SF_JFS:                   
			val = 0x0E;	
                        break ;
		case GEODE_V4L2_VBI_SF_VIDEO:                 
			val = 0x0F;
                        break ;
		case GEODE_V4L2_VBI_SF_TS_EURO:                 
			val = 0x09;
                        break ;
		case GEODE_V4L2_VBI_SF_TS_US:                 
			val = 0x0A;
                        break ;

		default: val = 0x0F;
	}
    return val;
}

/* ------------------------------------------------------------------------
 * Description : Sets up the decoder register depending on the formats which 
 *	need to be streamed. This function uses the stream_on flag to decide 
 *	whether the format needs to be set or not. 
 * Return      : void 
 * Side effects: None 
 * Notes: Ensure that this is called after stream_on is set. 
 * ------------------------------------------------------------------------*/
static void set_vbi_lines(struct vbi_device_open * dev)
{
#ifndef NO_DECODER
	int i,j,k;
	int start_line_odd= 10, start_line_even= 263;
        int num_lines;
        int vbi_start_line;
	unsigned long flags;
	
	for(i = 0 ; i < MAX_VBI_LINES ; i++)
	{
		dev->dev->vbi_lines_odd[i] = 0xff; /* First field */
		dev->dev->vbi_lines_even[i] = 0xff;/* Second field */
	}
	
	switch(VIN_STD)
	{
                case VSOURCE_STD_NTSC:
                        num_lines = VBI_NUM_LINES_NTSC;
                        vbi_start_line = 10;
                        for(i = 9 ; i <= 21 ; i++)
                        {
                                dev->dev->vbi_lines_odd[i] = 0x77; /* First field */
                                dev->dev->vbi_lines_even[i] = 0x77;/* Second field */
                        }
                        break;
                case VSOURCE_STD_PAL:
                        num_lines = VBI_NUM_LINES_PAL;
                        start_line_odd = 6;
                        start_line_even = 313;
                        vbi_start_line = 6;
                        for(i = 5 ; i <= 22 ; i++)
                        {
                                dev->dev->vbi_lines_odd[i] = 0x77; /* First field */
                                dev->dev->vbi_lines_even[i] = 0x77;/* Second field */
                        }
                    break;
                default:
                    err_msg("set_vbi_lines:vin standard not supported\n");
                    return;
	}
	for(j = 0; j < MAX_VBI_OPENS ; j++)
	{
		if(dev->dev->open_data[j].stream_on)
		{
			unsigned char slicer_nibble = get_slicer_byte(
				dev->dev->open_data[j].vbifmt.sample_format);
                        for(k = 0; k < num_lines; k++) {
                                if(GET_BIT(k,dev->dev->open_data[j].line_bitmap_odd)){
                                        dev->dev->vbi_lines_odd[k+vbi_start_line]
                                                = slicer_nibble;
                                }        
                        }        
                        for(k = 0; k < num_lines; k++) {
                                if(GET_BIT(k,dev->dev->open_data[j].line_bitmap_even)){
                                        dev->dev->vbi_lines_even[k+vbi_start_line]
                                                = slicer_nibble;
                                }        
                        }        
		}
	}
	/* Set the decoder registers */
	save_flags(flags);
	cli();
	for(i = 2; i <= 24 ; i++)
	{
		unsigned char slicer_byte = 
			(dev->dev->vbi_lines_odd[i] << 4) & 0xF0;
		slicer_byte |= (dev->dev->vbi_lines_even[i] & 0x0F);
		//info_msg("DBG: slicer byte %d: %x\n",i,slicer_byte);
		tv_i2c_saa7114_write_byte_data( 0x3F + i, slicer_byte );
	}
	restore_flags(flags);
#endif
}

/* ------------------------------------------------------------------------
 * Description : start VBI streaming 
 * Return      : int (0 on success & error on faliure) 
 * Side effects: Task A of the decoder is disabled if sliced data is requested
 *	       : This will affect DVIP.
 * Notes: 
 * ------------------------------------------------------------------------*/
int vbi_stream_on(struct vbi_device_open * dev)
{  

	unsigned long flags;
	int samples_per_line = 720 ;
	int num_lines = 0;
	save_flags(flags);
	cli();	       
#ifndef NO_DECODER		
#if 1
	if(dev->vbifmt.sample_format == V4L2_VBI_SF_UBYTE)
	{
		/* disable slicer for capturing raw VBI */
		tv_i2c_saa7114_write_byte_data(0x86,0x75);
		/* Enable task A + Task B */
		tv_i2c_saa7114_write_byte_data(0x80,0x30);
	}
	else /* Some kind of sliced data requested */
	{
		debug_msg("Slicer on\n");
		/* enable slicer */
		tv_i2c_saa7114_write_byte_data(0x86,0xe5);
		/* disable task A, Enable task B */
		tv_i2c_saa7114_write_byte_data(0x80,0x20);
	}
#endif        
	if(VIN_STD == VSOURCE_STD_NTSC)
	{
		tv_i2c_saa7114_write_byte_data(0x5A,0x06);
		samples_per_line = NUM_VBI_SAMPLES_PER_LINE_NTSC;
		num_lines = VBI_NUM_LINES_NTSC; /* 10 to 21 */
	}
	else if(VIN_STD == VSOURCE_STD_PAL)
	{
		tv_i2c_saa7114_write_byte_data(0x5A,0x03);
		samples_per_line = NUM_VBI_SAMPLES_PER_LINE_PAL;
		num_lines = VBI_NUM_LINES_PAL; /* 6 to 22 */
	}
#endif		
	/* DEBUG num_lines=38; */
	
	/* Now, allocate the VBI buffers */
	if(!dev->dev->stream_buffers_allocated)
		if(!allocate_vbi_buffers(samples_per_line,num_lines,
	                    		MAX_VBI_STREAM_BUFFERS, dev->dev))
			return -ENOMEM;  
	restore_flags(flags);
	dev->stream_on = TRUE;
	set_vbi_lines(dev);	
	allocate_open_buffers(dev,MAX_VBI_OPEN_BUFFERS);
	VL_VbiCapture( 1 );
	  //print_all_formats();
//	print_decoder_slicer_regs();
	return 0;
}

/* ------------------------------------------------------------------------
 * Description : implements the VBI ioctls 
 * Return      : int (0 on success & error on faliure) 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
int
v4l2_vbi_ioctl(void		    *id,
	           unsigned int	cmd, /* the ioctl command */
	           void		    *arg /* argument to the ioctl */
               )
{
	struct vbi_device_open *o = (struct vbi_device_open *)id;
	struct vbi_device *dev = o->dev;

	switch(cmd)
	{
	case VIDIOC_QUERYCAP:
	{
		struct v4l2_capability *b = arg;
		strcpy(b->name, dev->v.name);
		b->type = V4L2_TYPE_VBI;
		b->flags = V4L2_FLAG_READ
				| V4L2_FLAG_STREAMING
				| V4L2_FLAG_SELECT
				| V4L2_FLAG_TUNER
				| V4L2_FLAG_DATA_SERVICE
				| GEODE_V4L2_VBI_SF_CC_US		
				| GEODE_V4L2_VBI_SF_CC_EURO	
				| GEODE_V4L2_VBI_SF_VPS		
				| GEODE_V4L2_VBI_SF_WSS		
				| GEODE_V4L2_VBI_SF_WST		
				| GEODE_V4L2_VBI_SF_INTERCAST	
				| GEODE_V4L2_VBI_SF_TELETEXT	
				| GEODE_V4L2_VBI_SF_NABTS		
				| GEODE_V4L2_VBI_SF_MOJI		
				| GEODE_V4L2_VBI_SF_JFS		
				| GEODE_V4L2_VBI_SF_RAW
				| GEODE_V4L2_VBI_SF_VIDEO;
		b->inputs = 3;
		b->outputs = 1;
		b->audios = 0;
		b->maxwidth = MAX_WIDTH;
		b->maxheight = MAX_HEIGHT;
		b->minwidth = MIN_WIDTH;
		b->minheight = MIN_HEIGHT;
		b->maxframerate = 30;
		return 0;
	}
	case VIDIOC_G_FMT:
	{
		struct v4l2_format *fmt = arg;
		memcpy( &fmt->fmt.vbi, &o->vbifmt, sizeof(struct v4l2_vbi_format) );
		return 0;
	}

	case VIDIOC_S_FMT:
	{
		struct v4l2_format *fmt = arg;
		/* sanity check */
		if(o->stream_on) 
		{	
			debug_msg("Cannot S_FMT while streaming\n");
			return -EINVAL; /* streaming going on */
		}	
		if(!vbi_set_format(&fmt->fmt.vbi, o))
		{
			debug_msg("DBG: vbi_set_format: failed\n");	
			return -EINVAL;
		}	
		memcpy( &o->vbifmt, &fmt->fmt.vbi, sizeof(struct v4l2_vbi_format) );
		o->vbifmt.count[0] = o->line_count_odd;
		o->vbifmt.count[1] = o->line_count_even;
		o->format_set = TRUE;
		return 0;
	}
	case VIDIOC_STREAMON:
	{
		/* check if already started or format not set */	
		if(o->stream_on || (!o->format_set)) return -EINVAL;
		/* DO not allow VBI when using MPEG input */
		if(capture_dev.input == 3)
			return -EINVAL;	
		return vbi_stream_on(o);	
	}

	case VIDIOC_STREAMOFF:
	{
		int i;
		//unsigned long flags;

		if(!o->stream_on) return -EINVAL; /* not streaming */
		o->stream_on = FALSE;	
		/* Check if other opens are streaming*/
		for(i = 0; i < MAX_VBI_OPENS; i++)
		{
			if(o->dev->open_data[i].stream_on)
				return 0;
		}
		/* nobody needs vbi now, lets turn it off */
		VL_VbiCapture(0);
		slicer_off();
		if(vbi_dev.stream_buffers_allocated)
		cleanup_vbi_buffers(dev);
#ifndef NO_DECODER		
#if 0		
		save_flags(flags);
		cli();
		/* The following two are required for DVIP to work */
		/* disable slicer for capturing raw VBI */
		tv_i2c_saa7114_write_byte_data(0x86,0xe5);
		/* Enable task A + Task B */
		tv_i2c_saa7114_write_byte_data(0x80,0x30);
		restore_flags(flags);
#endif		
#endif	/* NO_DECODER */	
		return 0;
	} 
        case  GEODE_IOCTL_S_SIGNAL:
        {
#ifdef SIGNAL_USER		
		struct signal_info_t * sig_info = arg;
		/* Allow either SIGUSR1 or SIGUSR2 only */
		if( (sig_info->signal_number != SIGUSR1) &&
		    (sig_info->signal_number != SIGUSR2) )
			return -EINVAL;	 
		/* TODO:validate signal_on_count  properly*/	
		/* a zero value will disable signals */
		if(!(sig_info->signal_on_length >= 0))
			return -EINVAL;	
		o->proc = current;
		o->signal_on_count = sig_info->signal_on_length;		
		o->signal_number = sig_info->signal_number;
		return 0;
#else		
                return -EINVAL;
#endif		
        }
        break;

	case VIDIOC_ENUM_PIXFMT:
	case VIDIOC_G_COMP:
	case VIDIOC_S_COMP:
	case VIDIOC_REQBUFS:
	case VIDIOC_QUERYBUF:
	case VIDIOC_QBUF:
	case VIDIOC_DQBUF:
	case VIDIOC_ENUM_FBUFFMT:
	case VIDIOC_G_FBUF:		
	case VIDIOC_S_FBUF:		
	case VIDIOC_G_WIN:		
	case VIDIOC_S_WIN:		
	case VIDIOC_PREVIEW:	
	case VIDIOC_G_PERF:
	case VIDIOC_G_INPUT:
	case VIDIOC_S_INPUT:
	case VIDIOC_G_PARM:
	case VIDIOC_S_PARM:
	case VIDIOC_G_STD:
	case VIDIOC_S_STD:
	case VIDIOC_ENUMSTD:
	case VIDIOC_ENUMINPUT:
	case VIDIOC_QUERYCTRL:
	case VIDIOC_QUERYMENU:
	case VIDIOC_G_CTRL:
	case VIDIOC_S_CTRL:
	case VIDIOC_G_TUNER:
	case VIDIOC_S_TUNER:
	case VIDIOC_G_FREQ:	
	case VIDIOC_S_FREQ:	
	case VIDIOC_G_AUDIO:
	case VIDIOC_S_AUDIO:	
		debug_msg("ioctl %d\n", _IOC_NR(cmd));
		debug_msg("v4l2_vbi_ioctl(): invalid IOCTL\n");
		return -EINVAL;

	default:
		return -ENOIOCTLCMD;
	}
	return 0;
}

/* ------------------------------------------------------------------------
 * Description : memory mapping for the vbi device 
 * Return      : int 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
int v4l2_vbi_mmap( void	 *id, struct vm_area_struct *vma )
{
	return -EINVAL;

}

/* ------------------------------------------------------------------------
 * Description : poll information for the vbi device 
 * Return      : int 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
int v4l2_vbi_poll( void *id, struct file *file, poll_table *table )
{
	struct vbi_device_open *o = (struct vbi_device_open *)id;
		
	poll_wait(file,&o->read_wait, table);	
	if(o->num_captured_buffers)
	{
		return( POLLIN | POLLRDNORM );
	}
	return 0;
}


/* ------------------------------------------------------------------------
 * Description : read the vbi buffer & copy to the user space  
 * Return      : long 
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
long v4l2_vbi_read( void *id, /* pointer to the vbi open device */
		    char *buf, /* user buffer */
		    unsigned long count, /* length of user buffer */
		    int noblock /* flag to specify if blocking */ 
		    )
{
	struct vbi_device_open *o = (struct vbi_device_open *)id;
	struct vbi_buffer * vbi_buffer;
/* jyothin */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	struct wait_queue wait={current, NULL};
#else
	DECLARE_WAITQUEUE(wait,current);
#endif
	ssize_t ret = 0;
	ssize_t num_buffers = 1;
	ssize_t offset = 0;
	unsigned long cpu_flags;
	int	i;

	/* check if we have enough of buffer space */
	if(o->buff_length == 0)
		return 0;
	if (count < o->buff_length)
	{	
		return -EINVAL; /* we must have space for at-least one
				   buffer
				 */  
	} else if ( count > o->buff_length )
	{ 
	/* The user might need more than one buffer, compute how many
	 *  vbi_buffers the buffer that the user passed us can hold
	 */
	     num_buffers = count / o->buff_length ;
	}
	add_wait_queue(&o->read_wait,&wait);
	current->state = TASK_INTERRUPTIBLE;
	do {
		if (o->num_captured_buffers >= num_buffers)
			break; /* we have enough */	
		if (noblock) 
		{
			ret = -EAGAIN;
			goto get_out;
		}
		if (signal_pending(current) )
		{
			ret = -ERESTARTSYS ;
			goto get_out;
		}
		schedule(); /* we might have enough buffer at the 
		 	         * next schedule 
			         */
	} while(1);

	save_flags(cpu_flags);
	cli();	
	for(i = 0; i < num_buffers ; i ++)
	{
		vbi_buffer = v4l2_q_del_tail(&o->q_done);
		if(!vbi_buffer) 
		{
			debug_msg("vbi_read: cannot get open buff\n");
			continue;
		}
		o->num_captured_buffers--;
		copy_to_user(buf+offset,vbi_buffer->data ,o->buff_length);
		ret += o->buff_length;
		offset += o->buff_length;
		v4l2_q_add_head(&o->q_free,(struct v4l2_q_node*) vbi_buffer);
	}
	restore_flags(cpu_flags);	
	
	get_out:	
	current->state = TASK_RUNNING;
	remove_wait_queue(&o->read_wait,&wait); 	
	return ret;
}

/* ------------------------------------------------------------------------
 * Description : completing initialization of vbi device  
 * Return      : int (0 on success, -1 on faliure)
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
int v4l2_vbi_init_done( struct v4l2_device *v )
{
	struct vbi_device *dev = (struct vbi_device *)v;

	v4l2_q_init(&dev->q_done);
	v4l2_q_init(&dev->q_free);

	if( !vbi_decode_init( &vbi_dev ) )
	{
		debug_msg(" vbi_decode_init() failed\n");
		return -1;
	}

	return 0;
}
/* ------------------------------------------------------------------------
 * Description : Copy one line of VBI data from the VIP memory to the 
 *               open_device buffers. This also fills the 2 byte header.
 * Return      : void
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
static inline void vip_to_open_buff(unsigned char * vip_line,
                                    unsigned char * buffer_ptr,
                                    int             length)
{
//        unsigned short val  = 0;
//        val = 	 LINE_NUMBER((int)vip_line[2],(int)vip_line[3] );
        /* if the number of sliced bytes are > 0, we assume that
           this line is valid
         */
//	buffer_ptr[0] = 0;
	//buffer_ptr[1] = 0;
//	buffer_ptr[1] = val & 0x00FF;
//	buffer_ptr[0] = (val >> 8) & 0x00FF; 
//	printk("%d\n",LINE_NUMBER((int)vip_line[2],(int)vip_line[3] ));
	 
//	if( (NUM_VALID_BYTES(vip_line) - 2) > 0 ) {
//		buffer_ptr[0] |= 0x40;
//        } else {
//                buffer_ptr[0] &= (~(0x40));
//        }        
//       printk("DBG valid bytes: %d\n",NUM_VALID_BYTES(vip_line));
//	if(FIELD(vip_line[2])) {
//		buffer_ptr[0] |= 0x80;
//		//printk("odd\n");
//      } else {
//                buffer_ptr[0] &= (~(0x80));
//		//printk("even\n");
//        }
   //     printk("DBG Header %d\n",val);
//	memcpy(&buffer_ptr[2],&vip_line[4],length);
	memcpy(buffer_ptr,&vip_line[4],length);

}
/* ------------------------------------------------------------------------
 * Description : Decode sliced/raw vbi lines and copy into buffer.
 * Return      : void
 * Side effects: None 
 * Notes: 
 * ------------------------------------------------------------------------*/
static inline void decode_vbi(unsigned char sdid, /* The Expected SDID bits 
                                                   * programmed in the 
                                                   * decoder 
                                                   */
	   struct vbi_device_open * o_dev, /* The open device */
	   struct vbi_stream_buffer * buffer /* buffer which has been captured */
)
{
	int i;		
	int pitch = gfx_get_vbi_pitch();
	int length  = o_dev->vbifmt.samples_per_line;
	struct vbi_buffer * vbi_buf;
	unsigned char * vbi_line = (unsigned char *) buffer->vaddress ;
	unsigned char * open_buffer_ptr;
	unsigned long bitfield= 0;
	vbi_buf = v4l2_q_del_tail(&o_dev->q_free);
	/* If there are no free buffers avaliable, we pull out one
	 *  buffer from the done queue and use it. This will cause a loss of
	 *  captured data which hasn't been read yet.
	 */
	if(!vbi_buf) {
		vbi_buf = v4l2_q_del_head(&o_dev->q_done);
		if(!vbi_buf) {
			info_msg("Can't get vbi_buffer\n");
			return;
		}
		o_dev->num_captured_buffers --;
	}
        if(buffer->vidbuf.flags & V4L2_BUF_FLAG_ODDFIELD) {
                bitfield = o_dev->line_bitmap_odd;
	}
        else if(buffer->vidbuf.flags & V4L2_BUF_FLAG_EVENFIELD) {
                bitfield = o_dev->line_bitmap_even;
	}
	open_buffer_ptr = vbi_buf->data;
	memset(open_buffer_ptr,0,o_dev->buff_length);	

        for(i = 0; i < o_dev->dev->num_vbi_lines; i++) {
                if( GET_BIT(i,bitfield)) { /* Yes we need to copy this line */
                        vip_to_open_buff( vbi_line,
                                          open_buffer_ptr,
                                          length );        
                        //open_buffer_ptr += (length + 2);
                        open_buffer_ptr += (length);
                }
                vbi_line += pitch;
        }
	v4l2_q_add_head(&o_dev->q_done,(struct v4l2_q_node*)vbi_buf);
	o_dev->num_captured_buffers ++;
}
 

