/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Geode Video 4 Linux 2 Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */
 /* File Contents	: structure definitions and function declarations for vbi 
 * device
 *
 * [Notes]
 *
 * Project		: GeodeV4l2 linux driver
 *  Author		: Sojan James
 *    Date		:
 * -------------------------------------------------------------------------*/
#ifndef VBI_H
#define VBI_H

#define MAX_CAPTURE_BUFFERS		20
#define MAX_VBI_OPENS			5
#define MAX_VBI_LINES			25

/* buffer to hold RAW VBI "images" */
typedef struct vbi_stream_buffer
{
	struct v4l2_q_node	qnode;
	struct v4l2_buffer	vidbuf;
	int 			requested;
	__u8			*vaddress; /* Virtual address of the buffer*/
	unsigned long		paddress;  /* Physical address of the buffer*/
} vbi_stream_buffer_t;

struct vbi_device_open
{
        int                     isopen;
	int			format_set;
        struct vbi_device       *dev;
	int			noncapturing;
	int			stream_on; 
        struct v4l2_vbi_format  vbifmt; /* Each open can have it's own
					   format */
	struct	v4l2_queue	q_done; /* already captured */
	struct  v4l2_queue	q_free; /* free buffers */
	int			buff_length; /* length of the buffer */
	int			num_captured_buffers; /* to keep a count */
/* jyothin */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	struct wait_queue 	*read_wait;
#else
	wait_queue_head_t	read_wait;				   
#endif
	unsigned long           line_bitmap_odd;
	unsigned long           line_bitmap_even;
	int			line_count_odd;
	int			line_count_even;

#ifdef SIGNAL_USER
	struct  task_struct	*proc; /* process to signal to*/	
	int			signal_on_count; /* signal the user when data
						    is available */
	int 			signal_number;    
#endif		
};


typedef struct vbi_buffer
{
	struct v4l2_q_node	qnode;
	unsigned char		*data;
        unsigned char           field; /* 0 or 1 */
} vbi_buffer;

struct vbi_device
{
        struct v4l2_device      v;      /*  Must be first */
        char                    shortname[16];
        int                     is_registered;
        struct vbi_device_open  open_data[MAX_VBI_OPENS];
        int                     irq;
	int			capturing_opens;
	int			open_count;
	struct	tq_struct 	vbi_bottom_half; /* for bottom half routine */

	/* capture state */
	int			ready_to_capture;
	int			streaming;
	struct vbi_stream_buffer	stream_buf[MAX_CAPTURE_BUFFERS];
	int			stream_buffers_allocated;
	int 			stream_buffers_mapped;
	struct	v4l2_queue	q_done; /* already captured */
	struct  v4l2_queue	q_free; /* free buffers */
	struct vbi_stream_buffer * odd_capturing;
	struct vbi_stream_buffer * even_capturing;
	unsigned char		vbi_lines_odd[MAX_VBI_LINES];
	unsigned char   	vbi_lines_even[MAX_VBI_LINES];
	int			num_vbi_lines; /* this is used quite often */
};

/* Function declarations */
int
v4l2_vbi_open(struct v4l2_device *v, int flags, void **idptr);
void
v4l2_vbi_close(void *id);
long
v4l2_vbi_write(void  *id,
             const char  *buf,
             unsigned long  count,
	     int noblock);
int
v4l2_vbi_ioctl(void *id, unsigned int cmd, void *arg);
int
v4l2_vbi_mmap( void *id, struct vm_area_struct *vma );
int
v4l2_vbi_poll( void *id, struct file *file, poll_table *table );
long
v4l2_vbi_read( void *id, char *buf, unsigned long count, int noblock );
int
v4l2_vbi_init_done( struct v4l2_device *v );	
int config_vbi_device(struct vbi_device *dev);
int vbi_decode_init( struct vbi_device* dev );


#endif /* VBI_H */


