/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor IR Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */

#ifndef __INCLUDED_COMMON_H__
#  define __INCLUDED_COMMON_H__



typedef unsigned char      BYTE,  *PBYTE;
typedef unsigned short int WORD,  *PWORD;
typedef unsigned long int  DWORD, *PDWORD;

#define K_BYTES(n)	               ((n) * 1024)

#define N_BITS_IN(type_or_var)     (sizeof(type_or_var) * 8)

#define N_ELEMENTS_IN_TABLE(tab)   ((sizeof(tab)/sizeof(tab[0]))
#define ENDOF_TABLE(tab)	       ((void *)((byte *)(tab) + sizeof(tab) - 1))

#define OFFSETOF(type, member)     (&(((type *)0)->member))

#define CHAR_2_DIGIT(c)            ((c) - '0')
#define END_OF_STRING              '\0'

#ifndef MIN
#define MIN(x, y)	               ((x) < (y) ? (x) : (y))
#endif
#ifndef MAX
#define MAX(x, y)	               ((x) > (y) ? (x) : (y))
#endif

typedef struct _RANGE {
    DWORD Min;
    DWORD Max;
} RANGE, *PRANGE;

#define IS_IN_RANGE(val, min, max)          ((val) >= (min) && (val) <= (max))
#define IS_IN_STRICT_RANGE(val, min, max)   ((val) >  (min) && (val) <  (max))
#define IS_IN_STRICT_LEFT_RANGE(val, min, max) ((val) >  (min) && (val) <= (max))
#define IS_IN_STRICT_RIGHT_RANGE(val, min, max) ((val) >= (min) && (val) <  (max))
#define LOW_WORD  0
#define HIGH_WORD 1

#define LOW_BYTE_IN_WORD  0
#define HIGH_BYTE_IN_WORD 1

#define INT_MASK_4_BIT(bit)         ((UINT) (1 << (bit)))
#define DWORD_MASK_4_BIT(bit)       ((DWORD)(1 << (bit)))
#define WORD_MASK_4_BIT(bit)        ((WORD) (1 << (bit)))
#define BYTE_MASK_4_BIT(bit)        ((BYTE) (1 << (bit)))

                /* Casting to 'type' of 0xFFFFFFFF to avoid the lint warning:
                 * ANSI C treats constant as unsigned: op ">>"
                 */

#define MASK_4_BITGROUP(type, offset, size) ( ((type)((type)0xFFFFFFFF << (offset))) & ((type)((type)0xFFFFFFFF >> (N_BITS_IN(type) - (offset) - (size)))) )

#define DWORD_MASK_4_BITGROUP(offset, size)  MASK_4_BITGROUP(DWORD, offset, size)
#define WORD_MASK_4_BITGROUP(offset, size) MASK_4_BITGROUP(WORD,  offset, size)
#define BYTE_MASK_4_BITGROUP(offset, size) MASK_4_BITGROUP(BYTE,  offset, size)

#endif /* __INCLUDED_COMMON_H__ */

