/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor IR Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * File Name	  : detect.h						*
 *									*
 * Project Name	  : ASTB						*
 *									*
 * Module Name 	  : NSCIR						*
 *									*
 * Author Name	  : Arun Kumar, Niraj Kumar				*
 *									*
 * Language	  : C							*
 *									*
 * Date		  : 08/10/01						*
 *									*
 * Description	  : This file contains detection defination which are   *
 *		    used for decoding sejin protocol.			*
 *									*
 ************************************************************************/

#include "common.h"

#define BOOLEAN bool

typedef enum _IR_CODING_TYPE 
{	
	IR_CODE_FIRST,
	ALL_IR_CODING_TYPES,
	IR_CODE_LAST
} IR_CODING_TYPE;


#ifndef __INCLUDED_DETECT_H__
  #define __INCLUDED_DETECT_H__


    // Bit value for energy and no energy as read from Rx FIFO
#define ENERGY_LOW_LEVEL                1
#define ENERGY_HIGH_LEVEL               0

#define NEW_BIT_0       ((BYTE)0x00)
#define NEW_BIT_1       ((BYTE)0x01)

/*--------------------------------------------------------------------------
 *            General Detection Information
 *------------------------------------------------------------------------*/


typedef enum _DETECTION_STATE 
{ DETECTION_STATE_WAIT_4_HEADER,
    DETECTION_STATE_WAIT_4_DATA,
    DETECTION_SUBSTATE_WAIT_4_PULSE,
    DETECTION_SUBSTATE_WAIT_4_SPACE,
    DETECTION_SUBSTATE_WAIT_4_2ND_PULSE
} DETECTION_STATE;

typedef enum _DETECTION_EVENT 
{ DETECTION_NO_EVENT,
    DETECTION_SILENCE_EVENT,
    DETECTION_KEY_PRESSED_EVENT,
    DETECTION_KEY_REPEATED_EVENT,
    DETECTION_KEY_RELEASED_EVENT,
    DETECTION_FORCED_KEY_RELEASED_EVENT
} DETECTION_EVENT;

#define DETECTION_IS_ENERGY_BYTE(b)         (((b) & 0x80) == (ENERGY_HIGH_LEVEL << 7))
#define DETECTION_GET_RUN_LENGTH_COUNTER(b)  ((BYTE)(((b) & 0x7F) + 1))

#define MAX_NUMBER_OF_DATA_BITS      64
#define MAX_NUMBER_OF_DATA_DWORDS    ((MAX_NUMBER_OF_DATA_BITS + N_BITS_IN(DWORD) - 1) / N_BITS_IN(DWORD))


typedef struct _DETECTION_INFO {
    IR_CODING_TYPE  CodingType;

    RANGE           HeaderPulseCounter;
    RANGE           HeaderSpaceCounter;

    RANGE           Bit0PulseCounter;
    RANGE           Bit0SpaceCounter;
    RANGE           Bit1PulseCounter;
    RANGE           Bit1SpaceCounter;

    RANGE           ExtendedPulseCounter;    // For SHIFT protocol
    RANGE           ExtendedSpaceCounter;    // For SHIFT protocol

    RANGE           DataPulseCounter;        // For TWIRP only
    RANGE           DataSpaceCounter;        // For TWIRP only
    RANGE           DelimiterSpaceCounter;   // For TWIRP only

    RANGE           SpaceCounter;
    RANGE           InactivityCounter;

    BYTE            CodeLength[2];
    BYTE            AtomSize;        // Number of bits in each symbol in stream
    int            MsbIsFirst;

    //UIR_BR_PRESCALE 
    int  	    BaudRatePrescale;
    WORD            BaudRateDivisor;

    DETECTION_STATE InitialState;
    DETECTION_STATE State;
    DETECTION_STATE SubState;

    DWORD           NoEnergyCounter;
    DWORD           EnergyCounter;

    BYTE            NumberOfSamplesInTPeriod;

    int 		Bit0Candidate;
    int 		Bit1Candidate;
    int 	DelimiterSpaceSuspected;  // For TWIRP only
    BYTE            NumberOfChips;            // For 4PPM only
} DETECTION_INFO, *PDETECTION_INFO;

#define SIZEOF_DETECTION_INFO    (sizeof(DETECTION_INFO))

typedef struct _DETECTION_DATA {
    DWORD           Data[MAX_NUMBER_OF_DATA_DWORDS];
    BYTE            NumberOfBits;
    int 	KeyPressed;
    DETECTION_EVENT DetectionEvent;
} DETECTION_DATA, *PDETECTION_DATA;


#define SIZEOF_DETECTION_DATA    (sizeof(DETECTION_DATA))


#define LENGTH_2_COUNTER(length, tPeriod, nSamples) \
            (((length) * (nSamples)) / (tPeriod))

#define LOCAL_LENGTH_2_COUNTER(length)    \
            LENGTH_2_COUNTER(length, tPeriodLength, nSamples)

#define IS_IN_DETECTION_RANGE(value, range) \
            IS_IN_RANGE(value, (range).Min, (range).Max)

#define IS_ENERGY_IN_RANGE(value)             \
            IS_IN_DETECTION_RANGE(            \
                DetectionInfo->EnergyCounter, \
                DetectionInfo->value          \
            )  

#define IS_NO_ENERGY_IN_RANGE(value)            \
            IS_IN_DETECTION_RANGE(              \
                DetectionInfo->NoEnergyCounter, \
                DetectionInfo->value            \
            )

//     Calculations of the needed baud rate divisor and prescale value
// ===========================================================================



    // The carrier frequency is fixed at 36KHz (later referred as 
    // <carrier-frequency-in-KHz>). It means that 36000 pulses are generated
    // each second or 36 pulses each milisecond (note that the number is equal to
    // the <carrier-frequency-in-KHz>). 
    // Each bit in a data word in RC-5 consists of a burst of 32 pulses (reffered
    // as <number-of-pulses-in-a-bit>).
    // Time of one pulse in miliseconds is 1 / <carrier-frequency-in-KHz>, i.e.
    // 1 / 36 = 0.0277 msec or 27.8 usec.
    // Pulses for one bit have the following structure:
    //      __          __          __                           __          __          
    //     |  |________|  |________|  |________   . . . .  _____|  |________|  |
    //
    // 1/4-th of pulse time is high, 3/4-th - low; so the high part time is 
    // 27.8 usec / 4 = 6.95 usec. More exact result is:
    //     <pulse-high-part-time-in-msec> = 1 / <carrier-frequency-in-KHz> / 4
    // This gives more precious result of 6.94 usecs. 
    // Time of one bit in msecs is 
    //   <time-of-one-bit-in-msec> = 
    //    (<number-of-pulses-in-a-bit> - 1) / <carrier-frequency-in-KHz> 
    //   +
    //    <pulse-high-part-time-in-msec> 
    // The final result is 0.8681 msec or 868.1 usec.
    // Since each data bit value is defined by the kind of level change inside
    // a bit (low to high - 1, high to low - 0), we should sample each bit twice
    // to be able see the kind of transitions 
    // (referred as <samples-number-in-a-bit>).
    // So we should have one sample each 
    //    (<time-of-one-bit-in-msec> / <samples-number-in-a-bit>) msec or
    // 868.1 usec / 2 = 434.05 usec.
    // The sampling rate will be 1,000,000 / 434.05 = 2303.882 Bps
    // Generally,
    //     <sampling-rate-in-bps> = 
    //        (1000 * <samples-number-in-a-bit>) / <time-of-one-bit-in-msec>
    //
    // The internal clock of the baud rate divisor is 24MHz. It is divided internally
    // by 16, by prescale value and by the setting of the programmable baud rate
    // divisor.
    //     <sampling-rate-in-bps> = 
    //            24,000,000 / 
    //            16 / 
    //            <prescale-value> / 
    //            <baud-rate-divisor>
    // So, the needed <baud-rate-divisor> value is:
    //     <baud-rate-divisor> = 
    //            24,000,000 / 
    //            16 / 
    //            <prescale-value> / 
    //            <sampling-rate-in-bps>
    // We should calculate <baud-rate-divisor> for each possible <prescale-value>
    // (1, 1.625 or 13) aand see which of them gives less error for 
    // <baud-rate-divisor>.
    // In our case,
    //     Prescale = 1
    //        <baud-rate-divisor> = 24,000,000 / 16 / 1 / 2303.882 =
    //                            = 651.075
    //     Prescale = 1.625
    //        <baud-rate-divisor> = 24,000,000 / 16 / 1.625 / 2303.882 =
    //                            = 400.662
    //     Prescale = 13
    //        <baud-rate-divisor> = 24,000,000 / 16 / 13 / 2303.882 =
    //                            = 50.083
    // Minimum errors can be achieved for prescale value 1 and the divisor equal
    // to 651


#endif /* __INCLUDED_DETECT_H__ */







