/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor IR Drivers
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * File Name	  : rc5.c						*
 *									*
 * Project Name	  : ASTB						*
 *									*
 * Module Name 	  : NSCIR						*
 *									*
 * Author Name	  : Arun Kumar, Niraj Kumar				*
 *									*
 * Language	  : C							*
 *									*
 * Date		  : 08/10/01						*
 *									*
 * Description	  : This file contains the Rc5 protocol decoding part.  *
 *									*
 ************************************************************************/

#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>			// inb and outb
#include <linux/tqueue.h>	// for bottom-half
#include <linux/interrupt.h>
#include <linux/sched.h>	// for current
#include <linux/poll.h>		// poll method
#include <linux/string.h>

#include "common.h"
#include "detect.h" 
#include "rc5.h"

#define TRUE 1
#define FALSE 0
#define bool int
#define RC5_PROTO 3

#define SHIFT_CODING_ENABLED
#ifdef SHIFT_CODING_ENABLED


#define RC5_NO_SAMPLES  3
#define RC5_T_PERIOD    889
#define RC5_MOD_FREQ_HZ   36000
// #define RC5_NUM_CODE_BITS   14
#define RC5_INACTIVITY_LENGTH     5000000         // 5 secs   


DETECTION_INFO  *DetectionInfo;
DETECTION_DATA  *DetectionData;

static DETECTION_EVENT NscCirDetectionHandleByteRc5( BYTE );

void	NscCirInitializeDetectionRC5();
int     NscIrRawDataRC5(BYTE);

//EXPORT_SYMBOL_NOVERS(NscCirInitializeDetectionRC5);
//EXPORT_SYMBOL_NOVERS(NscIrRawDataRC5);

static unsigned char Rc5Map2StdKey[0x40]=
{
//0x0,0x1,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
  11,   2,   3,   4,   5,   6,   7,   8,   9,  10,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
 126, 127,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
0x00,0x01, 122,0x03, 120, 121, 124,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f
};

int reset_counter;


EXPORT_NO_SYMBOLS;



/************************************************************************
 * Syntax	: init_module                                           *
 *                                                                     	*
 * Description	: This function loads the driver as a module in the    	*
 *		  Linux kernel.						*
 *                                                                     	*
 * Parameters	: None.                                                	*
 *                                                                     	*
 * Return 	: 0 on success, errno on failure.	               	*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

int init_module(void)
{
	if( DetectionInfo ) return 0;

	if( (DetectionInfo = kmalloc(sizeof(DETECTION_INFO),GFP_KERNEL))<= 0)
	{
		printk(" erron in alocating kernel mem, DetectInfo \n");
	        return 0;
	}

	if( (DetectionData = kmalloc(sizeof(DETECTION_DATA),GFP_KERNEL)) <= 0)
	{
		printk(" erron in alocating kernel mem, DetectDAta \n");
	        return 0;
	}

	memset(DetectionInfo,0, sizeof(DETECTION_INFO));
	memset(DetectionData,0, sizeof(DETECTION_DATA));
	if(0==NscIrRegProto(RC5_PROTO, NscCirInitializeDetectionRC5,NscIrRawDataRC5,0))
	{
		printk(" not able to load the RC5 module \n"); 
	}

	return 0;
}


/************************************************************************
 * Syntax	: cleanup_module                                        *
 *                                                                     	*
 * Description	: This function clears the driver module from the Linux	*
 *		  kernel.						*
 *                                                                     	*
 * Parameters	: None.                                                	*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void cleanup_module(void)
{
	if(0==NscIrUnRegProto(RC5_PROTO))
	{
		printk(" not able to load the Rc5 module \n"); 
	}

   	kfree(DetectionInfo);
	kfree(DetectionData);
}


/************************************************************************
 * Syntax	: NscCirResetDetectionData                              *
 *                                                                     	*
 * Description	: This function  initializes the  Data buffer to zero,  *
 *		  so that the buffer is made ready to receive new set 	*
 *		  of data.						*
 *                                                                     	*
 * Parameters	: DetectionData                                         *
 *	    	    - This structure is used to  store data and its 	*
 *		      status.						*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void NscCirResetDetectionDataRc5(PDETECTION_DATA DetectionData)
{
    memset(&DetectionData->Data,0, MAX_NUMBER_OF_DATA_DWORDS * sizeof(DWORD));
    DetectionData->NumberOfBits = 0;
}


/************************************************************************
 * Syntax	: NscCirResetDetectionVariablesRc5                      *
 *                                                                     	*
 * Description	: This function resets the Detection structure so that	*
 *		  the structure is ready for the next set of data	*
 *                                                                     	*
 * Parameters	: DetectionInfo                                         *
 *	    	    - This structure is used to  store data and its 	*
 *		      status.						*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void NscCirResetDetectionVariablesRc5(PDETECTION_INFO DetectionInfo)
{
        // Check validity of the passed parameters
    if( DetectionInfo != NULL)
    {

    DetectionInfo->State                   = DetectionInfo->InitialState;
    DetectionInfo->SubState                = DETECTION_SUBSTATE_WAIT_4_PULSE;
    DetectionInfo->Bit0Candidate           = FALSE;
    DetectionInfo->Bit1Candidate           = FALSE;
    DetectionInfo->DelimiterSpaceSuspected = FALSE;
    DetectionInfo->NumberOfChips           = 0;
    }
}

/************************************************************************
 * Syntax	: NscCirInitializeDetectionRC5                          *
 *                                                                     	*
 * Description	: This function is used to initialise the variables     *
 *		  which are used in detecting the data 			*
 *                                                                     	*
 * Parameters	: None.		                                        *
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void NscCirInitializeDetectionRC5()
{
	BYTE  nSamples;
	DWORD tPeriodLength;
	DWORD baudRate;
	int   index;

        // Check validity of the passed parameters
	if(DetectionInfo <= 0)  return;

	DetectionInfo->InitialState = DETECTION_STATE_WAIT_4_DATA;
        DetectionInfo->CodingType               = 0;
	DetectionInfo->NoEnergyCounter          = 0;
	DetectionInfo->EnergyCounter            = 0;

	nSamples = RC5_NO_SAMPLES ;
    
	if (nSamples == 0) nSamples = 3;
	
	DetectionInfo->NumberOfSamplesInTPeriod = nSamples;

	tPeriodLength = RC5_T_PERIOD;

        // TODO: Improve algorithm to reach the least # of errors
        // Calculate the baud rate prescaler value.
        // Don't use prescale value equal to 1 since in some versions of
        // chip it actually works as prescale value 12 causing incorrect
        // calculations!!!
        // Calculate the baud rate value


        DetectionInfo->HeaderPulseCounter.Min = 0; 
        DetectionInfo->HeaderPulseCounter.Max = 0; 
	    
        DetectionInfo->HeaderSpaceCounter.Min = 0; 
        DetectionInfo->HeaderSpaceCounter.Max = 0;


	DetectionInfo->Bit0PulseCounter.Min =
          LOCAL_LENGTH_2_COUNTER(RC5_T_PERIOD) - 1;
	DetectionInfo->Bit0PulseCounter.Max =
          DetectionInfo->Bit0PulseCounter.Min + 2;


	DetectionInfo->Bit0SpaceCounter.Min =
          LOCAL_LENGTH_2_COUNTER(RC5_T_PERIOD) - 1;
	DetectionInfo->Bit0SpaceCounter.Max =
          DetectionInfo->Bit0SpaceCounter.Min + 2;

        DetectionInfo->Bit1PulseCounter.Min =
          LOCAL_LENGTH_2_COUNTER(RC5_T_PERIOD) - 1;
	DetectionInfo->Bit1PulseCounter.Max =
          DetectionInfo->Bit1PulseCounter.Min + 2;

	DetectionInfo->Bit1SpaceCounter.Min =
          LOCAL_LENGTH_2_COUNTER(RC5_T_PERIOD) - 1;
        DetectionInfo->Bit1SpaceCounter.Max =
          DetectionInfo->Bit1SpaceCounter.Min + 2;

	ShiftCodeInitializeDetection();

	DetectionInfo->InactivityCounter.Min =
          LOCAL_LENGTH_2_COUNTER(RC5_INACTIVITY_LENGTH) ;
	
	DetectionInfo->InactivityCounter.Max =
          DetectionInfo->InactivityCounter.Min + 1;

        DetectionInfo->CodeLength[0] = RC5_NUM_CODE_BITS;
	DetectionInfo->CodeLength[1] = RC5_NUM_CODE_BITS;

	DetectionInfo->MsbIsFirst = TRUE ;

	DetectionData->KeyPressed = FALSE;

	NscCirResetDetectionDataRc5(DetectionData);
	DetectionInfo->NumberOfChips = 0;

	NscCirResetDetectionVariablesRc5(DetectionInfo);

}


#define NO_NEW_DATA     ((BYTE)0xFF)

/************************************************************************
 * Syntax	: ShiftCodeInitializeDetection                          *
 *                                                                     	*
 * Description	: This function is used in initializing the data 	*
 *                structures  which are used in shift code decoding.	*
 *                                                                     	*
 * Parameters	: None.		                                        *
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

int ShiftCodeInitializeDetection()
{
    DWORD maxSinglePulse;
    DWORD maxSingleSpace;
    BYTE  nSamples = RC5_NO_SAMPLES;
    DWORD tPeriodLength = RC5_T_PERIOD;

    DetectionInfo->AtomSize = 1;

    DetectionInfo->ExtendedPulseCounter.Min =
      DetectionInfo->Bit0PulseCounter.Min + DetectionInfo->Bit1PulseCounter.Min;
    DetectionInfo->ExtendedPulseCounter.Max =
      DetectionInfo->Bit0PulseCounter.Max + DetectionInfo->Bit1PulseCounter.Max;
    maxSinglePulse = MAX(DetectionInfo->Bit0PulseCounter.Max,
                         DetectionInfo->Bit1PulseCounter.Max);
    if (DetectionInfo->ExtendedPulseCounter.Min <= maxSinglePulse)
        DetectionInfo->ExtendedPulseCounter.Min = maxSinglePulse + 1;

   DetectionInfo->ExtendedSpaceCounter.Min =
      DetectionInfo->Bit0SpaceCounter.Min + DetectionInfo->Bit1SpaceCounter.Min;
   DetectionInfo->ExtendedSpaceCounter.Max =
      DetectionInfo->Bit0SpaceCounter.Max + DetectionInfo->Bit1SpaceCounter.Max;
   maxSingleSpace = MAX(DetectionInfo->Bit0SpaceCounter.Max,
                         DetectionInfo->Bit1SpaceCounter.Max);
   if (DetectionInfo->ExtendedSpaceCounter.Min <= maxSingleSpace)
        DetectionInfo->ExtendedSpaceCounter.Min = maxSingleSpace + 1;

    DetectionInfo->SpaceCounter.Min = 2 +
        DetectionInfo->ExtendedSpaceCounter.Max;
    DetectionInfo->SpaceCounter.Max = 2 +
        LOCAL_LENGTH_2_COUNTER(100889);
}

#endif /* SHIFT_CODING_ENABLED */

/************************************************************************
 * Syntax	: ShiftCodeDetectionHandleByte                          *
 *                                                                     	*
 * Description	:  This function is used in decoding the Shift code data*
 *                                                                     	*
 * Parameters	: DetectionInfo	  					*
 *		    - holds the information about the data		*
 *		  DetectionData					        *
 *		    - holds the detection data				*
 *		  NewByte						*
 *		    - holds the data to be decoded			*
 *		  *NewBitPair						*
 *		    - holds the  decoded data				*
 *		  *DetectionEvent 					*
 *		    - holds the event on detection of data		*
 *		  *ResetDetection    					*
 *		    - set when there is detection reset encountered	*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void ShiftCodeDetectionHandleByte(
    PDETECTION_INFO DetectionInfo,
    PDETECTION_DATA DetectionData,
    BYTE            NewByte,
    BYTE            *NewBit,
    DETECTION_EVENT *DetectionEvent,
    BOOLEAN         *ResetDetection
)
{
    BOOLEAN isPulse0 = FALSE;
    BOOLEAN isPulse1 = FALSE;
    BOOLEAN isExtendedPulse = FALSE;
    BOOLEAN isSpace0 = FALSE;
    BOOLEAN isSpace1 = FALSE;
    BOOLEAN isExtendedSpace = FALSE;


    if (IS_ENERGY_IN_RANGE(Bit0PulseCounter)) {
        isPulse0 = TRUE;
        }
    if (IS_ENERGY_IN_RANGE(Bit1PulseCounter)) {
        isPulse1 = TRUE;
        }

    switch (DetectionInfo->SubState) {
    case DETECTION_SUBSTATE_WAIT_4_PULSE:
        if ((DetectionData->NumberOfBits == 0) && DetectionData->KeyPressed)
            NscCirResetDetectionDataRc5(DetectionData);

        if (IS_ENERGY_IN_RANGE(ExtendedPulseCounter))
            isExtendedPulse = TRUE;

        if (isPulse0 && isExtendedPulse)
            *ResetDetection = TRUE;

        else if (isPulse0)
        {
        	if (DetectionInfo->Bit0Candidate)
        	{
                	*NewBit = NEW_BIT_0;
	                DetectionInfo->Bit0Candidate = FALSE;
         	}
            else if (DetectionInfo->Bit1Candidate)
            	 {
                        *NewBit = NEW_BIT_1;
                        DetectionInfo->Bit1Candidate = FALSE;
            	 }
            else {
                if (DetectionData->NumberOfBits == 0) {
                    *NewBit = NEW_BIT_1;
                  }
                else {
                    DetectionInfo->Bit0Candidate = TRUE;
                     }
            }
            DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_SPACE;
        }
        else if (isExtendedPulse) {
            if (DetectionInfo->Bit0Candidate) {
                *NewBit = NEW_BIT_0;
                DetectionInfo->Bit0Candidate = FALSE;
                DetectionInfo->Bit1Candidate = TRUE;
                DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_SPACE;
            }
            else if (DetectionInfo->Bit1Candidate) {
                *NewBit = NEW_BIT_1;
                DetectionInfo->Bit1Candidate = FALSE;
                DetectionInfo->Bit0Candidate = TRUE;
                DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_SPACE;
            }
            else {
                    // Unexpected extended pulse is detected
                *ResetDetection = TRUE;
            }
        }
        else {
                // Too short or too long pulse is detected
            *ResetDetection = TRUE;
        }
        break;

    case DETECTION_SUBSTATE_WAIT_4_SPACE:
        if (IS_NO_ENERGY_IN_RANGE(Bit0SpaceCounter)) {
            isSpace0 = TRUE;
        }
        if (IS_NO_ENERGY_IN_RANGE(Bit1SpaceCounter)) {
            isSpace1 = TRUE;
        }

        if (IS_NO_ENERGY_IN_RANGE(ExtendedSpaceCounter)) {
            isExtendedSpace = TRUE;
        }

        if (isSpace0 && isExtendedSpace) {
                // Seems that remote control uses not SHIFT coding scheme
            *ResetDetection = TRUE;
        }
        else if (isSpace0) {
            if (DetectionInfo->Bit1Candidate) {
                *NewBit = NEW_BIT_1;
                DetectionInfo->Bit1Candidate = FALSE;
            }
            else if (DetectionInfo->Bit0Candidate) {
                *NewBit = NEW_BIT_0;
                DetectionInfo->Bit0Candidate = FALSE;
            }
            else {
                DetectionInfo->Bit1Candidate = TRUE;
            }
            DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;
        }
        else if (isExtendedSpace) {
            if (DetectionInfo->Bit1Candidate) {
                *NewBit = NEW_BIT_1;
                DetectionInfo->Bit1Candidate = FALSE;
                DetectionInfo->Bit0Candidate = TRUE;
                DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;
            }
            else if (DetectionInfo->Bit0Candidate) {
                *NewBit = NEW_BIT_0;
                DetectionInfo->Bit0Candidate = FALSE;
                DetectionInfo->Bit1Candidate = TRUE;
                DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;
            }
            else {
                    // Unexpected extended space is detected
                *ResetDetection = TRUE;
            }
        }
        else {
                // Too short or too long space is detected
            *ResetDetection = TRUE;
        }
        break;

    default:
        break;
    }
}



/************************************************************************
 * Syntax	: NscIrRawDataRC5		                        *
 *                                                                     	*
 * Description	:  this function is used to detecting the Rc5 data	*
 *                                                                     	*
 * Parameters	: NewByte 				                *
 *		    - new data to be detected				*
 *                                                                     	*
 * Return 	: Returns the event detected.				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

int NscIrRawDataRC5(BYTE NewByte)
{
	int event;
	event = NscCirDetectionHandleByteRc5( NewByte );
	return event;
}


/************************************************************************
 * Syntax	: NscCirDetectionHandleByte		                *
 *                                                                     	*
 * Description	: This function is used to detecting the RC5 data       *
 *                which is encoded using shift code protocol            *
 *                                                                     	*
 * Parameters	: NewByte 				                *
 *		    - new data to be detected				*
 *                                                                     	*
 * Return 	: Returns the event detected.				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

static DETECTION_EVENT NscCirDetectionHandleByteRc5( BYTE  NewByte)
{
    DETECTION_EVENT detectionEvent = DETECTION_NO_EVENT;
    BOOLEAN         isEnergy       = DETECTION_IS_ENERGY_BYTE(NewByte);
    BOOLEAN         isNoEnergy     = !isEnergy;
    BYTE            counter        = DETECTION_GET_RUN_LENGTH_COUNTER(NewByte);
    BOOLEAN         resetDetection = FALSE;
    BOOLEAN         resetData      = TRUE;
    BYTE            newData        = NO_NEW_DATA;
    int             index;

    #define RESET_DETECTION(str)  { resetDetection =TRUE; if(NewByte!=0xff) reset_counter++;/*printk("%s\n",str);*/ }

    if (isNoEnergy)
    {
        	   		// Increment no energy counter
        DetectionInfo->NoEnergyCounter += counter;
          			// Reset energy counter
        DetectionInfo->EnergyCounter = 0;
        
        if(DetectionInfo->NoEnergyCounter > DetectionInfo->InactivityCounter.Max) 
        {
        		            // Notify silence
            detectionEvent = DETECTION_SILENCE_EVENT;
                // Reset detection in any case
            RESET_DETECTION("Inactivity is detected");
        } 
        else 
      	if (DetectionInfo->NoEnergyCounter > DetectionInfo->SpaceCounter.Max) 
     	{
            	if (DetectionData->KeyPressed) 
		{
	                    // Notify key release 
                	detectionEvent = DETECTION_KEY_RELEASED_EVENT;
                	DetectionData->KeyPressed = FALSE;
            		}
               	 // Reset detection in any case
            	RESET_DETECTION("Too much silence is detected");
            	NscCirResetDetectionDataRc5(DetectionData);
            	resetData = FALSE;
        	}
    }
    else 
    {
        		  	 // Increment energy counter
        DetectionInfo->EnergyCounter += counter;
          			 // Reset no energy counter
        DetectionInfo->NoEnergyCounter = 0;
    }


    if (detectionEvent == DETECTION_SILENCE_EVENT)
        goto ResetDetection;

    
    StateMachine:
    //---------//
    
    
switch (DetectionInfo->State) 
{
            // We're looking for the beginning of the code which is
            // preluded by a header. 
  case DETECTION_STATE_WAIT_4_HEADER:

      switch (DetectionInfo->SubState) 
	{
	        
               case DETECTION_SUBSTATE_WAIT_4_PULSE:
	    {
                    if (isEnergy) 
	     {
                        if (IS_ENERGY_IN_RANGE(HeaderPulseCounter)) 
	         {
                        if (DetectionInfo->HeaderSpaceCounter.Max == 0) 
			{                // No space in header
                           DetectionInfo->State = DETECTION_STATE_WAIT_4_DATA;
                        }
                      DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_SPACE;
                  }
                  else 
	         {
                       // Too short or too long header pulse is detected
              RESET_DETECTION("Too short or too long header pulse is detected");
                  }
             }
              else 
	     {
                            // Else ignore any silence before header
 		;
                    }
	    }
                    break;




                case DETECTION_SUBSTATE_WAIT_4_SPACE:
	    {   

                    if (isNoEnergy) 
	     {
                        if (IS_NO_ENERGY_IN_RANGE(HeaderSpaceCounter)) 
	         	{
                         DetectionInfo->State    = DETECTION_STATE_WAIT_4_DATA;
                         DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;

			 DetectionInfo->NumberOfChips =0;
			 DetectionInfo->NoEnergyCounter =0;
			 DetectionInfo->EnergyCounter =0;
		
                        }
                        else {
                                // Too short or too long pause is detected
                   RESET_DETECTION("Too short or too long header space is detected");
                        }
                    }
                    else 
	     {
                            // Unexpected energy is detected
                   RESET_DETECTION("Unexpected energy is detected");
                    }
	    }
                    break;


                default:
			printk(" Invalid det Substate ");
                    break;

            }
            break;




        case DETECTION_STATE_WAIT_4_DATA:
            switch (DetectionInfo->SubState) 
            {

	     
                case DETECTION_SUBSTATE_WAIT_4_PULSE:
	   {
                    if (isEnergy) 
	     {
	         ShiftCodeDetectionHandleByte(
		 		    DetectionInfo,
                                    DetectionData, 
                                    NewByte, 
                                    &newData,
                                    &detectionEvent,
                                    &resetDetection

	         );
	//	reset_counter =0;
              } // End of if energy detected
                  else 
	     {
                        if (DetectionInfo->HeaderPulseCounter.Max == 0 && 
                            (DetectionData->NumberOfBits == 0                            ||
                             DetectionData->NumberOfBits == DetectionInfo->CodeLength[0] ||
                             DetectionData->NumberOfBits == DetectionInfo->CodeLength[1])
                           )
                        {
                                // Here number of bits should be 0, check this assumption
//		printk("Ignoring silence before the first pulse (no header)");
                        }
                        else 
	         {
                                // Unexpected no energy is detected
                    RESET_DETECTION("Unexpected no energy is detected");
                        }
                    }
	   }
                    break;



                case DETECTION_SUBSTATE_WAIT_4_SPACE:
              if (isNoEnergy) 
	     {
                        if (IS_NO_ENERGY_IN_RANGE(SpaceCounter)) 
	         {
#define IR_CODING_SHIFT 0
                     if (DetectionInfo->CodingType == IR_CODING_SHIFT) 
	             {
		  
                      if (DetectionInfo->Bit0Candidate) 
		      {
                                    newData = NEW_BIT_0;
                       }
                     }

                     detectionEvent = DETECTION_KEY_PRESSED_EVENT;
                     DetectionData->KeyPressed = TRUE;


                            NscCirResetDetectionVariablesRc5(DetectionInfo);
                        }
                        else 
	         	{

		  	ShiftCodeDetectionHandleByte(
 					DetectionInfo,
                                        DetectionData,
                                        NewByte, 
                                        &newData,
                                        &detectionEvent,
                                        &resetDetection
		  			);
                        }
                    }
                    else 
	     	{
                        if (DetectionInfo->DelimiterSpaceSuspected) 
	         {
                                // Change state end redo the state machine
                   DetectionInfo->State = DetectionInfo->InitialState;
                   DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;
                   goto StateMachine;
                  }
                 else
	         {
                            // Unexpected energy is detected, reset detection
                         RESET_DETECTION("Unexpected energy is detected");
                        }
                    }
                    break;

                default:
                    break;

            }
            break;


        default:
            break;
    }    // End of switch of the detection state




    if (newData != NO_NEW_DATA) 
    {
        int numberOfBits ;
        int atomSize     = DetectionInfo->AtomSize;
        numberOfBits = DetectionData->NumberOfBits;


        if (numberOfBits + atomSize <= MAX_NUMBER_OF_DATA_BITS) 
        {
            DWORD mask;
            int   bitInDword = numberOfBits % N_BITS_IN(DWORD);
            int   dwordNo    = numberOfBits / N_BITS_IN(DWORD);     
            int   index;

            if (DetectionInfo->MsbIsFirst) 
            {
                mask  = newData << (N_BITS_IN(DWORD) - atomSize - bitInDword);
                index = MAX_NUMBER_OF_DATA_DWORDS - 1 - dwordNo;
            }
            else 
            {
                mask  = newData << bitInDword;
                index = dwordNo;
            }

            DetectionData->Data[index] |= mask;

            DetectionData->NumberOfBits += (BYTE)atomSize;

        if(DetectionData->NumberOfBits >= 14)
		{
			Rc5AnalyzeData();
			resetDetection =1;
		}
        }
        else 
        {
                // Too many data bits are detected
            RESET_DETECTION("Too many data bits are detected");
        }
    }
    

    ResetDetection:
    //-------------
    if (resetDetection) 
    {
        NscCirResetDetectionVariablesRc5(DetectionInfo);
        if (resetData)
            NscCirResetDetectionDataRc5(DetectionData);
    }

    if(reset_counter >= 10) 
	{    reset_counter =0;
		return 0xfff;
	}

    return detectionEvent;

    #undef RESET_DETECTION
}

/************************************************************************
 * Syntax	: Rc5AnalyzeData                          		*
 *                                                                     	*
 * Description	: This function is used to analyse the data for Rc5     *
 *                protocol						*
 *                                                                     	*
 * Parameters	: None.		                                        *
 *                                                                     	*
 * Return 	: Returns the valid code.				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

BOOLEAN Rc5AnalyzeData()
{
    BOOLEAN   validCode  = FALSE;
unsigned char temp;

        // Check validity of the passed parameters

// printk(" RC5 data ready for analysis \n");
    if (DetectionData->NumberOfBits == DetectionInfo->CodeLength[0])
    {
        RC5_CODE rc5Code;
        BOOLEAN  startOk;

        rc5Code.Code = DetectionData->Data[MAX_NUMBER_OF_DATA_DWORDS - 1];
        rc5Code.Code >>= (N_BITS_IN(DWORD) - DetectionData->NumberOfBits);

        startOk =
            (rc5Code.Rc5Fields.Start == BYTE_MASK_4_BITGROUP(0, RC5_NUM_START_BITS));
        if (startOk)
        {
		reset_counter =0;

		// printk( " Command Code: %x \n", (REMOTE_CODE)rc5Code.Rc5Fields.Command);

		temp = Rc5Map2StdKey[rc5Code.Rc5Fields.Command];
		if( (temp > 119) && (temp < 128))
		{
			handle_scancode (56,1);
			handle_scancode (temp-118,1);
			handle_scancode (56,0);
		}
		else
			handle_scancode(Rc5Map2StdKey[rc5Code.Rc5Fields.Command],1);
            
		validCode = TRUE;
        }
        else
        {
              // Return the default FALSE
        }
    }
    else
    {
           // Return the default FALSE
    }

    return validCode;
}




