/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor IR-Drivers Library
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
*/
 /***********************************************************************
 *									*
 * File Name	  : sejin.c						*
 *									*
 * Project Name	  : ASTB						*
 *									*
 * Module Name 	  : NSCIR						*
 *									*
 * Author Name	  : Arun Kumar, Niraj Kumar				*
 *									*
 * Language	  : C							*
 *									*
 * Date		  : 08/10/01						*
 *									*
 * Description	  : This file contains the 4ppm and Sejin protocol      *
 *		    decoding part. 					*
 *									*
 ************************************************************************/


#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>			// inb and outb
#include <linux/tqueue.h>	// for bottom-half
#include <linux/interrupt.h>
#include <linux/sched.h>	// for current
#include <linux/poll.h>		// poll method
#include <linux/string.h>

#include "common.h"
#include "detect.h"
#include  "sejin.h"

#define   TRUE   1 
#define   FALSE  0 
#define   bool int
#define   SEJIN_PROTO 2
#define _4PPM_CODING_ENABLED

#ifdef _4PPM_CODING_ENABLED

#define SEJIN_NO_SAMPLES 13
#define SEJIN_T_PERIOD   312
#define SEJIN_MOD_FREQ_KHZ   38
#define SEJIN_MOD_FREQ_HZ   38800
#define SEJIN_NUM_CODE_BITS               34

#define   SEJIN_INACTIVITY_LENGTH     5000000         // 5 secs   


#define SJN_KBD    0
#define SJN_MOUSE  31
#define SJN_RMT    20

EXPORT_NO_SYMBOLS;

struct
{
	BYTE deviceId;
	BYTE button;
	BYTE modkey;
	BYTE data;
} NscSejinData;

DETECTION_INFO  *DetectionInfo;
DETECTION_DATA  *DetectionData;

int  _4ppmCodeInitializeDetection( );
void	NscCirInitializeDetection();
static DETECTION_EVENT NscCirDetectionHandleByte( BYTE );
int NscIrRawDataSejin(BYTE );
void NscSejinHandleData(void);
static void enque_mouse(BYTE,BYTE,BYTE,BYTE);
int nsc4ppmhit =0;
int reset_counter =0;

static unsigned char SjnMap2StdKey[0x8f]=
{
//0x0,0x1,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
0x00,0x01,  48,  15,  77,  57,  40,  54,  12,  26,0x0a,  28, 110,  75,0x0e,0x0f,
0x00,0x01,  30,  31,  32,  33,  36,0x07,  37,  38,0x0a,  39,  81,  79,0x0e,0x0f,
0x00,  56,  16,  17,  18,  19,  22,0x07,  23,  24,0x0a,  25,  27, 119,0x0e,0x0f,
0x00,0x01,  44,  45,  46,  47,  50,  54,  51,  52,0x0a,  53,0x0c,  49,0x0e,0x0f,
0x00, 111,  61,  01,  63,  06,  07,0x07,  64,  66,0x0a,  13,  14,  71,0x0e,0x0f,
0x00,0x01,  60,  59,  62,  34,  35,0x07,  65,  67,0x0a,  43,  72,  73,0x0e,  97,
0x00,0x01,  02,  03,  04,  05,   8,  07,   9,  10,0x0a,  11,  80,0x0d,0x0e,0x0f,
0x00,  56,  41,  58,  69,  20,  21,0x07,  87,  68,0x0a,  88,0x0c,  70,0x0e,0x0f
};


//EXPORT_SYMBOL_NOVERS(NscCirInitializeDetection);
//EXPORT_SYMBOL_NOVERS(NscIrRawDataSejin);


/************************************************************************
 * Syntax	: init_module                                           *
 *                                                                     	*
 * Description	: This function loads the driver as a module in the    	*
 *		  Linux kernel.						*
 *                                                                     	*
 * Parameters	: None.                                                	*
 *                                                                     	*
 * Return 	: 0 on success, errno on failure.	               	*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

int init_module(void)
{
	if( DetectionInfo ) return 0;

	if( (DetectionInfo = kmalloc(sizeof(DETECTION_INFO),GFP_KERNEL))<= 0)
	{
		printk(" erron in alocating kernel mem, DetectInfo \n");
	        return 0;
	}

	if( (DetectionData = kmalloc(sizeof(DETECTION_DATA),GFP_KERNEL)) <= 0)
	{
		printk(" erron in alocating kernel mem, DetectDAta \n");
	        return 0;
	}

	
	memset(DetectionInfo,0, sizeof(DETECTION_INFO));
	memset(DetectionData,0, sizeof(DETECTION_DATA));
	if(0==NscIrRegProto(SEJIN_PROTO, NscCirInitializeDetection,NscIrRawDataSejin,0))
	{
		printk(" not able to load the sejin module \n"); 
	}
	return 0;
}



/************************************************************************
 * Syntax	: cleanup_module                                        *
 *                                                                     	*
 * Description	: This function clears the driver module from the Linux	*
 *		  kernel.						*
 *                                                                     	*
 * Parameters	: None.                                                	*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void cleanup_module(void)
{
	if(0==NscIrUnRegProto(SEJIN_PROTO))
	{
		printk(" not able to load the sejin module \n"); 
	}
   	kfree(DetectionInfo);
	kfree(DetectionData);
}



/************************************************************************
 * Syntax	: NscCirResetDetectionData                              *
 *                                                                     	*
 * Description	: This function  initializes the  Data buffer to zero,  *
 *		  so that the buffer is made ready to receive new set 	*
 *		  of data.						*
 *                                                                     	*
 * Parameters	: DetectionData                                         *
 *	    	    - This structure is used to  store data and its 	*
 *		      status.						*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void NscCirResetDetectionData(PDETECTION_DATA DetectionData)
{
    memset(&DetectionData->Data,0, MAX_NUMBER_OF_DATA_DWORDS * sizeof(DWORD));
    DetectionData->NumberOfBits = 0;
}




/************************************************************************
 * Syntax	: NscCirResetDetectionVariables                         *
 *                                                                     	*
 * Description	: This function resets the Detection structure so that	*
 *		  the structure is ready for the next set of data	*
 *                                                                     	*
 * Parameters	: DetectionInfo                                         *
 *	    	    - This structure is used to  store data and its 	*
 *		      status.						*	
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void NscCirResetDetectionVariables(PDETECTION_INFO DetectionInfo)
{
        // Check validity of the passed parameters
    if( DetectionInfo != NULL)
    {

    DetectionInfo->State                   = DetectionInfo->InitialState;
    DetectionInfo->SubState                = DETECTION_SUBSTATE_WAIT_4_PULSE;
    DetectionInfo->Bit0Candidate           = FALSE;
    DetectionInfo->Bit1Candidate           = FALSE;
    DetectionInfo->DelimiterSpaceSuspected = FALSE;
    DetectionInfo->NumberOfChips           = 0;
    }
}


/************************************************************************
 * Syntax	: NscCirInitializeDetection                             *
 *                                                                     	*
 * Description	: This function is used to initialise the variables     *
 *		  which are used in detecting the data 			*
 *                                                                     	*
 * Parameters	: None.		                                        *
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void NscCirInitializeDetection()
{
    BYTE  nSamples;
    DWORD tPeriodLength;
    DWORD baudRate;
    int   index;

        // Check validity of the passed parameters
    if(DetectionInfo <= 0)  return;

    DetectionInfo->InitialState = DETECTION_STATE_WAIT_4_HEADER;
    
    DetectionInfo->CodingType               = 0;
    DetectionInfo->NoEnergyCounter          = 0;
    DetectionInfo->EnergyCounter            = 0;

    nSamples = SEJIN_NO_SAMPLES ; 
    
    if (nSamples == 0) nSamples = 3;
	
    DetectionInfo->NumberOfSamplesInTPeriod = nSamples;

    tPeriodLength = SEJIN_T_PERIOD;

        // TODO: Improve algorithm to reach the least # of errors
        // Calculate the baud rate prescaler value.
        // Don't use prescale value equal to 1 since in some versions of 
        // chip it actually works as prescale value 12 causing incorrect
        // calculations!!!
        // Calculate the baud rate value


        DetectionInfo->HeaderPulseCounter.Min = 
            LOCAL_LENGTH_2_COUNTER(3*SEJIN_T_PERIOD) - 1;
	    
        DetectionInfo->HeaderPulseCounter.Max = 
            DetectionInfo->HeaderPulseCounter.Min + 2;

        DetectionInfo->HeaderSpaceCounter.Min =0; 
        DetectionInfo->HeaderSpaceCounter.Max = 0;


    DetectionInfo->Bit0PulseCounter.Min = 
        LOCAL_LENGTH_2_COUNTER(SEJIN_T_PERIOD) - 1;
    DetectionInfo->Bit0PulseCounter.Max = 
        DetectionInfo->Bit0PulseCounter.Min + 2;

	

    DetectionInfo->Bit0SpaceCounter.Min = 
        LOCAL_LENGTH_2_COUNTER(SEJIN_T_PERIOD) - 1;
    DetectionInfo->Bit0SpaceCounter.Max = 
        DetectionInfo->Bit0SpaceCounter.Min + 2;
    
    DetectionInfo->Bit1PulseCounter.Min = 
        LOCAL_LENGTH_2_COUNTER(SEJIN_T_PERIOD) - 1;
    DetectionInfo->Bit1PulseCounter.Max = 
        DetectionInfo->Bit1PulseCounter.Min + 2;

    DetectionInfo->Bit1SpaceCounter.Min = 
        LOCAL_LENGTH_2_COUNTER(SEJIN_T_PERIOD) - 1;
    DetectionInfo->Bit1SpaceCounter.Max = 
        DetectionInfo->Bit1SpaceCounter.Min + 2;



    _4ppmCodeInitializeDetection();
     
     
     
    DetectionInfo->InactivityCounter.Min =     
        LOCAL_LENGTH_2_COUNTER(SEJIN_INACTIVITY_LENGTH) ;
	
    DetectionInfo->InactivityCounter.Max = 
        DetectionInfo->InactivityCounter.Min + 1;


    DetectionInfo->CodeLength[0] = SEJIN_NUM_CODE_BITS;
    DetectionInfo->CodeLength[1] = 0;

    DetectionInfo->MsbIsFirst = TRUE ; 

    DetectionData->KeyPressed               = FALSE;

    NscCirResetDetectionData(DetectionData);
    DetectionInfo->NumberOfChips            = 0;

    NscCirResetDetectionVariables(DetectionInfo);

}


#define NO_NEW_DATA     ((BYTE)0xFF)


/************************************************************************
 * Syntax	: _4ppmCodeInitializeDetection                          *
 *                                                                     	*
 * Description	: This function is used in initializing the data 	*
 *                structures  which are used in 4PPM decoding		*
 *                                                                     	*
 * Parameters	: None.		                                        *
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

int _4ppmCodeInitializeDetection()
{
    BYTE  nSamples = SEJIN_NO_SAMPLES;   
    DWORD tPeriodLength = SEJIN_T_PERIOD;

    DetectionInfo->AtomSize = 2;

        // In 38KHz 2-3 modulation pulses are added to the header
        // In 56KHz 5   modulation pulses are added to the header
    DetectionInfo->HeaderPulseCounter.Max += 
        LOCAL_LENGTH_2_COUNTER(1000000 * 5 /SEJIN_MOD_FREQ_HZ);
    
    
        // In 38KHz 2-3 modulation pulses are added to the pulse
        // In 56KHz 1-2 modulation pulese are added to the pulse
	
    DetectionInfo->DataPulseCounter.Min = 
        LOCAL_LENGTH_2_COUNTER(SEJIN_T_PERIOD) - 1;
	
    DetectionInfo->DataPulseCounter.Max = 
        LOCAL_LENGTH_2_COUNTER((2*SEJIN_T_PERIOD) + (1000000 * 3 / SEJIN_MOD_FREQ_HZ)) + 1;




        // In 38KHz 2-3 modulation pulses are substracted to the pulse
        // In 56KHz 1-2 modulation pulese are substracted to the pulse
    DetectionInfo->DataSpaceCounter.Min = 
        LOCAL_LENGTH_2_COUNTER( SEJIN_T_PERIOD - (1000000 * 3 / SEJIN_MOD_FREQ_HZ)) - 1;
    
    DetectionInfo->DataSpaceCounter.Max = 
        LOCAL_LENGTH_2_COUNTER(6 * SEJIN_T_PERIOD) + 1;

    DetectionInfo->SpaceCounter.Min = 2 + 
        DetectionInfo->DataSpaceCounter.Max;
    
    DetectionInfo->SpaceCounter.Max = 2 +
        LOCAL_LENGTH_2_COUNTER(80000);

  return 1;
}

#endif /* _4PPM_CODING_ENABLED */


/************************************************************************
 * Syntax	: _4ppmCodeDetectionHandleByte                          *
 *                                                                     	*
 * Description	:  This function is used in decoding the 4ppm data	*
 *                                                                     	*
 * Parameters	: DetectionInfo	  					*
 *		    - holds the information about the data		*
 *		  DetectionData					        *
 *		    - holds the detection data				*
 *		  NewByte						*
 *		    - holds the data to be decoded			*
 *		  *NewBitPair						*
 *		    - holds the  decoded data				*
 *		  *DetectionEvent 					*
 *		    - holds the event on detection of data		*
 *		  *ResetDetection    					*
 *		    - set when there is detection reset encountered	*
 *                                                                     	*
 * Return 	: None.	               					*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void _4ppmCodeDetectionHandleByte(
    PDETECTION_INFO DetectionInfo,
    PDETECTION_DATA DetectionData,
    BYTE            NewByte,
    BYTE            *NewBitPair,
    DETECTION_EVENT *DetectionEvent,
    int         *ResetDetection
)
{
    DWORD pulseChips;

    switch (DetectionInfo->SubState) {
    case DETECTION_SUBSTATE_WAIT_4_PULSE:
            // Round calculation.
        pulseChips = (DetectionInfo->EnergyCounter + 
                      DetectionInfo->NumberOfSamplesInTPeriod / 2) /
                     DetectionInfo->NumberOfSamplesInTPeriod;

      // printk(" PC-%d ", pulseChips);
        switch (pulseChips) {
        case 1: // it is a single pulse
            DetectionInfo->NumberOfChips++;
            break;

        case 2: // it is a double pulse - 11 followed by 00
            DetectionInfo->NumberOfChips += 2;
            *NewBitPair = 0;
            break;

        case 3: // it is a new header after the end of the data
                // The check for CodeLength + AtomSize is due to the fact
                // that the space after the data till the header was
                // thought to be AtomSize more bits.
            if (DetectionData->NumberOfBits == 
                (DetectionInfo->CodeLength[0] + DetectionInfo->AtomSize)) 
		{
                    // Fix NumberOfBits to actual value
                DetectionData->NumberOfBits = DetectionInfo->CodeLength[0];
                    // End of data
                *DetectionEvent = DETECTION_KEY_PRESSED_EVENT;
                DetectionData->KeyPressed = TRUE;
                    // Reset all detection variables
                NscCirResetDetectionVariables(DetectionInfo);
                    // adjust the state machine - header is already found
                DetectionInfo->State = DETECTION_STATE_WAIT_4_DATA;
            } 
	    else 
	    {
                    // Too short or too long space is detected
                *ResetDetection = TRUE;
            }
            break;

        default:
                // Too short or too long space is detected
            *ResetDetection = TRUE;
            break;
        }
        DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_SPACE;
        break;

    case DETECTION_SUBSTATE_WAIT_4_SPACE:
        if (IS_NO_ENERGY_IN_RANGE(DataSpaceCounter)) 
	{
            int spaceChips;
                
                // Verify that the data is reset before putting new one
            if (DetectionData->KeyPressed && (DetectionData->NumberOfBits == 0))
                NscCirResetDetectionData(DetectionData);

                // Round down
            spaceChips = (DetectionInfo->NoEnergyCounter + 
                          DetectionInfo->NumberOfSamplesInTPeriod -1) / 
                            DetectionInfo->NumberOfSamplesInTPeriod;

	if(! IS_IN_RANGE(spaceChips,1,6))	
		printk("\n out of range : spaceChips \n");

            DetectionInfo->NumberOfChips += (BYTE)spaceChips;
            
            *NewBitPair = (BYTE)(DetectionInfo->NumberOfChips % 4);
   
  
   nsc4ppmhit =1;
            DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;

        }
        else 
	{	
                // Space suspected
            if (DetectionData->NumberOfBits == DetectionInfo->CodeLength[0]) 
	    {
                    // End of data
                *DetectionEvent = DETECTION_KEY_PRESSED_EVENT;
                DetectionData->KeyPressed = TRUE;

            } 
            *ResetDetection = TRUE;
        }
        break;
        
    default:
            // Must not get here
        break;
    }
}


#define NO_NEW_DATA     ((BYTE)0xFF)


/************************************************************************
 * Syntax	: NscIrRawDataSejin		                        *
 *                                                                     	*
 * Description	:  this function is used to detecting the sejin data	*
 *                                                                     	*
 * Parameters	: NewByte 				                *
 *		    - new data to be detected				*
 *                                                                     	*
 * Return 	: Returns the event detected.				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

int NscIrRawDataSejin(BYTE NewByte)
{
	int event;
  	event = NscCirDetectionHandleByte( NewByte );
	return event; 
}



/************************************************************************
 * Syntax	: NscCirDetectionHandleByte		                *
 *                                                                     	*
 * Description	: This function is used to detecting the sejin data     *
 *                which is encoded using 4PPM protocol                  *
 *                                                                     	*
 * Parameters	: NewByte 				                *
 *		    - new data to be detected				*
 *                                                                     	*
 * Return 	: Returns the event detected.				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

static DETECTION_EVENT NscCirDetectionHandleByte( BYTE  NewByte)
{
    DETECTION_EVENT detectionEvent = DETECTION_NO_EVENT;
    BOOLEAN         isEnergy       = DETECTION_IS_ENERGY_BYTE(NewByte);
    BOOLEAN         isNoEnergy     = !isEnergy;
    BYTE            counter        = DETECTION_GET_RUN_LENGTH_COUNTER(NewByte);
    BOOLEAN         resetDetection = FALSE;
    BOOLEAN         resetData      = TRUE;
    BYTE            newData        = NO_NEW_DATA;
    int             index;

    #define RESET_DETECTION(str)  { resetDetection =TRUE; if(NewByte!=0xff) reset_counter++; }

 
    if (isNoEnergy) 
    {
        	   		// Increment no energy counter
        DetectionInfo->NoEnergyCounter += counter;
          			// Reset energy counter
        DetectionInfo->EnergyCounter = 0;
        
        if(DetectionInfo->NoEnergyCounter > DetectionInfo->InactivityCounter.Max) 
        {
        		            // Notify silence
            detectionEvent = DETECTION_SILENCE_EVENT;
                // Reset detection in any case
            RESET_DETECTION("Inactivity is detected");
        } 
        else 
      	if (DetectionInfo->NoEnergyCounter > DetectionInfo->SpaceCounter.Max) 
     	{
            	if (DetectionData->KeyPressed) 
		{
	                    // Notify key release 
                	detectionEvent = DETECTION_KEY_RELEASED_EVENT;
                	DetectionData->KeyPressed = FALSE;
            		}
               	 // Reset detection in any case
            	RESET_DETECTION("Too much silence is detected");
            	NscCirResetDetectionData(DetectionData);
            	resetData = FALSE;
        	}
    }
    else 
    {
        		  	 // Increment energy counter
        DetectionInfo->EnergyCounter += counter;
          			 // Reset no energy counter
        DetectionInfo->NoEnergyCounter = 0;
    }


    
    if (detectionEvent == DETECTION_SILENCE_EVENT)
        goto ResetDetection;

    
    
    
    StateMachine:
    
switch (DetectionInfo->State) 
{
            // We're looking for the beginning of the code which is
            // preluded by a header. 
  case DETECTION_STATE_WAIT_4_HEADER:

      switch (DetectionInfo->SubState) 
	{
	        
               case DETECTION_SUBSTATE_WAIT_4_PULSE:
	    {
                    if (isEnergy) 
	     {
                        if (IS_ENERGY_IN_RANGE(HeaderPulseCounter)) 
	         {
                        if (DetectionInfo->HeaderSpaceCounter.Max == 0) 
			{                // No space in header
                           DetectionInfo->State = DETECTION_STATE_WAIT_4_DATA;
                        }
                      DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_SPACE;
                  }
                  else 
	         {
                       // Too short or too long header pulse is detected
              RESET_DETECTION("Too short or too long header pulse is detected");
                  }
             }
	    }
                    break;




                case DETECTION_SUBSTATE_WAIT_4_SPACE:
	    {   

                    if (isNoEnergy) 
	     {
                        if (IS_NO_ENERGY_IN_RANGE(HeaderSpaceCounter)) 
	         {
                            DetectionInfo->State    = DETECTION_STATE_WAIT_4_DATA;
                            DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;

			 DetectionInfo->NumberOfChips =0;
			 DetectionInfo->NoEnergyCounter =0;
			 DetectionInfo->EnergyCounter =0;
                        }
                        else {
                                // Too short or too long pause is detected
                            RESET_DETECTION("Too short or too long header space is detected");
                        }
                    }
                    else 
	     {
                            // Unexpected energy is detected
                        RESET_DETECTION("Unexpected energy is detected");
                    }
	    }
                    break;


                default:
			printk(" Invalid det Substate ");
                    break;

            }
            break;




        case DETECTION_STATE_WAIT_4_DATA:
            switch (DetectionInfo->SubState) 
            {

	     
                case DETECTION_SUBSTATE_WAIT_4_PULSE:
	   {
                    if (isEnergy) 
	     {
	         _4ppmCodeDetectionHandleByte(
		      DetectionInfo, 
                                    DetectionData, 
                                    NewByte, 
                                    &newData,
                                    &detectionEvent,
                                    &resetDetection

	         );
              } // End of if energy detected
                  else 
	     {
                        if (DetectionInfo->HeaderPulseCounter.Max == 0 && 
                            (DetectionData->NumberOfBits == 0                            ||
                             DetectionData->NumberOfBits == DetectionInfo->CodeLength[0] ||
                             DetectionData->NumberOfBits == DetectionInfo->CodeLength[1])
                           )
                        {
                                // Here number of bits should be 0, check this assumption
		printk("Ignoring silence before the first pulse (no header)");
                        }
                        else 
	         {
                                // Unexpected no energy is detected
                            RESET_DETECTION("Unexpected no energy is detected");
                        }
                    }
	   }
                    break;



                case DETECTION_SUBSTATE_WAIT_4_SPACE:
              if (isNoEnergy) 
	     {
                        if (IS_NO_ENERGY_IN_RANGE(SpaceCounter)) 
	         {
#define IR_CODING_SHIFT 0
                     if (DetectionInfo->CodingType == IR_CODING_SHIFT) 
	             {
		  
                      if (DetectionInfo->Bit0Candidate) 
		      {
                                    newData = NEW_BIT_0;
                       }
                     }

                     detectionEvent = DETECTION_KEY_PRESSED_EVENT;
                     DetectionData->KeyPressed = TRUE;


                            NscCirResetDetectionVariables(DetectionInfo);
                        }
                        else 
	         	{

		  	_4ppmCodeDetectionHandleByte(
 			DetectionInfo, 
                                        DetectionData,
                                        NewByte, 
                                        &newData,
                                        &detectionEvent,
                                        &resetDetection
		  			);
                        }
                    }
                    else 
	     	{
                        if (DetectionInfo->DelimiterSpaceSuspected) 
	         {
                                // Change state end redo the state machine
                            DetectionInfo->State = DetectionInfo->InitialState;
                   DetectionInfo->SubState = DETECTION_SUBSTATE_WAIT_4_PULSE;
                            goto StateMachine;
                        }
                        else 
	         {
                            // Unexpected energy is detected, reset detection
                            RESET_DETECTION("Unexpected energy is detected");
                        }
                    }
                    break;


                default:
                    break;

            }
            break;


        default:
            break;
    }    // End of switch of the detection state




    if (newData != NO_NEW_DATA) 
    {
        int numberOfBits ;
        int atomSize     = DetectionInfo->AtomSize;
        numberOfBits = DetectionData->NumberOfBits;


        if (numberOfBits + atomSize <= MAX_NUMBER_OF_DATA_BITS) 
        {
            DWORD mask;
            int   bitInDword = numberOfBits % N_BITS_IN(DWORD);
            int   dwordNo    = numberOfBits / N_BITS_IN(DWORD);     
            int   index;

            if (DetectionInfo->MsbIsFirst) 
            {
                mask  = newData << (N_BITS_IN(DWORD) - atomSize - bitInDword);
                index = MAX_NUMBER_OF_DATA_DWORDS - 1 - dwordNo;
            }
            else 
            {
                mask  = newData << bitInDword;
                index = dwordNo;
            }

            DetectionData->Data[index] |= mask;

            DetectionData->NumberOfBits += (BYTE)atomSize;

        if(DetectionData->NumberOfBits >= 34)
		{
			SejinAnalyzeData();
			resetDetection =1;
		}
        }
        else 
        {
                // Too many data bits are detected
            RESET_DETECTION("Too many data bits are detected");
        }
    }
    

    ResetDetection:
    
    if (resetDetection) 
    {
        NscCirResetDetectionVariables(DetectionInfo);
        if (resetData)
            NscCirResetDetectionData(DetectionData);
    }

    if(reset_counter >= 450) 
	{    reset_counter =0;
		return 0xfff;
	}

    return detectionEvent;

    #undef RESET_DETECTION
}



/************************************************************************
 * Syntax	: SejinAnalyzeData                          		*
 *                                                                     	*
 * Description	: This function is used to analyse the data for Sejin   *
 *                protocol						*
 *                                                                     	*
 * Parameters	: None.		                                        *
 *                                                                     	*
 * Return 	: Returns the valid code.				*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

BOOLEAN SejinAnalyzeData()
{
    BOOLEAN              validCode          = FALSE;
    BOOLEAN              resetSejinAnalysis = FALSE;
    
    
    for(;;)
    {
        SEJIN_CODE                sejinCode;
        int                       i;
        int                       checkSumCalc, checkSumFound;
        BOOLEAN                   deviceFound = FALSE;

        if (DetectionData->NumberOfBits != DetectionInfo->CodeLength[0]) 
	{
	 
            resetSejinAnalysis = TRUE;
	    break;
        }
	reset_counter =0;

        for (i = 0; i < SEJIN_NUM_OF_CODE_DWORDS; i++) 
	{
            sejinCode.Code[i] = DetectionData->Data[i];
        }



            // Start Bits
        if (sejinCode.SejinCommonFields.StartBits != SEJIN_START_BITS_VALUE) 
	{
	    resetSejinAnalysis = TRUE;
	    break;
        }
        
            // Check Sum
        checkSumCalc  = sejinCode.SejinNibbleFields.Nibble0;
        checkSumCalc += sejinCode.SejinNibbleFields.Nibble1;
        checkSumCalc += sejinCode.SejinNibbleFields.Nibble2;
        checkSumCalc += sejinCode.SejinNibbleFields.Nibble3;
        checkSumCalc += sejinCode.SejinNibbleFields.Nibble4;
        checkSumCalc += sejinCode.SejinNibbleFields.Nibble5;
        checkSumCalc += sejinCode.SejinNibbleFields.Nibble6;
        checkSumCalc &= SEJIN_CHECKSUM_MASK;

        checkSumFound = sejinCode.SejinCommonFields.Checksum1 +
                        (sejinCode.SejinCommonFields.Checksum2 << 2);

        if (checkSumCalc != checkSumFound) 
	{
            resetSejinAnalysis = TRUE;
	    break;
        }
	else
	{ 
	
	 NscSejinData.deviceId = sejinCode.SejinCommonFields.DeviceID;
	 NscSejinData.button   = sejinCode.SejinNibbleFields.Nibble1 & 0x07;
	 NscSejinData.modkey	= ((sejinCode.SejinNibbleFields.Nibble2&0x0f) << 4) | sejinCode.SejinNibbleFields.Nibble3 &0x0f ;
	 NscSejinData.data	= ((sejinCode.SejinNibbleFields.Nibble4&0x0f) << 4) | sejinCode.SejinNibbleFields.Nibble5 &0x0f ;

         NscSejinHandleData();
        
         break;
	}


    }
    return validCode;
}


/************************************************************************
 * Syntax	: NscSejinHandleData                          		*
 *                                                                     	*
 * Description	: This function is used to generate the keycode for  	*
 *                some keys to simulates the standard keyboard		*
 *                                                                     	*
 * Parameters	: None.		                                        *
 *                                                                     	*
 * Return 	: None.							*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/

void NscSejinHandleData()
{

static BYTE sjn_shift =0;
static BYTE sjn_alt =0;
static BYTE sjn_ctrl =0;

   switch(NscSejinData.deviceId)
  {
   case SJN_KBD : 
    {
       switch(NscSejinData.data)
	{
	case 0x07: 
	case 0x37:
	case 0x5f:
	case 0x71:
	case 0x21:  // ignore scan code
		break;
	case 0x73:
	case 0x74:
	case 0x7d:
	handle_scancode(SjnMap2StdKey[NscSejinData.data],1);
	handle_scancode(SjnMap2StdKey[NscSejinData.data],0);
	break;
	default :
       {	
  	if( (NscSejinData.modkey & 0x80) != 0x80)
	{

	if(NscSejinData.modkey & 0x30) handle_scancode(42,1);
	if(NscSejinData.modkey & 0x0c) handle_scancode(56,1);
	if(NscSejinData.modkey & 0x03) handle_scancode(29,1);

	handle_scancode(SjnMap2StdKey[NscSejinData.data],1);

	if(NscSejinData.modkey & 0x30) handle_scancode(42,0);
	if(NscSejinData.modkey & 0x0c) handle_scancode(56,0);
	if(NscSejinData.modkey & 0x03) handle_scancode(29,0);

	}

	} break;

	}
	break;
	}
	case SJN_MOUSE: 
      enque_mouse(NscSejinData.button & 0x1, NscSejinData.button & 0x02,NscSejinData.modkey,NscSejinData.data);
	break;
	case SJN_RMT:break;
	default : break;
}

}


/************************************************************************
 * Syntax	: enque_mouse                          			*
 *                                                                     	*
 * Description	: This function is used to covert the mouse data so that*
 *	          it can be fed to standard mouse driver		*
 *                                                                     	*
 * Parameters	: l_but		                                        *
 *		    - Status of mouse left button 			*
 *		  r_but                                                 *
 *		    - Status of mouse right button.			*
 *		  x_pos							*
 *		    - Mouse pointer value on the x-axis.		*
 *		  y_pos							*
 *		    - Mouse pointer value on the y-axis.		*
 *                                                                     	*
 * Return 	: None.							*
 *                                                                     	*
 * Comment	: None.						   	*
 *                                                                     	*
 ************************************************************************/


static void enque_mouse(BYTE l_but,BYTE r_but,BYTE x_pos,BYTE y_pos)
{
	char delta_x =0;
	char delta_y =0;
	char x_sign=0;
	char y_sign =0;
	BYTE temp;
	
	if(x_pos >128) {  delta_x = -(x_pos -128)/10;    x_sign =1; }
	else  { delta_x =  (x_pos /2 ) +1;    x_sign =0; }
	if(y_pos >128) {  delta_y = -(y_pos -128)/10;    y_sign =1; }
	else  { delta_y =  (y_pos /2 ) +1;    y_sign =0; }
	temp =0x07;
	if(l_but) temp&=0xfb;
	if(r_but) temp&=0xfe;

	ansc_mv_mouse ( delta_x, delta_y ,temp);

}





