/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 *    NSC Analog Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 */
#ifndef __ANLG_VIDEO_H__
# define __ANLG_VIDEO_H__

/* Define this if we shouldn't open the devices NOIO 
 * V4L2 specifies two kinds of open for any device. The first is a normal
 * open and the second being a NOIO open. The current National Video
 * driver does not support NOIO opens. Define this to prevent NOIO opens
 */
//#define NO_NOIO

/* define NO_VBI if VBI is not to be used */

/* System Includes */
#include <unistd.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <errno.h>
#include<signal.h>
#include<unistd.h>

/* Includes for using the video4linux2 API */
#include<linux/fs.h>
#include<linux/kernel.h>
#include<linux/videodev.h>

/* include for the geode_v V4l2 driver */
#include <linux/ioctl_geode_v.h>

/* Video Input types defines */
/* Use these values for the VMW_AVS_SetCurrentInputFunction */
#define VIDEO_INPUT_TUNER		0	
#define VIDEO_INPUT_COMPOSITE		1
#define VIDEO_INPUT_SVIDEO		2
#define VIDEO_INPUT_MPEG_DECODER	3

/* Filename defines */
#define DEV_FILE_VIDEO "/dev/video"
#define DEV_FILE_VBI   "/dev/vbi"
#define DEV_FILE_VOUT  "/dev/vout"

/* status */
#define VIDEO_OPENED 		2	
#define VIDEO_OPENED_NOIO 	4
#define VBI_OPENED 		8
#define VBI_OPENED_NOIO 	16
#define VOUT_OPENED 	 	32
#define VOUT_OPENED_NOIO 	64
#define VBI_SIGNAL_REGISTERED   128

/* Error codes */
#define VMW_AVSE_OK             0  /* Everything is OK  */
#define VMW_AVSE_INVSTAT	1  /* Invalid status 	*/
#define VMW_AVSE_OPEN		2  /* Error opening  		*/	
#define VMW_AVSE_IOCTL		4  /* ioctl error    		*/
#define VMW_AVSE_READ		8  /* Read error     		*/
#define VMW_AVSE_WRITE		16 /* Write error     		*/
#define VMW_AVSE_OOPS		32 /* OOPS.. this shouldn't happen */
#define VMW_AVSE_INIT		64 /* Error initializing/allocating mem */

#define ENUM_PIXEL_FMT	1
#define ENUM_FRBUFF_FMT	2

/* Defines for video controls */
#define AVS_CONTROL_BRIGHT 	V4L2_CID_BRIGHTNESS
#define AVS_CONTROL_HUE		V4L2_CID_HUE
#define AVS_CONTROL_SAT		V4L2_CID_SATURATION
#define AVS_CONTROL_CONTRAST	V4L2_CID_CONTRAST
#define AVS_CONTROL_WHITENESS	V4L2_CID_WHITENESS

/* The Signal number to use, use either SIGUSR1 or SIGUSR2 */
/* The VBI driver can signal us when it has enough of VBI data. The Signal
 * can be either SIGUSR1 or SIGUSR2
 */
#define VBI_SIGNAL_NUM	SIGUSR1

#define MIN(a,b) ((a)<(b))?(a):(b)

/* Macros for debugging */

/************************************************************/
#ifdef DEBUG
#define DBG(args...) printf(__FILE__":%d:" args, __LINE__)
#else
#define DBG(args...)
#endif

#ifdef MESSAGE
#define MSG(args...) printf(__FILE__":%d:" args, __LINE__)
#else
#define MSG(args...)
#endif
/**************************************************************/

#define DO_IOCTL(fd,command,ptr) (ioctl(fd,command,ptr)?VMW_AVSE_IOCTL:VMW_AVSE_OK)

/* Typedefs to map the v4l2 names to the new names in the API */
typedef struct v4l2_capability VIDEO_CAPABILITY;
typedef struct v4l2_clip VIDEO_CLIP;
typedef struct v4l2_window VIDEO_WINDOW;
typedef struct v4l2_capture VIDEO_CAPTURE;
typedef struct v4l2_buffer VIDEO_BUFFER;
typedef struct v4l2_mmap VIDEO_MMAP;
typedef struct v4l2_key VIDEO_KEY ;
typedef struct v4l2_fmtdesc VIDEO_FMTDESC; 
typedef struct v4l2_format VIDEO_FORMAT;
typedef struct v4l2_vbi_format VBI_FORMAT;
typedef struct v4l2_pix_format PIX_FORMAT;
typedef struct v4l2_compression VIDEO_COMPRESSION;
typedef struct v4l2_standard VIDEO_STANDARD;
typedef struct v4l2_enumstd ENUM_STANDARD;
typedef struct v4l2_control VIDEO_CONTROL;

/* video input structures */
typedef struct v4l2_input VIDEO_INPUT;

/* tuner structures */
typedef struct v4l2_tuner VIDEO_TUNER; 

/* video audio structures */
typedef struct v4l2_audio VIDEO_AUDIO;

/* these structures are defined in ioctl_geode_v.h */
typedef overlay_info_t OVERLAY_INFO;
typedef alpha_blend_info_t ALPHA_BLEND_INFO;
typedef __u32 u32;
/***********************************************************/

typedef enum VMW_OVL_ModeType {
	VMW_OVL_COLOR_KEY = 1,
        VMW_OVL_CHROMA_KEY,
        VMW_OVL_COLOR_KEY_INSIDE_ALPHA_AND_VIDEO_OUTSIDE,
        VMW_OVL_CHROMA_KEY_INSIDE_ALPHA_AND_GRAPHICS_OUTSIDE,
        VMW_OVL_ALWAYS_ON
} VMW_OVL_ModeType;



/* The callback function for VBI */
/* The callback function returns a void and takes an integer. Even though
 * the user specifies the number of bytes to be captured after which the
 * callback should be called, more data might be actually available. The
 * integer tells the user the number of bytes actually available in the
 * buffer. The second parameter is a void pointer. The user can decide what 
 * this will actually point to. It is set at initialization time.
*/
typedef void (*pVBICallback)(int, void * );


/* We keep the state of all the open devices in this structure. This is
 * maintained globally and accessed by all the functions.
 */
struct VMW_video_struct
{
	int fd ;
	int fd_noio;  /* descriptor to use for NOIO calls */
	int fd_vbi;   /* descriptor for vbi device */
	int fd_vbi_noio;
	
	int fd_vout ;      /* descriptor for video out (not used yet) */
	int fd_vout_noio ; /* descriptor for video out (not used yet) */
	
	u32 status; 

	/* to be used by VBI */
	void * callback_data;    /* the user will set this */
	pVBICallback	callback;/* the callback function */
	unsigned char * buffer;  /* pointer to the user buffer */
	unsigned long buffer_length; /* length of user buffer */
	unsigned long vbi_read_buffer_length;
};

typedef struct VMW_video_struct VMW_VIDEO;
/*
 *	Declarations of the actual API functions 
 */

/* C++ compilers shouldn't complain */
#ifdef __cplusplus
extern "C" {
#endif

/* 
 *  Description : Initialize the the library. ( Opens all required devices )
 *  Params      : None
 *  Returns     : Zero on success, error code on failure.
 */
u32 VMW_AVS_VideoInit(); 

/*
   Description : This function obtains the capability information of the
		 device.
   Params      : Pointer to VIDEO_CAPABILITY structure
   Returns     : Zero on success, error code on failure.
*/   		 
u32 VMW_AVS_GetVideoCapabilities(VIDEO_CAPABILITY *pvidcap );

/*
   Description : Enumerate the formats of either the frame-buffer or the
                 pixel format.
   Params      :1) Chose between ENUM_PIXEL_FMT or ENUM_FRBUFF_FMT 
   		2) Pointer to VIDEO_FMTDESC structure.		 
   Returns     : Zero on success, error code on failure.		
*/
u32 VMW_AVS_EnumFormat(int type, VIDEO_FMTDESC * pvidfmtdesc );

/*
   Description : Retrieve the captured image format
   Params      : Pointer to VIDEO_FORMAT structure 
   Returns     : Zero on success, error code on failure.
*/
u32 VMW_AVC_GetCaptureFormat(VIDEO_FORMAT *pformat );

/*
   Description : Set the capture format.
   Params      : Pointer to VIDEO_FORMAT structure filled by user
   Returns     : Zero on success, error code on failure
*/
u32 VWM_AVC_SetCaptureFormat(VIDEO_FORMAT * pformat );

/*
   Description : Get the compression format
   Params      : Pointer to VIDEO_COMPRESSION structure
   Returns     : Zero on success, error code on failure
*/
u32 VMW_AVC_GetCompFormat(VIDEO_COMPRESSION *pvidcomp );

/* 
   Description : Set the compression format
   Params      : Pointer to VIDEO_COMPRESSION structure.
   Returns     : Zero on success, error code on failure.
*/
u32 VWM_AVC_SetCompFormat(VIDEO_COMPRESSION *pvidcomp ); 

/*
  Description  : Capture an image into memory ( Not supported by driver)
  Params       : 1) Pointer to buffer
  		 2) Size of buffer
  Returns      : Non zero value on success. ( number of bytes read)		 
*/
u32 VWM_AVS_Read(char * buffer, size_t count ); 

/*
  Description  : Retrieve the properties of an input into a VIDEO_INPUT
                 structure. The user user must allocate and fill in the
		 index field of the structure.
  Params       : Pointer to VIDEO_INPUT structure.
  Returns      : Zero on success, non-zero error code on failure.
*/
u32 VMW_AVS_EnumVideoInput( VIDEO_INPUT *pvidinp );

/*
  Description  : Get the index of the current video input
  Params       : Pointer to integer
  Returns      : Zero on success, non-zero error code on failure
*/
u32 VMW_AVS_GetCurrentVideoInput(int * pindex );

/*
  Description : Set the current video input
  Params      : Pointer to integer holding the index to switch to.
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_AVS_SetCurrentVideoInput(int * pindex );

/*
  Description : Get the tuner properties (current input must have a tuner)
  Params      : Pointer to VIDEO_TUNER structure.
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVT_GetTunerCapabilities( VIDEO_TUNER *ptuner );

/*
  Description : Set the tuner capabilities (current input must have a
                tuner)
  Params      : Pointer to VIDEO_TUNER structure.
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVT_SetTunerCapabilities( VIDEO_TUNER *ptuner );

/* 
  Description : Retrieve the current channel number.
  Params      : Pointer to unsigned long
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVT_GetTunerFrequency(unsigned long * pfrequency );

/*
  Description : Set the channel number
  Params      : Pointer to unsigned long holding the channel number
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_AVT_SetTunerFrequency(unsigned long * pfrequency );

/*
  Description : Get the audio properties of the current input
  Params      : Pointer to VIDEO_AUDIO structure
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVA_GetAudioCapabilities(VIDEO_AUDIO *paudio_cap );

/*
  Description : Set the audio properties of the current input
  Params      : Pointer to VIDEO_AUDIO structure
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVA_SetAudioCapabilities(VIDEO_AUDIO *paudio_cap );

/* 
  Description : Set the Video window position and size
  Params      : 1) X position
                2) Y position
		3) Width
		4) Height
  Returns     : Zero on success, non-zero error code on failure		
*/
u32 VMW_AVS_SetVideoWindow(int x, int y, int width, int height );

/*
  Description : Get the properties of the VBI device
  Params      : Pointer to VIDEO_CAPABILITY structure
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_AVV_GetVBICapabilities(VIDEO_CAPABILITY *pvidcap );

/*
  Description : Retrieve the VBI format
  Params      : Pointer to VIDEO_FORMAT structure
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_AVV_GetVBIFormat(VIDEO_FORMAT *pformat );

/*
  Description : Set the VBI format
  Params      : Pointer to VIDEO_FORMAT structure
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_AVV_SetVBIFormat(VIDEO_FORMAT *pformat );

/* 
  Description : enable/disable DVIP
  Params      : flag to set enable or disable
  Returns     : Zero on success, non-zero on faliure
 */
u32 VMW_AVS_EnableDVIP(int enable);  

/*
  Description : Retrieve the differen video standards supported ( user must
                fill in the index field)
  Params      : Pointer to ENUM_STANDARD structure.
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVS_EnumVideoStandards(ENUM_STANDARD *penumstd );

/*
  Description : Query the current video standard
  Params      : Pointer to VIDEO_STANDARD structure
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVS_GetVideoStandard(VIDEO_STANDARD *pstandard );

/*
  Description : Set the current video standard
  Params      : Pointer to VIDEO_STANDARD structure
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_AVS_SetVideoStandard(VIDEO_STANDARD *pstandard );

/*
  Description : Query the tuner properties.
  Params      : Pointer to VIDEO_TUNER structure.
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_AVT_GetTuner(VIDEO_TUNER *ptuner);

/*
  Description : Set the tuner properties.
  Params      : Pointer to VIDEO_TUNER structure.
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_SetTuner(VIDEO_TUNER *ptuner);

/*
  Description : Shut down the Video system and close all devices
  Params      : None
  Returns     : void
*/
void VMW_AVS_VideoClose();


/*
  Description : Set the Video Overlay mode.
  Params      :1)  mode.
	One off:
	1. VMW_OVL_COLOR_KEY;
	2. VMW_OVL_CHROMA_KEY :
        3. VMW_OVL_COLOR_KEY_INSIDE_ALPHA_AND_VIDEO_OUTSIDE :
        4. VMW_OVL_CHROMA_KEY_INSIDE_ALPHA_AND_GRAPHICS_OUTSIDE :
        5. VMW_OVL_ALWAYS_ON:
  Returns     : Zero on success, non-zero error code on failure		   
*/
u32 VMW_OVL_OverlayMode(VMW_OVL_ModeType  mode);

/*
  Description : Set the color key and color mask.
  Params      :1) color-key
  	       2) color-mask
  Returns     : Zero on success, non-zero error code on failure		   
*/
u32 VMW_OVL_ColorKey( u32 colorkey, u32 colormask);
	
/*
  Description : Enable video overlay
  Returns     : Zero on success, non-zero error code on failure		   
*/
u32 VMW_OVL_EnableOverlay();

/*
  Description : Disable video overlay
  Params      : None
  Returns     : Zero on success, non-zero error code on failure.
*/
u32 VMW_OVL_DisableOverlay();

/*
  Description : Enable an alpha blend window ( max of three windows)
  Params      : 1) X position
  		2) Y position
		3) Width
		4) Height
		5) Alpha value (0 - 255)
		6) Autoincrement/decrement value
		7) Alpha window number (0 - 2)
  Returns     : Zero on success, non-zero error code on failure		
*/
u32 VMW_OVL_EnableAlphaBlendWindow( u32 startX, u32 startY,
				    u32 width , u32 height,
				    u32 alphaValue, char alphaIncValue,
				    u32 windowNumber);

/*
  Description : Disable an alpha window
  Params      : Window number (0 -2)
  Returns     : Zero on success, non-zero error code on failure
*/
u32 VMW_OVL_DisableAlphaWindow(u32 windowNumber); 

/*
  Description : Set the value of a video control.
  Params      : 1) the control type
  			AVS_CONTROL_BRIGHT
  			AVS_CONTROL_HUE
			AVS_CONTROL_SAT	
			AVS_CONTROL_CONTRAST	
			AVS_CONTROL_WHITENESS
  		2) the new value of the control
  Returns     : Zero on success, non zero error code on failure		
*/
u32 VMW_AVS_SetControl(u32 control, int value);
int VMW_AVS_GetControl(u32 control);

/*
  Description : Sets up the VBI callback function
  Params      : 1) Pointer to callback function
  		2) pointer to buffer
		3) Length of buffer
		4) User specified pointer
  Returns     : Zero on success, non-zero error code on failure		
*/
u32 VMW_AVV_SetupVBICallback(pVBICallback pCallBack,unsigned char * buffer,int
				length, void *userData);

/*
  Description : Sets the VIP buffer addressess
  Params      : 1) even address
  		2) odd address
		3) pitch
  Returns     : Zero on success, non-zero error code on failure
*/		
u32 VMW_OVL_SetVIPBufferInfo(unsigned long even, unsigned long odd,
        		     unsigned long pitch);

/*
  Description : Start or stop VBI capture
  Params      : 1 = start
                0 = stop
  Returns     : Zero on success, non-zero error code on failure		
*/
u32 VMW_AVV_EnableVBICapture(int enable);

#ifdef __cplusplus
}
#endif

#endif  /* __ANLG_VIDEO_H__ */





