/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 ********************************************************************************
 * File Name        : nscstrparam.cpp						*
 * 										*
 * Subsystem Name   : IA Software                                               *
 *                               						*
 * Module Name      : Param							*
 *										*
 * Author Name	    : Arun Kumar						*
 *										*
 * Language         : C++							*
 *										*
 * Date             : 12/03/00							*
 *										*
 * Description      : This module acts as a bridge between GUI application and  *
 *		      player.							*
 *										*
 ********************************************************************************/


#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include "nscparam.h"





/************************************************************************
 * Syntax	: Param()						*
 *									*
 * Description	: This function is to  initialises the member 	*
 *		  variables.					 	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

Param::Param()
{ 	Play = Pause= FF =Rev= Read=Write=Loop=Quit = FALSE;
          Stop = TRUE;
	  StatusCh =FALSE;
	  Mute    = FALSE;
	  Open 	=  FALSE;
	  Cont 	= FALSE; 
	  GetDir = FALSE;
	  
	  owner=0;
	  dirList =(DIR_LIST *) malloc(sizeof(DIR_LIST));
	  dirList->dir_record = (DIR_RECORD *) malloc(sizeof(DIR_RECORD)*200);    
	  		//	allocating for 200 elenements
          if((dirList <=0)||(dirList->dir_record <=0))
             printf(" \n\n Param :   Not able to allocate mem for dir \n\n");
	  dirList->NoElements =0;

	  ErFlg =0;
}


/************************************************************************
 * Syntax	: GetStatus()						*
 *									*
 * Description	: This function is to return the status of the key  	*
 *		  pressed.					 	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: StatusCh						*
 *			Status of the key pressed.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
bool Param::GetStatus()
{	
	return StatusCh;
}





/************************************************************************
 * Syntax	: Show()						*
 *									*
 * Description	: This function is to prints the variable for debugging.*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment 	: Used for testing and debugging.			*
 ************************************************************************/

Param::Show()
	{
	printf(" pl	FF	REV	PAU	STOp	LOOP	SCROL	But pre  Butval   \n");
	printf(" %x	%x	%x	%x	%x	%x	%d	<  %d - %d >%d\n",Play,FF,Rev,Pause,Stop,Loop,ScrolPt,ButPress,butvalue,ScrolPt); 
	}





/************************************************************************
 * Syntax	: GetKey()						*
 *									*
 * Description	: This function is to return the value of the key  	*
 *		  pressed.					 	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: ButPress						*
 *			Value of the key pressed.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetKey()
{ 	return(ButPress);
}



/************************************************************************
 * Syntax	: GetKeyWithValue((int *butvalue)			*
 *									*
 * Description	: This function is to return the status of the key  	*
 *		  pressed and allows for the next key pressed.		*
 *									*
 * Parameters	: *butvalue						*
 *			- Value of the button pressed.			*
 *									*
 * Returns	: Status of the key pressed.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetKeyWithValue(int *butvalue)
{   	int retvalue;	
		*butvalue =0;
		retvalue= ButPress;
		switch(ButPress)
		{
		case COPEN  	: if (Open) 	*butvalue=1; 	break;	
		case CPLAY	:if(Play) 	*butvalue=1; 	break;
		case CFF   	:if(FF)   	*butvalue=1; 	break;
		case CREV 	:if(Rev)  	*butvalue=1; 	break;
		case CPAUSE	:if(Pause) 	*butvalue=1; 	break;
		case CSTP	:if(Stop & ~Quit) 	*butvalue=1; 	break;
		case CLOOP  	:if(Loop) 	*butvalue=1; 	break;
		case CSCROLL	:	  *butvalue=ScrolPt; 	break;
		case CMUTE      :if(Mute) 	*butvalue=1;  	break;
		case CCNTSER    :if(Cont) 	*butvalue=1; 	break;
		case CGETDIR	:if(GetDir)     *butvalue=1;    break;
		case CQUT	:if(Quit & Stop)	*butvalue=1;
				 printf("In CQUIT of nscparam case\n");    break;
		default   	:		*butvalue=0;    break;
		
		};

	   StatusCh = FALSE;
            ButPress =0;
	   return retvalue;
}


/************************************************************************
 * Syntax	: GetServNameDot(char *srvname)				*
 *									*
 * Description	: This function is to get the server name.	  	*
 *									*
 * Parameters	: *srvname						*
 *			- Pointer to the server name.			*
 *									*
 * Returns	: 0 for fail, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetServNameDot(char *srvname)
{ 	if(strncpy(srvname,ServerName,40) == 0) 
		return 0; 
		return 1; 
}


/************************************************************************
 * Syntax	: SetServNameDot(char *srvname)				*
 *									*
 * Description	: This function is to set the server name.	  	*
 *									*
 * Parameters	: *srvname						*
 *			- pointer to the server name.			*
 *									*
 * Returns	: 0 for fail, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetServNameDot(char *srvname)
{	if(strncpy(ServerName,srvname,40) == 0 ) 
	return 0; 
	return 1;  
}




/************************************************************************
 * Syntax	: GetFlName(char * flname)				*
 *									*
 * Description	: This function is to get the file name.	  	*
 *									*
 * Parameters	: *flname						*
 *			- Buffer to get the file name.			*
 *									*
 * Returns	: 0 for fail, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetFlName(char * flname)
{  	if( strncpy(flname,FileName,40) == 0) return 0;  
	return 1; 
}


/************************************************************************
 * Syntax	: SetFlName(char * flname)				*
 *									*
 * Description	: This function is to set the server name.	  	*
 *									*
 * Parameters	: *srvname						*
 *			- pointer to the server name.			*
 *									*
 * Returns	: 0 for fail, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetFlName(char * flname)
{
  	if(strncpy(FileName,flname,80) == 0) return 0;  
	return 1; 
}



/************************************************************************
 * Syntax	: SetDirectory(char *str)				*
 *									*
 * Description	: This function sets a command to get the directory 	*
 *		  listing.						*
 *									*
 * Parameters	: *str							*
 *		    - Directory name to be set.				*
 *									*
 * Returns	: -ve value for fail, +ve value for sucess.		*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetDirectory(char *str)
{ if(GetDir  )  return -1;
	  if(StatusCh)  return -2;
	  StatusCh = TRUE;
	  Play=Stop=FF=Rev=Pause=Open=0;
	  ButPress = CGETDIR;  //OPEN;
	  strncpy(DirName,str,50);
	  return 1;
}




/************************************************************************
 * Syntax	: GetRmtCWD()						*
 *									*
 * Description	: This function returns the directory name of the 	*
 *		  server.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: DirName						*
 *		     - Directory name of the server.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
char * Param::GetRmtCWD()
{

	return DirName;		
}




/************************************************************************
 * Syntax	: GetDirectory()					*
 *									*
 * Description	: This function returns the pointer to the directory	*
 *		  list.							*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: dirList						*
 *		     - Pointer to the directory list.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

DIR_LIST *Param::GetDirectory()
{  
	return dirList;
		
}

/************************************************************************
 * Syntax	: SetFilename4Dir(int indx, char * str1)		*
 *									*
 * Description	: This function is called to sets the file name in   	*
 *		  the directory list at the specified index.		*
 *									*
 * Parameters	: index							*
 *		     - Index for the file.				*
 *		  *str1							*
 *		     - Pointer to the file name.			*
 *									*
 * Returns	: Suceess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetFilename4Dir(int indx, char * str1)
{	
	DIR_RECORD *cur_rec;
	cur_rec =  dirList->dir_record; 
	cur_rec = cur_rec  +  indx;
	strncpy(cur_rec->flname,str1,50);
	return 1;
	
}



/************************************************************************
 * Syntax	: SetFileType4Dir(int indx, char type)			*
 *									*
 * Description	: This function is called to set the file at the     	*
 *		  location offset of the index.				*
 *									*
 * Parameters	: index							*
 *		     - Index for the file.				*
 *		  type							*
 *		     - File type.					*
 *									*
 * Returns	: Suceess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetFileType4Dir(int indx,char type)
{ 	DIR_RECORD *cur_rec;
	cur_rec =  dirList->dir_record; 
	cur_rec = cur_rec  +  indx;
	cur_rec->type =type;  
	return 1;
}



/************************************************************************
 * Syntax	: GetFileType4Dir(int indx)				*
 *									*
 * Description	: This function is called to returns the filetype    	*
 *		  for a file which is located at the offset of the index*
 *		  in a directory list.					*
 *									*
 * Parameters	: index							*
 *		     - Index for the file.				*
 *									*
 * Returns	: Type of the file.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

char Param::GetFileType4Dir(int indx)
{	
	DIR_RECORD *cur_rec;
	cur_rec =  dirList->dir_record; 
	cur_rec = cur_rec  +  indx;
	return cur_rec->type ;  
}



/************************************************************************
 * Syntax	: GetFilename4Dir(int indx, char * str)			*
 *									*
 * Description	: This function is called to fetches the file name   	*
 *		  from the directory list.				*
 *									*
 * Parameters	: index							*
 *		     - Index for the file.				*
 *		  *str							*
 *		     - Contains the pointer to the filename on 		*
 *                     successful execution.				*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void Param::GetFilename4Dir(int indx ,char *str)
{	
	DIR_RECORD *cur_rec;
	cur_rec =  dirList->dir_record; 
	cur_rec = cur_rec  +  indx;
	strncpy(str,cur_rec->flname,50);
}


/************************************************************************
 * Syntax	: SetNoRec4Dir(int NoRec)				*
 *									*
 * Description	: This function sets the number of elements in the   *
 *		  directory.						*
 *									*
 * Parameters	: NoRec							*
 *		     - Number of records in the directory.		*
 *									*
 * Returns	: -1 for unsuccessful.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetNoRec4Dir(int NoRec)
{	
	if(NoRec <=0) return -1;
	   else  dirList->NoElements=NoRec;
	   return NoRec;
}


/************************************************************************
 * Syntax	: GetNoRec4Dir()				   	*
 *									*
 * Description	: This function returns the number of records in the *
 *		  directory list.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Number of records in the directory list.		*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetNoRec4Dir()
{	return dirList->NoElements;
}


/************************************************************************
 * Syntax	: SetOpen()						*
 *									*
 * Description	: This function is used to issue the open request.   *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: -ve values for failure.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetOpen()
{	if( Open )  return -1;
	if(StatusCh)  return -2;
	  StatusCh = TRUE;
	  Play=Stop=FF=Rev=Pause=0;
	  ButPress = COPEN;
	  Open = TRUE;
	  return 1;

}



/************************************************************************
 * Syntax	: SetCont()						*
 *									*
 * Description	: This function is used to issue the server connect  *
 *		  request.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess and -ve values for failure.		*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetCont()
{  	if((Play)||(FF)||(Rev)) return -1;
	if(StatusCh)    return -2;  
	StatusCh =TRUE;
	   Stop = FF = Rev = Pause = 0;
	   ButPress = CCNTSER;
           Cont =TRUE;    
	   return 1;
}



/************************************************************************
 * Syntax	: SetOwner()						*
 *									*
 * Description	: This function is used to set the status of the     *
 *                server connection.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

Param::SetOwner(int value)
{ 	owner = value;
	
}


/************************************************************************
 * Syntax	: GetOwner()						*
 *									*
 * Description	: This function is used to return the status of the  *
 *		  server connection to the client.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Status of the server connection to the client.	*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetOwner()
{
	if(StatusCh) return -2;
	return owner;
}
	




/************************************************************************
 * Syntax	: SetPlay()						*
 *									*
 * Description	: This function is used to issue the play request.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetPlay()
{ 	if (Play)       return -1; //originally -1
	if(StatusCh)    return -2;  
	StatusCh =TRUE;
	Stop = FF = Rev = Pause = 0;
	ButPress = CPLAY;
        Play =TRUE;    
	return 1;
}



/************************************************************************
 * Syntax	: SetPause()						*
 *									*
 * Description	: This function is used to issue the Pause command.  *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetPause()    // toggle
{ 	if(Stop)   return -1;
	if(StatusCh)    return -2;  
	StatusCh =TRUE;
	ButPress = CPAUSE;
	Pause =!Pause;  
	return 1;
} 


/************************************************************************
 * Syntax	: SetStop()						*
 *									*
 * Description	: This function is used to issue the Stop command.   *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
int Param::SetStop()
{ 	if((Stop) || (Pause))   return -1;
	if(StatusCh)    return -2;  StatusCh =TRUE;
	Open = Play = Pause= FF =Rev = FALSE;
	ButPress=CSTP;
	Stop=1;  return 1;
}

/************************************************************************
 * Syntax	: SetQuit()						*
 *									*
 * Description	: This function is used to issue the Quit command.   *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
int Param::SetQuit()
{
	if(StatusCh)
		return -2;
	StatusCh = TRUE;
	Open = Play = Pause= FF =Rev = FALSE;
	Stop = 1;
	Quit = 1;
	ButPress = CQUT;
	return 1;
}

/************************************************************************
 * Syntax	: SetFF()						*
 *									*
 * Description	: This function is used to issue the Fast forward    *
 *		  play request.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetFF()
{ 	if((FF) || (Stop) || (Pause) )  return -1;
	if(StatusCh)    return -2;  
	StatusCh =TRUE;
	Play = Pause= Rev=  FALSE;
	FF=1;
	ButPress=CFF;
	return 1;
}


/************************************************************************
 * Syntax	: SetRev()						*
 *									*
 * Description	: This function is used to issue the reverse play    *
 *		  request.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetRev()
{ 	if( (Rev)|| (Stop) || (Pause) ) return -1;
	if(StatusCh)    return -2;  
	StatusCh =TRUE;
	Play = Pause= FF = FALSE;
	Rev=1;
	ButPress = CREV; 
	return 1;
}



/************************************************************************
 * Syntax	: SetRead()						*
 *									*
 * Description	: This function is used to issue the Read request.   *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 on sucess.						*
 *                                                                      *
 * Comment	: Not used.						*
 ************************************************************************/

int Param::SetRead()    // toggle
{ 	if( !Stop) return -1;
	   Read =1;  return 1;
}


/************************************************************************
 * Syntax	: SetWrite()						*
 *									*
 * Description	: This function is used to issue the write request.  *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 on sucess.						*
 *                                                                      *
 * Comment	: Not Used.						*
 ************************************************************************/

int Param::SetWrite()     // toggle
{ 	if(!Stop) return -1;
	Write = 1;  return 1;
}



/************************************************************************
 * Syntax	: SetLoop()						*
 *									*
 * Description	: This function is used to set the loop button.      *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetLoop()   // toggle
{
	  if((Pause) || (!Play) || (Stop)) return -1;
           
	  if(StatusCh)    return -2;
	  StatusCh =TRUE;
	  ButPress = CLOOP;
	  Loop = ! Loop;  
	  return 1;
}



/************************************************************************
 * Syntax	: SetMute()						*
 *									*
 * Description	: This function is used to set the Mute flag.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetMute()
{
	 if(Pause)     return -1;
	 if(StatusCh) return -2; 
	 StatusCh = TRUE;
         Mute = !Mute;
	 ButPress=CMUTE;
	 return 1;

}


/************************************************************************
 * Syntax	: SetScrolPt(int scrol-loc)				*
 *									*
 * Description	: This function is used to set the scrol point 	*
 *		  variable.						*
 *									*
 * Parameters	: scrol_loc						*
 *			- Sets scrolPt variable.			*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::SetScrolpt(int scrol_loc)
{ 	if( (scrol_loc<0) || (scrol_loc>100) ) return -1;
	if(StatusCh)    return -2;  
	StatusCh =TRUE;
        ButPress = CSCROLL;
	ScrolPt =scrol_loc; 
	return 1;
}



/************************************************************************
 * Syntax	: GetScrolpt()						*
 *									*
 * Description	: This function is used to get the scrol point value.*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: ScrolPt						*
 *		     - Scrol point.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetScrolpt()
{ 	return ScrolPt;
}



/************************************************************************
 * Syntax	: GetScrolPoint()					*
 *									*
 * Description	: This function is used to return the file index.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: ScrolPoint						*
 *			- Index of the file.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::GetScrolPoint()
{ 	return ScrolPoint;
}



/************************************************************************
 * Syntax	: UpdateScrolpt(int )					*
 *									*
 * Description	: This function is used to update the scrol point 	*
 *		  variable.						*
 *									*
 * Parameters	: scrol_pt						*
 *			- Scrol point value.				*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int Param::UpdateScrolpt(int scrol_pt)         // update scroll pt as it plays
{  	
		if( (scrol_pt < 0) || (scrol_pt > 100) ) return -1;
	  ScrolPoint = scrol_pt; 
	  return 1;  

}


/************************************************************************
 * Syntax	: postErMsg( char *  )					*
 *									*
 * Description	: This function is used to copies error message from 	*
 *		  the generator to Param::ErrMsg..			*
 *									*
 * Parameters	: ermsg							*
 *		     - Error message.					*
 *									*
 * Returns	: 1 for sucess.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void Param::postErMsg( char * ermsg )
{
  strncpy(ErrMsg,ermsg,80);
  ErFlg=1;
  	
}



/************************************************************************
 * Syntax	: getErMsg()						*
 *									*
 * Description	: This function is used to get the error message.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Address of the latest error message.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

char * Param::getErMsg()
{  if(ErFlg) 
	{ErFlg =0;
	 return ErrMsg;
	}
   else return 0;
}



/************************************************************************
 * Syntax	: GetAudioL()						*
 *									*
 * Description	: This function returns the volume of the left 		*
 *		  speaker.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: audioVolL                                             *
 *			-volume for the left speaker.			*
 *									*
 * Comment	: None.							*
 ************************************************************************/

int Param::GetAudioL()
{	return audioVolL;
}




/************************************************************************
 * Syntax	: GetAudioR)						*
 *									*
 * Description	: This function returns the volume of the right		*
 *		  speaker.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: audioVolR                                             *
 *			-volume for the right speaker.			*
 *									*
 * Comment	: None.							*
 ************************************************************************/

int Param::GetAudioR()
{	return audioVolR;
}


/************************************************************************
 * Syntax	: SetAudioL(int val)					*
 *									*
 * Description	: This function is used to tuned the left speaker. 	*
 *									*
 * Parameters	: val							*
 *		   - volume to the left speaker.			*
 *									*
 * Returns	: Non -ve value on sucess.                              *
 *									*
 * Comment	: None.							*
 ************************************************************************/

int  Param::SetAudioL(int val)
{	
	if((Mute ) ||(Pause) )return -1;
	if(StatusCh)    return -2; 
	StatusCh =TRUE;
        ButPress = CADIOL;
	audioVolL = val;
	return 1;
}




/************************************************************************
 * Syntax	: SetAudioR(int val)					*
 *									*
 * Description	: This function is used to tuned the right speaker. 	*
 *									*
 * Parameters	: val							*
 *		   - volume to the left speaker.			*
 *									*
 * Returns	: Non -ve value on sucess.                              *
 *									*
 * Comment	: None.							*
 ************************************************************************/

int  Param::SetAudioR(int val)
{       if((Mute )||(Pause))  return -1;
	if(StatusCh)    return -2; 
	StatusCh =TRUE;
        ButPress = CADIOR;
	audioVolR = val;
	return 1;
}
