/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 

 ********************************************************************************
 * File Name        : nscplayer.cc						*
 * 										*
 * Project Name     : ASTB							*
 *										*
 * Subsystem Name   : Streamer		                                        *
 *                               						*
 * Module Name      : nscplayer							*
 *										*
 * Author Name	    : Arun Kumar						*
 *										*
 * Language         : C++							*
 *										*
 * Date             : 12/03/00							*
 *										*
 * Description      : This module is responsible for playing the MPEG stream.   *
 *										*
 ********************************************************************************/

#include <malloc.h>
#include "nscplayer.h"

static Param * pParamT ;
static ClientCom * connectionT ;

void *NSCNetBufWrite(NscPlayer *);



/************************************************************************
 * Syntax	: GetKey()						*
 *									*
 * Description	: This function returns the valid key value. 		*
 *									*
 * Parameters	: None	                                                *
 *									*
 * Returns	: Return key value for valid key and returns zero for   *
 *		  invalid key.						*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
		
 int  NscPlayer::GetKey()
{
   int temp;
       temp = KeyPress;	 
	switch ( KeyPress ) 
	{
		case PLAY_KEY: 	 	 	
		case FF_KEY: 		 
		case REV_KEY:		
		case STOP_KEY:	
		case PAUSE_KEY:
		case LOOP_KEY:
		case QUT_KEY:
			break;
		default: temp = 0; 	
			break;
	}
	KeyPress =0;
	return temp;
}
	

		
/************************************************************************
 * Syntax	: SetKey(int key)					*
 *									*
 * Description	: This function sets the keypressed variable to key 	*
 *		  pressed.						*
 *									*
 * Parameters	: key	                                                *
 *		   - Key value to be set.				*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::SetKey(int key )    
	// Gui thread usses this on Button press event;    
{
	KeyPress = key;
}


/************************************************************************
 * Syntax	: NscPlayer()						*
 *									*
 * Description	: This construcor,					*
 *		  - Creates sigma object , Initialises sigma card with  *
 *		    audio ON                                            *
 *		  - Initialises  all the class variables                *
 *									*
 * Parameters	: None	                                                *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: It is constractor for the NscPlayer class.		*
 ************************************************************************/

NscPlayer::NscPlayer()
{
	
	pSigma = new sigma();
	if( pSigma == 0)  
		printf(" Not able to create NscSigma object \n");
	pSigma->InitSigma();
	pSigma->AudioOnSigma();
	ActualPtrInFl = 0;
		
   	blocksPlayed = 0;
  	bytesProcessed = 0;
  	PSOB = 0;//Player start of byte
  	bytes2BProcessed = 0;
  	blocksToReceive = 0;
  	doFastMode = 0;
	KeyPress = 0;
	flag = 1;
	loop = 0;
	fileSize = 0;
	ActualPtrInFl = 0;
	skipMode = 0;
	AllocBuffer();
	CreateNetReader();
}


/************************************************************************
 * Syntax	: SetConnection( ClientCom *cnt )			*
 *									*
 * Description	: This function sets the connection value to the  	*
 *		  Clientcom object.					*
 *									*
 * Parameters	: *cnt 	                                                *
 *		     - Pointer to the Clientcom object.			*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::SetConnection( ClientCom *cnt )
{
	connection = cnt;
}


/************************************************************************
 * Syntax	: Setparam( Param *prm )				*
 *									*
 * Description	: This function sets the pParam to the Param object.	*
 *									*
 * Parameters	: *prm 	                                                *
 *		     - Pointer to the Param object.			*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::Setparam( Param *prm )
{
 	pParam = prm;	
}




/************************************************************************
 * Syntax	: ~NscPlayer()						*
 *									*
 * Description	: This distructor deletes the sigma object.		*
 *									*
 * Parameters	: None.	                                                *
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: It is a distructor.					*
 ************************************************************************/

NscPlayer::~NscPlayer()
{       
	for(int i = 0; i < NUMOFBUFFERS; i++)
 	  {	
		  if ( 0 != dataBuffer[i] ) 
			  delete dataBuffer[i];   
	  }
	delete pSigma;
}



/************************************************************************
 * Syntax	: AllocBuffer()						*
 *									*
 * Description	: This function allocates a set of  data buffers ,   *
 *                which are used for storing transport stream received. *
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::AllocBuffer()
{
	for(int i = 0; i < NUMOFBUFFERS; i++)
 		dataBuffer[i] = new char[BLOCK_SZ];   
}




/************************************************************************
 * Syntax	: ResetBuffers()					*
 *									*
 * Description	: This function resets the buffer.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: At present this  function has  no effect on system.	*
 ************************************************************************/

void NscPlayer::ResetBuffers()
{
	for(int i = 0; i < NUMOFBUFFERS ; i++ )
		{ 	bufferStatus[i] = BUFFER_WRITE; 
			bufferSize[i] = 0;
		}
}


/************************************************************************
 * Syntax	: GetStartIFrameFromServer() 				*
 *									*
 * Description	: This function get the address where the fast mode  	*
 *		  data is being called.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Used to receive data from the server.			*
 *                                                                      *
 * Comment	: called when seting the fast mode paly.		*
 ************************************************************************/

unsigned long NscPlayer::GetStartIFrameFromServer()
{
	unsigned long temp;
		//get from where the fast mode data is beng played
		
	connection->ReceiveData((char *)&temp, sizeof(temp));
		
	return(temp);   
}
	

/************************************************************************
 * Syntax	: SetIndexSize(unsigned long Indx)			*
 *									*
 * Description	: This function initialises the maxIdex.		*
 *									*
 * Parameters	: Indx 	                                                *
 *		     - Index ny=umber.					*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::SetIndexSize(unsigned long Indx)
{ 
	maxIndex = Indx;
}


/************************************************************************
 * Syntax	: GetIndexSize()					*
 *									*
 * Description	: This function returns the maximum index for the 	*
 *		  MPEG stream.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Maximum index for the mpeg stream.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

unsigned long NscPlayer::GetIndexSize()
{ 	
	return maxIndex;	
}



/************************************************************************
 * Syntax	: GetfileSize()						*
 *									*
 * Description	: This function returns the total stream size.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Total stream size.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

unsigned long NscPlayer::GetfileSize()
{ 	
	return fileSize;
}




/************************************************************************
 * Syntax	: SetfileSize( unsigned long flsz )			*
 *									*
 * Description	: This function sets the class varible fileSize with *
 *		  stream size.						*
 *									*
 * Parameters	: flsz							*
 *		    - Stream size.					*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::SetfileSize( unsigned long flsz )
{  	
	fileSize = flsz;
}



/************************************************************************
 * Syntax	: MapToFile( int index )				*
 *									*
 * Description	: This function maps the scroll index to the stream  *
 *		  and returns corresponding stream  pointer		*
 *									*
 * Parameters	: index							*
 *		    - scroll index.					*
 *									*
 * Returns	: Stream pointer.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

unsigned long  NscPlayer::MapToFile( int index )
{ 	
	return ( GetfileSize() / 100 ) * index ; 
}




/************************************************************************
 * Syntax	: MapToScrol( unsigned long flsz )			*
 *									*
 * Description	: This function maps the stream pointer to the       	*
 *		  scroll  bar.						*
 *									*
 * Parameters	: flsz							*
 *		    - stream pointer.					*
 *									*
 * Returns	: Index to the scroll bar ( 0- 100).			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
int NscPlayer::MapToScrol( unsigned long flsz )
{	
	return ( flsz ) / ( GetfileSize() /100 );
}




/************************************************************************
 * Syntax	: SetSliderPoint( int slPt )				*
 *									*
 * Description	: This function sets the variable SliderPointer to	*
 *		  the  value specifed.If the value specifed is out range*
 *		  then value is not assigned.The SliderPoint varible    *
 *		  is used by GUI to display the pointer position of     *
 * 		  scroll bar.                                           *
 *									*
 * Parameters	: slPt							*
 *		    - scroll bar index.					*
 *									*
 * Returns	: 0 for failure,1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::SetSliderPoint( int slPt )
{  	
	if( ( slPt <= 100 ) && ( slPt >= 0 ) )
		{	SliderPoint = slPt; 
			return  1 ;
		}
   	return 0;	
}




/************************************************************************
 * Syntax	: GetSliderPoint()					*
 *									*
 * Description	: This function is indirectly used by Gui to  set the	*
 *		  pointer position of scroll bar.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: SliderPoint						*
 *			- Slide point.					*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/


int NscPlayer::GetSliderPoint()
{ 	
	return SliderPoint;
}





/************************************************************************
 * Syntax	: GetSkipLocFromSlider()				*
 *									*
 * Description	: This function used to skip the stream pointer when 	*
 *		  player changes the state to skip mode.		*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: current pointer of the stream.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

unsigned long NscPlayer::GetSkipLocFromSlider()
{ 	
	return ( GetfileSize() / 100 ) * GetSliderPoint() ;
}








/************************************************************************
 * Syntax	: ShowDirectory( char * dir )				*
 *									*
 * Description	: This function gets the directory listing from the  	*
 *		  srever.						*
 *									*
 * Parameters	: *dir							*
 *		     - path of the directory.				*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::ShowDirectory( char * dir )
{
 	FindData fileAttr;
 	char 	cwd[_MAX_PATH];
 	char 	directory[32];
	char 	tempStr[100];
	char 	fltype;
 	int 	nextdirloc=0;
 	int 	loop=1, dirSize, dirFound;
 	int 	iItem=0, iSubItem=0, iActualItem=0;
	int 	ind =0;
	

	
 	strncpy ( directory, dir, 31);

 	if( !connection->SendDirectoryListing() )	     
	{ 	
		pParam->postErMsg("Fails to send request for Dir");
		return 0;
	}
 	dirSize = strlen((const char *)directory);

 	if( !connection->SendData( (char *)&dirSize, sizeof(dirSize)) )
	{
		pParam->postErMsg("Fails to send request for Dir size");
		return 0;
	}

 	//send the filename whose contents are to be received
	
 	if( !connection->SendData(directory, strlen(directory)) ) 
	{ 	
		pParam->postErMsg(" fails to get directory ");
		return 0;
	}

 	//receive the length of the cwd directory name
	
 	if( !connection->ReceiveData((char *)&dirSize, sizeof(dirSize)) )	
       	{  	
		pParam->postErMsg("Fails to get dir size ");
	   	return 0;
 	}

 	
	//receive the cwd directory name
	
 	if(!connection->ReceiveData(cwd, dirSize)) 
	{ 
		pParam->postErMsg(" Fails to receive CWD ");
	  	return 0;
	}

 	//Null terminate
	
 	cwd[dirSize]=0x0;
	

 	//receive the length of the cwd directory name
	
 	if( !connection->ReceiveData((char *)&dirFound, sizeof(dirFound)))
	{	
		pParam->postErMsg("Fils to get size of directory ");
		return 0;
	}
	
 	if(dirFound)
 	{ 	
		//directory not found
 		//printf("\n\n dir  Not FOUND \n");
 	}

	ind =0;
	
 	while(loop)
	{
 		//receive the cwd directory name
 		if( !connection->ReceiveData(fileAttr.data,sizeof(fileAttr.data)) )
 			return 0;
 		if(fileAttr.fmt.name[0] == 0x0) //endof list
 			loop=0;
 		else  
		{
			if(fileAttr.fmt.attrib & _A_SUBDIR)  	fltype = 'D';
 			if(fileAttr.fmt.attrib & _A_HIDDEN)  	fltype = 'H';
			if(fileAttr.fmt.attrib & _A_SYSTEM) 	fltype = 'S';
			if(fileAttr.fmt.attrib & _A_RDONLY) 	fltype = 'R';
			if(fileAttr.fmt.attrib & _A_ARCH)  	fltype = 'A';
			
			pParam->SetFilename4Dir( ind, fileAttr.fmt.name );  
			pParam->SetFileType4Dir( ind, fltype );
			
			ind++;
			
			pParam->SetNoRec4Dir( ind );
			
 			if( !connection->NextPacket() )	
					return 0;
			
			if(ind >= 99 ) break;
	      }
		      
  		
 	}
	pParam->SetNoRec4Dir(ind);
	
 	//Send the end of transmission
 	if( !connection->EndOfTransmission() )
 		return 0;
 	
 	return 1;
 }

	





/************************************************************************
 * Syntax	: SetupFileTransfer(char * file)			*
 *									*
 * Description	: This function set the server  for file transfer 	*
 *		  mode.							*
 *									*
 * Parameters	: *file							*
 *		      - File name.					*
 *									*
 * Returns	: 0 for fail, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
int NscPlayer::SetupFileTransfer(char * file)
 {
       	unsigned long 	fileSize=0, temp;
       	char 		filename[32];	
	int 		retvalue = 1 ;  // By default returns success
    	
       	strncpy( filename, file, 32 );
		
        	//Send the command
       	if( !connection->SendFileData() )		
	{	
		retvalue = 0;
	}
	else
	{	
       	  fileSize = strlen(filename);
		
          if( !connection->SendData((char *)&fileSize,sizeof(fileSize)) )
	  {	pParam->postErMsg("SetupFileTransfer:  ERR 01");   
		retvalue = 0;
	  }
	  else	
       	  if( !connection->SendData((char *)&filename,fileSize) )
	  {	
	        pParam->postErMsg("SetupFileTransfer:  ERR 02"); 
		retvalue = 0;
	  }
	  else
	  {
       		fileSize = 0;            
				//Get the file size in bytes
        	if(!connection->ReceiveData((char *) &fileSize,sizeof (fileSize))) 
		{	
		        pParam->postErMsg(" SetupFileTransfer:  ERR 03");
			return 0;
		}	
		
		SetfileSize(fileSize);
			
        	if(!connection->ReceiveData((char *)&temp,sizeof(temp)))  	
		{	
			pParam->postErMsg("SetupFileTransfer: ERR 04 ");
			return 0;
		}
        
		bFastMode = temp;
		bytesProcessed =0;
        	
        	if(bFastMode)
        	{
        	    if(!connection->ReceiveData((char *)&temp,sizeof(temp)))  	
		    {
		        pParam->postErMsg("Fails to get No of records in Index file ");	
			return 0;
		    }
        		
		
		    SetIndexSize( temp );
		    
        	    FMList = (FastModeTable *) malloc( sizeof(FastModeTable) * GetIndexSize() ); 
		    
        	    if( !connection->ReceiveData((char *)FMList,
			               sizeof(FastModeTable) *  GetIndexSize()) )
			    
		    {	
			pParam->postErMsg("Fails to get  Index file ");
        		return 0;
		    }	
        		
#if 0
    			printf(": SlNo   Start       End   \n ");
        		for(temp=0;temp < GetIndexSize() ; temp ++)
        		{
        		printf(" %d    %d    %d\n", temp,FMList[temp].start, FMList[temp].end);
			usleep(100);
        		}
#endif
               }
	  }
	}
	/*
        if( !connection->ReceiveData((char *)FMList,
               sizeof(FastModeTable) *  GetIndexSize()) )
	    
	    {	
		pParam->postErMsg("Fails to get  Index file ");
       		return 0;
	    }	
        */	
       	return fileSize;
}




/************************************************************************
 * Syntax	: GetNearestIFrame(unsigned long fileptr)		*
 *									*
 * Description	: This function serches  Iframe   and returns		*
 *									*
 * Parameters	: fileptr						*
 *		     - stream pointer.					*
 *									*
 * Returns	: 0 for failur and stream pointer  on sucess.		*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

unsigned long NscPlayer::GetNearestIFrame(unsigned long fileptr)
{
		int i;
		int max_ind;
		max_ind = GetIndexSize();
	
		for(i=0;i<max_ind;i++)
		{
			if(FMList[i].start >= fileptr)
				return FMList[i].start;
		}
		return 0;
}



/************************************************************************
 * Syntax	: SetInPlayMode(BOOL skip)				*
 *									*
 * Description	:  This function sets the player to Play mode , if 	*
 *		   skip flag then plays  the stream from the skip 	*
 *		   location else plays from the begining.		*
 *									*
 * Parameters	: skip							*
 *		    - mode flag.					*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::SetInPlayMode(BOOL skip)
{
        	unsigned long SkipTo = 0;
		int  retvalue = 1;     // return success
		
		printf("SetInPlayMode\n");
		PSOB = 0;
		bytes2BProcessed = 0;
		blocksToReceive = 0;

		pSigma->ClearSigma();

		sleep(1);
        	if( skip )
        	{	
			SkipTo = MapToFile( GetSliderPoint() );  
			PSOB = 0;
			ActualPtrInFl = SkipTo;
			skipMode = 1;
        	}
		else
		{	ActualPtrInFl = 0;
			skipMode =0;
		}

		pParam->UpdateScrolpt(MapToScrol(ActualPtrInFl));

              	
		if ( connection->PlayMode(0 , PSOB, 0 , SkipTo) == FALSE )
		{	
			
			pParam->postErMsg("SetInPlaymode:  ERR 0 ");
			
			retvalue = 0 ;   	
		} 
		else 	//get from where the play mode data is beng played
		if ( 0 == connection->ReceiveData( (char *)&PSOB, sizeof(PSOB)) )
		{	
			pParam->postErMsg("SetInPlaymode:  ERR 1 ");

			retvalue = 0; 
		}
		else //get how many bytes of fastmode pktsin total till BOF/EOF
		if ( 0 == connection->ReceiveData( (char *)&bytes2BProcessed, sizeof(bytes2BProcessed)))
		{	
			pParam->postErMsg("SetInPlaymode: Fails to receive #bytetes2proceess ");

			retvalue = 0; 
		}	 //how many blocks will be trnitted
		if ( 0 == connection->ReceiveData( (char *)&blocksToReceive, sizeof(blocksToReceive)) )
		{	
			pParam->postErMsg("SetInPlaymode: Fails to Receive #Block of data to receive  ");

			retvalue = 0;
		}
		

		bytesProcessed = 0;
		return  retvalue ;
  }




/************************************************************************
 * Syntax	: SetFastFarward( unsigned long processedBytes )	*
 *									*
 * Description	: This function switch to  fast mode  from the 		*
 *		  location specified.					*
 *									*
 * Parameters	: processedBytes					*
 *			- offset from the begining to start play in 	*
 *			  Fast forward mode.				*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
int NscPlayer::SetFastFarward( unsigned long processedBytes )
{ 	
	if( processedBytes >= GetfileSize() )  
		return 0;
	Direction = 1;
	return FastMode( processedBytes );	
}



/************************************************************************
 * Syntax	: SetFastFarward()					*
 *									*
 * Description	: This function used to switch to the fast mode from 	*
 *		  the current location.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/
	
int NscPlayer::SetFastFarward()
{ 	
	Direction = 1;
	return FastMode( bytesProcessed );	
}



/************************************************************************
 * Syntax	: SetReverseMode( unsigned long processedBytes )	*
 *									*
 * Description	: This function used to switch to rev mode from the  	*
 *		  location specified.					*
 *									*
 * Parameters	: processedBytes					*
 *			- offset from beging to strat play in the 	*
 *			  reverse mode.					*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::SetReverseMode( unsigned long processedBytes )
{ 	Direction = -1; 
	return FastMode( processedBytes );	
}




/************************************************************************
 * Syntax	: SetReverseMode()					*
 *									*
 * Description	: This function used to swith to revers mode , starts	*
 *		  playing the stream in reverse mode from the  current 	*
 *		 location.						*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 0 for failure , 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::SetReverseMode()
{ 	Direction = -1;
	return FastMode( bytesProcessed );	
}

	


/************************************************************************
 * Syntax	: FastMode( unsigned long processedBytes )		*
 *									*
 * Description	: This function used to set the server to fast mode	*
 *									*
 * Parameters	: processedBytes 					*
 *			- Offset from beging of the stream.		*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::FastMode( unsigned long processedBytes )
{	
	int retvalue = 1;
	
	 pSigma->ClearSigma();

		
         processedBytes = MapToFile( GetSliderPoint() );
	 
	sleep(1);
    	
	if( (Direction != 1 )   && (Direction !=-1))  
	{	pParam->postErMsg("FastMode: Fast/Rev mode is not set  ");
		retvalue = 0;
	}
	else
	if( !connection->FastMode( Direction,processedBytes ))
	{  	
		pParam->postErMsg("FastMode: Fails to set in fast mode <Err 1> ");
		retvalue  = 0;
	}
	else
       	if(0 == connection->ReceiveData((char *)&doFastMode, sizeof(doFastMode)))
	{	pParam->postErMsg("FastMode: Fails to set in fast mode <Err 02> ");
		retvalue = 0 ;
	}
	else			
	if(doFastMode) 
	{	PSOB = GetStartIFrameFromServer(); 
			//get from where the fast mode data is beng played
			
		if( 0 == connection->ReceiveData( (char *)&bytes2BProcessed, sizeof(bytes2BProcessed)) )
		{  
			pParam->postErMsg("FastMode: Fails to get #bytes2Bprocess");
			retvalue = 0;	; 
		}
		else	 //get how many bytes of fastmode packets in total till the BOF/EOF
		if( 0 == connection->ReceiveData((char *)&blocksToReceive, sizeof(blocksToReceive)) )
		{       pParam->postErMsg("FastMode: Fails to receive #Blocks to receive ");
			retvalue = 0; 
		}
			bytesProcessed = 0;
	}

	return retvalue;
 }
   




/************************************************************************
 * Syntax	: NscKeyCheck()						*
 *									*
 * Description	: This function used to read the key pressed when the	*
 *		  control is one of the play modes.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 0 => maintain control within the mode  ,            	*
 *                1 =>  give up the control from the  current mode	*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/


int NscPlayer::NscKeyCheck()
{ 	
	int value;
  	int retvalue =0; 
		
	if(pParam->GetStatus())
	{
		switch( pParam->GetKey() )
		{
		case  CMUTE:   	pSigma->AudioSwitch(); 
				pParam->GetKeyWithValue( &value ); // kbd buf cleared
				retvalue = 0;
				break;
				
		 case CADIOL:   pParam->GetKeyWithValue( &value );
		 		SetAudioLeft(pParam->GetAudioL());
		 		retvalue = 0;
		 		break;
				
		 case CADIOR:	pParam->GetKeyWithValue( &value );
		 		SetAudioRight(pParam->GetAudioR());
		 		retvalue = 0;
		 		break;		
				
		 case  CPAUSE:  pParam->GetKeyWithValue( &value );
		 		pParam->GetKeyWithValue( &value ); 
					// kbdbuf cleared	
					
		 		while( pParam->GetKey() != CPAUSE ) 
				{	
					sleep(1);  	//PAUSE key has to be accounted  
					
				}
				pParam->GetKeyWithValue( &value ); // but buf cleared
				retvalue =0;
				break;		    
				
				
		 case  CLOOP:   loop = !loop;
		 		pParam->GetKeyWithValue( &value ); // kbd buf cleared
		 		retvalue = 0;
		 		break;
				
				//  now main loop will take control

		 case  CQUT:	
		 case  CSTP :	
		 case  CGETDIR:
		 case  CCNTSER:
		 case  CPLAY:
		 case  CSKIP:
		 case  CFF:
		 case  CREV:
		 case  CSCROLL:	
	 			retvalue =1;
		 		break;
				
		 default: retvalue =0; 
		 	break;
		}
			
	}
	return retvalue;
}
	




/************************************************************************
 * Syntax	: NSCFastPlay()						*
 *									*
 * Description	: This function used to plays the stream in fast mode	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::NSCFastPlay()
{    	
	unsigned long expect,expect1;
	long 	alreadyRead =0;
	int 	findex=0;
	int 	scrlPt =0;
	int 	i=0,i_buf=0;	
	int 	retvalue =1;
	  

	if ( bytes2BProcessed >= BLOCK_SZ )
	do
	{
	   if( Direction == 1) 	findex = PSOB +i;
		          else 	findex= PSOB -i;
			     
	   expect = FMList[ findex].end - FMList[ findex ].start;  
                        
	   alreadyRead = alreadyRead + (BLOCK_SZ - expect);
				     
	   if(alreadyRead >= expect)
	   {  
		alreadyRead -= expect;
	      	i++;
	      	if( Direction == 1 ) 	findex = PSOB + i;
	       			else 	findex = PSOB - i;
	      expect = FMList[ findex].end - FMList[ findex ].start; 
	   }
			
			
	   if (0 == connection->ReceiveData(dataBuffer[i_buf % NUMOFBUFFERS],BLOCK_SZ))
	   { 	retvalue = 0;
	  	pParam->postErMsg("FastPlay : Fails to receive data     ");
	  	break;	
	   }
			
           bytes2BProcessed -= BLOCK_SZ;  //expect;
  	   bytesProcessed   += BLOCK_SZ;   //expect ; //BLOCK_SZ; 

	   ActualPtrInFl = FMList[findex].start ;
	   scrlPt = MapToScrol(ActualPtrInFl);
	   pParam->UpdateScrolpt(scrlPt);
	   SetSliderPoint(scrlPt);
	   usleep(10);
			
	if(Direction == 1)	// FF 
	pSigma->PlaySigma(dataBuffer[i_buf % NUMOFBUFFERS],BLOCK_SZ,SIG_FF_REV) ;

	i = ( i + 1 ) ;
	if (Direction == -1)	//REV
	{
		if( i_buf % NUMOFBUFFERS == 0)
			
		for(int j = 0; j < NUMOFBUFFERS; j++ )
	  	      pSigma->PlaySigma( dataBuffer[j%NUMOFBUFFERS], BLOCK_SZ, SIG_FF_REV );
	}
	
	i_buf++;
 	if( NscKeyCheck() )  break;  
	
	if( bytes2BProcessed >= BLOCK_SZ )  
	{
		if( ! connection->NextPacket() )
		{ retvalue = 0;
		  break;
		}
	}
			
	}
   	while( (bytes2BProcessed >= BLOCK_SZ) ) ;
	
	connection-> EndOfTransmission();
	doFastMode=0;
	usleep(1000);
	return retvalue;
 }



/************************************************************************
 * Syntax	: GetOffset(char * bufpt,long sz)			*
 *									*
 * Description	: This function used to get the offset address to the	*
 *		  start of the frame.					*
 *									*
 * Parameters	: bufpt							*
 *		    - Buffer pointer.					*
 *		  sz							*
 *		   - Size of the buffer.				*
 *									*
 * Returns	: current pointer of the stream.			*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

long NscPlayer::GetOffset(char * bufpt,long sz)
{
	int i;
	for ( i=0; i<sz; i++)
	{
		if( ( *bufpt == 0x47) && (*(bufpt+1)== 0x50))
		{  	
			return i;
		}
		else bufpt++;
	}
	return 0;
		
	}



/************************************************************************
 * Syntax	: SemCreate( int semkey)				*
 *									*
 * Description	: This function facilitates to create a semaphore.	*
 *									*
 * Parameters	: semkey						*
 *		      - semaphore key.					*
 *									*
 * Returns	: Semphore Id on Sucess and -1 on failure.		*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::SemCreate( int semkey)
 {
	 int semid = semget(semkey,1,IPC_CREAT|0666);
	 
	 if   ( semid  < 0 )
 		{ 
			printf("error in sem create ");
 		  	return -1;	
 		}
		
 	return (semid);
 }


/************************************************************************
 * Syntax	: LockSem(int semid)					*
 *									*
 * Description	: This function facilitates to capture a semaphore.	*
 *									*
 * Parameters	: semid							*
 *		     - Semaphore id.					*
 *									*
 * Returns	: returns Semphore Id on Sucess and 0 on failure.	*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::LockSem(int semid)
 {
   	if( semop ( semid, &op_lock[0], 2 ) < 0 )  
	{ 	
		printf("error in lock ");
		return 0;
	}
	return 1;
  }




/************************************************************************
 * Syntax	: UnlockSem(int semid)					*
 *									*
 * Description	: This function facilitates to release a lock.		*
 *									*
 * Parameters	: semid							*
 *		     - semaphore id.					*
 *									*
 * Returns	: returns Semphore Id on Sucess and 0 on failure.	*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int NscPlayer::UnlockSem(int semid)
 {
 	if( semop( semid, &op_unlock[0], 1 ) < 0 )  
	{ 
		printf("error in unlock ");
		return 0 ;
	}
	return 1;
 }




/************************************************************************
 * Syntax	: LockNetCount()					*
 *									*
 * Description	: This function to hold NetCountSemID.			*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	:  used to synchronise the usage of NetCount variable   *
 *                 from player and a thread which reads data from net.  *
 ************************************************************************/

void NscPlayer::LockNetCount()
{
	LockSem( NetCountSemID );
}




/************************************************************************
 * Syntax	: UnLockNetCount()					*
 *									*
 * Description	: This function facilitates to Release  NetCountSemID*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: used to synchronise the usage of NetCount variable    *
 *                from player and a thread which reads data from net.	*
 ************************************************************************/

void NscPlayer::UnLockNetCount()
{      
	UnlockSem( NetCountSemID );
}



/************************************************************************
 * Syntax	: LockThreadStatus()					*
 *									*
 * Description	: This function facilitates facilitates to hold  	*
 *		  ThdStatusSemId.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: used to synchronise the usage of ThreadStatus variable*
 *                from player and a thread which reads data from net.	*
 ************************************************************************/

void NscPlayer::LockThreadStatus()
{ 	
	LockSem( ThdStatusSemID );
}
  



/************************************************************************
 * Syntax	: UnLockThreadStatus()					*
 *									*
 * Description	: This function facilitates facilitates to release  	*
 *		  ThdStatusSemId.					*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: used to synchronise the usage of ThreadStatus variable*
 *                from player and a thread which reads data from net.	*
 ************************************************************************/

void NscPlayer::UnLockThreadStatus()
{ 	
	UnlockSem( ThdStatusSemID );
}




/************************************************************************
 * Syntax	: IncCount()						*
 *									*
 * Description	: This function safely increments the netcount, which	*
 *		  indicates Number of ready buffers in the circular que.*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::IncCount()
{
	LockNetCount();
	netCount++;
	UnLockNetCount();

}



/************************************************************************
 * Syntax	: DecCount()						*
 *									*
 * Description	: This function safely decrements the netcount ready 	*
 *		  buffers in the circular que.				*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: Called in a player.					*
 ************************************************************************/

void NscPlayer::DecCount()
{
		LockNetCount();
		netCount--;
		UnLockNetCount();
}




/************************************************************************
 * Syntax	: CreateNetReader()					*
 *									*
 * Description	: This function						*
 * 			-Intialises semaphores ,			*
 *                 	-creates a thread to read data from the net	*
 *                      -Creates circular buffer to read data from net	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: called in a player constructor. 			*
 ************************************************************************/

int  NscPlayer::CreateNetReader()
{

	int i;
	
	op_lock[0].sem_num = 0;
	op_lock[0].sem_op  = 0;
	op_lock[0].sem_flg = 0;
	
	op_lock[1].sem_num = 0;
	op_lock[1].sem_op  = 1;
	op_lock[1].sem_flg = 0;

	op_unlock[0].sem_num = 0;
	op_unlock[0].sem_op  = -1;
	op_unlock[0].sem_flg = IPC_NOWAIT;
	
	netCount = 0;
	netReadIndex  = 0;
	netWriteIndex = 0;
 	NetThreadStatus = 0;
	
	netthdHandle = pthread_create( &f_th, NULL, NSCNetBufWrite, this );
	
        if( netthdHandle != 0)
                {      
			printf("Failed to create Net Reader Thread !!!\n");
                        return -1;  
                }    
		
	for( i=0; i < MAXNETCOUNT; i++ )
	{
		glbNetInBuffer[i] = new char[BLOCK_SZ];
	}
	NetCountSemID  = SemCreate(726);
	ThdStatusSemID = SemCreate(727);
}



/************************************************************************
 * Syntax	: NSCNetBufWrite(NscPlayer *pPlayer)			*
 *									*
 * Description	: This function checks,					*
 *			 -If thread status is active then reads the data*
 *			  from the net  and fills the  circular buffer.	*
 *			 -If the circular buffer is full waits for the 	*
 *			  player to read from the buffer.		*
 *									*
 *									*
 * Parameters	: *pPlayer						*
 *			- pointer to the player buffer.			*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void *NSCNetBufWrite(NscPlayer *pPlayer)
{
       int expect; 
       int netWriteIndex =0;
       int retn;
       int noResponseCnt = 0;


    while(1)
    {
    if ( pPlayer->IsNscRun() )
    {

	if (pPlayer->NetB2Proc >= BLOCK_SZ )    
		 		expect = BLOCK_SZ;
	               else     expect = pPlayer->NetB2Proc ;
				   
  	if (0 == (retn = connectionT->ReceiveData( pPlayer->glbNetInBuffer[netWriteIndex], expect)) )
	{ 	
		pParamT->postErMsg("NetRead    : Fails to receive data     ");
		
		if(25 != noResponseCnt++)
		{
			usleep(10000);
		}
		else
		{ 	pPlayer->NSCNetDeactiveThread();
		}
		
		continue;
	}
	else
	{
		netWriteIndex = ( netWriteIndex + 1 ) % MAXNETCOUNT;
		pPlayer->IncCount();
		pPlayer->NetB2Proc -= expect;
		noResponseCnt = 0;
	}
	
	while ( pPlayer->netCount >= MAXNETCOUNT -1 ) ;
		 
	if ( pPlayer->NetThreadStatus ) 
	{
		if ( pPlayer->NetB2Proc < BLOCK_SZ)
		{
		 	pPlayer->NSCNetDeactiveThread();
	
		}
		else	
		if ( ! connectionT->NextPacket() )
		{   	
			pParamT->postErMsg("NetRead :Server is not responding ");
	
		}
	}
     }
     else
	{
		 usleep(10);
	}
    };
}





/************************************************************************
 * Syntax	: NSCNetBufRead()					*
 *									*
 * Description	: This function checkes					*
 *			-If there is a unread data block in the  	*
 *			 circular buffer then return the address of the *
 *			 data block to the caller 			*
 * Parameters	: None.							*
 *									*
 * Returns	: current pointer of the stream.			*
 *                                                                      *
 * Comment	: called from the player to read data block from  	*
 *		  circular list .					*
 ************************************************************************/

char * NscPlayer::NSCNetBufRead()
{
	int netIndex;
	
	if ( netCount )
	{
	   netIndex = netReadIndex ;
	   netReadIndex = ( netReadIndex + 1 ) % MAXNETCOUNT ;
	   DecCount();
	   return glbNetInBuffer[netIndex];	
	}
	else return 0;
}





/************************************************************************
 * Syntax	: IsNscRun()						*
 *									*
 * Description	: This function used to return the thread status.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: True or False.					*
 *                                                                      *
 * Comment	: Called in a NscNetbufWrite thread .			*
 ************************************************************************/

volatile bool NscPlayer::IsNscRun()
{
	if ( NetThreadStatus == 0 )  
				return false;
		        else  	return true;
}




/************************************************************************
 * Syntax	: NSCResetNetBuffer()					*
 *									*
 * Description	: This function used to reset the circular buffer.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: called wheever the change in mode of play.		*
 ************************************************************************/

void NscPlayer::NSCResetNetBuffer()
{
   	int temp ;
   
   	temp = NSCNetDeactiveThread();
   	usleep( 100000 );

   	for ( int i=0; i < MAXNETCOUNT ; i++ )
   		bzero( glbNetInBuffer[i], BLOCK_SZ ) ;
	
   	netWriteIndex = 0;
   	netReadIndex = 0;
   	netCount = 0 ;
   	if ( temp )
   		NSCNetActiveThread();
	
}




/************************************************************************
 * Syntax	: NSCNetActiveThread()					*
 *									*
 * Description	: This function request to activates the thread.	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: Previous thread status.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int  NscPlayer::NSCNetActiveThread()
{       
	int status = NetThreadStatus;
	LockThreadStatus();
	NetThreadStatus = 1;
	UnLockThreadStatus();
	pParamT = pParam;
	connectionT = connection;
	return status;
}




/************************************************************************
 * Syntax	: NSCNetActiveThread( unsigned long b2p)		*
 *									*
 * Description	: This function request to activates the thread.	*
 *									*
 * Parameters	: b2p							*
 *		    - oofset to play.					*
 *									*
 * Returns	: Previous thread status.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int  NscPlayer::NSCNetActiveThread( unsigned long b2p)
{       
	int status = NetThreadStatus;
	LockThreadStatus();
	NetThreadStatus = 1;
	UnLockThreadStatus();
	NetB2Proc = b2p;
	pParamT = pParam;
	connectionT = connection;
	return status;
}





/************************************************************************
 * Syntax	: NSCNetDeactiveThread()				*
 *									*
 * Description	: This function request to deactivates the thread	*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: previous ThreadStatus.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int  NscPlayer::NSCNetDeactiveThread()
{	
	
	int sts = NetThreadStatus;
	LockThreadStatus();
	NetThreadStatus = 0;
	UnLockThreadStatus();
	return sts;
}




/************************************************************************
 * Syntax	: NSCNormalPlay()					*
 *									*
 * Description	: This function used to Plays the stream in normal 	*
 *		  mode.							*
 *									*
 * Parameters	: None.							*
 *									*
 * Returns	: 0 for failure, 1 for sucess.				*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

int  NscPlayer::NSCNormalPlay()
{ 	int i=0;
	int retvalue =1;
  	unsigned long expect=0;     
	char *temp;
	int  noResponseCnt =0;
		
	do{   
		NSCResetNetBuffer();
		sleep(1);
		NSCNetActiveThread(bytes2BProcessed);

		while ( bytes2BProcessed >= BLOCK_SZ )
       		{
               		if( bytes2BProcessed > BLOCK_SZ)	
				expect = BLOCK_SZ;
                	else   	expect = bytes2BProcessed;

			temp = NSCNetBufRead();
	
			if ( temp != 0 )
			{
				bzero( dataBuffer[i], BLOCK_SZ );
				memcpy( dataBuffer[i], temp, expect );
				noResponseCnt =0;
			}
			else  	
			{  	
				
				if ( 20 != noResponseCnt++ )
				{      usleep(1000);	
					continue;
				}
				else
				{	retvalue =0;
					break;
				}
			}
			
			
                  	bytes2BProcessed -= expect;
		  	bytesProcessed   += expect;
			if ( skipMode )
			{       
			  pSigma->PlaySigma( dataBuffer[i], expect,SIG_SKIP  );
			  skipMode =0;	
			}
			else
			{
	         	  pSigma->PlaySigma(dataBuffer[i], expect, SIG_NORMAL);
			}
			
			ActualPtrInFl += expect;
			

			pParam->UpdateScrolpt(MapToScrol(ActualPtrInFl));
			

			if ( NscKeyCheck() ) 
			{
				break;
			}   // other commands  override LOOP
			i = ( i+1 ) % NUMOFBUFFERS;
       		};
		
		connection-> EndOfTransmission();    
		
		
		 NSCNetDeactiveThread();
		 
		usleep(10000);
		
		if ( retvalue == 0 ) 	break;
		else
		if ( bytes2BProcessed > BLOCK_SZ ) break;
		else
		if( loop )
		 {	 skipMode = 1;
			 SetInPlayMode(0);
			 pSigma->AudioOnSigma();
		 }
			
	   } while (loop);
	   return retvalue ;

}
	




/************************************************************************
 * Syntax	: SetAudioLeft(int vol)					*
 *									*
 * Description	: This function used to set the left volume.		*
 *									*
 * Parameters	: vol							*
 *		    - Left volume.					*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::SetAudioLeft(int vol)
{
	pSigma->SetLVolSigma( vol )  ;
}




/************************************************************************
 * Syntax	: SetAudioRight(int vol)				*
 *									*
 * Description	: This function used to sets the Right volume.		*
 *									*
 * Parameters	: vol							*
 *		    - Right volume.					*
 *									*
 * Returns	: None.							*
 *                                                                      *
 * Comment	: None.							*
 ************************************************************************/

void NscPlayer::SetAudioRight(int vol)
{
	pSigma->SetRVolSigma( vol )  ;
}

