/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor Video Middleware
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 

 ********************************************************************************
 * File Name        : nscplayer.h						*
 * 										*
 * Project Name	    : ASTB							*
 * 										*
 * Subsystem Name   : Streamer	                                                *
 *                               						*
 * Module Name      : nscplayer							*
 *										*
 * Author Name	    : Arun Kumar						*
 *										*
 * Language         : C++							*
 *										*
 * Date             : 12/03/00							*
 *										*
 * Description      : This file defiens the funtion prototype and define values *
 *		      used in nscplayer.cpp file.				*
 *										*
 ********************************************************************************/



#ifndef __NSCPLAYER_H
#define __NSCPLAYER_H


#include "nscsigma.h"
#include "clientcom.h"
#include "defines.h"
#include "nscparam.h"
#include <sys/sem.h>
#include <pthread.h>

enum {PLAY_KEY =1,FF_KEY,REV_KEY,STOP_KEY,PAUSE_KEY,LOOP_KEY,QUT_KEY};

#define _MAX_PATH 50
#define MAXNETCOUNT 10


/************************************************************************
 * class Description : The Nscplayer is a class which contains various 	*
 *		  functions as its member which are used to play the 	*
 *		  MPEG stream.					 	*
 *									*
 ************************************************************************/

	
class  NscPlayer
{
	private:
	
	sigma *pSigma;
	//buckets for carrying data and its charateristics
	char 		*dataBuffer[NUMOFBUFFERS];
	int 		bufferStatus[NUMOFBUFFERS];
	unsigned long 	bufferSize[NUMOFBUFFERS];
	unsigned long   ActualPtrInFl;
	Param      *  pParam;
	int netthdHandle;
        char netmsg[2];
        pthread_t f_th;
	struct sembuf op_lock[2];             // used for capturing lock
	struct sembuf op_unlock[1];           //  used to release the lock
	int NetCountSemID;
        int ThdStatusSemID;
        //Play mode related
	unsigned long 	blocksPlayed;
	unsigned long 	bytesProcessed;
	unsigned long 	PSOB;//Player start of byte
	unsigned long 	bytes2BProcessed;
	unsigned long 	blocksToReceive;
	unsigned long 	doFastMode;
	unsigned long  fileSize;
	unsigned long  maxIndex;
	unsigned long  bytesToSkip;
	unsigned char  bFastMode;
 	ClientCom *connection;
        int Direction;
	int SliderPoint;
        int fl_sz;
	int mode_flag;
	int skipMode;
	int KeyPress;
	int flag;
	BOOL loop;
        FastModeTable *	FMList;
			
	
	protected :

	int 	GetKey();
	int 	NscKeyCheck();
	unsigned long  MapToFile(int aa);
	long 	GetOffset(char * bufpt,long sz);
	int 	MapToScrol(unsigned long flsz);
	void 	SetIndexSize(unsigned long Indx);
	void 	SetfileSize(unsigned long flsz);
        char * 	NSCNetBufRead();
	void    NSCResetNetBuffer();
	int  	CreateNetReader();
	int  	UnlockSem(int semid);
	int  	LockSem(int semid);
        int  	SemCreate( int semkey);
	int  	NSCNetActiveThread();
	int  	NSCNetActiveThread(unsigned long);


	public:
	
	int netCount ;
	int netReadIndex ;
        int netWriteIndex ;
        unsigned long  NetB2Proc;
        char 	*glbNetInBuffer[MAXNETCOUNT];
	volatile int NetThreadStatus ;
	void   	IncCount();
        void 	DecCount();
        void 	LockNetCount();
	void 	UnLockNetCount();
        void 	LockThreadStatus();
	void 	UnLockThreadStatus();
        int 	NSCNetDeactiveThread();
	volatile bool 	IsNscRun();
	void AllocBuffer();
        void  ResetBuffers();
        int  SetSliderPoint(int slPt);
        void SetKey(int key );    // Gui thread usses this on Button press event;
        NscPlayer();
        ~NscPlayer();
        void SetConnection(ClientCom *);
        void Setparam(Param *);
        unsigned long GetStartIFrameFromServer() ;
        unsigned long GetIndexSize();
        unsigned long  GetfileSize();
	int GetSliderPoint();
        unsigned long GetSkipLocFromSlider();
        int ShowDirectory(char * dir);
	int SetupFileTransfer(char * file);
        unsigned long GetNearestIFrame(unsigned long fileptr);
        int  SetInPlayMode(BOOL skip);
        int SetFastFarward(unsigned long processedBytes);
	int SetFastFarward();
        int SetReverseMode(unsigned long processedBytes);
        int SetReverseMode();
        int FastMode(unsigned long processedBytes);
        int  NSCFastPlay();
	int NSCNormalPlay();
	void NscPlayerStop();
        void SetAudioLeft(int);
        void SetAudioRight(int);
	
};

#endif

