/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor NSC-ACB I2c modules 
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
*/
#include <linux/version.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <asm/io.h>
#include <linux/kernel.h>
#include <linux/stddef.h>
#include <linux/sched.h>
#include <linux/delay.h>

#include <linux/i2c.h>
#include <linux/init.h>

#define ACB_I2C_VERSION "1.0.0"

#define my_outb_p(val,port) { printk("outb_p port=%03X val=%02X\n", port,val);  outb_p( val, port);  }

#ifndef I2C_ALGO_ACB 
    /* in case if it is not in i2c-id.h */
    #define I2C_ALGO_ACB    0x070000 /* NSC ACCESS.bus */
#endif  //I2C_ALGO_ACB 

#ifndef I2C_HW_ACB_SC1200
    /* in case if it is not in i2c-id.h */
    #define I2C_HW_ACB_SC1200	0
    #define I2C_HW_ACB_SC1400	1
#endif //I2C_HW_ACB_SC1200

/* initialization states */
#define INIT_ACB1	0x1
#define INIT_ACB2	0x2

/* Base address for ACB registers */
#define ACB_BASE_1200_AB1	0x810
#define ACB_BASE_1200_AB2	0x820
#define ACB_BASE_1400_AB1	0x810
#define ACB_BASE_1400_AB2	0x820

/* ACB registers */
#define ACBSDA		0
#define ACBST		1
#define ACBCST		2
#define ACBCTL1		3
#define ACBADDR		4
#define ACBCTL2		5

/* timeout */
#define MAX_TIMEOUT (5 * 1000 / HZ)

/* pci vendor and device id for pci to isa bridge */
#define PCI_DEVICE_ID_CYRIX_SC1400_BRIDGE	0x0405 
#define PCI_DEVICE_ID_NS_SC1200_BRIDGE  	0x0505 

struct i2c_algo_acb_data {
	int  iobase;
	int  acbindex;
	int  deviceid;
	int  status;
};

/* debug stuff */
#define DEB1(X)      //X
#define DEB2(X)      //X
#define DEB3(X)      //X
#define TRACE(X)     //X
#define DEBERR(X)    //X
#define DEBPROTO(X)  //X

static void nscacb_inc_use(struct i2c_adapter *adapter);
static void nscacb_dec_use(struct i2c_adapter *adapter);
static u32 nscacb_func(struct i2c_adapter *adapter);
static int nscacb_master_xfer(struct i2c_adapter *adapter,
		    struct i2c_msg msgs[], int num);

#ifdef MODULE
static
#else
extern
#endif
int __init nscacb_init(void);
static int __init nscacb_cleanup(void);

#ifdef MODULE
extern int init_module(void);
extern int cleanup_module(void);
#endif	/* MODULE */

/***********************************************/
/*  NSC ACCESS.bus (ACB) algorithm             */
/***********************************************/ 

/* This is the actual algorithm we define */
static struct i2c_algorithm nscacb_algorithm = {
	/* name */ "ACCESS.bus (ACB) algorithm",
	/* id */ I2C_ALGO_ACB,
	/* master_xfer */ nscacb_master_xfer,
	/* smbus_access */ NULL,
	/* slave_send */ NULL,
	/* slave_rcv */ NULL,
	/* algo_control */ NULL,
	/* functionality */ &nscacb_func,
};

static mydelay(int seconds)
{
	unsigned long timeout = jiffies + seconds * HZ;
	while(jiffies < timeout)
		;
}

static u32 nscacb_func(struct i2c_adapter *adapter)
{
	return I2C_FUNC_SMBUS_EMUL |  
	       I2C_FUNC_PROTOCOL_MANGLING; 
}

static void acb_start(struct i2c_algo_acb_data *acb_data) 
{
	unsigned char reg;

    //TRACE(printk("i2c-nscacb.o acb_start()\n"));
    //DEBPROTO(printk("S "));

    reg = inb_p(acb_data->iobase + ACBCTL1);
    reg = reg | 0x01;
    outb_p( reg, acb_data->iobase + ACBCTL1 );

}

static void acb_stop(struct i2c_algo_acb_data *acb_data) 
{
    unsigned char reg;

	//TRACE(printk("i2c-nscacb.o acb_stop()\n"));
	//DEBPROTO(printk("P\n"));
	reg = inb_p(acb_data->iobase + ACBCTL1);
	reg = reg | 0x02;
	outb_p( reg, acb_data->iobase + ACBCTL1 );
}

static void acb_reset(struct i2c_algo_acb_data *acb_data)
{
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_reset()\n"));
	// Disable the ACCESS.bus device and
	// Configure the SCL frequency: 56 clock cycles
	outb_p( 0x70, acb_data->iobase + ACBCTL2 );

	// Configure no interrupt mode (polling) and
	// Disable global call address
	outb_p(0x0, acb_data->iobase + ACBCTL1 );
 
	// Disable slave address
	outb_p(0x0, acb_data->iobase + ACBADDR );

	// Enable the ACCESS.bus device
 	reg = inb_p(acb_data->iobase + ACBCTL2);
	reg = reg | 0x01;
	outb_p(reg, acb_data->iobase + ACBCTL2);

  	// Issue STOP event
	acb_stop(acb_data);

	// Clear NEGACK, STASTR and BER bits
	outb_p( 0x38, acb_data->iobase + ACBST );

	// Clear BB (BUS BUSY) bit
 	reg = inb_p(acb_data->iobase + ACBCST);
	reg = reg | 0x02;
	outb_p( reg, acb_data->iobase + ACBCST );

}
			
static void acb_init_hardware(struct i2c_algo_acb_data *acb_data)
{
	u16 index_port;
	u16 data_port;
	u8  val1, val2;
	unsigned long reg = 0;

	TRACE(printk("i2c-nscacb.o acb_init_hardware()\n"));
	TRACE(printk("i2c-nscacb.o deviceid=%X acbindex=%X iobase=%X status=%0X\n", acb_data->deviceid, acb_data->acbindex, acb_data->iobase, acb_data->status));
 	reg = inb_p(acb_data->iobase + ACBCTL2);
	reg = reg & 0xFe;
	outb_p( reg, acb_data->iobase + ACBCTL2 );
        udelay(1000);
	// enable ACB
 	reg = inb_p(acb_data->iobase + ACBCTL2);
	reg = reg | 0x01;
	outb_p( reg, acb_data->iobase + ACBCTL2 );

	acb_reset(acb_data);
}

static void acb_start_clock(struct i2c_algo_acb_data *acb_data)
{
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_start_clock()\n"));
	reg = inb_p(acb_data->iobase + ACBCTL2);
	reg = reg | 0x01;
	outb_p( reg, acb_data->iobase + ACBCTL2 );
}

static void acb_stop_clock(struct i2c_algo_acb_data *acb_data)
{	
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_stop_clock()\n"));
	reg = inb_p(acb_data->iobase + ACBCTL2);
	reg = reg & 0xFE;
	outb_p( reg, acb_data->iobase + ACBCTL2 );
}

static void acb_stall_after_start(struct i2c_algo_acb_data *acb_data, unsigned char state )
{
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_stall_after_start()\n"));
	reg = inb_p(acb_data->iobase + ACBCTL1);
	if ( state)
		reg = reg | 0x80;
	else
		reg = reg & 0x7f;
	outb_p( reg, acb_data->iobase + ACBCTL1 );

	if ( ! state)
	{
		reg = inb_p(acb_data->iobase + ACBST);
		reg = reg | 0x08;
		outb_p( reg, acb_data->iobase + ACBST );
	}
}

static void acb_clear_push_nak(struct i2c_algo_acb_data *acb_data)
{
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_clear_push_nak()\n"));
	// Push Ack onto I2C bus
	reg = inb_p(acb_data->iobase + ACBCTL1);
	reg = reg & 0xE7;
	outb_p( reg, acb_data->iobase + ACBCTL1 );
}

static void acb_set_push_nak(struct i2c_algo_acb_data *acb_data)
{
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_set_push_nak()\n"));
	// Push negAck onto I2C bus
	reg = inb_p(acb_data->iobase + ACBCTL1);
	reg = reg | 0x10;
	outb_p( reg, acb_data->iobase + ACBCTL1 );
} 

static void acb_abort_data(struct i2c_algo_acb_data *acb_data)
{
	TRACE(printk("i2c-nscacb.o acb_abort_data()\n"));
	acb_stop(acb_data);
	acb_set_push_nak(acb_data);
}

static void acb_bus_recovery(struct i2c_algo_acb_data *acb_data)
{
	TRACE(printk("i2c-nscacb.o acb_bus_recovery()\n"));
	acb_stop(acb_data);
	acb_set_push_nak(acb_data);
	acb_reset(acb_data);
}

static int acb_request_master(struct i2c_algo_acb_data *acb_data)
{
	unsigned long start;
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_request_master()\n"));

	acb_start(acb_data);
	start = jiffies;
	while (1) {
		reg = inb_p(acb_data->iobase + ACBST);
		if (reg & 0x60)		// check SDAST and BER
			break;
		
		if (jiffies > (start+MAX_TIMEOUT))
		{
			DEBERR(printk("i2c-nscacb.o: Timeout in acb_request_master\n"));
			acb_bus_recovery(acb_data);
			return 0;
		}
		udelay(1000); //microseconds
	}
	if ((reg & 0x20) != 0) {		// bus error (BER == 1)
		DEBERR(printk("i2c-nscacb.o: BER in acb_request_master\n"));
		acb_abort_data(acb_data);
		return 0;
	}
	if ((reg & 0x02) == 0) {		// Master request (MASTER != 1)
		DEBERR(printk("i2c-nscacb.o: Arbitration loss in acb_request_master\n"));
		acb_abort_data(acb_data);
		return 0;
	}
	return 1;
}

static int acb_wait_for_ack(struct i2c_algo_acb_data *acb_data)
{
	unsigned long start;
	unsigned char reg;

	TRACE(printk("i2c-nscacb.o acb_wait_for_ack()\n"));
	// Receive Ack from I2C bus
	start = jiffies;
	while (1) {
		reg = inb_p(acb_data->iobase + ACBST);
		if ((reg & 0x70) != 0)		// check SDAST, BER and NEGACK
			break;
		if (jiffies > (start+MAX_TIMEOUT))
		{
			DEBERR(printk("Timeout in acb_wait_for_ack\n"));
			acb_bus_recovery(acb_data);
			return 0;
		}
		//udelay(1000); //microseconds
	}
	if ((reg & 0x20) != 0) {		// bus error (BER == 1)
		acb_bus_recovery(acb_data);
		return 0;
	}
	if ((reg & 0x10) != 0) {		// negative acknowledge (NEGACK == 1)
		acb_abort_data(acb_data);
		return 0;
	}
	return 1;
}

static inline int acb_send_address(struct i2c_algo_acb_data *acb_data,
        struct i2c_msg *msg, int retries) 
{
	unsigned short flags = msg->flags;
	unsigned long start;
	unsigned char addr;
	unsigned char reg;
	int ret;

	DEB1(printk("i2c-nscacb.o acb_send_address()\n"));
	if ( (flags & I2C_M_TEN)  ) {
		return 0;
	}

	/* normal 7bit address	*/
	addr = ( msg->addr << 1 );
	if (flags & I2C_M_RD )
		addr |= 1;
	if (flags & I2C_M_REV_DIR_ADDR )
		addr ^= 1;

	acb_stall_after_start(acb_data, 1);

	// Write the data
	outb_p( addr, acb_data->iobase + ACBSDA );
	start = jiffies;
	while (1) {
		reg = inb_p(acb_data->iobase + ACBST);
		if ((reg & 0x38) != 0)	// check STASTR, BER and NEGACK
			break;

		if (jiffies > (start+MAX_TIMEOUT))
		{
			DEBERR(printk("Timeout in acb_send_address\n"));
			acb_bus_recovery(acb_data);
			return 0;
		}
		udelay(1000); //microseconds
	}

	if ((reg & 0x20) != 0) {	
		// bus error (BER == 1)
		acb_bus_recovery(acb_data);
		return 0;
	}
	if ((reg & 0x10) != 0) {
		// negative acknowledge (NEGACK == 1)
		acb_abort_data(acb_data);
		return 0;
	}
	acb_stall_after_start(acb_data, 0);
	
	return 1;
}

static int acb_sendbytes(struct i2c_adapter *adapter, const char *buf, int count)
{
	struct i2c_algo_acb_data *acb_data = adapter->algo_data;
	unsigned long start;
	unsigned char reg;
	int wrcount, status, timeout;
    
	TRACE(printk("i2c-nscacb.o acb_sendbytes()\n"));
	for (wrcount=0; wrcount<count; ++wrcount) {
		DEBERR(printk("i2c-nscacb.o: %s i2c_write: writing %2.2X\n",
		      adapter->name, buf[wrcount]&0xff));

		// wait for data ready to write
		start = jiffies;
		while (1) {
			reg = inb_p(acb_data->iobase + ACBST);
			if ((reg & 0x70) != 0)		// check SDAST, BER and NEGACK
				break;
			if (jiffies > (start+MAX_TIMEOUT))
			{
				DEBERR(printk("i2c-nscacb.o: "
					"Timeout waiting for ready in acb_sendbytes\n"));
				acb_bus_recovery(acb_data);
				return (-EREMOTEIO);
			}
			udelay(1000); //microseconds
		}
		if ((reg & 0x20) != 0) {		// bus error (BER == 1)
			acb_bus_recovery(acb_data);
			return (-EREMOTEIO);
		}
		if ((reg & 0x10) != 0) {		// bus error (BER == 1)
			acb_abort_data(acb_data);
			return (-EREMOTEIO);
		}

		outb_p( buf[wrcount], acb_data->iobase + ACBSDA );

		if(! acb_wait_for_ack(acb_data)) {
			printk("i2c-nscacb.o: %s acb_sendbytes: "
			       "error - timeout.\n", adapter->name);
			return (-EREMOTEIO);
		}
	}
	return (wrcount);
}

static int acb_readbytes(struct i2c_adapter *adapter, char *buf, int count)
{
	int rdcount=0, i, status, timeout, dummy=1;
	struct i2c_algo_acb_data *acb_data = adapter->algo_data;
	unsigned char reg;
	unsigned long start;

	DEB1(printk("i2c-nscacb.o acb_readbytes()\n"));
	for (i=0; i<count; ++i) {
 
		if(i==(count-1)) acb_set_push_nak(acb_data);
		//else acb_clear_push_nak(acb_data);

		// wait for data ready to read
		start = jiffies;
		while (1) {
			reg = inb_p(acb_data->iobase + ACBST);
			if ((reg & 0x60) != 0)		// check SDAST, BER and NEGACK
				break;
			if (jiffies > (start+MAX_TIMEOUT))
			{
				DEBERR(printk("Timeout waiting for data ready failed in acb_readbytes\n"));
				acb_bus_recovery(acb_data);
				return (-EREMOTEIO);
			}
			udelay(1000); //microseconds
		}
		if ((reg & 0x20) != 0) {		// bus error (BER == 1)
			acb_bus_recovery(acb_data);
			return (-EREMOTEIO);
		}

		// 
		acb_stop_clock(acb_data);
		buf[rdcount] = inb_p(acb_data->iobase + ACBSDA);
		rdcount++;
		acb_start_clock(acb_data);
	}

	return (rdcount);
}

/***********************************************/ 
/*  nscacb_master_xfer                         */
/***********************************************/ 

static int nscacb_master_xfer(struct i2c_adapter *adapter,
		    struct i2c_msg msgs[], 
		    int num)
{
	struct i2c_algo_acb_data *acb_data = adapter->algo_data;
	struct i2c_msg *pmsg;
	int msgidx = 0;
	int ret, timeout, status;
    
	DEB1(printk("i2c-nscacb.o nscacb_master_xfer()\n"));

 	if( ! acb_request_master(acb_data)) 
	{
		return -EIO;
	}

	acb_clear_push_nak(acb_data);
   
	msgidx=0;
	for(msgidx=0; msgidx<num; msgidx++) {
		pmsg = &msgs[msgidx];

		if(msgidx>0 && (pmsg->flags & I2C_M_RD)) {
			acb_start(acb_data);
			acb_set_push_nak(acb_data);
		}

		if( !(ret = acb_send_address(acb_data, pmsg, adapter->retries))) {
			DEBERR(printk("i2c-nscacb.o: " 
			"acb_send_address failed in acb_master_xfer\n"));
			acb_stop(acb_data); // mpj
			acb_bus_recovery(acb_data); //mpj
			return (-EREMOTEIO);
		}
    
		/* Check for bus busy */
		if(! acb_wait_for_ack(acb_data)) {
			DEBERR(printk("i2c-nscacb.o: " 
			"wait_for_ack failed in acb_master_xfer\n"));
			acb_stop(acb_data); // mpj
			return (-EREMOTEIO);
		}
    
		DEB1(printk("i2c-nscacb.o: Msg %d, addr=0x%x, flags=0x%x, len=%d\n",
		    msgidx, msgs[msgidx].addr, msgs[msgidx].flags, msgs[msgidx].len));
    
		if (pmsg->flags & I2C_M_RD) {
			/* Read */
			/* read bytes into buffer*/

			//acb_set_push_nak(acb_data);

			ret = acb_readbytes(adapter, pmsg->buf, pmsg->len);
			if (ret != pmsg->len) {
				DEBERR(printk("i2c-nscacb.o: fail: "
							"only read %d bytes.\n",ret));
			} else {
				DEBERR(printk("i2c-nscacb.o: read %d bytes.\n",ret));
			}
		} 
		else { 
			/* Write */
		/* Write bytes from buffer */
			ret = acb_sendbytes(adapter, pmsg->buf, pmsg->len);
	
			if (ret != pmsg->len) {
				DEBERR(printk("i2c-nscacb.o: fail: "
							"only wrote %d bytes.\n",ret));
			} else {
				DEBERR(printk("i2c-nscacb.o: wrote %d bytes.\n",ret));
			}
		}
	}

	acb_stop(acb_data);
	return (num);
}

static int nscacb_algo_control(struct i2c_adapter *adapter, 
	unsigned int cmd, unsigned long arg)
{
	DEB1(printk("i2c-nscacb.o nscacb_algo_control()\n"));

	return 0;
}

/***********************************************/
/*  NSC ACB Adapter Specific Module            */
/***********************************************/ 

struct i2c_algo_acb_data nscacb_data[2] = 
{
	{
		/* iobase */   0x810,
		/* acbindex */ 0,
		/* deviceid */ 0,
		/* status */   0
	},
	{
		/* iobase */   0x820,
		/* acbindex */ 1,
		/* deviceid */ 0,
		/* status */   0
	}
};

/* There can only be two adapters in any pc */
static struct i2c_adapter nscacb_adapter[2] = {
	{
		/* name */ "NSC SC1200 ACB adapter (ACB1)",
		/* id */ I2C_ALGO_ACB | I2C_HW_ACB_SC1200,
		/* algorithm */ &nscacb_algorithm,
		/* algo_data */ (void*) &nscacb_data[0],
		/* inc_use */ &nscacb_inc_use,
		/* dec_use */ &nscacb_dec_use,
		/* data */ NULL,
		/* Other fields not initialized */
	},
	{
		/* name */ "NSC SC1200 ACB adapter (ACB2)",
		/* id */ I2C_ALGO_ACB | I2C_HW_ACB_SC1200,
		/* algorithm */ &nscacb_algorithm,
		/* algo_data */ (void*) &nscacb_data[1],
		/* inc_use */ &nscacb_inc_use,
		/* dec_use */ &nscacb_dec_use,
		/* data */ NULL,
		/* Other fields not initialized */
	}
};

/* Used in nscacb_init/cleanup */
static int __initdata nscacb_initialized;

void nscacb_inc_use(struct i2c_adapter *adapter)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

void nscacb_dec_use(struct i2c_adapter *adapter)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

int __init nscacb_init(void)
{
	struct pci_dev *dev;
	int num = 0;
	int geode_id=0;
	int res;

	printk("i2c-nscacb.o version %s\n", ACB_I2C_VERSION);

#ifdef DEBUG
	if (nscacb_initialized) {
		printk
		    ("i2c-nscacb.o: Oops, nscacb_init called a second time!\n");
		return -EBUSY;
	}
#endif
	nscacb_initialized = 0;
	geode_id = 0;
	dev = NULL;

	do {
		if ((dev = pci_find_device(PCI_VENDOR_ID_NS,
					   PCI_DEVICE_ID_NS_SC1200_BRIDGE,
					   dev))) {
			if (!num) {
				geode_id = 0x1200;
			}
			num++;
		}
	} while (dev);

	dev = NULL;
	do {
		if ((dev = pci_find_device(PCI_VENDOR_ID_CYRIX,
					   PCI_DEVICE_ID_CYRIX_SC1400_BRIDGE, 
					   dev))) {
			if (!num) {
				geode_id = 0x1400;
			}
			num++;
		}
	} while (dev);

	if (num <= 0) {
		return -ENODEV;
	}
	
	if (num > 1) {
		DEB1(KERN_INFO
	       "i2c-nscacb.o: warning: too many devices found. only 1 supported.\n");
	}

	DEB1(printk("i2c-nscacb.o: %d nsc%04x found.\n", num, geode_id));

	// initailize adapter structure.
	if(geode_id == 0x1400) {
		strcpy(nscacb_adapter[0].name, "NSC SC1400 ACB adapter (ACB1)");
		strcpy(nscacb_adapter[1].name, "NSC SC1400 ACB adapter (ACB2)");
		nscacb_adapter[0].id = I2C_ALGO_ACB | I2C_HW_ACB_SC1400;
		nscacb_adapter[1].id = I2C_ALGO_ACB | I2C_HW_ACB_SC1400;
	}
    else {
		strcpy(nscacb_adapter[0].name, "NSC SC1200 ACB adapter (ACB1)");
		strcpy(nscacb_adapter[1].name, "NSC SC1200 ACB adapter (ACB2)");
		nscacb_adapter[0].id = I2C_ALGO_ACB | I2C_HW_ACB_SC1200;
		nscacb_adapter[1].id = I2C_ALGO_ACB | I2C_HW_ACB_SC1200;
    }
	nscacb_data[0].iobase = 0x810;
	nscacb_data[0].acbindex = 0;
	nscacb_data[0].deviceid = geode_id;
	nscacb_data[0].status = 0;
	nscacb_data[1].iobase = 0x820;
	nscacb_data[1].acbindex = 1;
	nscacb_data[1].deviceid = geode_id;
	nscacb_data[1].status = 0;
	if(geode_id == 0x1400) {
		nscacb_data[0].iobase = 0x810;
		nscacb_data[1].iobase = 0x820;
	}

	if(0 && geode_id == 0x1400) {
		// 1400 has only one ACB
	}
	else if(check_region(nscacb_data[0].iobase, 8) < 0) {
		printk("i2c-nscacb.o: ACB1 Adapter io-base %03X is in use\r\n",
			nscacb_data[0].iobase);
	}
	else {
		if ((res = i2c_add_adapter(&nscacb_adapter[0]))) {
			printk("i2c-nscacb.o: ACB1 Adapter registration failed , "
		       	"module i2c-nscacb.o is not inserted\n.");
		}
		else {
			printk("i2c-nscacb.o: nscacb bus ACB1 for i2c modules initialized.\n");
			request_region(nscacb_data[0].iobase, 8, "ACB1");
			nscacb_initialized |= INIT_ACB1;
			acb_init_hardware(nscacb_adapter[0].algo_data);
		}
	}

	if(nscacb_data[0].iobase == nscacb_data[1].iobase) {
		// do nothing
	}
	else if(check_region(nscacb_data[1].iobase, 8) < 0) {
		printk("i2c-nscacb.o: ACB2 Adapter io-base %03X is in use\r\n",
			nscacb_data[1].iobase);
	}
	else {
		if ((res = i2c_add_adapter(&nscacb_adapter[1]))) {
			printk("i2c-nscacb.o: ACB2 Adapter registration failed , "
		       	"module i2c-nscacb.o is not inserted\n.");
		}
		else {
			printk("i2c-nscacb.o: nscacb bus ACB2 for i2c modules initialized.\n");
			request_region(nscacb_data[1].iobase, 8, "ACB2");
			nscacb_initialized |= INIT_ACB2;
			acb_init_hardware(nscacb_adapter[1].algo_data);
		}
	}

	if(!(nscacb_initialized & (INIT_ACB1 | INIT_ACB2))) {
		printk("i2c-nscacb.o: Both registrations failed, module not inserted\n");
		nscacb_cleanup();
		return res;
	}
	return 0;
}

int __init nscacb_cleanup(void)
{
	int res;

	if (nscacb_initialized & INIT_ACB1) {
		if ((res = i2c_del_adapter(&nscacb_adapter[0]))) {
			printk
			    ("i2c-nscacb.o: Adapter ACB1 deregistration failed, module not removed.\n");
			return res;
		} else {
			nscacb_initialized &= (~INIT_ACB1);
			release_region(nscacb_data[0].iobase, 8);
		}
	}

	if (nscacb_initialized & INIT_ACB2) {
		if ((res = i2c_del_adapter(&nscacb_adapter[1]))) {
			printk("i2c-nscacb.o: Adapter ACB2 deregistration failed, module not removed.\n");
			return res;
		} else {
			nscacb_initialized &= (~INIT_ACB2);
			release_region(nscacb_data[1].iobase, 8);
		}
	}
	return 0;
}

EXPORT_NO_SYMBOLS;

#ifdef MODULE

MODULE_AUTHOR("Frodo Looijaard <frodol@dds.nl>");
MODULE_DESCRIPTION("Nsc ACCESS.bus access through i2c");

int init_module(void)
{
	return nscacb_init();
}

int cleanup_module(void)
{
	return nscacb_cleanup();
}

#endif	/* MODULE */


/***********************************************/
/*  end of file                                */
/***********************************************/ 
