/*
 * $Workfile: galfns.c $
 * $Revision: 30 $
 *
 * File Contents: This file contains the main functions of the Geode 
 *                frame buffer device drivers GAL function definitions.
 *
 * Project:       Geode Frame buffer device driver
 *
 */

/* 
 * NSC_LIC_ALTERNATIVE_PREAMBLE
 *
 * Revision 1.0
 *
 * National Semiconductor Alternative GPL-BSD License
 *
 * National Semiconductor Corporation licenses this software 
 * ("Software"):
 *
 * Geode frame buffer driver
 *
 * under one of the two following licenses, depending on how the 
 * Software is received by the Licensee.
 * 
 * If this Software is received as part of the Linux Framebuffer or
 * other GPL licensed software, then the GPL license designated 
 * NSC_LIC_GPL applies to this Software; in all other circumstances 
 * then the BSD-style license designated NSC_LIC_BSD shall apply.
 *
 * END_NSC_LIC_ALTERNATIVE_PREAMBLE */


/* NSC_LIC_BSD
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * Geode frame buffer driver
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * END_NSC_LIC_BSD */


/* NSC_LIC_GPL
 *
 * National Semiconductor Corporation Gnu General Public License for 
 *
 * Geode frame buffer driver
 *
 * (GPL License with Export Notice)
 *
 * Copyright (c) 1999-2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted under the terms of the GNU General 
 * Public License as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later version  
 *
 * In addition to the terms of the GNU General Public License, neither 
 * the name of the National Semiconductor Corporation nor the names of 
 * its contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE, 
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE. See the GNU General Public License for more details. 
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this file; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * END_NSC_LIC_GPL */

#ifndef XFree86LOADER
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#endif

#include "galproto.h"

/*
 * Compile time constants
 */
#define FBDEV_NAME  "/dev/nscgal"

/*
 * Cool Macros to access the structures
 */
#define INIT_GAL(x) \
		((GAL_BASE *)(x))->dwSignature = FBGAL_SIGNATURE;\
		((GAL_BASE *)(x))->dwSize = sizeof(x);\
		((GAL_BASE *)(x))->dwVersion = FBGAL_VERSION;
/*
 * Variables public for this file
 */
static int ifbdev_handle;

/*------------------------------------------------------------------------
 * create_devicenode
 *
 * Description:	This function creates nscgal device node in the device
 *					directory.
 * parameters :	none
 *	
 *	return:	'0' was return on creating the galdevice node.
 *----------------------------------------------------------------------*/
int create_devicenode()
{

#if 1
	FILE *pfdevices;
	char line[200], devname[200];
	int majdev;

	/* remove fails if device is open */
	remove("/dev/nscgal");
	
	if ((pfdevices = fopen("/proc/devices","r"))) {
		while(fgets(line, sizeof(line), pfdevices)) {
			if (sscanf(line, "%d%*[ \t]%s", &majdev, devname) == 2) {
				if (strstr(devname, "nscgal"))
					mknod("/dev/nscgal", S_IFCHR|S_IRUSR|S_IWUSR, 
					makedev(majdev,0));       
            }
		}
		fclose(pfdevices);
	}
	return 1;
#endif

}

/*------------------------------------------------------------------------
 * Gal_initialize_interface 
 *
 * Description:	This function intializes the  nscgal device .
 * parameters :	none
 *	
 *	return:	 '1' was returned on intialization of the galdevice 
 *					otherwise '0' was returned on failure.
 *----------------------------------------------------------------------*/
BOOLEAN Gal_initialize_interface()
{
	create_devicenode();
	if ((ifbdev_handle = open(FBDEV_NAME, O_RDONLY)) == -1)
		return 0;
	return 1;
}

/*------------------------------------------------------------------------
 * Gal_cleanup_interface
 *
 * Description:	This function closes the  nscgal device .
 * parameters :	none
 *	
 *	return:	 '1' was returned on closing the galdevice.
 *----------------------------------------------------------------------*/
BOOLEAN Gal_cleanup_interface()
{
	if (ifbdev_handle != -1)
		close(ifbdev_handle);
	return 1;
}

/*---------------------------------------------------------------------------
 * Gal_write_register
 *
 * Description:	This function writes the data to the hardware register
 *					of the nscgal device .
 *  parameters:	
 *        type:	It specifies the hardware access type.
 *      offset: It specifies the offset address the register to be accessed.
 *       value:	It specifies the data value to be written into the register.
 *        size: It specifies the size of the data to be written. 
 *	
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_write_register(int type, unsigned long offset, unsigned long value,	
						   int size)
{
	GAL_HWACCESS hwAccess;

	INIT_GAL(&hwAccess);
	hwAccess.dwSubfunction = GALFN_WRITEREG;
	hwAccess.dwType = type;
	hwAccess.dwOffset = offset;
	hwAccess.dwValue = value;
	hwAccess.dwByteCount = size;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &hwAccess))
		return 0;
	else {
		return 1;
	}   
}

/*---------------------------------------------------------------------------
 * Gal_read_register
 *
 * Description:	This function reads the data from the hardware register
 *					of the nscgal device .
 *  parameters:	
 *        type:	It specifies the hardware access type.
 *      offset: It specifies the offset address of the register to be accessed.
 *       value:	It specifies the pointer to hold the data  to be read from
 *					the gal hardware register.
 *        size: It specifies the size of the data to be read
 *	
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_read_register(int type, unsigned long offset, unsigned long *value,
						  int size)
{
	GAL_HWACCESS hwAccess;
	
	INIT_GAL(&hwAccess);
	hwAccess.dwSubfunction = GALFN_READREG;
	hwAccess.dwType = type;
	hwAccess.dwOffset = offset;
	hwAccess.dwByteCount = size;
    
    if (ioctl(ifbdev_handle, FBIOGAL_API, &hwAccess))
		return 0;
	else {
		*value = hwAccess.dwValue;
		return 1;
	}   
}

/*---------------------------------------------------------------------------
 * Gal_get_adapter_info
 *
 * Description:	This function gets the adapter information of the
 *					nscgal device .
 *  parameters:	
 *pAdapterInfo: It specifies the adapter information structure.     
 *	
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_adapter_info(PGAL_ADAPTERINFO pAdapterInfo)
{
	INIT_GAL(pAdapterInfo);
	
	pAdapterInfo->dwSubfunction = GALFN_GETADAPTERINFO;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, pAdapterInfo))
		return 0;
	else {
		return 1;
	}   
}

/*---------------------------------------------------------------------------
 * Gal_set_softvga_state 
 *
 * Description:	This function sets the softvga state of the platform device .
 *  parameters:	
 *     bEnable:	It specifies the softvga state enable state.    
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_softvga_state(BOOLEAN bEnable)
{
	GAL_SOFTVGASTATE sSoftVgaState;
	
    INIT_GAL(&sSoftVgaState);
	sSoftVgaState.dwSubfunction = GALFN_SETSOFTVGASTATE;
	sSoftVgaState.bSoftVgaEnable = bEnable;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sSoftVgaState))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_get_softvga_state 
 *
 * Description:	This function gets the softvga state of the platform device .
 *  parameters:	
 *     bEnable:	get the softvga state.    
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_softvga_state(int *bState)
{
	GAL_SOFTVGASTATE sSoftVgaState;
	
    INIT_GAL(&sSoftVgaState);
	sSoftVgaState.dwSubfunction = GALFN_GETSOFTVGASTATE;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sSoftVgaState))
		return 0;
	else {
		*bState = sSoftVgaState.bSoftVgaEnable;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_set_crt_state
 *
 * Description:	This function sets the crt state of the device .
 *  parameters:	
 *    crtState:	It specifies the crt state of the galdevice.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_crt_state(int crtState)
{
	GAL_CRTSTATE sCrtState;
	
	INIT_GAL(&sCrtState);
	sCrtState.dwSubfunction = GALFN_SETCRTSTATE;
	sCrtState.wCrtState = crtState;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sCrtState))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_is_display_mode_supported 
 *
 * Description:	This function checks the display mode is supported or not.
 *  parameters:	
 *        xres:	It specifies x co-ordinate resolution.
 *        Yres:	It specifies y co-ordinate resolution.  
 *	   bpp:	It specifies the bits per pixel (8/16 bits).
 *	    hz:	It specifies the frequency of the display mode.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_is_display_mode_supported(int xres, int yres, int bpp, int hz,
									  int *supported)
{
	GAL_DISPLAYMODE sDisplayMode;
	
	*supported = 0;
	INIT_GAL(&sDisplayMode);
	sDisplayMode.dwSubfunction = GALFN_ISDISPLAYMODESUPPORTED;
	sDisplayMode.wXres = xres;
	sDisplayMode.wYres = yres;
	sDisplayMode.wBpp = bpp;
	sDisplayMode.wRefresh = hz;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayMode))
		return 0;
	else
	{
		*supported = sDisplayMode.dwSupported;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_set_display_mode
 *
 * Description:	This function sets the display mode of the galdevice.
 *  parameters:	
 *        xres:	It specifies x co-ordinate resolution.
 *        Yres:	It specifies y co-ordinate resolution.  
 *	   bpp:	It specifies the bits per pixel (8/16 bits).
 *	    hz:	It specifies the frequency of the display mode.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_display_mode(int xres, int yres, int bpp, int hz)
{
	GAL_DISPLAYMODE sDisplayMode;
	INIT_GAL(&sDisplayMode);
	sDisplayMode.dwSubfunction = GALFN_SETDISPLAYMODE;
	sDisplayMode.wXres = xres;
	sDisplayMode.wYres = yres;
	sDisplayMode.wBpp = bpp;
	sDisplayMode.wRefresh = hz;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayMode))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_get_display_mode
 *
 * Description:	This function gets the display mode of the galdevice.
 *  parameters:	
 *        xres:	It specifies x co-ordinate resolution.
 *        Yres:	It specifies y co-ordinate resolution.  
 *	   bpp:	It specifies the bits per pixel (8/16 bits).
 *	    hz:	It specifies the frequency of the display mode.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_display_mode(int *xres, int *yres, int *bpp, int *hz)
{
	GAL_DISPLAYMODE sDisplayMode;
	
    INIT_GAL(&sDisplayMode);
	sDisplayMode.dwSubfunction = GALFN_GETDISPLAYMODE;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayMode))
		return 0;
    else
	{
		*xres = sDisplayMode.wXres;   
		*yres = sDisplayMode.wYres;   
		*bpp = sDisplayMode.wBpp;    
		*hz = sDisplayMode.wRefresh;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_set_display_bpp
 *
 * Description:	This function sets the number bits per pixel in the display
 *					mode of the galdevice.
 *  parameters:	
 *	   bpp:	It specifies the bits per pixel (8/16 bits).
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_display_bpp(unsigned short bpp)
{
	GAL_DISPLAYPARAMS sDisplayParams;
	 
	INIT_GAL(&sDisplayParams);
	sDisplayParams.dwSubfunction = GALFN_SETDISPLAYBPP;
	sDisplayParams.wBpp = bpp;
	
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayParams))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_get_display_bpp
 *
 * Description:	This function gets the number bits per pixel in the display
 *					mode of the galdevice.
 *  parameters:	
 *	   bpp:	It specifies the bits per pixel (8/16 bits).
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_display_bpp(unsigned short *bpp)
{
	GAL_DISPLAYPARAMS sDisplayParams;
    
	INIT_GAL(&sDisplayParams);
	sDisplayParams.dwSubfunction = GALFN_GETDISPLAYBPP;

	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayParams))
		return 0;
	else {
		*bpp = sDisplayParams.wBpp;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_set_display_pitch
 *
 * Description:	This function sets the display pitch of the galdevice.
 *  parameters:	
 *       pitch:	It specifies pitch of the display mode.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_display_pitch(unsigned short pitch)
{
	GAL_DISPLAYPARAMS sDisplayParams;
    
	INIT_GAL(&sDisplayParams);
	sDisplayParams.dwSubfunction = GALFN_SETDISPLAYPITCH;
	sDisplayParams.wPitch = pitch;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayParams))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_get_display_pitch
 *
 * Description:	This function gets the display pitch of the galdevice.
 *  parameters:	
 *       pitch:	It specifies pitch of the display mode.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_display_pitch(unsigned short *pitch)
{
	GAL_DISPLAYPARAMS sDisplayParams;
 
	INIT_GAL(&sDisplayParams);
	sDisplayParams.dwSubfunction = GALFN_GETDISPLAYPITCH;

	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayParams))
		return 0;
	else {
		*pitch = sDisplayParams.wPitch;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_set_display_offset
 *
 * Description:	This function sets the offset of display parameters.
 *  parameters:	
 *      offset:	It specifies the offset address of display parameters.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_display_offset(unsigned long offset)
{
	GAL_DISPLAYPARAMS sDisplayParams;
    
	INIT_GAL(&sDisplayParams);
	sDisplayParams.dwSubfunction = GALFN_SETDISPLAYOFFSET;
	sDisplayParams.dwOffset = offset;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayParams))
		return 0;
	else
		return 1;    
}

/*---------------------------------------------------------------------------
 * Gal_get_display_offset
 *
 * Description:	This function gets the offset of display parameters.
 *  parameters:	
 *      offset:	It specifies the offset address of display parameters.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_display_offset(unsigned long *offset)
{
	GAL_DISPLAYPARAMS sDisplayParams;
    
	INIT_GAL(&sDisplayParams);
	sDisplayParams.dwSubfunction = GALFN_GETDISPLAYOFFSET;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDisplayParams))
		return 0;
	else {
		*offset = sDisplayParams.dwOffset;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_get_refreshrate_from_dotclock
 *
 * Description:	This function gets the refreshrate from dotclock.
 *  parameters:	
 *        xres:	It specifies x co-ordinate resolution.
 *        Yres:	It specifies y co-ordinate resolution.  
 *	   bpp:	It specifies the bits per pixel (8/16 bits).
 *	    hz:	It is a pointer which holds the refresh rate of the display.
 *   frequency:	It spcifies the frequency of the dotclock.
 *	return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_refreshrate_from_dotclock(int xres, int yres, int bpp, int *hz,
										  unsigned long frequency)
{
	GAL_DOTCLKTOREFRESH sDclkToRefresh;
    
	INIT_GAL(&sDclkToRefresh);
	sDclkToRefresh.dwSubfunction = GALFN_DOTCLKTOREFRESH;
	sDclkToRefresh.wXres = xres;
	sDclkToRefresh.wYres = yres;
	sDclkToRefresh.wBpp = bpp;
	sDclkToRefresh.dwDotClock = frequency;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sDclkToRefresh))
		return 0;
	else {
		*hz = sDclkToRefresh.wRefreshRate;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_get_display_timing
 *
 *    Description:	This function gets the display timing from galdevice.
 *     parameters:	
 * pDisplayTiming:	It specifies the display timing of the galdevice.
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_display_timing(PGAL_DISPLAYTIMING pDisplayTiming)
{
	INIT_GAL(pDisplayTiming);
	pDisplayTiming->dwSubfunction = GALFN_GETDISPLAYTIMINGS;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, pDisplayTiming))
		return 0;
	else {
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_set_display_timing
 *
 *    Description:	This function sets the display timing of the galdevice. 
 *     parameters:	
 * pDisplayTiming:	It specifies the display timing of the galdevice.
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_display_timing(PGAL_DISPLAYTIMING pDisplayTiming)
{
	INIT_GAL(pDisplayTiming);
	pDisplayTiming->dwSubfunction = GALFN_SETDISPLAYTIMINGS;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, pDisplayTiming))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_set_fixed_timings
 *
 *    Description:	This function sets the fixed display timings of the 
 *						galdevice. 
 *     parameters:
 *	  pnlXres:	It specifies the panel X resolution.
 *	  pnlYres:	It specifies the panel Y resolution.
 *        totXres:  It specifies the total X resolution.  
 *        totYres:  It specifies the total Y resolution.
 *	      bpp:	It specifies the bits per pixel (8/16 bits).
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_fixed_timings( int pnlXres, int pnlYres,int totXres,
								int totYres, int bpp)
{
	GAL_DISPLAYTIMING DisplayTiming;

	INIT_GAL(&DisplayTiming);
	DisplayTiming.dwSubfunction = GALFN_SETFIXEDTIMINGS;
	DisplayTiming.wHActive = pnlXres; /* panel Xres */
	DisplayTiming.wVActive = pnlYres; /* panel Yres */
	DisplayTiming.wHTotal = totXres; /* Total Xres */
	DisplayTiming.wVTotal = totYres; /* Total Yres */
	DisplayTiming.wBpp = bpp;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &DisplayTiming))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_set_display_palette_entry
 *
 *    Description:	This function sets the display palette entry of the 
 *						galdevice. 
 *     parameters:
 *	    index:	It specifies the palette index,
 *        palette:	It specifies the palette of the galdevice. 
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_display_palette_entry(unsigned long index, 
        unsigned long palette)
{
	GAL_PALETTE_ENTRY sPalette;
    
	INIT_GAL(&sPalette);
	sPalette.dwSubfunction = GALFN_SETPALETTE_ENTRY;
	sPalette.dwIndex = index;
	sPalette.dwPalette = palette;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sPalette))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_get_display_palette_entry
 *
 *    Description:	This function gets the display palette entry of the 
 *						galdevice. 
 *     parameters:
 *          index:	It specifies the palette index,
 *        palette:	It is a pointer to the palette of the galdevice. 
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_display_palette_entry(unsigned long index, unsigned long *palette)
{
	GAL_PALETTE_ENTRY sPalette;
    
	INIT_GAL(&sPalette);
	sPalette.dwSubfunction = GALFN_GETPALETTE_ENTRY;
	sPalette.dwIndex = index;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sPalette))
		return 0;
	else {
		*palette = sPalette.dwPalette;
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_set_display_palette_entry
 *
 *    Description:	This function sets the display palette entry of the 
 *						galdevice. 
 *     parameters:
 *       pPalette:	It specifies the palette structure of the galdevice. 
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_display_palette(PGAL_PALETTE pPalette)
{
	INIT_GAL(pPalette);
	pPalette->dwSubfunction = GALFN_SETPALETTE;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, pPalette))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_get_display_palette_entry
 *
 *    Description:	This function gets the display palette entry of the 
 *						galdevice. 
 *     parameters:
 *       pPalette:	It specifies the palette structure of the galdevice. 
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_get_display_palette(PGAL_PALETTE pPalette)
{
	INIT_GAL(pPalette);
	pPalette->dwSubfunction = GALFN_GETPALETTE;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, pPalette))
		return 0;
	else {
		return 1;
	}
}

/*---------------------------------------------------------------------------
 * Gal_wait_until_idle
 *
 *    Description:	This function waits until the graphics engine is idle. 
 *     parameters:	none.
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_wait_until_idle(void)
{
	GAL_WAITUNTILIDLE sWaitIdle;
    
	INIT_GAL(&sWaitIdle);
	sWaitIdle.dwSubfunction = GALFN_WAITUNTILIDLE;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sWaitIdle))
		return 0;
	else
		return 1;
}

/*---------------------------------------------------------------------------
 * Gal_wait_vertical_blank
 *
 *    Description:	This function wait until start of vertical blank.
 *     parameters:	none.
 *	   return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_wait_vertical_blank(void)
{
	GAL_WAITVERTICALBLANK sWaitVerticalBlank;
    
	INIT_GAL(&sWaitVerticalBlank);
	sWaitVerticalBlank.dwSubfunction = GALFN_WAITVERTICALBLANK;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sWaitVerticalBlank))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_set_cursor_enable
 *
 * Description:	This function enable or disable the hardware cursor.
 *  parameters:
 *      enable:	This specifies the enable value of the cursor.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/
BOOLEAN Gal_set_cursor_enable(int enable)
{
        GAL_CURSORENABLE sCursorEnable;
    
	INIT_GAL(&sCursorEnable);
	sCursorEnable.dwSubfunction = GALFN_SETCURSORENABLE;
	sCursorEnable.bCursorEnable = enable ? 1 : 0;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sCursorEnable))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_set_cursor_position
 *
 * Description:	This function sets the position of the cursor.
 *  parameters:
 *   memoffset:	It specifies the memory offset of the cursor position.
 *        xpos: It specifies the X co-ordinate position of the cursor.
 *        ypos: It specifies the Y co-ordinate position of the cursor.
 *    xhotspot: It specifies the X hotspot location for current cursor shape.
 *    yhotspot: It specifies the Y hotspot location for current cursor shape. 
 *      return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/        
BOOLEAN Gal_set_cursor_position(unsigned long memoffset, 
					unsigned short xpos, unsigned short ypos, 
				unsigned short xhotspot, unsigned short yhotspot)
{
	GAL_CURSORPOSITION sCursorPos;
    
	INIT_GAL(&sCursorPos);
	sCursorPos.dwSubfunction = GALFN_SETCURSORPOSITION;
	sCursorPos.dwMemOffset = memoffset;
	sCursorPos.wXPos = xpos;
	sCursorPos.wYPos = ypos;
	sCursorPos.wXHot = xhotspot;
	sCursorPos.wYHot = yhotspot;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sCursorPos))
		return 0;
	else
		return 1;
}

BOOLEAN Gal_get_cursor_position(unsigned long *memoffset, 
                             unsigned short *xpos, unsigned short *ypos, 
                             unsigned short *xhotspot, unsigned short *yhotspot)
{
    GAL_CURSORPOSITION sCursorPos;
    
    INIT_GAL(&sCursorPos);
    sCursorPos.dwSubfunction = GALFN_GETCURSORPOSITION;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sCursorPos))
        return 0;
    else {
		*memoffset = sCursorPos.dwMemOffset;
		*xpos = sCursorPos.wXPos;
		*ypos = sCursorPos.wYPos;
		*xhotspot = sCursorPos.wXHot;
		*yhotspot = sCursorPos.wYHot;
        return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_set_cursor_shape32
 *
 * Description:	This function loads 32x32 cursor pattern.
 *  parameters:
 *   memoffset:	It specifies the graphics memory offset for cursor shape.
 *     andmask:	It is a pointer to 32 DWORD of AND data.
 *     xormask:	It is a pointer to 32 DWORD of XOR data.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/       
BOOLEAN Gal_set_cursor_shape32(unsigned long memoffset, 
                            unsigned long *andmask, unsigned long *xormask)
{
	GAL_SETCURSORSHAPE sCursorShape;
    
	INIT_GAL(&sCursorShape);
	sCursorShape.dwSubfunction = GALFN_SETCURSORSHAPE;
	sCursorShape.dwMemOffset = memoffset;
     
        memcpy(sCursorShape.dwAndMask, andmask,	sizeof(sCursorShape.dwAndMask));

        memcpy(sCursorShape.dwXorMask, xormask, sizeof(sCursorShape.dwXorMask));
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sCursorShape))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_set_cursor_colors
 *
 * Description:	This function sets the colors of the hardware cursor.
 *  parameters:
 *     bkcolor:It specifies the RGB value for the background color.
 *     fgcolor:It specifies the RGB value for the foreground color.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/  
BOOLEAN Gal_set_cursor_colors(unsigned long bkcolor, unsigned long fgcolor)
{
	GAL_CURSORCOLORS sCursorColor;
    
	INIT_GAL(&sCursorColor);
	sCursorColor.dwSubfunction = GALFN_SETCURSORCOLORS;
	sCursorColor.dwBgColor = bkcolor;
	sCursorColor.dwFgColor = fgcolor;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sCursorColor))
		return 0;
	else
		return 1;
}

BOOLEAN Gal_get_cursor_colors(unsigned long *bkcolor, unsigned long *fgcolor)
{
    GAL_CURSORCOLORS sCursorColor;
    
    INIT_GAL(&sCursorColor);
    sCursorColor.dwSubfunction = GALFN_GETCURSORCOLORS;
    
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sCursorColor))
        return 0;
    else
	{
		*bkcolor = sCursorColor.dwBgColor;
		*fgcolor = sCursorColor.dwFgColor;
        return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_set_solid_pattern
 *
 * Description:	This function sets a solid pattern color for future rendering.
 *  parameters:
 *       color:	It specifies the pattern color in proper format for current 
 *					display mode.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/  
BOOLEAN Gal_set_solid_pattern(unsigned long color)
{
	GAL_SETSOLIDPATTERN sSetSoildPat;
    
	INIT_GAL(&sSetSoildPat);
	sSetSoildPat.dwSubfunction = GALFN_SETSOLIDPATTERN;
	sSetSoildPat.dwColor = color;
	
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetSoildPat))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_set_solid_source
 *
 * Description:	This function specifies a constant source data value for 
 *				raster operations that use both pattern 
 *				and source data.
 *  parameters:
 *       color:	It specifies the source color.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *-------------------------------------------------------------------------*/ 
BOOLEAN Gal_set_solid_source(unsigned long color)
{
	GAL_SETSOLIDSOURCE sSetSolidSrc;

	INIT_GAL(&sSetSolidSrc);
	sSetSolidSrc.dwSubfunction = GALFN_SETSOLIDSOURCE;
	sSetSolidSrc.dwColor = color;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetSolidSrc))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_set_mono_pattern
 *
 * Description:	This function specifies an 8x8 monochrome pattern 
 *				used in future rendering operations.
 *  parameters:
 *     bkcolor: It specifies the background color.
 *     fgcolor: It specifies the foreground color.
 *		 data0: It specifies the bits of 8x8 monochrome pattern.
 *       data1: It specifies the bits of 8x8 monochrome pattern.
 *transparency:	It specifies the transparency flag.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_set_mono_pattern(unsigned long bgcolor, unsigned long fgcolor, 
							unsigned long data0, unsigned long data1, 
							unsigned char transparency)
{
	GAL_SETMONOPATTERN sSetMonoPat;
    
	INIT_GAL(&sSetMonoPat);
	sSetMonoPat.dwSubfunction = GALFN_SETMONOPATTERN;
	sSetMonoPat.dwFgColor = fgcolor;
	sSetMonoPat.dwBgColor = bgcolor;
	sSetMonoPat.dwData0 = data0;
	sSetMonoPat.dwData1 = data1;
	sSetMonoPat.cTransparency = transparency;
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetMonoPat))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_set_raster_operation
 *
 * Description:	This function specifies the raster operation for 
 *					future rendering.  
 *  parameters:
 *         rop: It specifies the ternary raster operation
 *					(pattern/source/destination).
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_set_raster_operation(unsigned char rop)
{
	GAL_SETRASTEROPERATION sSetRop;
        
	INIT_GAL(&sSetRop);
	sSetRop.dwSubfunction = GALFN_SETRASTEROPERATION;
	sSetRop.cRop = rop;

	if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetRop))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_pattern_fill
 *
 * Description:	This function renders pattern data to a rectangular 
 *					region. 
 *  parameters:
 *           x:	It specifies the screen X position, in pixels.
 *           y:	It specifies the screen Y position, in pixels.
 *       width:	It specifies the width of rectangle, in pixels.
 *      height:	It specifies the height of rectangle, in pixels.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pattern_fill(unsigned short x, unsigned short y,
						unsigned short width, unsigned short height)
{
	GAL_PATTERNFILL sPatternFill;
    
	INIT_GAL(&sPatternFill);
	sPatternFill.dwSubfunction = GALFN_PATTERNFILL;
	sPatternFill.wXPos = x;
	sPatternFill.wYPos = y;
	sPatternFill.wWidth = width;
	sPatternFill.wHeight = height;

	if (ioctl(ifbdev_handle, FBIOGAL_API, &sPatternFill))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_screen_to_screen_blt
 *
 * Description:	This function is used to perform a screen to screen 
 *				BLT operation. 
 *  parameters:
 *        srcx:	It specifies the source X position.
 *        srcy:	It specifies the source Y position.
 *        dstx:	It specifies the destination X position.
 *        dsty:	It specifies the destination Y position.
 *       width: It specifies the width of BLT, in pixels.
 *      height: It specifies the height of BLT, in pixels.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_screen_to_screen_blt(unsigned short srcx, unsigned short srcy,
				unsigned short dstx, unsigned short dsty, unsigned short width,
				unsigned short height)
{
	GAL_SCREENTOSCREENBLT sScreenBlt;
    
	INIT_GAL(&sScreenBlt);
	sScreenBlt.dwSubfunction = GALFN_SCREENTOSCREENBLT;
	sScreenBlt.wXStart = srcx;
	sScreenBlt.wYStart = srcy;
	sScreenBlt.wXEnd = dstx;
	sScreenBlt.wYEnd = dsty;
	sScreenBlt.wWidth = width;
	sScreenBlt.wHeight = height;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sScreenBlt))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_screen_to_screen_xblt
 *
 * Description:	This function is used to perform a screen to screen 
 *				BLT operation using a transparency color. 
 *  parameters:
 *        srcx:	It specifies the source X position.
 *        srcy:	It specifies the source Y position.
 *        dstx:	It specifies the destination X position.
 *        dsty:	It specifies the destination Y position.
 *       width: It specifies the width of BLT, in pixels.
 *      height: It specifies the height of BLT, in pixels.
 *       color: It specifies the transparency color.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_screen_to_screen_xblt(unsigned short srcx, unsigned short srcy,
								unsigned short dstx, unsigned short dsty, 
								unsigned short width, unsigned short height,
								unsigned short color)
{
	GAL_SCREENTOSCREENXBLT sScreenXBlt;
    
	INIT_GAL(&sScreenXBlt);
	sScreenXBlt.dwSubfunction = GALFN_SCREENTOSCREENXBLT;
	sScreenXBlt.wXStart = srcx;
	sScreenXBlt.wYStart = srcy;
	sScreenXBlt.wXEnd = dstx;
	sScreenXBlt.wYEnd = dsty;
	sScreenXBlt.wWidth = width;
	sScreenXBlt.wHeight = height;
	sScreenXBlt.wColor = color;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sScreenXBlt))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_bresenham_line
 *
 * Description:	This function is used to draw a single pixel line
 *					using the specified Bresenham parameters. 
 *  parameters:
 *           x:	It specifies the starting X position.
 *           y:	It specifies the starting Y position.
 *      length:	It specifies the length of the vector, in pixels.
 *     initerr:	It specifies the Bresenham initial error term.
 *    axialerr:	It specifies the Bresenham axial error term
 *					(moving in major direction only).
 *     diagerr: It specifies Bresenham diagonal error term
 *					(moving in major and minor direction.
 *       flags: It specifies the flag.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_bresenham_line(unsigned short x, unsigned short y, 
					unsigned short length, unsigned short initerr, 
					unsigned short axialerr, unsigned short diagerr, 
					unsigned short flags)
{
	GAL_BRESENHAMLINE sBresenhamLine;
    
	INIT_GAL(&sBresenhamLine);
	sBresenhamLine.dwSubfunction = GALFN_BRESENHAMLINE;
	sBresenhamLine.wX1 = x;
	sBresenhamLine.wY1 = y;
	sBresenhamLine.wLength = length;
	sBresenhamLine.wErr = initerr;
	sBresenhamLine.wE1 = axialerr;
	sBresenhamLine.wE2 = diagerr;
	sBresenhamLine.wFlags = flags;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sBresenhamLine))
		return 0;
	else
		return 1;
}

/*------------------------------------------------------------------------
 * Gal_set_compression_state
 *
 *       Description:	This function enables or disables display 
 * 			compression.
 *	  parameters:
 * bCompressionState:	It specifies the display compression state.  
 *  	      return:	'1' was returned on success otherwise 
 *			'0' was returned.
 *----------------------------------------------------------------------*/ 
BOOLEAN Gal_set_compression_state(BOOLEAN bCompressionState)
{
	GAL_COMPRESSIONSTATE sCompState;
    
	INIT_GAL(&sCompState);
	sCompState.dwSubfunction = GALFN_SETCOMPRESSIONSTATE;
	sCompState.bCompressionState = bCompressionState;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sCompState))
		return 0;
	else
		return 1;
}

/*------------------------------------------------------------------------
 * Gal_get_compression_state
 *
 * 	Description:	This function gets the compression state.
 *
 *	  parameters:
 * bCompressionState:	gets the display compression state.  
 *	      return:	'1' was returned on success otherwise 
 *			'0' was returned.
 *----------------------------------------------------------------------*/ 
BOOLEAN Gal_get_compression_state(int *bCompressionState)
{
    GAL_COMPRESSIONSTATE sCompState;
    
    INIT_GAL(&sCompState);
    sCompState.dwSubfunction = GALFN_GETCOMPRESSIONSTATE;
    
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sCompState))
        return 0;
    else {
		*bCompressionState = sCompState.bCompressionState;
        return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_set_compression_parameters
 *
 * Description:	This function sets the compression parameters of the
 * 			frame buffer device.
 *  parameters:
 *       flags:	It specifies the flag.
 *      offset:	It specifies the base offset in graphics memory for the 
 *			compression buffer.
 *	 pitch:	It specifies the pitch of compression buffer, in bytes.
 *        size:	It specifies the maximum line size of the compression buffer
 *			in bytes.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_set_compression_parameters(unsigned long flags,
					unsigned long offset, unsigned short pitch, 
					unsigned short size)
{
	GAL_COMPRESSIONPARAMS sCompParams;
    
	INIT_GAL(&sCompParams);
	sCompParams.dwSubfunction = GALFN_SETCOMPRESSIONPARAMS;
	sCompParams.dwFlags = flags;
	sCompParams.dwCompOffset = offset;
	sCompParams.dwCompPitch = pitch;
	sCompParams.dwCompSize = size;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sCompParams))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_get_compression_parameters
 *
 * Description:	This function gets the compression parameters of the
 *				frame buffer device.
 *  parameters:
 *       flags:	It specifies the flag.
 *      offset:	gets the base offset in graphics memory for the 
 *					compression buffer.
 *		 pitch:	gets the pitch of compression buffer, in bytes.
 *        size:	gets the maximum line size of the compression buffer
 *					in bytes.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_get_compression_parameters(unsigned long flags, 
                        unsigned long *offset,
                        unsigned short *pitch, 
						unsigned short *size)
{
    GAL_COMPRESSIONPARAMS sCompParams;
    
    INIT_GAL(&sCompParams);
    sCompParams.dwSubfunction = GALFN_GETCOMPRESSIONPARAMS;
    sCompParams.dwFlags = flags;
    
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sCompParams))
        return 0;
    else {
		*offset = sCompParams.dwCompOffset;
		*pitch = sCompParams.dwCompPitch;
		*size = sCompParams.dwCompSize;
        return 1;
	}
}
/*--------------------------------------------------------------------------
 * Gal_vga_mode_switch
 *
 * Description:This function signals the beginning or end of a 
 *				mode switch.  
 *  parameters:
 *      active:	It specifies the mode switch.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_vga_mode_switch(int active)
{
	GAL_VGAMODEDATA sVgaData;
    
	INIT_GAL(&sVgaData);
	sVgaData.dwSubfunction = GALFN_VGAMODESWITCH;
	sVgaData.dwFlags = active;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sVgaData))
		return 0;
	else
		return 1;    
}

/*--------------------------------------------------------------------------
 * Gal_vga_clear_extended
 *
 * Description:	This will clear the Svga data.
 *  parameters:	none.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_vga_clear_extended(void)
{
	GAL_VGAMODEDATA sVgaData;
    
	INIT_GAL(&sVgaData);
	sVgaData.dwSubfunction = GALFN_VGACLEARCRTEXT;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, &sVgaData))
		return 0;
	else
		return 1;
}

/*--------------------------------------------------------------------------
 * Gal_vga_pitch
 *
 * Description:	This function sets VGA register values in VGA 
 *					structure for specified pitch. 
 *  parameters:
 *    pVgaData: It specifies the vga structure.
 *		 pitch:	It specifies the number of bytes between scanlines.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_vga_pitch(PGAL_VGAMODEDATA pVgaData, unsigned short pitch)
{
	INIT_GAL(pVgaData);
	pVgaData->dwSubfunction = GALFN_VGASETPITCH;
	pVgaData->dwFlags = pitch;
    
	if (ioctl(ifbdev_handle, FBIOGAL_API, pVgaData))
		return 0;
	else {
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_vga_restore
 *
 * Description:	This function sets the VGA state to the values in the
 *				VGA structure.  
 *  parameters:
 *    pVgaData: It specifies the vga structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_vga_restore(PGAL_VGAMODEDATA pVgaData)
{
	INIT_GAL(pVgaData);
	pVgaData->dwSubfunction = GALFN_VGARESTORE;

	if (ioctl(ifbdev_handle, FBIOGAL_API, pVgaData))
		return 0;
	else
		return 1;    
}

/*--------------------------------------------------------------------------
 * Gal_vga_save
 *
 * Description:	This function saves the current VGA state in the
 *					VGA structure.  
 *  parameters:
 *    pVgaData: It specifies the vga structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_vga_save(PGAL_VGAMODEDATA pVgaData)
{
	INIT_GAL(pVgaData);
	pVgaData->dwSubfunction = GALFN_VGASAVE;

	if (ioctl(ifbdev_handle, FBIOGAL_API, pVgaData))
		return 0;
	else {
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_vga_mode
 *
 * Description:	This function sets VGA register values in VGA 
 *					structure for specified mode. 
 *  parameters:
 *    pVgaData: It specifies the vga structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_vga_mode(PGAL_VGAMODEDATA pVgaData)
{
	INIT_GAL(pVgaData);
	pVgaData->dwSubfunction = GALFN_VGASETMODE;

	if (ioctl(ifbdev_handle, FBIOGAL_API, pVgaData))
		return 0;
	else {
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_pnl_enabled_in_bios
 *
 * Description:	This function gets the status of the FP in BIOS.
 *  parameters:
 *       status: returns the state of FP in Bios.
 *	pParam:	It specifies the panel parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_enabled_in_bios(int *state)
{
	GAL_PNLBIOS pStat;

	INIT_GAL(&pStat);
	pStat.dwSubfunction = GALFN_PNLBIOSENABLE;

	if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
		return 0;
	else {
		*state = pStat.state;
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_pnl_info_from_bios
 *
 * Description:	This function gets the parameters of the FP in BIOS.
 *  parameters:
 *       status: returns the state of FP in Bios.
 *	pParam:	It specifies the panel parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_info_from_bios(int *xres, int *yres, int *bpp, int *hz)
{
	GAL_PNLBIOS pStat;

	INIT_GAL(&pStat);
	pStat.dwSubfunction = GALFN_PNLBIOSINFO;

	if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
		return 0;
	else {
		*xres = pStat.XRes;
		*yres = pStat.YRes;
		*bpp = pStat.Bpp;
		*hz = pStat.Freq;
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_pnl_set_params
 *
 * Description:	This function sets the panel parameters. 
 *  parameters:
 *       flags: 
 *	pParam:	It specifies the panel parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_set_params(unsigned long flags, PPnl_PanelParams pParam)
{
	GAL_PNLPARAMS pStat;

	INIT_GAL(&pStat);
	pStat.dwSubfunction = GALFN_PNLSETPARAMS;
	pParam->Flags = flags;
	memcpy( &(pStat.PanelParams), pParam, sizeof(Pnl_PanelParams));

	if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
		return 0;
	else {
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_pnl_get_params
 *
 * Description:	This function gets the panel parameters. 
 *  parameters:
 *       flags:
 *	pParam:	It specifies the panel parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_get_params(unsigned long flags, PPnl_PanelParams pParam)
{
	GAL_PNLPARAMS pStat;

	INIT_GAL(&pStat);
	pStat.dwSubfunction = GALFN_PNLGETPARAMS;
	memcpy(&(pStat.PanelParams), pParam, sizeof(Pnl_PanelParams));
	pStat.PanelParams.Flags = flags;

	if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
		return 0;
	else {
		memcpy(pParam,  &(pStat.PanelParams), sizeof(Pnl_PanelParams));
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_pnl_init
 *
 * Description:	This function initializes the panel parameters. 
 *  parameters:
 *	pParam:	It specifies the panel parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_init(PPnl_PanelParams pParam)
{
	GAL_PNLPARAMS pStat;

	INIT_GAL(&pStat);
	pStat.dwSubfunction = GALFN_PNLINITPANEL;
	memcpy( &(pStat.PanelParams), pParam, sizeof(Pnl_PanelParams));

	if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
		return 0;
	else 
		return 1;
	
}

/*--------------------------------------------------------------------------
 * Gal_pnl_save
 *
 * Description:	This function saves the current panel parameters. 
 *  parameters:	none.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_save(void)
{
    GAL_PNLPARAMS pStat;

    INIT_GAL(&pStat);
    pStat.dwSubfunction = GALFN_PNLSAVESTATE;

    if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
        return 0;
    else
    {
        return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_pnl_restore
 *
 * Description:	This function restores the current panel parameters. 
 *  parameters:	none.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_restore(void)
{
    GAL_PNLPARAMS pStat;

    INIT_GAL(&pStat);
    pStat.dwSubfunction = GALFN_PNLRESTORESTATE;

    if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
        return 0;
    else
    {
        return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_pnl_powerup
 *
 * Description:	This function powers up the panel. 
 *  parameters:	none.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_powerup(void)
{
    GAL_BASE pStat;

    INIT_GAL(&pStat);
    pStat.dwSubfunction = GALFN_PNLPOWERUP;

    if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
        return 0;
    else
    {
        return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_pnl_powerdown
 *
 * Description:	This function powers down the panel. 
 *  parameters:	none.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_pnl_powerdown(void)
{
    GAL_BASE pStat;

    INIT_GAL(&pStat);
    pStat.dwSubfunction = GALFN_PNLPOWERDOWN;

    if (ioctl(ifbdev_handle, FBIOGAL_API, &pStat))
        return 0;
    else
    {
        return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_tv_set_params
 *
 * Description:	This function sets the tv parameters of
 *					tvparameters structure. 
 *  parameters:
 *       flags:
 *	   pTV:	It specifies the tv parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_tv_set_params(unsigned long flags, PGAL_TVPARAMS pTV)
{
	INIT_GAL(pTV);
	pTV->dwSubfunction = GALFN_SETTVPARAMS;
	pTV->dwFlags = flags;

	if (ioctl(ifbdev_handle, FBIOGAL_API, pTV))
		return 0;
	else {
		return 1;
	}
}

/*--------------------------------------------------------------------------
 * Gal_tv_get_params
 *
 * Description:	This function gets the tv parameters of
 *					tvparameters structure. 
 *  parameters:
 *       flags: Dummy flag
 *	   pTV:	It specifies the tv parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_tv_get_params(unsigned long flags, PGAL_TVPARAMS pTV)
{
	INIT_GAL(pTV);
	pTV->dwSubfunction = GALFN_GETTVPARAMS;
	pTV->dwFlags = flags;

	if (ioctl(ifbdev_handle, FBIOGAL_API, pTV))
		return 0;
	else
		return 1;
    
}

/*--------------------------------------------------------------------------
 * Gal_tv_set_timings
 *
 * Description:	This function sets the tv timing registers.
 *  parameters:
 *       flags:	Dummy flag.
 *	   pTV:	It specifies the tv parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_tv_set_timings(unsigned long flags, PGAL_TVTIMING pTV)
{
    INIT_GAL(pTV);
    pTV->dwSubfunction = GALFN_SETTVTIMING;
    pTV->dwFlags = flags;

    if (ioctl(ifbdev_handle, FBIOGAL_API, pTV))
        return 0;
    else
    {
        return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_tv_get_timings
 *
 * Description:	This function gets the tv timing registers.
 *  parameters:
 *       flags:	Dummy flag.
 *	   pTV:	It specifies the tv parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_tv_get_timings(unsigned long flags, PGAL_TVTIMING pTV)
{
    INIT_GAL(pTV);
    pTV->dwSubfunction = GALFN_GETTVTIMING;
    pTV->dwFlags = flags;

    if (ioctl(ifbdev_handle, FBIOGAL_API, pTV))
        return 0;
    else
    {
        return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_set_tv_enable
 *
 * Description:	This function sets the tv state of the device .
 *  parameters: 
 *     bState : set the tv state.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_set_tv_enable(int bState)
{
    GAL_TVPARAMS pTV;

    INIT_GAL(&pTV);
    pTV.dwSubfunction = GALFN_SETENABLE;
    pTV.bState = bState;

    if (ioctl(ifbdev_handle, FBIOGAL_API, &pTV))
        return 0;
    else
    {
        return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_get_tv_enable
 *
 * Description:	This function gets the tv state of the device .
 *  parameters: 
 *     bState : get the tv state.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_get_tv_enable(int *bState)
{
    GAL_TVPARAMS pTV;

    INIT_GAL(&pTV);
    pTV.dwSubfunction = GALFN_GETENABLE;

    if (ioctl(ifbdev_handle, FBIOGAL_API, &pTV))
	{
		*bState = 0;
		return 0;
    }
	else
    {
		*bState = pTV.bState; 
		return 1;
    }
}

/*--------------------------------------------------------------------------
 * Gal_is_tv_mode_supported
 *
 * Description:	This function checks the tv mode is supported or not.
 *  parameters:
 *       flags: Dummy flag
 *	   pTV:	It specifies the tv parameters structure.
 *      return:	'1' was returned on success otherwise '0' was returned.
 *------------------------------------------------------------------------*/ 
BOOLEAN Gal_is_tv_mode_supported(unsigned long flags, PGAL_TVPARAMS pTV,
		int *bState)
{
	INIT_GAL(pTV);
	pTV->dwSubfunction = GALFN_ISTVMODESUPPORTED;
	pTV->dwFlags = flags;

	if (ioctl(ifbdev_handle, FBIOGAL_API, pTV)) {
		return 0;
	} else {
		*bState = pTV->bState;
		return 1;
	}
}

/** Video **********************************************************/
BOOLEAN Gal_set_video_enable(int enable)
{
    GAL_SETVIDEOENABLE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOENABLE;
    sSetVideo.enable = enable;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_format(int format)
{
    GAL_SETVIDEOFORMAT sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOFORMAT;
    sSetVideo.format = format;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_size(unsigned short width, unsigned short height)
{
    GAL_SETVIDEOSIZE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOSIZE;
    sSetVideo.width = width;
    sSetVideo.height = height;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_offset(unsigned long offset)
{
    GAL_SETVIDEOOFFSET sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOOFFSET;
    sSetVideo.offset = offset;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_window(short x, short y, short w, short h)
{
    GAL_SETVIDEOWINDOW sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOWINDOW;
    sSetVideo.x = x;
    sSetVideo.y = y;
    sSetVideo.w = w;
    sSetVideo.h = h;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_scale(unsigned short srcw, unsigned short srch,
                            unsigned short dstw, unsigned short dsth)
{
    GAL_SETVIDEOSCALE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOSCALE;
    sSetVideo.srcw = srcw;
    sSetVideo.srch = srch;
    sSetVideo.dstw = dstw;
    sSetVideo.dsth = dsth;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_filter(int xfilter, int yfilter)
{
    GAL_SETVIDEOFILTER sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOFILTER;
    sSetVideo.xfilter = xfilter;
    sSetVideo.yfilter = yfilter;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_color_key(unsigned long key, unsigned long mask, int bluescreen)
{
    GAL_SETVIDEOCOLORKEY sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOCOLORKEY;
    sSetVideo.key = key;
    sSetVideo.mask = mask;
    sSetVideo.bluescreen = bluescreen;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_downscale_enable(int enable)
{
    GAL_SETVIDEODOWNSCALEENABLE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEODOWNSCALEENABLE;
    sSetVideo.enable = enable;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_downscale_config(unsigned short type, unsigned short m)
{
    GAL_SETVIDEODOWNSCALECONFIG sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEODOWNSCALECONFIG;
    sSetVideo.type = type;
    sSetVideo.m = m;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_downscale_coefficients(unsigned short coef1,
                                             unsigned short coef2,
                                             unsigned short coef3,
                                             unsigned short coef4)
{
    GAL_SETVIDEODOWNSCALECOEFF sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEODOWNSCALECOEFF;
    sSetVideo.coef1 = coef1;
    sSetVideo.coef2 = coef2;
    sSetVideo.coef3 = coef3;
    sSetVideo.coef4 = coef4;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_source(int source)
{
    GAL_SETVIDEOSOURCE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOSOURCE;
    sSetVideo.source = source;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_interlaced(int enable)
{
    GAL_SETVIDEOINTERLACED sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOINTERLACED;
    sSetVideo.enable = enable;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_color_space(int enable)
{
    GAL_SETVIDEOCOLORSPACE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOCOLORSPACE;
    sSetVideo.enable = enable;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_cursor(unsigned long key,
                             unsigned long mask,
                             unsigned short select_color2,
                             unsigned long color1,
                             unsigned long color2)
{
    GAL_SETVIDEOCURSOR sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOCURSOR;
    sSetVideo.key = key;
    sSetVideo.mask = mask;
    sSetVideo.select_color2 = select_color2;
    sSetVideo.color1 = color1;
    sSetVideo.color2 = color2;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_request(short x, short y)
{
    GAL_SETVIDEOREQUEST sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOREQUEST;
    sSetVideo.x = x;
    sSetVideo.y = y;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_alpha_enable(int enable)
{
    GAL_SETVIDEOALPHAENABLE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOALPHAENABLE;
    sSetVideo.enable = enable;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_alpha_window(short x, short y, 
                             unsigned short width,
                             unsigned short height)
{
    GAL_SETVIDEOALPHAWINDOW sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOALPHAWINDOW;
    sSetVideo.x = x;
    sSetVideo.y = y;
    sSetVideo.width = width;
    sSetVideo.height = height;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_alpha_value(unsigned char alpha, char delta)
{
    GAL_SETVIDEOALPHAVALUE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOALPHAVALUE;
    sSetVideo.alpha = alpha;
    sSetVideo.delta = delta;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_alpha_priority(int priority)
{
    GAL_SETVIDEOALPHAPRIORITY sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOALPHAPRIORITY;
    sSetVideo.priority = priority;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_alpha_color(unsigned long color)
{
    GAL_SETVIDEOALPHACOLOR sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOALPHACOLOR;
    sSetVideo.color = color;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_select_alpha_region(int region)
{
    GAL_SETVIDEOALPHAREGION sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOALPHAREGION;
    sSetVideo.region = region;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_outside_alpha(int enable)
{
    GAL_SETVIDEOOUTSIDEALPHA sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOOUTSIDEALPHA;
    sSetVideo.enable = enable;
    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}

BOOLEAN Gal_set_video_palette(unsigned long* palette)
{
    GAL_SETVIDEOPALETTE sSetVideo;
    
    INIT_GAL(&sSetVideo);
    sSetVideo.dwSubfunction = GALFN_SETVIDEOPALETTE;

    if(palette==NULL) {
        sSetVideo.identity = 1;
    } else {
        sSetVideo.identity = 0;
        memcpy(sSetVideo.palette, palette, 256*sizeof(*palette));
    }

    if (ioctl(ifbdev_handle, FBIOGAL_API, &sSetVideo))
        return 0;
    else
        return 1;
}
/** Video **********************************************************/


