/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 *
 * File Name      : dvd_gui.cc                             *
 *                                                         *
 * Project Name   : ASTB                                   *
 *                                                         *
 * Module Name    : Integrated Test App                    *
 *                                                         *
 * Author Name    : Lakshmi B G                            *
 *                                                         *
 * Language       : C++                                    *
 *                                                         *
 * Date           : 2/02/01                                *
 *							   *
 * Description    : This is the source file for the        *
 *		    dvd gui functionalities 		   *
 **********************************************************/

#include "dvd_gui.h"

extern int exit_app;

/************************************************************************
 * Syntax       : DvdGUI(int,int,int,int,char*)                         *
 * Description  : DvdGUI class constructor				*
 * Parameters   : x,y 							*
 *		  - Starting x and y position values			*
 *		  w,h							*
 *		  - Width and height of the gui widget 			*
 *		  label							*
 *		  - label for the widget, by default it is null 	*
 * Returns      : void                                                  *
 * Comment      : None                                                  *
************************************************************************/
DvdGUI::DvdGUI(int x, int y, int w, int h, char *label=0):
	Fl_Box(x,y,w,h,label)
{
                topx = x;
                topy = y;
                width = w;
                ht = h;
                Create_gui();
                hide();
}

/************************************************************************
 * Syntax       : Create_gui()                                          *
 * Description  : Creates all the gui components required               *
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void DvdGUI::Create_gui()
{
 	int bwidth;
	bwidth = (width-10)/3-10;
	select_box = new Fl_Box(topx, topy, width, 100);
	select_box->box(FL_ROUNDED_BOX);
	select_grp = new Fl_Group(topx, topy, width, 100);
	select_grp->begin();
	
	/* Menu button creation */
	menu_b = new Fl_Button(topx+30,topy+10,50,15);
	menu_b->labelsize(10);
	menu_b->label("DvdMenu");


	/* Up, down, left, right, select and control buttons */
	up_b    = new Fl_Button(topx+bwidth+15, topy+30, bwidth, 20, "@#-22<-");
	dn_b    = new Fl_Button(topx+bwidth+15, topy+70, bwidth, 20, "@#-22->");
	left_b  = new Fl_Button(topx+15, topy+50, bwidth, 20, "@#-2<-");
	right_b = new Fl_Button(topx+2*bwidth+15, topy+50, bwidth, 20, "@#-2->");
	select_b= new Fl_Button(topx+bwidth+15, topy+50, bwidth, 20, "Slct");
	select_b->labelsize(10);
	
	ctrl_b  = new Fl_Button(topx+3*bwidth+5, topy+100, bwidth, 25, "Ctrl");
	ctrl_b->labelsize(10);

	/* Create for dvd status info */
	/* All button callback functions */
	
	menu_b->callback(Dvd_select_cb, this);
	up_b->callback(Dvd_select_cb, this);
	dn_b->callback(Dvd_select_cb, this);
	left_b->callback(Dvd_select_cb, this);
	right_b->callback(Dvd_select_cb, this);
	select_b->callback(Dvd_select_cb, this);
	ctrl_b->callback(Dvd_select_cb, this);

	
	select_grp->end();

	/* DVD program search functions group */ 
 	prog_search_grp = new Fl_Group(topx, topy+100, width, 120);
 	prog_search_grp->begin();
	
	/* Next, previous, top, chapter, title, goup, subtitle, language
         * selections, angle selection, buttons creation 
	 */
	next_b = new Fl_Button(topx+5, topy+100, bwidth, 25, "Nxt");
	next_b->labelsize(10);
	prev_b = new Fl_Button(topx+5+bwidth, topy+100, bwidth, 25, "Prv");
	prev_b->labelsize(10);
	top_b = new Fl_Button(topx+5+2*bwidth, topy+100, bwidth, 25, "Top");
	top_b->labelsize(10);
	
	chap_b = new Fl_Button(topx+5, topy+130, bwidth, 25, "Chpr");
	chap_b->labelsize(10);
	titl_b = new Fl_Button(topx+5+bwidth, topy+130, bwidth, 25, "Ttl");
	titl_b->labelsize(10);
	goup_b = new Fl_Button(topx+2*bwidth+5, topy+130, bwidth, 25, "Goup");	
	goup_b->labelsize(10);
	dvd_subtitl_b = new Fl_Button(topx+3*bwidth+5, topy+130, bwidth, 25, "Sub");
	dvd_subtitl_b->labelsize(10);
	dvd_lang_b = new Fl_Button(topx+bwidth+5, topy+155, bwidth, 25, "Lang");
	dvd_lang_b->labelsize(10);
	dvd_angle_b = new Fl_Button(topx+2*bwidth+5,topy+155,bwidth,25,"Agl");
	dvd_angle_b->labelsize(10);

	/* The previous created buttons callback */
	next_b->callback(Dvd_prog_sel_cb, this);
	prev_b->callback(Dvd_prog_sel_cb, this);
	top_b->callback(Dvd_prog_sel_cb, this);
	chap_b->callback(Dvd_prog_sel_cb, this);
	titl_b->callback(Dvd_prog_sel_cb, this);
	goup_b->callback(Dvd_prog_sel_cb, this);
	dvd_subtitl_b->callback(Dvd_prog_sel_cb, this);
	dvd_lang_b->callback(Dvd_prog_sel_cb, this);
	dvd_angle_b->callback(Dvd_prog_sel_cb, this);

 	prog_search_grp->end();

 	int bwdth = (width-20)/3;
	/* Keypad for titles and chapter selections */
 	num_select = new Keypad_Dvd(topx+5, topy+185, width-10, 70);
	num_select->box(FL_ROUNDED_BOX);
 	num_select->hide();

	/* Play control buttons */
 	ctrl_box = new Fl_Box(topx+5, topy+200, width-10, 60);
 	ctrl_box->box(FL_ROUNDED_BOX);
 	ctrl_grp = new Fl_Group(topx+5, topy+200, width-10, 60);
 	ctrl_grp->begin();
	play_b = new Fl_Button(topx+10,topy+210, bwdth, 20, "@#-2>");
	stop_b = new Fl_Button(topx+bwdth+10,topy+210, bwdth, 20, "@#-2circle");
	pause_b= new Fl_Button(topx+2*bwdth+10, topy+210, bwdth, 20,"@#-2||");
	fwd_b =  new Fl_Button(topx+10,topy+230, bwdth, 20,"@#-2>>");
	rev_b =  new Fl_Button(topx+bwdth+10,topy+230, bwdth, 20,"@#-2<<");
	ft_sl = new Fl_Button(topx+2*bwdth+10,topy+230, bwdth, 20, "Ft/Sl");
	ft_sl->labelsize(10);


 	ctrl_grp->end();
	
	/* Buttons generate callbacks when their type is set to toggle */
 	play_b->type(102);
 	stop_b->type(102);
 	fwd_b->type(102);
 	fwd_b->type(FL_TOGGLE_BUTTON);
 	rev_b->type(102);
 	rev_b->type(FL_TOGGLE_BUTTON);
 	pause_b->type(102);
 	pause_b->type(FL_TOGGLE_BUTTON);
	ft_sl->type(102);
	ft_sl->type(FL_TOGGLE_BUTTON);
	
	/* Callbacks */
 	play_b->callback(Dvd_playctrl_cb, this);	
 	stop_b->callback(Dvd_playctrl_cb, this);
	fwd_b->callback(Dvd_playctrl_cb, this);
 	rev_b->callback(Dvd_playctrl_cb, this);
 	//mute_b->callback(Dvd_playctrl_cb, this);
 	pause_b->callback(Dvd_playctrl_cb, this);
	ft_sl->callback(Dvd_playctrl_cb,this);

 	dvd_hide_b = new Fl_Button(topx+40, topy+260, 40, 20, "Ok");
	dvd_hide_b->labelsize(10);
 	dvd_hide_b->callback(Dvd_hide_cb, this);
}

/************************************************************************
 * Syntax       : hide()                                                *
 * Description  : This function hides all the DVD components		*
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void DvdGUI::hide()
{
	Fl_Box::hide();
	select_box->hide();
	select_grp->hide();
	prog_search_grp->hide();
	num_select->hide();
 	dvd_hide_b->hide();
	ctrl_box->hide();
 	ctrl_grp->hide();
}

/************************************************************************
 * Syntax       : show()                                                *
 * Description  : This function shows all the DVD components		*
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void DvdGUI::show()
{
 	Fl_Box::show();
 	select_grp->show();
 	prog_search_grp->show();
 	dvd_hide_b->show();
 	ctrl_box->show();
 	ctrl_grp->show();
}

/************************************************************************
 * Syntax       : Dvd_select_cb(Fl_Widget*, void* )			*
 * Description  : This is the callback function for the DVD menu buttons*
 * Parameters   : pointer to the component on which action happened     *
 *			pointer to the parent on the component		*
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Dvd_select_cb(Fl_Widget* wid, void* ptr)
{
  	DvdGUI* parent = ptr;
	printf("DVD select button callback \n");

  	if (wid == parent->up_b)
	{
		if(VMW_DVD_ButtonSelect(DVD_UPPERBUTTON) != VMW_OK)
		{
			printf("Error in button select at upperbutton\n");
		}
	}	
  	else 
	if(wid == parent->dn_b)
        {	
		if(VMW_DVD_ButtonSelect(DVD_LOWERBUTTON) != VMW_OK)
		{
			printf("Error in button select at lowerbutton\n");
		}
	}	
  	else 
	if(wid == parent->left_b)
	{
		if(VMW_DVD_ButtonSelect(DVD_LEFTBUTTON) != VMW_OK)
		{
			printf("Error in button select at leftbutton\n");
		}
	}	
  	else 
	if(wid == parent->right_b)
	{
		if(VMW_DVD_ButtonSelect(DVD_RIGHTBUTTON) != VMW_OK)
			printf("Error in button select at rightbutton\n");	
	}

  	else if(wid == parent->select_b)
		VMW_DVD_ButtonActivate();

  	else if (wid == parent->ctrl_b)
  	{
		parent->ctrl_box->show();
		parent->ctrl_grp->show();
  	}
  	else if(wid == parent->menu_b) 
	{
	//	VMW_DVD_ActivateMenu(0x03); 
		VMW_DVD_ActivateMenu(DVD_MENU_ID_ROOT);
	}
  	else
		printf ("invalid button presses\n");
	
}

/************************************************************************
 * Syntax       : Dvd_prog_sel_cb(Fl_Widget*, void* )			*
 * Description  : this is the callback function for the DVD program     *
 *			select buttons.					*
 * Parameters   : pointer to the component on which action happened     *
 *			pointer to the parent on the component		*
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Dvd_prog_sel_cb(Fl_Widget *wid, void* ptr)
{
  	DvdGUI* parent = ptr;
  	static int angleNumber = 0;
  	static int subPicture = 0;
  	static int audioStream = 0;
	static char*  lang_select = 0;

  	if ((wid == parent->chap_b) || (wid == parent->titl_b))
  	{
		parent->ctrl_box->hide();
		parent->ctrl_grp->hide();
		parent->num_select->show();
  	}
  	else if (wid == parent->goup_b)
 		VMW_DVD_Search(DVD_GOUP_SRCH,NULL);
	
  	else if (wid == parent->next_b) 
		VMW_DVD_Search(DVD_NEXT_SRCH,NULL); 
  
	else if (wid == parent->prev_b) 
		VMW_DVD_Search(DVD_PREV_SRCH,NULL);
  
	else if (wid == parent->top_b) 
		VMW_DVD_Search(DVD_TOP_SRCH,NULL);

  	else if (wid == parent->dvd_subtitl_b)
  	{
            	if (subPicture >= 7)
            	{
                 	subPicture = 0;
                  	VMW_DVD_SubPictureStreamSelect(subPicture, 0);
                  	printf("infostream is %d\n",subPicture);
            	}
            	else
                  	printf("SubTitle is %d\n",subPicture);
            
		VMW_DVD_SubPictureStreamSelect(subPicture,1);
            	subPicture=subPicture++;
 	}

  	else if (wid == parent->dvd_angle_b)
  	{
		VMW_DVD_AngleSelect(angleNumber);
		angleNumber += 1;
  	}            
  	else if (wid == parent->dvd_lang_b)
	{
		printf("Entering into callback of lang_select\n");
		int i = VMW_DVD_LanguageSelect(lang_select);
		printf("language value %d\n",i);
	}
  	else
		printf ("invalid button pressed\n");
}
 
/************************************************************************
 * Syntax       : Dvd_hide_cb(Fl_Widget*, void* )			*
 * Description  : this is the callback function for the hide button	*
 * Parameters   : pointer to the component on which action happened     *
 *			pointer to the parent on the component		*
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Dvd_hide_cb(Fl_Widget *wid, void *ptr)
{
	DvdGUI* parent = ptr;
	VMW_OVL_DisableAlphaWindow(0);
	parent->hide();
}

/************************************************************************
 * Syntax       : Dvd_playctrl_cb(Fl_Widget*, void* )			*
 * Description  : this is the callback function for the DVD program     *
 *			play control buttons.				*
 * Parameters   : pointer to the component on which action happened     *
 *			pointer to the parent on the component		*
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void Dvd_playctrl_cb(Fl_Widget *wid, void *ptr)
{
	static int pauseflag;
	static int ft_sl_flag;
	static float fast_speed = 2;
	static float slow_speed = 2;
	
	DvdGUI* parent = ptr;

	if (wid == parent->play_b)
	{
		printf("Coming to play \n");
		fast_speed = 2;
		slow_speed = 2;
		int i = VMW_DVD_Play();
		SetAudio(AV_CHAN_MPEG,0,1);
		printf("Playing status %d \n",i);
		parent->play_b->clear();
	}
	else 
	if (wid == parent->fwd_b)
	{		
		printf("Coming into fast forward callback");
	
		if(ft_sl_flag == 0)
		{	
			if(fast_speed > 8) fast_speed = 2;
			printf("Forwardscan success for the speed %d \n",fast_speed);
			int i = VMW_DVD_ForwardScan (fast_speed,TRUE);
			fast_speed *= 2;
			parent->fwd_b->clear();
		}
		else
		{
			if(slow_speed > 8) slow_speed = 2;	
			int i = VMW_DVD_ForwardScan(slow_speed, FALSE);
			printf("Slow motion speed %d\n",slow_speed);
			slow_speed *= 2;
			parent->fwd_b->clear();
		}
	}
	else 
	if (wid == parent->rev_b)
	{
		if(ft_sl_flag == 0)
		{	
			if(fast_speed > 8) fast_speed = 2;
			int i = VMW_DVD_BackwardScan (fast_speed,TRUE);
			printf("Forwardscan success %d \n",i);
			fast_speed *= 2;
			parent->rev_b->clear();
		}
		else
		{
			if(slow_speed > 8) slow_speed = 2;	
			int i = VMW_DVD_BackwardScan(slow_speed, FALSE);
			printf("Slow motion %d\n",i);
			slow_speed *= 2;
			parent->rev_b->clear();
		}
	}
	else if (wid == parent->stop_b)
	{
		VMW_DVD_Stop();
		parent->stop_b->clear();
	}
	else 
	if(wid == parent->ft_sl)
	{
		if(ft_sl_flag == 0)
			ft_sl_flag = 1;		
		else	
		{
			ft_sl_flag = 0;
			parent->ft_sl->clear();
		}
	}
	if (wid == parent->pause_b)
	{
		if (pauseflag)
		{
			VMW_DVD_PauseOff();
		   	pauseflag=0;
		   	parent->pause_b->clear();
		}
		else
		{
		   	VMW_DVD_PauseOn();
		   	pauseflag=1;
		}
	}
	else
		printf("invalid button pressed \n");	
}
