/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 * File Name      : netstrm_gui.h                          *
 *                                                         *
 * Project Name   : ASTB                                   *
 *                                                         *
 * Subsystem Name : Integrated Test App                    *
 *                                                         *
 * Module Name    : Netstream                              *
 *                                                         *
 * Author Name    : Lakshmi B G                            *
 *							   *
 * Description	  : This defines the class for streamer gui*
 **********************************************************/ 		    

/* System includes */
#include <string.h>
#include <stdio.h>
#include <iostream.h>

/* Installed middleware includes */
#include <anlg_video.h>
#include <nscstrapi.h>

#include "netstrm_gui.h"

/* Global Param class object */
Param* ppParam;
char pathname[80];

 /***********************************************************************
 * Syntax       : NetStrmGUI(int,int,int,int,char*)                     *
 * Description  : class constructor 					*
 * Parameters   : x,y - starting window positions		 	*
 		  w,h - width and height of the application
		  label - label name of the widget(default is null ) 	
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
NetStrmGUI::NetStrmGUI(int x,int y,int w,int h,char* label=0):
		Fl_Box(x,y,w,h,label)
{
	topx = x;
        topy = y; width = w; ht = h;
        create_gui();
        hide();
}

/************************************************************************
 * Syntax       : create_gui()                                          *
 * Description  : this function creates all the gui components required *
 *			for net streamer				*
 * Parameters   : None                                                  *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void NetStrmGUI::create_gui(void)
{
 	int bht=20;
 	int bwdth = (width-30)/3;
 	Fl::enable_symbols();
 	ppParam = new Param();

 	server_titl = new Fl_Box(topx+5, topy+25, width-20, bht, "Server :");
 	server_ip = new Fl_Input(topx+5,topy+bht+30,width-55, bht);
 	conn_b = new Fl_Button(topx+(width-52),topy+bht+30, 45, bht, "Conn");
 	disconn_b = new Fl_Button(topx+(width-52),topy+bht+30, 45, bht, 
			"Discon");
 	conn_b->callback(net_sconn_cb, this);
 	disconn_b->callback(net_sconn_cb, this);

 	ctrl_box = new Fl_Box(topx+5, topy+2*bht+35, width-10, 3*bht+10);
 	ctrl_box->box(FL_BORDER_BOX);
 	ctrl_grp = new Fl_Group(topx+5, topy+2*bht+35, width-10, 3*bht+10);
 	ctrl_grp->begin();

 	play_b =  new Fl_Button(topx+10,topy+2*bht+40, bwdth, bht,"@>");
 	stop_b =  new Fl_Button(topx+bwdth+10,topy+2*bht+40, bwdth, bht, 
		"@#-2circle");
 	pause_b = new Fl_Button(topx+2*bwdth+10, topy+2*bht+40, bwdth, bht,
		"@#-2||");
 	fwd_b =   new Fl_Button(topx+10,topy+3*bht+40, bwdth, bht,"@#-2>>");
 	rev_b =   new Fl_Button(topx+bwdth+10, topy+3*bht+40, bwdth, bht,
		"@#-2<<");
 	mute_b =  new Fl_Button(topx+2*bwdth+10,topy+3*bht+40, bwdth, bht, 
		"@#1+");
 	loop_b =  new Fl_Button(topx+bwdth+10,topy+4*bht+40, bwdth,bht,
		"@#-2<->");
 	ctrl_grp->end();
	ctrl_grp->deactivate();

	/* Buttons toggle feature is set */ 
 	play_b->type(102);
 	stop_b->type(102);
 	fwd_b->type(102);
 	rev_b->type(102);
 	mute_b->type(FL_TOGGLE_BUTTON);
 	loop_b->type(FL_TOGGLE_BUTTON);
 	pause_b->type(FL_TOGGLE_BUTTON);

	/* Callback functions are set here */
 	play_b->callback(net_ctrl_cb, this);
 	stop_b->callback(net_ctrl_cb, this);
 	fwd_b->callback(net_ctrl_cb, this);
 	rev_b->callback(net_ctrl_cb, this);
 	mute_b->callback(net_ctrl_cb, this);
 	pause_b->callback(net_ctrl_cb, this);
 	loop_b->callback(net_ctrl_cb, this);

 	int grpy = topy+5*bht+90;
	
	progressBar = new Fl_Value_Slider(topx+5,grpy-35,width-10,15);	
	progressBar->type(FL_HOR_NICE_SLIDER);
	progressBar->selection_color(FL_GREEN);
	progressBar->scrollvalue(0,0,0,100);
	progressBar->when(FL_WHEN_RELEASE);

 	file_grp = new Fl_Group(topx+5, grpy-15, width-10, 180);
 	file_grp->begin();
		filelist = new Fl_Select_Browser(topx+5, grpy-15, width/2, 70);
		file_show_b = new Fl_Button(topx+15+width/2,grpy+10,40,20);
		file_show_b->labeltype(FL_EMBOSSED_LABEL);
		file_show_b->labelfont(FL_COURIER_ITALIC);
		file_show_b->label("Show");
		file_show_b->deactivate();
	
		file_ok_b = new Fl_Button(topx+15+width/2, grpy+30, 40, 20);
		file_ok_b->labeltype(FL_EMBOSSED_LABEL);
		file_ok_b->labelfont(FL_COURIER_ITALIC);
		file_ok_b->label("Ok");
 	file_grp->end();

 	progressBar->callback(updatePrgBar_cb,this);
 	file_show_b->callback(net_file_cb, this);
 	file_ok_b->callback(net_file_cb, this);
 	filelist->callback(net_file_cb, this);
 	net_hide_b = new Fl_Button(topx+40, grpy-15+70+5, 40, 20, "Ok");
	net_hide_b->labeltype(FL_EMBOSSED_LABEL);
	net_hide_b->labelfont(FL_COURIER_ITALIC);
	net_hide_b->label("Ok");

 	net_hide_b->callback(net_hide_cb, this);
}

 /************************************************************************
 * Syntax       : show()             	                                 *
 * Description  : this function shows all the gui components required    *
 *			for net streamer				 *
 * Parameters   : None                                                   *
 * Returns      : void                                                   *
 * Comment      : None                                                   *
 *************************************************************************/
void NetStrmGUI::show(void)
{
 	Fl_Box::show();
 	server_titl->show();
 	server_ip->show();
 	conn_b->show();
 	ctrl_box->show();
 	ctrl_grp->show();
 	file_grp->show();
 	file_ok_b->hide();
 	progressBar->show();
 	net_hide_b->show();
}
/*************************************************************************
 * Syntax       : hide()             	                                 *
 * Description  : this function hide all the gui components required     *
 *			for net streamer				 *
 * Parameters   : None                                                   *
 * Returns      : void                                                   *
 * Comment      : None                                                   *
 *************************************************************************/
void NetStrmGUI::hide(void)
{
 	Fl_Box::hide();
 	server_titl->hide();
 	server_ip->hide();
 	conn_b->hide();
 	disconn_b->hide();
 	ctrl_box->hide();
 	ctrl_grp->hide();
 	file_grp->hide();
 	progressBar->hide();
 	net_hide_b->hide();
}

/************************************************************************
 * Syntax       : update(void*)             	                        *
 * Description  : this function updates the progress bar 	 	*
 * Parameters   : pointer to the gui                                    *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void update(void* p)
{
	int new_value;
	NetStrmGUI* parent = p;
        /* get current point */
        new_value = NscStrGetPlayStatus();

        /* update scroll bar*/
        parent->progressBar->value(new_value);

        /* set timeout */
        Fl::add_timeout(1.0, update,parent);
}

/************************************************************************
 * Syntax       : updatePrgBar_cb(Fl_Widget* wid, void* p)              *
 * Description  : this callback function for updateing the progress bar *
 * Parameters   : pointer to the component on which action happened	*
 *			pointer to the parent of the component          *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void updatePrgBar_cb(Fl_Widget* wid, void* p)
{
 	Fl_Slider *s =(Fl_Slider*)wid;
        if((s->value() > 0)  && (s->value() <100))
                NscStrPlaySkip(s->value());
	update(p);
}

/************************************************************************
 * Syntax       : net_hide_cb(Fl_Widget* wid, void* p)             	*
 * Description  : this callback function to hide button			*
 * Parameters   : pointer to the component on which action happened	*
 *			pointer to the parent of the component          *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void net_hide_cb(Fl_Widget* wid, void *ptr)
{
	NetStrmGUI *parent = ptr;
	VMW_OVL_DisableAlphaWindow(0);
	parent->hide();
}

/************************************************************************
 * Syntax       : net_sconn_cb(Fl_Widget* wid, void* p)             	*
 * Description  : this callback function for the server connect buttons	*
 * Parameters   : pointer to the component on which action happened	*
 *			pointer to the parent of the component          *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/	 
void net_sconn_cb(Fl_Widget* wid, void *ptr)
{
 	NetStrmGUI *parent = ptr;
	int i, servContFlag;

	char *sname;
	if(wid == parent->conn_b)
	{
	  	parent->conn_b->hide();
		sname = parent->server_ip->value();
		printf(" Server %s \n",sname);
		parent->ctrl_grp->activate();
		parent->file_show_b->activate();
		if (0==NscStrSetSrvName(sname))
			printf ("cannot set server name\n");
                if(0==NscStrHook2Srv(sname))
                {
                        printf("\n Not able to connect to the server\n");
                        return;
                }
		else
		{
			printf("Connected to server\n");
		}
		
		/* set path to initial */
		strcat(pathname, "\\");
		NscStrSetDirectory(pathname);
		
		parent->conn_b->hide();
		parent->disconn_b->show();
	}
        else if (wid == parent->disconn_b)
        {
                if(0==NscStrUnHook2Srv())
                {
                        printf("\n Not able to disconnect from the server");
                        return;
                }
		(*pathname) = 0;	
		(*sname) = 0;
		parent->server_ip->value("");
		parent->filelist->clear();
	  	parent->disconn_b->hide();
		parent->conn_b->show();
        }
}

/************************************************************************
 * Syntax       : net_ctrl_cb(Fl_Widget* wid, void* p)             	*
 * Description  : this callback function for the play control buttons	*
 * Parameters   : pointer to the component on which action happened	*
 *			pointer to the parent of the component          *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void net_ctrl_cb(Fl_Widget* wid, void* ptr)
{	
	NetStrmGUI *parent = ptr;
	int i, retval;
	SetAudio(AV_CHAN_MPEG,0,1);

	if(wid == parent->play_b)
	{
		
		retval = NscStrPlay();
		parent->play_b->clear();
		parent->stop_b->clear();
		if (retval <= 0)
		{
			parent->play_b->clear();	
		}
		else
		{
		/* Played deactivate all buttons */
			parent->disconn_b->deactivate();
			parent->file_show_b->deactivate();
			parent->file_ok_b->deactivate();
			parent->filelist->deactivate();
			Fl::add_timeout(1.0,update,parent);
		}	
	}

	else if(wid == parent->fwd_b)
	{
		retval = NscStrPlayFastForward();
		if (retval)
			return;
		else
		{
			printf ("ff timed out\n");
		}
	}
        else if(wid == parent->rev_b)
        {
		retval = NscStrPlayReverse();
		if (retval <= 0)
	          	printf(" REV TIME OUT GUI \n");
        }		

	else if(wid == parent->stop_b)
	{
		NscStrStop();
		parent->stop_b->clear();

		parent->disconn_b->activate();
		parent->file_show_b->activate();
		parent->file_ok_b->activate();
		parent->filelist->activate();
	}
	else if (wid == parent->mute_b)
	{
		retval = NscStrMute();
		if (retval > 0)
		{
			return;
		}
		else
		{
			printf ("mute timed out\n");
		}	
	}
	else if(wid == parent->loop_b)
	{
		NscStrLoop();

        } 
	else if(wid == parent->pause_b)
	{
		NscStrPause();
        } 
	else 
		printf ("invalid button pressed\n");	
}
/************************************************************************
 * Syntax       : net_file_cb(Fl_Widget* wid, void* p)             	*
 * Description  : this callback function for the file select components	*
 * Parameters   : pointer to the component on which action happened	*
 *			pointer to the parent of the component                *
 * Returns      : void                                                  *
 * Comment      : None                                                  *
 ************************************************************************/
void net_file_cb(Fl_Widget* wid, void* ptr)
{
	int index, loop, ret;
	char *buf;
	int itemsel;
	char* fname;
	char type;

	NetStrmGUI *parent = ptr;
	
	if (wid == parent->filelist)
	{
	/* check if a dir selected else set filename */

		itemsel = parent->filelist->value();
		fname = parent->filelist->text(itemsel);
		type = NscStrGetFileType(itemsel);
		if(type ==  TYPE_DIR)
		{
			if (strcmp(pathname, "\\") != 0)
			/* path other than root, add slashes */
				strcat(pathname, "\\");
			strcat(pathname, fname);
			if (0 == NscStrSetDirectory(pathname))
			{
				parent->filelist->clear();
				printf(" fail to set dir \n");
			}
			else
			{
				sleep(5);
				index = NscStrGetNoFiles();
				if( (buf = (char*)malloc(80*index)) == NULL)
				{
				   printf (" fail to alloc \n");
				}
                   
				NscStrGetFileList(buf, index*80);
				if (index != 0)
				   parent->filelist->clear();
				
				for (loop=0; loop<index; loop++)
				{
			      	   parent->filelist->insert(loop, buf+(80*loop), 					0);				      
				   printf("buf = %s ",buf+(80*loop));
			
				}
				free(buf);
			}
		}
		else
			parent->file_ok_b->show(); 
		
	}	
	if (wid == parent->file_show_b)
	{
		printf ("get file list & display\n");
		index = NscStrGetNoFiles();
                printf(" no of rec = %d\n ", index);
		buf = (char*)malloc(80*index);
		NscStrGetFileList(buf, index*80);
		/* initialize the file list dialog */
		for (loop=0; loop<index; loop++)
		{
			parent->filelist->insert(loop, buf+(80*loop), 0);
		}
                free(buf);
	}
	else if(wid == parent->file_ok_b)
	{
		char* fname;
		int lineno;
		lineno = parent->filelist->value();
		printf ("selected line %d\n", lineno);
		if (lineno == 0)
			lineno = 1;
		fname = parent->filelist->text(lineno);
		strcat(pathname, "\\");
		strcat(pathname, fname);
		printf ("file name = %s\n", fname);
		ret = NscStrSelectFile(pathname);
		ret = NscStrOpenCurrentFile();
	}
	else
		printf ("invalid button pressed\n");
}
