/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 * File Name      : tv_screen.cc                           *
 *                                                         *
 * Project Name   : ASTB                                   *
 *                                                         *
 * Module Name    : Integrated Test App                    *
 *                                                         *
 * Author Name    : Sojan James                            *
 *                                                         *
 * Language       : C++                                    *
 *                                                         *
 * Date           :                                        *
 *
 * Description    : Source file for tv_screen class 	   *
		    simulation.
 **********************************************************/

#include "tv_screen.h"
#include "includes.h"

/*************************************************************
 * Syntax       :       TV_Screen(int,int,int,int,label)     *
 *				int,int,int)        	     *
 *                                                           *
 * Description  :       Class constructor 		     *
 *                                                           *
 * Parameters   :       x,y
 *                      - Starting x and y co-ordinates      *
 *			w,h				     *
 *			- Width and height of the screen     *
 *			label				     *
 *			- label for the widget, by default 0 *
 * Comment      :       None                                 *
 ************************************************************/
TV_Screen::TV_Screen(int x, int y, int w, int h, char * label = 0):Fl_Widget(x,y,w,h,label)
{
	image_data = 0; level = 0;
}

void TV_Screen::formTile(TV_Screen* t)
{
	screen_tile = new Fl_Tile(image_x,image_y+image_height,
			image_width,image_height);
	tv_msg_output= new Fl_Output(image_x-310,image_height+380-30,
			image_width+310,30);
	tv_msg_output->type(FL_DOWN_BOX);
	screen_tile->add(t);
	screen_tile->add(tv_msg_output);
	screen_tile->resizable(tv_msg_output);
	screen_tile->resizable(t);
	screen_tile->end();
}
/*************************************************************
 * Syntax       :       setImage(const unsigned char*, int,  *
 *				int,int,int)        	     *
 *                                                           *
 * Description  :       The image position widht and height  *
 *                      has been set for video display       *
 *                                                           *
 * Parameters   :       data
 *                      - data holds the image to be display *
 *		 	x,y				     *
 *			- starting x and y co-ordinates      *
 *			w,h				     *
 *			- width and height of the image      *
 * Comment      :       None                                 *
 ************************************************************/
void TV_Screen::setImage(const unsigned char * data, int w, int h,int x,int y) 
{
	image_data = data;
	image_height = h;
	image_width = w;
	image_x = x;
	image_y = y;
}

/*************************************************************
 * Syntax       :       displayImage()			     *
 *                                                           *
 * Description  :       Display the image in the tvscreen at *
 *                      a user specified location            *
 *                                                           *
 * Parameters   :       None     
 * Comment      :       None                                 *
 ************************************************************/
void TV_Screen::displayImage()
{
	if(image_data)
	fl_draw_image(image_data,image_x,image_y,image_width,image_height);	
}	


void TV_Screen::setLevel(int lev)
{
	if(lev > 0 && lev <= 100)
		level = lev;
}


void TV_Screen::setBackground(unsigned char r,unsigned char g, unsigned
char b)
{
	bgcol = 128;
	Fl::get_color(bgcol,r,g,b);
}


void TV_Screen::draw()
{
	fl_rectf(x(),y(),w(),h(),255,0,255);
	drawLabels();
}


void TV_Screen::setLabel(const char * str,int lab)
{
	label[lab] = str;	
}

void TV_Screen::setLabelAttrib(Fl_Color clr, int fontface, int fontsize,int lab)
{
	labelColor[lab] = clr;
	face[lab] = fontface;
	size[lab] = fontsize;
}

void TV_Screen::setLabelPos(int x, int y, int lab)
{
	labelx[lab] = x;
	labely[lab] = y;
}

void TV_Screen::drawLabels()
{
	Fl_Color old_color = fl_color();
	int oldface = fl_font(),oldsize=fl_size();
	for(int i = 0 ; i < TV_MAX_LABELS ; i++)
	{
		if(label[i] != "")
		{
			fl_color(labelColor[i]);
			fl_font(face[i],size[i]);

		fl_draw	(label[i].c_str(), label[i].size(),
		(float)(labelx[i]+x()),(float)(labely[i]+y()) );
		}
	}
	// restore the color, font and font size to what it was
	fl_color(old_color);
	fl_font(oldface,oldsize);
}

