/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
 * File Name      : tv_tuner_gui.cc                        *
 *                                                         *
 * Project Name   : ASTB                                   *
 *                                                         *
 * Module Name    : Tv_Tuner				   *
 *                                                         *
 * Author Name    : Lakshmi B G                            *
 *                                                         *
 * Language       : C++                                    *
 *                                                         *
 * Date           : 					   *
 *                                                         *
 * Description	  :					   *
 *		    This file defines the functions of 	   *
 *		    TV_Tuner class 			   *
 **********************************************************/	

#include <iostream.h>
#include <anlg_video.h>

#include "tv_tuner_gui.h"
//#include "newtrial.cc"
#include "vbi.h"

/*************************************************************
 * Syntax       :       TV_Tuner(int,int,int,int)            *
 *                                                           *
 * Description  :       TV_Tuner class constructor     	     *
 *                                                           *
 * Parameters   :       x,y				     *
 *                      - Starting x and y positions of the  *
 *			  gui				     *
 *			w,h				     *
 *			- Width and height of the gui 	     *
 * Comment      :       None                                 *
 ************************************************************/
TV_Tuner::TV_Tuner(int x,int y,int w,int h):Fl_Box(x,y,w,h)
{
	sx = x; wid = w;
        sy = y; hei = h;
        tuner_create_gui(x,y,w,h);
        hide();
}

/*************************************************************
 * Syntax       :       tv_tuner_cb(Fl_Widget*, void*)	     *
 *                                                           *
 * Description  :       TV_Tuner class callback		     *
 *                                                           *
 * Parameters   :       w				     *
 *			- widget which has been activated    *
 *			d				     *
 *			- Class component to which widget    *
 *			  belongs to.			     *
 * Comment      :       None                                 *
 ************************************************************/
/* Functionality of all callback functions are done here */
void tv_tuner_cb(Fl_Widget* w, void* d)
{
	TV_Tuner* t = d;
	int input;
	if(w == t->channel_select)
	{
		t->tuner_key->show();
		input = VIDEO_INPUT_TUNER;
		if(VMW_AVS_SetCurrentVideoInput(&input) != 0)
		{
			printf("Error in setting up the video input for tuner\n");
		}
		t->channel_select->clear();
	}
	else
	if(w == t->done_button)
	{
		t->hide();
	}

	else
	if(w == t->svideo_button)
	{
		t->vcp_group->hide();
		input = VIDEO_INPUT_SVIDEO;
		SetAudio(AV_CHAN_SVIDEO,0,0);
		if(VMW_AVS_SetCurrentVideoInput(&input) != 0)
		{
			printf("Error in setting up the video input for svideo\n");
		}
		SetAudio(AV_CHAN_SVIDEO,0,1);
	}
	else
	if(w == t->comp_button)
	{
		t->vcp_group->hide();
		input = VIDEO_INPUT_COMPOSITE;
		if(VMW_AVS_SetCurrentVideoInput(&input) != 0)
		{
			printf("Error in setting up the video input for composite\n");
		}
		SetAudio(AV_CHAN_VIDEO,0,1);
	}
	else
	if(w == t->vbi_button)
	{
		t->vcp_group->hide();
		VMW_AVV_SetupVBICallback(user_callback, user_buf, 4, user_buf);
		VMW_AVV_EnableVBICapture(1);
	}
}

/*************************************************************
 * Syntax       :       tuner_create_gui(Fl_Widget*, void*)  *
 *                                                           *
 * Description  :       TV_Tuner class callback		     *
 *                                                           *
 * Parameters   :       w				     *
 *			- widget which has been activated    *
 *			d				     *
 *			- Class component to which widget    *
 *			  belongs to.			     *
 * Comment      :       None                                 *
 ************************************************************/
void TV_Tuner::tuner_create_gui(int x,int y,int w,int h)
{
	
	tuner_grp = new Fl_Group(x+5,y+30,w-10,h);
	tuner_grp->begin();
		channel_select = new Fl_Button(x+15,y+30,80,20);
		channel_select->labelfont(FL_COURIER_BOLD_ITALIC);
		channel_select->label("Ch Select");
		channel_select->show();
		tuner_key = new Keypad_Tun(x+5,y+50,w-20,75);
	tuner_grp->end();

	vcp_group = new Fl_Group(x+5,y+30,w-10,h);
	vcp_group->begin();
		svideo_button = new Fl_Button(x+20,y+45,80,20);
		svideo_button->labelfont(FL_COURIER_BOLD);
		svideo_button->label("Svideo");
		comp_button = new Fl_Button(x+20,y+80,80,20,"Composite");
		comp_button->labelfont(FL_COURIER_BOLD);
		comp_button->label("Composite");
		vbi_button = new Fl_Button(x+20,y+120,80,20);
		vbi_button->labelfont(FL_COURIER_BOLD);
		vbi_button->label("VBI");
	vcp_group->end();
	done_button = new Fl_Button(w/2-10,h-30,30,20);
	done_button->labelfont(FL_COURIER_BOLD_ITALIC);
	done_button->label("Ok");
	done_button->callback(tv_tuner_cb,this);
	svideo_button->callback(tv_tuner_cb,this);
	comp_button->callback(tv_tuner_cb,this);
	vbi_button->callback(tv_tuner_cb,this);
	channel_select->callback(tv_tuner_cb,this);
}
/*************************************************************
 * Syntax       :       hide()				     *
 *                                                           *
 * Description  :       hides all the buttons in the gui     *
 *                                                           *
 * Parameters   :       None				     *
 *			             			     *
 * Comment      :       None                                 *
 ************************************************************/
void TV_Tuner::hide()
{
	Fl_Box::hide();
	tuner_grp->hide();
	vcp_group->hide();
	done_button->hide();
}

/*************************************************************
 * Syntax       :       show()				     *
 *                                                           *
 * Description  :       shows all the buttons in the gui     *
 *                                                           *
 * Parameters   :       None				     *
 *			             			     *
 * Comment      :       None                                 *
 ************************************************************/
void TV_Tuner::show(Fl_Widget* wid)
{
	Fl_Box::show();
	if(wid == 0)
		tuner_grp->show();
	else
		vcp_group->show();	
	done_button->show();
}	

