/*
 *
 * National Semiconductor Corporation Open Source License for 
 *
 * National Semiconductor VidTest4linux for Microwindows
 *
 * (BSD License with Export Notice)
 *
 * Copyright (c)  2001
 * National Semiconductor Corporation.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   * Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *
 *   * Redistributions in binary form must reproduce the above 
 *     copyright notice, this list of conditions and the following 
 *     disclaimer in the documentation and/or other materials provided 
 *     with the distribution. 
 *
 *   * Neither the name of the National Semiconductor Corporation nor 
 *     the names of its contributors may be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * NATIONAL SEMICONDUCTOR CORPORATION OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE,
 * INTELLECTUAL PROPERTY INFRINGEMENT, OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF 
 * YOUR JURISDICTION. It is licensee's responsibility to comply with 
 * any export regulations applicable in licensee's jurisdiction. Under 
 * CURRENT (2001) U.S. export regulations this software 
 * is eligible for export from the U.S. and can be downloaded by or 
 * otherwise exported or reexported worldwide EXCEPT to U.S. embargoed 
 * destinations which include Cuba, Iraq, Libya, North Korea, Iran, 
 * Syria, Sudan, Afghanistan and any other country to which the U.S. 
 * has embargoed goods and services. 
*/

#include <anlg_video.h>

volatile int	decoder_cc_enabled = 0;
volatile int	encoder_cc_enabled = 0;
unsigned char	encoder_cc_buf[ 64 ];
unsigned char	decoder_cc_screen[ 8 ][ 32 ];
unsigned char	decoder_cc_off_screen[ 8 ][ 32 ];

#define RAW_VBI_LEN		(720*10)

int parse_cc( unsigned char cc1, unsigned char cc2 )
{  
	static unsigned char	offset = 0;
	static unsigned char	row = 0;
	static unsigned char	colidx = 0;
	int		update_screen = 0;
	int		i = 0;
	
	
	if( colidx > 32 )
	{
		colidx = 32;
	}
	
	cc1 &= 0x7F;
	cc2 &= 0x7F;
	
	//	check for row codes
	if( ( cc1 >= 0x11 ) && ( cc1 <= 0x14 ) && ( cc2 & 0x40 ) )
{
		offset = ( cc2 & 0x20 ) ? 1 : 0;
		colidx = 0;
		switch( cc1 )
		{
			case 0x11:
				row = offset;
				break;
			case 0x12:
				row = offset + 2;
				break;
			case 0x13:
				row = offset + 4;
				break;
			case 0x14:
				row = offset + 6;
				break;
		}
		return update_screen ;
	}
	
	if( ( cc1 != 0x11 ) && ( cc1 != 0x14 ) && ( cc1 != 0x17 ) )
	{
		//	regular data (text) byte
		if( cc1 )
		{
			decoder_cc_off_screen[ row ][ colidx++ ] = (char)cc1;
		}
		if( cc2 )
		{
			decoder_cc_off_screen[ row ][ colidx++ ] = (char)cc2;
		}
		return update_screen;
	}
	
	switch( cc1 )
	{
		case 0x11:
		case 0x19:
			//	attribute
			switch( cc2 )
			{
				case 0x20:
					//	white
					break;
				case 0x21:
					//	white underline
					break;
				case 0x22:
					//	green
					break;
				case 0x23:
					//	green underline
					break;
				case 0x24:
					//	blue
					break;
				case 0x25:
					//	blue underline
					break;
				case 0x26:
					//	cyan
					break;
				case 0x27:
					//	cyan underline
					break;
				case 0x28:
					//	red
					break;
				case 0x29:
					//	red underline
					break;
				case 0x2A:
					//	yellow
					break;
				case 0x2B:
					//	yellow underline
					break;
				case 0x2C:
					//	magenta
					break;
				case 0x2D:
					//	magenta underline
					break;
				case 0x2E:
					//	italics
					decoder_cc_off_screen[ row ][ colidx++ ] = ' ';
					break;
				case 0x2F:
					//	italics underline
					break;
			}
			break;
		case 0x14:
		case 0x1C:
			switch( cc2 )
			{
				case 0x20:
					for( i = 0; i < 32; i++ )
					{
						//	start of caption
						decoder_cc_off_screen[ 0 ][ i ] = 0x00;
						decoder_cc_off_screen[ 1 ][ i ] = 0x00;
						decoder_cc_off_screen[ 2 ][ i ] = 0x00;
						decoder_cc_off_screen[ 3 ][ i ] = 0x00;
						decoder_cc_off_screen[ 4 ][ i ] = 0x00;
						decoder_cc_off_screen[ 5 ][ i ] = 0x00;
						decoder_cc_off_screen[ 6 ][ i ] = 0x00;
						decoder_cc_off_screen[ 7 ][ i ] = 0x00;
					}
					break;
				case 0x21:
					//	backspace
					break;
				case 0x24:
					//	delete to end of row
					//printf("<delete to end of row>\n");
					break;
				case 0x25:
					//	roll-up captions 2 rows
					//printf("<roll-up captions 2 rows>\n");
					break;
				case 0x26:
					//	roll-up captions 3 rows
					//printf("<roll-up captions 3 rows>\n");
					break;
				case 0x27:
					//	roll-up captions 4 rows
					//printf("<roll-up captions 4 rows>\n");
					break;
				case 0x28:
					//	flash on
					break;
				case 0x29:
					//	resume direct captioning
					//printf("<resume direct captioning>\n");
					break;
				case 0x2A:
					//	text restart
					//printf("<text restart>\n");
					break;
				case 0x2B:
					//	resume text display
					//printf("<resume text display>\n");
					break;
				case 0x2C:
					update_screen = 1;
					for( i = 0; i < 32; i++ )
					{
						//	clear previous caption
						decoder_cc_screen[ 0 ][ i ] = 0x00;
						decoder_cc_screen[ 1 ][ i ] = 0x00;
						decoder_cc_screen[ 2 ][ i ] = 0x00;
						decoder_cc_screen[ 3 ][ i ] = 0x00;
						decoder_cc_screen[ 4 ][ i ] = 0x00;
						decoder_cc_screen[ 5 ][ i ] = 0x00;
						decoder_cc_screen[ 6 ][ i ] = 0x00;
						decoder_cc_screen[ 7 ][ i ] = 0x00;
					}
					break;
				case 0x2E:
					//	erase non-displayed memory
					break;
				case 0x2D:
					//	carriage return
					//	fall through
					//printf("<carriage return>\n");
				case 0x2F:
					update_screen = 1;
					for( i = 0; i < 32; i++ )
					{
						//	display caption
						decoder_cc_screen[ 0 ][ i ] = decoder_cc_off_screen[ 0 ][ i ];
						decoder_cc_screen[ 1 ][ i ] = decoder_cc_off_screen[ 1 ][ i ];
						decoder_cc_screen[ 2 ][ i ] = decoder_cc_off_screen[ 2 ][ i ];
						decoder_cc_screen[ 3 ][ i ] = decoder_cc_off_screen[ 3 ][ i ];
						decoder_cc_screen[ 4 ][ i ] = decoder_cc_off_screen[ 4 ][ i ];
						decoder_cc_screen[ 5 ][ i ] = decoder_cc_off_screen[ 5 ][ i ];
						decoder_cc_screen[ 6 ][ i ] = decoder_cc_off_screen[ 6 ][ i ];
						decoder_cc_screen[ 7 ][ i ] = decoder_cc_off_screen[ 7 ][ i ];
					}
					break;
				default:
					break;
			}
			break;
		case 0x17:
		case 0x1F:
			switch( cc2 )
			{
				case 0x21:
					//	tab offset 1 column
					break;
				case 0x22:
					//	tab offset 2 columns
					break;
				case 0x23:
					//	tab offset 3 columns
					break;
			}
			break;
	}
	return update_screen;
}

/*
	This is *very* simple.  It does not support color, italics,
	special characters, scrolling, etc.  It is based on a subset
	of the FCC telecommunications code Title 47, Volume 1, Part 15.
	HINT: It will render PBS CC well.
*/

void render_cc()
{
	char		row_buf[ 32 ];
	int		i;
	
	/* render the rows */
	for( i = 0; i < 8; i++ )
	{
		memset( row_buf, 0, sizeof(row_buf) );
		strncpy( row_buf, (const char*)decoder_cc_screen[ i ], 32 );
		if( strlen(row_buf) != 0 )
		{
			printf( "%s\n", row_buf );
		}
	}
}

int decode_cc( int vbi ) 
{
	__u8		vbi_data[ 12 ];
	int		len;
	int i = 0;

	memset( &vbi_data, 0, sizeof(vbi_data) );

	len = read( vbi, vbi_data, 12 );

	if( len < 0 )
	{
//		printf("failed to read\n");
	}
	else
	{
		for(i = 2 ; i <= 10; i +=4) {

	//	printf("read ok\n");
			if( ( vbi_data[i] != 0x00 ) && ( vbi_data[i+1] != 0x00 ) )
			{
#if 0
				// This is used to debug order data received.
				printf("%c%c\n", vbi_data[2]&0x7f, vbi_data[3]&0x7f);
#else
				if( parse_cc( vbi_data[i], vbi_data[i+1] ) )
				{
					render_cc();
				}
#endif
			}
		}
	}
	return 1;
}




unsigned char user_buf[15];

void user_callback(int len, void* ptr){
	char *data = (char*)ptr;
	int i;
	for(i = 2 ; i <= len; i +=4) {

		if( ( data[i] != 0x00 ) && ( data[i+1] != 0x00 ) )
		{
#if 0
			// This is used to debug order data received.
			printf("%c%c\n", data[2]&0x7f, data[3]&0x7f);
#else
			if( parse_cc( data[i], data[i+1] ) )
			{
				render_cc();
			}
#endif
		}
	}
}

/*
void main()
{
	int input, enable;
	VIDEO_FORMAT fmt;
	enable =1;
		fmt.fmt.vbi.sampling_rate = 60;
                fmt.fmt.vbi.offset = 0;
                fmt.fmt.vbi.samples_per_line = 2;
                fmt.fmt.vbi.sample_format =  GEODE_V4L2_VBI_SF_CC_US;
                fmt.fmt.vbi.start[0] = 21;
                fmt.fmt.vbi.start[1] = 284;
                fmt.fmt.vbi.count[0] = 1;
                fmt.fmt.vbi.count[1] = 1;
                fmt.fmt.vbi.flags = 0;

	input = VIDEO_INPUT_COMPOSITE;
	VMW_AVS_VideoInit();
	VMW_AVS_SetVideoWindow(0,0,720,480);
	VMW_OVL_EnableOverlay(0x0000f000f0,0x0000f0f0f0,0);
 	VMW_OVL_EnableAlphaBlendWindow(0,
                                          0, 720, 480, 150, 0, 0);
	VMW_AVS_SetCurrentVideoInput(&input);
	printf ("input set to composite\n");
	VMW_AVV_SetVBIFormat(&fmt);
	VMW_AVV_SetupVBICallback(&user_callback, user_buf, 4, user_buf);
	VMW_AVV_EnableVBICapture(enable);
	getchar();
	VMW_AVS_SetVideoWindow(20, 20, 300, 300);
 	VMW_OVL_EnableAlphaBlendWindow(20, 20, 300, 300, 150, 0, 0);
	input = VIDEO_INPUT_SVIDEO;
	VMW_AVS_SetCurrentVideoInput(&input);
	printf ("input set to Svideo, press enter to play mpeg \n");
	getchar();
	enable = 0;
	VMW_AVV_EnableVBICapture(enable);
	input = VIDEO_INPUT_MPEG_DECODER;
	VMW_AVS_SetCurrentVideoInput(&input);
	printf("input set to mpeg, press enter to exit\n");
	getchar();
	VMW_AVS_VideoClose();
}
*/	


